@@ -3,8 +3,8 @@
  * Module:        Binary array module
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.35 $
- * Last edited:   $Date: 2003/10/09 18:08:39 $
+ * Revision:      $Revision: 1.39 $
+ * Last edited:   $Date: 2004/11/07 00:38:39 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -303,7 +303,7 @@ Gan_Bool
  * array.
  */
 Gan_BitArray *
- gan_bit_array_invert_s ( Gan_BitArray *ba )
+ gan_bit_array_invert_s ( const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
    Gan_BitArray *ba_dst;
@@ -331,7 +331,7 @@ Gan_BitArray *
  * writing the result into \a ba_dst.
  */
 Gan_Bool
- gan_bit_array_and_i ( Gan_BitArray *ba_dst, Gan_BitArray *ba )
+ gan_bit_array_and_i ( Gan_BitArray *ba_dst, const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
 
@@ -358,7 +358,7 @@ Gan_Bool
  * \a ba_dst, writing the result into \a ba_dst.
  */
 Gan_Bool
- gan_bit_array_nand_i ( Gan_BitArray *ba_dst, Gan_BitArray *ba )
+ gan_bit_array_nand_i ( Gan_BitArray *ba_dst, const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
 
@@ -386,7 +386,7 @@ Gan_Bool
  * writing the result into \a ba_dst.
  */
 Gan_Bool
- gan_bit_array_or_i ( Gan_BitArray *ba_dst, Gan_BitArray *ba )
+ gan_bit_array_or_i ( Gan_BitArray *ba_dst, const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
 
@@ -413,7 +413,7 @@ Gan_Bool
  * \a ba_dst, writing the result into \a ba_dst.
  */
 Gan_Bool
- gan_bit_array_eor_i ( Gan_BitArray *ba_dst, Gan_BitArray *ba )
+ gan_bit_array_eor_i ( Gan_BitArray *ba_dst, const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
 
@@ -440,7 +440,7 @@ Gan_Bool
  * \a ba_dst, writing the result into \a ba_dst.
  */
 Gan_Bool
- gan_bit_array_andnot_i ( Gan_BitArray *ba_dst, Gan_BitArray *ba )
+ gan_bit_array_andnot_i ( Gan_BitArray *ba_dst, const Gan_BitArray *ba )
 {
    unsigned int ctr = ba->no_words;
 
@@ -467,7 +467,7 @@ Gan_Bool
  * returning the result as a new bit array.
  */
 Gan_BitArray *
- gan_bit_array_and_s  ( Gan_BitArray *ba1, Gan_BitArray *ba2 )
+ gan_bit_array_and_s  ( const Gan_BitArray *ba1, const Gan_BitArray *ba2 )
 {
    Gan_BitArray *ba_dst;
    unsigned int ctr = ba1->no_words;
@@ -503,7 +503,7 @@ Gan_BitArray *
  * \a ba2, returning the result as a new bit array.
  */
 Gan_BitArray *
- gan_bit_array_nand_s ( Gan_BitArray *ba1, Gan_BitArray *ba2 )
+ gan_bit_array_nand_s ( const Gan_BitArray *ba1, const Gan_BitArray *ba2 )
 {
    Gan_BitArray *ba_dst;
    unsigned int ctr = ba1->no_words;
@@ -540,7 +540,7 @@ Gan_BitArray *
  * returning the result as a new bit array.
  */
 Gan_BitArray *
- gan_bit_array_or_s   ( Gan_BitArray *ba1, Gan_BitArray *ba2 )
+ gan_bit_array_or_s ( const Gan_BitArray *ba1, const Gan_BitArray *ba2 )
 {
    Gan_BitArray *ba_dst;
    unsigned int ctr = ba1->no_words;
@@ -576,7 +576,7 @@ Gan_BitArray *
  * \a ba2, returning the result as a new bit array.
  */
 Gan_BitArray *
- gan_bit_array_eor_s  ( Gan_BitArray *ba1, Gan_BitArray *ba2 )
+ gan_bit_array_eor_s ( const Gan_BitArray *ba1, const Gan_BitArray *ba2 )
 {
    Gan_BitArray *ba_dst;
    unsigned int ctr = ba1->no_words;
@@ -612,7 +612,7 @@ Gan_BitArray *
  * \a ba2, returning the result as a new bit array.
  */
 Gan_BitArray *
- gan_bit_array_andnot_s ( Gan_BitArray *ba1, Gan_BitArray *ba2 )
+ gan_bit_array_andnot_s ( const Gan_BitArray *ba1, const Gan_BitArray *ba2 )
 {
    Gan_BitArray *ba_dst;
    unsigned int ctr = ba1->no_words;
@@ -652,8 +652,8 @@ Gan_BitArray *
  * Insert part of bit array \a src in bit array \a dst
  */
 Gan_Bool
- gan_bit_array_insert ( Gan_BitArray *src, unsigned int offset_src,
-                        Gan_BitArray *dst, unsigned int offset_dst, 
+ gan_bit_array_insert ( const Gan_BitArray *src, unsigned int offset_src,
+                              Gan_BitArray *dst, unsigned int offset_dst, 
                         unsigned int no_bits )
 {
    unsigned int lbits, ubits, lmask, umask,
@@ -1297,7 +1297,7 @@ Gan_Bool
  * Copy bit array \a ba_source to \a ba_dest.
  */
 Gan_Bool
- gan_bit_array_copy_q ( Gan_BitArray *ba_source, Gan_BitArray *ba_dest )
+ gan_bit_array_copy_q ( const Gan_BitArray *ba_source, Gan_BitArray *ba_dest )
 {
    /* make sure destination bit array can hold source bit array */
    if ( ba_dest->words_alloc < ba_source->no_words &&
@@ -1321,7 +1321,7 @@ Gan_Bool
  * Return a copy of bit array \a ba_source.
  */
 Gan_BitArray *
- gan_bit_array_copy_s ( Gan_BitArray *ba_source )
+ gan_bit_array_copy_s ( const Gan_BitArray *ba_source )
 {
    Gan_BitArray *ba_dest = gan_bit_array_alloc(ba_source->no_bits);
 
@@ -1346,8 +1346,8 @@ Gan_BitArray *
  * Expand bit array \a ba by padding it with zeros in a reference bit array
  * \a ref_ba.
  */
-Gan_Bool
- gan_bit_array_expand_q ( Gan_BitArray *ba, Gan_BitArray *ref_ba,
+Gan_BitArray *
+ gan_bit_array_expand_q ( const Gan_BitArray *ba, const Gan_BitArray *ref_ba,
                           Gan_BitArray *exp_ba )
 {
    unsigned int i, j;
@@ -1359,7 +1359,7 @@ Gan_Bool
       if ( exp_ba == NULL )
       {
          gan_err_register ( "gan_bit_array_expand_q", GAN_ERROR_FAILURE, "" );
-         return GAN_FALSE;
+         return NULL;
       }
    }
    else
@@ -1368,7 +1368,7 @@ Gan_Bool
            !gan_bit_array_set_size ( exp_ba, ref_ba->no_bits ) )
       {
          gan_err_register ( "gan_bit_array_expand_q", GAN_ERROR_FAILURE, "" );
-         return GAN_FALSE;
+         return NULL;
       }
    
    gan_bit_array_fill ( exp_ba, GAN_FALSE );
@@ -1387,11 +1387,11 @@ Gan_Bool
       gan_err_flush_trace();
       gan_err_register ( "gan_bit_array_expand_q", GAN_ERROR_INCOMPATIBLE,
                          "" );
-      return GAN_FALSE;
+      return NULL;
    }
 
    /* success */
-   return GAN_TRUE;
+   return exp_ba;
 }
 
 /**
@@ -1404,12 +1404,11 @@ Gan_Bool
  * \a ref_ba.
  */
 Gan_BitArray *
- gan_bit_array_expand_s ( Gan_BitArray *ba, Gan_BitArray *ref_ba )
+ gan_bit_array_expand_s ( const Gan_BitArray *ba, const Gan_BitArray *ref_ba )
 {
    Gan_BitArray *exp_ba = gan_bit_array_alloc(ref_ba->no_bits);
 
-   if ( ref_ba == NULL ||
-        !gan_bit_array_expand_q ( ba, ref_ba, exp_ba ) )
+   if ( ref_ba == NULL || gan_bit_array_expand_q ( ba, ref_ba, exp_ba ) == NULL )
    {
       gan_err_register ( "gan_bit_array_expand_s", GAN_ERROR_FAILURE, "" );
       return NULL;
@@ -1419,7 +1418,325 @@ Gan_BitArray *
    return exp_ba;
 }
 
- /**
+static Gan_Bool
+ dilate_1bit ( Gan_BitArray *ba )
+{
+   unsigned bit;
+   int ctr = (int)ba->no_words-1;
+   Gan_BitWord word, end_bit=0, next_end_bit;
+
+   if ( ctr < 0 ) return GAN_TRUE;
+   
+   bit = ba->no_bits % GAN_BITWORD_SIZE;
+   if ( bit > 0 )
+   {
+      word = ba->data[ctr];
+
+      /* clear last bit before combining */
+#ifdef WORDS_BIGENDIAN
+      next_end_bit = word & GAN_MSB_SET;
+      end_bit = GAN_MSB_SET >> bit;
+#else /* #ifndef WORDS_BIGENDIAN */
+      next_end_bit = word & GAN_LSB_SET;
+      end_bit = GAN_LSB_SET << bit;
+#endif
+      if ( word & end_bit ) word ^= end_bit;
+      ba->data[ctr] |= (word << 1) | (word >> 1);
+#ifdef WORDS_BIGENDIAN
+      end_bit = ba->data[ctr] & GAN_MSB_SET;
+      if ( ctr > 0 && (ba->data[ctr-1] & GAN_LSB_SET) )
+         ba->data[ctr] |= GAN_MSB_SET;
+#else /* #ifndef WORDS_BIGENDIAN */
+      end_bit = ba->data[ctr] & GAN_LSB_SET;
+      if ( ctr > 0 && (ba->data[ctr-1] & GAN_MSB_SET) )
+         ba->data[ctr] |= GAN_LSB_SET;
+#endif
+      end_bit = next_end_bit;
+      ctr--;
+   }
+
+   for ( ; ctr >= 0; ctr-- )
+   {
+      word = ba->data[ctr];
+#ifdef WORDS_BIGENDIAN
+      next_end_bit = word & GAN_MSB_SET;
+      ba->data[ctr] = (word << 1) | word | (word >> 1);
+      if ( end_bit ) ba->data[ctr] |= GAN_LSB_SET;
+      if ( ctr > 0 && (ba->data[ctr-1] & GAN_LSB_SET) )
+         ba->data[ctr] |= GAN_MSB_SET;
+#else /* #ifndef WORDS_BIGENDIAN */
+      next_end_bit = word & GAN_LSB_SET;
+      ba->data[ctr] = (word << 1) | word | (word >> 1);
+      if ( end_bit ) ba->data[ctr] |= GAN_MSB_SET;
+      if ( ctr > 0 && (ba->data[ctr-1] & GAN_MSB_SET) )
+         ba->data[ctr] |= GAN_LSB_SET;
+#endif
+      end_bit = next_end_bit;
+   }
+
+   /* success */
+   return GAN_TRUE;
+   
+}
+
+/**
+ * \brief Dilate bit array and write result into another
+ * \param ba_source Input bit array
+ * \param no_bits Number of bits to dilate bit array
+ * \param ba_dest Result bit array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Dilate input bit array \a ba_source by \a no_bits, and write result into
+ * another bit array \a ba_dest.
+ */
+Gan_Bool
+ gan_bit_array_dilate_q ( Gan_BitArray *ba_source, unsigned int no_bits,
+                          Gan_BitArray *ba_dest )
+{
+   int bit;
+
+   if ( ba_source != ba_dest &&
+        !gan_bit_array_copy_q ( ba_source, ba_dest ) )
+   {
+      gan_err_register ( "gan_bit_array_dilate_q", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   for ( bit = (int)no_bits-1; bit >= 0; bit-- )
+      if ( !dilate_1bit ( ba_dest ) )
+      {
+         gan_err_register ( "gan_bit_array_dilate_q", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Dilate bit array in-place
+ * \param ba Input bit array
+ * \param no_bits Number of bits to dilate bit array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Dilate input bit array \a ba by \a no_bits in-place.
+ */
+Gan_BitArray *
+ gan_bit_array_dilate_s ( Gan_BitArray *ba, unsigned int no_bits )
+{
+   Gan_BitArray *ba_dest = gan_bit_array_alloc(ba->no_bits);
+
+   if ( ba_dest == NULL || !gan_bit_array_dilate_q ( ba, no_bits, ba_dest ) )
+   {
+      gan_err_register ( "gan_bit_array_dilate_s", GAN_ERROR_FAILURE, "" );
+      return NULL;
+   }
+
+   /* success */
+   return ba_dest;
+}
+
+static Gan_Bool
+ shift_right ( Gan_BitArray *ba, unsigned int no_bits )
+{
+   unsigned int no_words, word;
+
+   // shift whole words first
+   no_words = no_bits / GAN_BITWORD_SIZE;
+   if(no_words > 0)
+   {
+      for(word=ba->no_words-1; word>=no_words; word--)
+         ba->data[word] = ba->data[word-no_words];
+
+      // pad first words with zero
+      for(word=0; word<no_words; word++)
+         ba->data[word] = 0;
+   }
+
+   // now deal with remaining bits
+   no_bits = no_bits % GAN_BITWORD_SIZE;
+   if(no_bits != 0)
+   {
+      Gan_BitWord rollover=0;
+
+      for(word=0; word<ba->no_words; word++)
+      {
+#ifdef WORDS_BIGENDIAN
+         // remember the bits being shifted out
+         Gan_BitWord new_rollover = ba->data[word] << (GAN_BITWORD_SIZE - no_bits);
+
+         // apply shift and insert any rolled over bits
+         ba->data[word] = (ba->data[word] >> no_bits) | rollover;
+#else
+         // remember the bits being shifted out
+         Gan_BitWord new_rollover = ba->data[word] >> (GAN_BITWORD_SIZE - no_bits);
+
+         // apply shift and insert any rolled over bits
+         ba->data[word] = (ba->data[word] << no_bits) | rollover;
+#endif         
+         // remember discarded bits for next word
+         rollover = new_rollover;
+      }
+   }
+   
+   return GAN_TRUE;
+}
+
+static Gan_Bool
+ shift_left ( Gan_BitArray *ba, unsigned int no_bits )
+{
+   unsigned int no_words, word;
+
+   // shift whole words first
+   no_words = no_bits / GAN_BITWORD_SIZE;
+   if(no_words > 0)
+   {
+      for(word=no_words; word<ba->no_words; word++)
+         ba->data[word-no_words] = ba->data[word];
+
+      // pad last words with zero
+      for(word=ba->no_words-no_words; word<ba->no_words; word++)
+         ba->data[word] = 0;
+   }
+
+   // now deal with remaining bits
+   no_bits = no_bits % GAN_BITWORD_SIZE;
+   if(no_bits != 0)
+   {
+      Gan_BitWord rollover=0;
+      int iword;
+
+      for(iword=(int)ba->no_words-1; iword >= 0; iword--)
+      {
+#ifdef WORDS_BIGENDIAN
+         // remember the bits being shifted out
+         Gan_BitWord new_rollover = ba->data[iword] >> (GAN_BITWORD_SIZE - no_bits);
+
+         // apply shift and insert any rolled over bits
+         ba->data[iword] = (ba->data[iword] << no_bits) | rollover;
+#else
+         // remember the bits being shifted out
+         Gan_BitWord new_rollover = ba->data[iword] << (GAN_BITWORD_SIZE - no_bits);
+
+         // apply shift and insert any rolled over bits
+         ba->data[iword] = (ba->data[iword] >> no_bits) | rollover;
+#endif         
+         // remember discarded bits for next word
+         rollover = new_rollover;
+      }
+   }
+   
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Shift bit array and write result into another
+ * \param ba_source Input bit array
+ * \param no_bits Number of bits to shift bit array
+ * \param ba_dest Result bit array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Shift input bit array \a ba_source by \a no_bits, and write result into
+ * another bit array \a ba_dest. If \a no_bits is >= 0,
+ * the shift is towards the end of the bit array, and vice veraa.
+ */
+Gan_Bool
+ gan_bit_array_shift_q ( Gan_BitArray *ba_source, int no_bits,
+                         Gan_BitArray *ba_dest )
+{
+   if ( ba_source != ba_dest &&
+        !gan_bit_array_copy_q ( ba_source, ba_dest ) )
+   {
+      gan_err_register ( "gan_bit_array_shift_q", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   /* if we're shifting the whole bit array out of itself, fill with zeros */
+   if(abs(no_bits) >= (int)ba_dest->no_bits)
+   {
+      if(!gan_bit_array_fill ( ba_dest, GAN_FALSE ))
+      {
+         gan_err_register ( "gan_bit_array_shift_q", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+
+      return GAN_TRUE;
+   }
+
+   if(no_bits >= 0)
+   {
+      if ( !shift_right ( ba_dest, (unsigned int)no_bits ) )
+      {
+         gan_err_register ( "gan_bit_array_shift_q", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+   }
+   else
+   {
+      if ( !shift_left ( ba_dest, (unsigned int)(-no_bits) ) )
+      {
+         gan_err_register ( "gan_bit_array_shift_q", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+   }
+
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Shift bit array in-place
+ * \param ba Input bit array
+ * \param no_bits Number of bits to shift bit array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Shift input bit array \a ba by \a no_bits in-place. If \a no_bits is >= 0,
+ * the shift is towards the end of the bit array, and vice veraa.
+ */
+Gan_BitArray *
+ gan_bit_array_shift_s ( Gan_BitArray *ba, int no_bits )
+{
+   Gan_BitArray *ba_dest = gan_bit_array_alloc(ba->no_bits);
+
+   if ( ba_dest == NULL || !gan_bit_array_shift_q ( ba, no_bits, ba_dest ) )
+   {
+      gan_err_register ( "gan_bit_array_shift_s", GAN_ERROR_FAILURE, "" );
+      return NULL;
+   }
+
+   /* success */
+   return ba_dest;
+}
+
+/**
+ * \brief Get the first set bit in a bit array
+ * \return The bit that is set or \c UINT_MAX if all are zero
+ */
+unsigned int
+ gan_bit_array_get_first_set_bit(Gan_BitArray *ba)
+{
+   unsigned int word;
+   int bit;
+
+   for(word=0; word<ba->no_words; word++)
+      if(ba->data[word] != 0)
+      {
+         /* find the bit */
+#ifdef WORDS_BIGENDIAN
+         for(bit = GAN_BITWORD_SIZE-1; bit >= 0; bit-- )
+            if(ba->data[word] & ( GAN_MSB_SET >> bit ))
+               return word*GAN_BITWORD_SIZE+bit;
+#else
+         for(bit = GAN_BITWORD_SIZE-1; bit >= 0; bit-- )
+            if(ba->data[word] & ( GAN_LSB_SET << bit ))
+               return word*GAN_BITWORD_SIZE+bit;
+#endif
+            
+      }
+
+   /* didn't find a bit */
+   return UINT_MAX;
+}
+
+/**
  * \brief Fill part of a bit array.
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  *
@@ -1636,7 +1953,7 @@ Gan_Bool
  * \brief Print bit array in ASCII to file.
  */
 void
- gan_bit_array_fprint ( FILE *fp, Gan_BitArray *ba, int indent )
+ gan_bit_array_fprint ( FILE *fp, const Gan_BitArray *ba, int indent )
 {
    unsigned int i;
 
