@@ -3,8 +3,8 @@
  * Module:        Common package test program
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.14 $
- * Last edited:   $Date: 2003/01/31 18:56:41 $
+ * Revision:      $Revision: 1.15 $
+ * Last edited:   $Date: 2004/11/07 00:38:39 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -211,6 +211,22 @@ static Gan_Bool
    return GAN_TRUE;
 }
 
+/* shift bit array */
+static Gan_Bool
+ safe_bit_array_shift ( Gan_BitArray *source, int no_bits,
+                        Gan_BitArray *dest )
+{
+   int i;
+
+   gan_bit_array_fill(dest, GAN_FALSE);
+   for ( i = 0; i < (int)source->no_bits; i++ )
+      if(i+no_bits >= 0 && i+no_bits<(int)source->no_bits &&
+         gan_bit_array_get_bit ( source, i ))
+         cu_assert ( gan_bit_array_set_bit ( dest, (unsigned)(i+no_bits) ) );
+
+   return GAN_TRUE;
+}
+
 /* fill part of bit array */
 static Gan_Bool
  safe_bit_array_fill_part ( Gan_BitArray *ba, unsigned int offset,
@@ -490,6 +506,22 @@ static Gan_Bool test_bit_array(void)
 #endif /* #if GAN_BITWORD_SIZE == 64 */
    cu_assert ( bit_arrays_equal ( b1, b1s ) );
 
+   /* test bit shifting */
+   {
+      int it, ishift;
+      for(it=0; it<100; it++)
+      {
+         cu_assert ( random_bit_array ( b1 ) );
+         cu_assert ( gan_bit_array_copy_q ( b1, b1s ) );
+
+         /* choose a random shift between -100 and 100 */
+         ishift = (gan_random() % 200) - 100;
+         cu_assert ( gan_bit_array_shift_q ( b1, ishift, b2 ) );
+         cu_assert ( safe_bit_array_shift ( b1s, ishift, b2s ) );
+         cu_assert ( bit_arrays_equal ( b2, b2s ) );
+      }
+   }
+         
    /* success */
    gan_bit_array_free_va ( b2s, b1s, b2, b1, NULL );
    return GAN_TRUE;
