@@ -3,8 +3,8 @@
  * Module:        Little and big-endian I/O for architecture independent I/O
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.4 $
- * Last edited:   $Date: 2002/05/16 08:43:10 $
+ * Revision:      $Revision: 1.10 $
+ * Last edited:   $Date: 2005/05/13 17:12:53 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -51,7 +51,7 @@
  * order.
  */
 Gan_Bool
- gan_fread_lendian_i16 ( FILE *fp, gan_i16 *ai16, size_t nitems )
+ gan_fread_lendian_i16 ( FILE *fp, gan_int16 *ai16, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -101,7 +101,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fwrite_lendian_i16 ( FILE *fp, gan_i16 *ai16, size_t nitems )
+ gan_fwrite_lendian_i16 ( FILE *fp, gan_int16 *ai16, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -152,7 +152,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fread_lendian_ui16 ( FILE *fp, gan_ui16 *aui16, size_t nitems )
+ gan_fread_lendian_ui16 ( FILE *fp, gan_uint16 *aui16, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -202,7 +202,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fwrite_lendian_ui16 ( FILE *fp, gan_ui16 *aui16, size_t nitems )
+ gan_fwrite_lendian_ui16 ( FILE *fp, gan_uint16 *aui16, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -256,7 +256,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fread_lendian_i32 ( FILE *fp, gan_i32 *ai32, size_t nitems )
+ gan_fread_lendian_i32 ( FILE *fp, gan_int32 *ai32, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -308,7 +308,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fwrite_lendian_i32 ( FILE *fp, gan_i32 *ai32, size_t nitems )
+ gan_fwrite_lendian_i32 ( FILE *fp, gan_int32 *ai32, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -360,7 +360,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fread_lendian_ui32 ( FILE *fp, gan_ui32 *aui32, size_t nitems )
+ gan_fread_lendian_ui32 ( FILE *fp, gan_uint32 *aui32, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -412,7 +412,7 @@ Gan_Bool
  * order.
  */
 Gan_Bool
- gan_fwrite_lendian_ui32 ( FILE *fp, gan_ui32 *aui32, size_t nitems )
+ gan_fwrite_lendian_ui32 ( FILE *fp, gan_uint32 *aui32, size_t nitems )
 {
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
@@ -455,62 +455,279 @@ Gan_Bool
 
 #endif /* #ifdef GAN_INT32 */
 
+#ifdef GAN_INT64
+
 /**
- * \brief Reads an array of floats from a file
+ * \brief Reads an array of signed 64-bit integers from a file
  * \param fp Pointer to a binary file opened for reading
- * \param af Array of floats
+ * \param ai64 Array of integers
  * \param nitems The number of integers to be read into the array
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  *
- * Reads an array of floats from a file in little-endian byte order.
+ * Reads an array of unsigned 64-bit integers from a file in little-endian byte
+ * order.
  */
 Gan_Bool
- gan_fread_lendian_f ( FILE *fp, float *af, size_t nitems )
+ gan_fread_lendian_i64 ( FILE *fp, gan_int64 *ai64, size_t nitems )
+{
+   /* reverse bytes if on big-endian architecture */
+#ifdef WORDS_BIGENDIAN
+   {
+      char ca[8];
+      int i;
+
+      for ( i = (int)nitems-1; i >= 0; i--, ai64++ )
+      {
+         /* read bytes */
+         if ( fread ( ca, 8, 1, fp ) != 1 )
+         {
+            gan_err_flush_trace();
+            gan_err_register ( "gan_fread_lendian_i64",
+                               GAN_ERROR_READING_FROM_FILE, "" );
+            return GAN_FALSE;
+         }
+
+         /* reverse bytes */
+         ((char *)ai64)[7] = ca[0];
+         ((char *)ai64)[6] = ca[1];
+         ((char *)ai64)[5] = ca[2];
+         ((char *)ai64)[4] = ca[3];
+         ((char *)ai64)[3] = ca[4];
+         ((char *)ai64)[2] = ca[5];
+         ((char *)ai64)[1] = ca[6];
+         ((char *)ai64)[0] = ca[7];
+      }
+   }
+#else
+   /* little-endian architecture: read bytes directly */
+   if ( fread ( ai64, 8, nitems, fp ) != nitems )
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_fread_lendian_i64",
+                         GAN_ERROR_READING_FROM_FILE, "" );
+      return GAN_FALSE;
+   }
+#endif
+
+   /* success */
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Writes an array of signed 64-bit integers to a file
+ * \param fp Pointer to a binary file opened for reading
+ * \param ai64 Array of integers
+ * \param nitems The number of integers to be written from the array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Writes an array of signed 64-bit integers to a file in little-endian byte
+ * order.
+ */
+Gan_Bool
+ gan_fwrite_lendian_i64 ( FILE *fp, gan_int64 *ai64, size_t nitems )
+{
+   /* reverse bytes if on big-endian architecture */
+#ifdef WORDS_BIGENDIAN
+   {
+      char ca[8];
+      int i;
+
+      for ( i = (int)nitems-1; i >= 0; i--, ai64++ )
+      {
+         /* build reversed bytes */
+         ca[0] = ((char *)ai64)[7];
+         ca[1] = ((char *)ai64)[6];
+         ca[2] = ((char *)ai64)[5];
+         ca[3] = ((char *)ai64)[4];
+         ca[4] = ((char *)ai64)[3];
+         ca[5] = ((char *)ai64)[2];
+         ca[6] = ((char *)ai64)[1];
+         ca[7] = ((char *)ai64)[0];
+
+         /* write reversed bytes */
+         if ( fwrite ( ca, 8, 1, fp ) != 1 )
+         {
+            gan_err_flush_trace();
+            gan_err_register ( "gan_fwrite_lendian_i64",
+                               GAN_ERROR_WRITING_TO_FILE, "" );
+            return GAN_FALSE;
+         }
+      }
+   }
+#else
+   /* little-endian architecture: write bytes directly */
+   if ( fwrite ( ai64, 8, nitems, fp ) != nitems )
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_fwrite_lendian_i64",
+                         GAN_ERROR_WRITING_TO_FILE, "" );
+      return GAN_FALSE;
+   }
+#endif
+
+   /* success */
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Reads an array of unsigned 64-bit integers from a file
+ * \param fp Pointer to a binary file opened for reading
+ * \param aui64 Array of integers
+ * \param nitems The number of integers to be read into the array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Reads an array of unsigned 64-bit integers from a file in little-endian byte
+ * order.
+ */
+Gan_Bool
+ gan_fread_lendian_ui64 ( FILE *fp, gan_uint64 *aui64, size_t nitems )
+{
+   /* reverse bytes if on big-endian architecture */
+#ifdef WORDS_BIGENDIAN
+   {
+      char ca[8];
+      int i;
+
+      for ( i = (int)nitems-1; i >= 0; i--, aui64++ )
+      {
+         /* read bytes */
+         if ( fread ( ca, 8, 1, fp ) != 1 )
+         {
+            gan_err_flush_trace();
+            gan_err_register ( "gan_fread_lendian_ui64",
+                               GAN_ERROR_READING_FROM_FILE, "" );
+            return GAN_FALSE;
+         }
+
+         /* reverse bytes */
+         ((char *)aui64)[7] = ca[0];
+         ((char *)aui64)[6] = ca[1];
+         ((char *)aui64)[5] = ca[2];
+         ((char *)aui64)[4] = ca[3];
+         ((char *)aui64)[3] = ca[4];
+         ((char *)aui64)[2] = ca[5];
+         ((char *)aui64)[1] = ca[6];
+         ((char *)aui64)[0] = ca[7];
+      }
+   }
+#else
+   /* little-endian architecture: read bytes directly */
+   if ( fread ( aui64, 8, nitems, fp ) != nitems )
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_fread_lendian_ui64",
+                         GAN_ERROR_READING_FROM_FILE, "" );
+      return GAN_FALSE;
+   }
+#endif
+
+   /* success */
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Writes an array of unsigned 64-bit integers to a file
+ * \param fp Pointer to a binary file opened for reading
+ * \param aui64 Array of integers
+ * \param nitems The number of integers to be written from the array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Writes an array of unsigned 64-bit integers to a file in little-endian byte
+ * order.
+ */
+Gan_Bool
+ gan_fwrite_lendian_ui64 ( FILE *fp, gan_uint64 *aui64, size_t nitems )
+{
+   /* reverse bytes if on big-endian architecture */
+#ifdef WORDS_BIGENDIAN
+   {
+      char ca[8];
+      int i;
+
+      for ( i = (int)nitems-1; i >= 0; i--, aui64++ )
+      {
+         /* build reversed bytes */
+         ca[0] = ((char *)aui64)[7];
+         ca[1] = ((char *)aui64)[6];
+         ca[2] = ((char *)aui64)[5];
+         ca[3] = ((char *)aui64)[4];
+         ca[4] = ((char *)aui64)[3];
+         ca[5] = ((char *)aui64)[2];
+         ca[6] = ((char *)aui64)[1];
+         ca[7] = ((char *)aui64)[0];
+
+         /* write reversed bytes */
+         if ( fwrite ( ca, 8, 1, fp ) != 1 )
+         {
+            gan_err_flush_trace();
+            gan_err_register ( "gan_fwrite_lendian_ui64",
+                               GAN_ERROR_WRITING_TO_FILE, "" );
+            return GAN_FALSE;
+         }
+      }
+   }
+#else
+   /* little-endian architecture: write bytes directly */
+   if ( fwrite ( aui64, 8, nitems, fp ) != nitems )
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_fwrite_lendian_ui64",
+                         GAN_ERROR_WRITING_TO_FILE, "" );
+      return GAN_FALSE;
+   }
+#endif
+
+   /* success */
+   return GAN_TRUE;
+}
+
+#endif /* #ifdef GAN_INT64 */
+
+#ifdef GAN_FLOAT32
+
+/**
+ * \brief Reads an array of 32-bit floats from a file
+ * \param fp Pointer to a binary file opened for reading
+ * \param af32 Array of floats
+ * \param nitems The number of values to be read into the array
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Reads an array of 32-bit floats from a file in little-endian byte order.
+ */
+Gan_Bool
+ gan_fread_lendian_f32 ( FILE *fp, gan_float32 *af32, size_t nitems )
 {
    
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
    {
-      char ca[SIZEOF_FLOAT];
+      char ca[4];
       int i;
 
-      for ( i = (int)nitems-1; i >= 0; i--, af++ )
+      for ( i = (int)nitems-1; i >= 0; i--, af32++ )
       {
          /* read bytes */
          if ( fread ( ca, sizeof(float), 1, fp ) != 1 )
          {
             gan_err_flush_trace();
-            gan_err_register ( "gan_fread_lendian_f",
+            gan_err_register ( "gan_fread_lendian_f32",
                                GAN_ERROR_READING_FROM_FILE, "" );
             return GAN_FALSE;
          }
 
          /* reverse bytes */
-#if (SIZEOF_FLOAT == 4)
-         ((char *)af)[3] = ca[0];
-         ((char *)af)[2] = ca[1];
-         ((char *)af)[1] = ca[2];
-         ((char *)af)[0] = ca[3];
-#elif (SIZEOF_FLOAT == 8)
-         ((char *)af)[7] = ca[0];
-         ((char *)af)[6] = ca[1];
-         ((char *)af)[5] = ca[2];
-         ((char *)af)[4] = ca[3];
-         ((char *)af)[3] = ca[4];
-         ((char *)af)[2] = ca[5];
-         ((char *)af)[1] = ca[6];
-         ((char *)af)[0] = ca[7];
-#else
-#error "unsupported float size"      
-#endif
+         ((char *)af32)[3] = ca[0];
+         ((char *)af32)[2] = ca[1];
+         ((char *)af32)[1] = ca[2];
+         ((char *)af32)[0] = ca[3];
       }
    }
 #else
    /* little-endian architecture: read bytes directly */
-   if ( fread ( af, sizeof(float), nitems, fp ) != nitems )
+   if ( fread ( af32, sizeof(gan_float32), nitems, fp ) != nitems )
    {
       gan_err_flush_trace();
-      gan_err_register ( "gan_fread_lendian_f",
+      gan_err_register ( "gan_fread_lendian_f32",
                          GAN_ERROR_READING_FROM_FILE, "" );
       return GAN_FALSE;
    }
@@ -521,50 +738,37 @@ Gan_Bool
 }
 
 /**
- * \brief Writes an array of floats to a file
+ * \brief Writes an array of 32-bit floats to a file
  * \param fp Pointer to a binary file opened for writing
- * \param af Array of floats
- * \param nitems The number of integers to be written from the array
+ * \param af32 Array of floats
+ * \param nitems The number of values to be written from the array
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  *
- * Writes an array of floats to a file in little-endian byte order.
+ * Writes an array of 32-bit floats to a file in little-endian byte order.
  */
 Gan_Bool
- gan_fwrite_lendian_f ( FILE *fp, float *af, size_t nitems )
+ gan_fwrite_lendian_f32 ( FILE *fp, gan_float32 *af32, size_t nitems )
 {
    
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
    {
-      char ca[SIZEOF_FLOAT];
+      char ca[4];
       int i;
 
-      for ( i = (int)nitems-1; i >= 0; i--, af++ )
+      for ( i = (int)nitems-1; i >= 0; i--, af32++ )
       {
          /* build reversed bytes */
-#if (SIZEOF_FLOAT == 4)
-         ca[0] = ((char *)af)[3];
-         ca[1] = ((char *)af)[2];
-         ca[2] = ((char *)af)[1];
-         ca[3] = ((char *)af)[0];
-#elif (SIZEOF_FLOAT == 8)
-         ca[0] = ((char *)af)[7];
-         ca[1] = ((char *)af)[6];
-         ca[2] = ((char *)af)[5];
-         ca[3] = ((char *)af)[4];
-         ca[4] = ((char *)af)[3];
-         ca[5] = ((char *)af)[2];
-         ca[6] = ((char *)af)[1];
-         ca[7] = ((char *)af)[0];
-#else
-#error "unsupported float size"      
-#endif
+         ca[0] = ((char *)af32)[3];
+         ca[1] = ((char *)af32)[2];
+         ca[2] = ((char *)af32)[1];
+         ca[3] = ((char *)af32)[0];
 
          /* write reversed bytes */
          if ( fwrite ( ca, sizeof(float), 1, fp ) != 1 )
          {
             gan_err_flush_trace();
-            gan_err_register ( "gan_fwrite_lendian_f",
+            gan_err_register ( "gan_fwrite_lendian_f32",
                                GAN_ERROR_WRITING_TO_FILE, "" );
             return GAN_FALSE;
          }
@@ -572,10 +776,10 @@ Gan_Bool
    }
 #else
    /* little-endian architecture: write bytes directly */
-   if ( fwrite ( af, sizeof(float), nitems, fp ) != nitems )
+   if ( fwrite ( af32, sizeof(gan_float32), nitems, fp ) != nitems )
    {
       gan_err_flush_trace();
-      gan_err_register ( "gan_fwrite_lendian_f",
+      gan_err_register ( "gan_fwrite_lendian_f32",
                          GAN_ERROR_WRITING_TO_FILE, "" );
       return GAN_FALSE;
    }
@@ -585,62 +789,56 @@ Gan_Bool
    return GAN_TRUE;
 }
 
+#endif /* #ifdef GAN_FLOAT32 */
+
+#ifdef GAN_FLOAT64
+
 /**
- * \brief Reads an array of doubles from a file
+ * \brief Reads an array of 64-bit floats from a file
  * \param fp Pointer to a binary file opened for reading
- * \param ad Array of doubles
+ * \param af64 Array of 64-bit floats
  * \param nitems The number of values to be read into the array
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  *
- * Reads an array of doubles from a file in little-endian byte order.
+ * Reads an array of 64-bit floats from a file in little-endian byte order.
  */
 Gan_Bool
- gan_fread_lendian_d ( FILE *fp, double *ad, size_t nitems )
+ gan_fread_lendian_f64 ( FILE *fp, gan_float64 *af64, size_t nitems )
 {
-   
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
    {
-      char ca[SIZEOF_DOUBLE];
+      char ca[8];
       int i;
 
-      for ( i = (int)nitems-1; i >= 0; i--, ad++ )
+      for ( i = (int)nitems-1; i >= 0; i--, af64++ )
       {
          /* read bytes */
-         if ( fread ( ca, sizeof(double), 1, fp ) != 1 )
+         if ( fread ( ca, sizeof(gan_float64), 1, fp ) != 1 )
          {
             gan_err_flush_trace();
-            gan_err_register ( "gan_fread_lendian_d",
+            gan_err_register ( "gan_fread_lendian_f64",
                                GAN_ERROR_READING_FROM_FILE, "" );
             return GAN_FALSE;
          }
 
          /* reverse bytes */
-#if (SIZEOF_DOUBLE == 4)
-         ((char *)ad)[3] = ca[0];
-         ((char *)ad)[2] = ca[1];
-         ((char *)ad)[1] = ca[2];
-         ((char *)ad)[0] = ca[3];
-#elif (SIZEOF_DOUBLE == 8)
-         ((char *)ad)[7] = ca[0];
-         ((char *)ad)[6] = ca[1];
-         ((char *)ad)[5] = ca[2];
-         ((char *)ad)[4] = ca[3];
-         ((char *)ad)[3] = ca[4];
-         ((char *)ad)[2] = ca[5];
-         ((char *)ad)[1] = ca[6];
-         ((char *)ad)[0] = ca[7];
+         ((char *)af64)[7] = ca[0];
+         ((char *)af64)[6] = ca[1];
+         ((char *)af64)[5] = ca[2];
+         ((char *)af64)[4] = ca[3];
+         ((char *)af64)[3] = ca[4];
+         ((char *)af64)[2] = ca[5];
+         ((char *)af64)[1] = ca[6];
+         ((char *)af64)[0] = ca[7];
       }
-#else
-#error "unsupported double size"      
-#endif
    }
 #else
    /* little-endian architecture: read bytes directly */
-   if ( fread ( ad, sizeof(double), nitems, fp ) != nitems )
+   if ( fread ( af64, sizeof(gan_float64), nitems, fp ) != nitems )
    {
       gan_err_flush_trace();
-      gan_err_register ( "gan_fread_lendian_d",
+      gan_err_register ( "gan_fread_lendian_f64",
                          GAN_ERROR_READING_FROM_FILE, "" );
       return GAN_FALSE;
    }
@@ -651,50 +849,41 @@ Gan_Bool
 }
 
 /**
- * \brief Writes an array of doubles to a file.
+ * \brief Writes an array of 64-bit floats to a file.
  * \param fp Pointer to a binary file opened for writing
- * \param ad Array of doubles
+ * \param af64 Array of 64-bit floats
  * \param nitems The number of values to be written from the array
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  *
- * Writes an array of doubles to a file in little-endian byte order.
+ * Writes an array of 64-bit floats to a file in little-endian byte order.
  */
 Gan_Bool
- gan_fwrite_lendian_d ( FILE *fp, double *ad, size_t nitems )
+ gan_fwrite_lendian_f64 ( FILE *fp, gan_float64 *af64, size_t nitems )
 {
    
    /* reverse bytes if on big-endian architecture */
 #ifdef WORDS_BIGENDIAN
    {
-      char ca[SIZEOF_DOUBLE];
+      char ca[8];
       int i;
 
-      for ( i = (int)nitems-1; i >= 0; i--, ad++ )
+      for ( i = (int)nitems-1; i >= 0; i--, af64++ )
       {
          /* build reversed bytes */
-#if (SIZEOF_DOUBLE == 4)
-         ca[0] = ((char *)ad)[3];
-         ca[1] = ((char *)ad)[2];
-         ca[2] = ((char *)ad)[1];
-         ca[3] = ((char *)ad)[0];
-#elif (SIZEOF_DOUBLE == 8)
-         ca[0] = ((char *)ad)[7];
-         ca[1] = ((char *)ad)[6];
-         ca[2] = ((char *)ad)[5];
-         ca[3] = ((char *)ad)[4];
-         ca[4] = ((char *)ad)[3];
-         ca[5] = ((char *)ad)[2];
-         ca[6] = ((char *)ad)[1];
-         ca[7] = ((char *)ad)[0];
-#else
-#error "unsupported double size"      
-#endif
+         ca[0] = ((char *)af64)[7];
+         ca[1] = ((char *)af64)[6];
+         ca[2] = ((char *)af64)[5];
+         ca[3] = ((char *)af64)[4];
+         ca[4] = ((char *)af64)[3];
+         ca[5] = ((char *)af64)[2];
+         ca[6] = ((char *)af64)[1];
+         ca[7] = ((char *)af64)[0];
 
          /* write reversed bytes */
-         if ( fwrite ( ca, sizeof(double), 1, fp ) != 1 )
+         if ( fwrite ( ca, sizeof(gan_float64), 1, fp ) != 1 )
          {
             gan_err_flush_trace();
-            gan_err_register ( "gan_fwrite_lendian_d",
+            gan_err_register ( "gan_fwrite_lendian_f64",
                                GAN_ERROR_WRITING_TO_FILE, "" );
             return GAN_FALSE;
          }
@@ -702,10 +891,10 @@ Gan_Bool
    }
 #else
    /* little-endian architecture: write bytes directly */
-   if ( fwrite ( ad, sizeof(double), nitems, fp ) != nitems )
+   if ( fwrite ( af64, sizeof(gan_float64), nitems, fp ) != nitems )
    {
       gan_err_flush_trace();
-      gan_err_register ( "gan_fwrite_lendian_d",
+      gan_err_register ( "gan_fwrite_lendian_f64",
                          GAN_ERROR_WRITING_TO_FILE, "" );
       return GAN_FALSE;
    }
@@ -715,6 +904,8 @@ Gan_Bool
    return GAN_TRUE;
 }
 
+#endif /* #ifdef GAN_FLOAT64 */
+
 /**
  * \}
  */
