@@ -3,8 +3,8 @@
  * Module:        Linked list
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.30 $
- * Last edited:   $Date: 2003/10/13 10:25:26 $
+ * Revision:      $Revision: 1.31 $
+ * Last edited:   $Date: 2004/05/14 08:32:19 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -126,7 +126,7 @@ Gan_List*
  * pointed to by the node should not to be freed.
  */
 void
- gan_list_free        ( Gan_List* list, void (*free_func)(void *))
+ gan_list_free        ( Gan_List* list, Gan_FreeFunc free_func)
 {
    gan_list_delete_all(list, free_func);
 
@@ -468,7 +468,7 @@ Gan_Bool
  * by the node should not to be freed
  */
 Gan_Bool
- gan_list_delete_first( Gan_List* list, void (*free_func)(void *))
+ gan_list_delete_first( Gan_List* list, Gan_FreeFunc free_func)
 {
    Gan_List_Node *tmp;
 
@@ -525,7 +525,7 @@ Gan_Bool
  * not to be freed.
  */
 Gan_Bool
- gan_list_delete_next ( Gan_List* list, void (*free_func)(void *))
+ gan_list_delete_next ( Gan_List* list, Gan_FreeFunc free_func)
 {
    Gan_List_Node* tmp;
 
@@ -592,7 +592,7 @@ Gan_Bool
  * Moves the current position to the next node in the list.
  */
 Gan_Bool
- gan_list_delete_current ( Gan_List *list, void (*free_func)(void *) )
+ gan_list_delete_current ( Gan_List *list, Gan_FreeFunc free_func )
 {
    Gan_List_Node *tmp;
 
@@ -651,7 +651,7 @@ Gan_Bool
  * doubly linked \a list.
  */
 Gan_Bool
- gan_list_delete_prev ( Gan_List *list, void (*free_func)(void *) )
+ gan_list_delete_prev ( Gan_List *list, Gan_FreeFunc free_func )
 {
    Gan_List_Node *tmp;
 
@@ -712,7 +712,7 @@ Gan_Bool
  * not to be freed.
  */
 Gan_Bool
- gan_list_delete_at( Gan_List *list, void (*free_func)(void *), int pos )
+ gan_list_delete_at( Gan_List *list, Gan_FreeFunc free_func, int pos )
 {
    Gan_List_Node *tmp;
    int i;
@@ -785,7 +785,7 @@ Gan_Bool
  * by the node should not to be freed.
  */
 Gan_Bool
- gan_list_delete_last ( Gan_List *list, void (*free_func)(void *))
+ gan_list_delete_last ( Gan_List *list, Gan_FreeFunc free_func)
 {
    Gan_List_Node *tmp;
 
@@ -833,7 +833,7 @@ Gan_Bool
  * pointed to by the nodes should not to be freed
  */
 void
- gan_list_delete_all ( Gan_List *list, void (*free_func)(void *) )
+ gan_list_delete_all ( Gan_List *list, Gan_FreeFunc free_func )
 {
    Gan_List_Node *tmp, *tmp_del;
    int i;
@@ -1341,10 +1341,17 @@ Gan_List*
  gan_list_copy( Gan_List *list )
 {
    int      i;
-   Gan_List *list_new = gan_list_new();
+   Gan_List *list_new;
+   Gan_List_Node* pNode;
 
-   Gan_List_Node* pNode = list->p_first;
+   list_new = gan_list_new();
+   if(list_new == NULL)
+   {
+      gan_err_register("gan_list_copy", GAN_ERROR_FAILURE, "");
+      return NULL;
+   }
 
+   pNode = list->p_first;
    for ( i = 0; i < list->node_count; i++ )
    {
       gan_list_insert_next( list_new, pNode->p_data );
@@ -1354,6 +1361,47 @@ Gan_List*
    return list_new;
 }
 
+/**
+ * \brief Copies all the nodes of a list, creating a new list.
+ * \param list The linked list
+ * \param copy_func The function used to copy each node
+ * \return The copy of the list, or \c NULL on failure.
+ *
+ * Copies all the nodes of a \a list, creating a new list, creating copies
+ * of each node using the provided function \a copy_func.
+ */
+Gan_List*
+ gan_list_copy_with_data( Gan_List *list, Gan_CopyFunc copy_func )
+{
+   int      i;
+   Gan_List *list_new;
+   Gan_List_Node* pNode;
+
+   list_new = gan_list_new();
+   if(list_new == NULL)
+   {
+      gan_err_register("gan_list_copy", GAN_ERROR_FAILURE, "");
+      return NULL;
+   }
+
+   pNode = list->p_first;
+   for ( i = 0; i < list->node_count; i++ )
+   {
+      void *pNewData = copy_func(pNode->p_data);
+      if(pNewData == NULL)
+      {
+         gan_err_flush_trace();
+         gan_err_register("gan_list_copy", GAN_ERROR_FAILURE, "");
+         return NULL;
+      }
+
+      gan_list_insert_next( list_new, pNewData );
+      pNode = pNode->p_next;
+   }
+
+   return list_new;
+}
+
 /**
  * \brief Sorts the nodes in the list in ascending order.
  * \param list The linked list
