@@ -3,9 +3,9 @@
  * Module:        Binary images
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.40 $
- * Last edited:   $Date: 2003/08/08 18:37:40 $
- * Author:        $Author: pm $
+ * Revision:      $Revision: 1.48 $
+ * Last edited:   $Date: 2005/03/02 16:46:32 $
+ * Author:        $Author: jps $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
 
@@ -105,6 +105,15 @@
  * \}
  */
 
+/**
+ * \defgroup ImageMinMax Minumum/maximum pixel value across an image
+ * \{
+ */
+
+/**
+ * \}
+ */
+
 /**
  * \}
  */
@@ -126,6 +135,7 @@
 #define GAN_IMAGE_FILL_CONST gan_image_fill_const_b
 #define GAN_IMAGE_GET_ACTIVE_SUBWINDOW gan_image_get_active_subwindow_b
 #define GAN_IMAGE_MASK_WINDOW gan_image_mask_window_b
+#define GAN_IMAGE_CLEAR_WINDOW gan_image_clear_window_b
 #define GAN_IMAGE_PIXEL_ZERO_VAL GAN_FALSE
 #define GAN_FILL_ARRAY gan_fill_array_b
 #define GAN_COPY_ARRAY gan_copy_array_b
@@ -153,6 +163,7 @@
 #undef GAN_IMAGE_FILL_CONST
 #undef GAN_IMAGE_GET_ACTIVE_SUBWINDOW
 #undef GAN_IMAGE_MASK_WINDOW
+#undef GAN_IMAGE_CLEAR_WINDOW
 #undef GAN_IMAGE_PIXEL_ZERO_VAL
 #undef GAN_FILL_ARRAY
 #undef GAN_COPY_ARRAY
@@ -180,7 +191,7 @@
  * or #GAN_FALSE otherwise.
  */
 Gan_Bool
- gan_image_bit_get_pix_4group ( Gan_Image *image,
+ gan_image_bit_get_pix_4group ( const Gan_Image *image,
                                 unsigned row, unsigned col )
 {
    /* consistency check */
@@ -205,7 +216,7 @@ Gan_Bool
  * (true), or #GAN_FALSE otherwise.
  */
 Gan_Bool
- gan_image_bit_get_pix_5group ( Gan_Image *image,
+ gan_image_bit_get_pix_5group ( const Gan_Image *image,
                                 unsigned row, unsigned col )
 {
    /* consistency check */
@@ -224,6 +235,38 @@ Gan_Bool
            ? GAN_TRUE : GAN_FALSE);
 }
 
+/**
+ * \brief Test local group of nine binary pixels.
+ *
+ * Return #GAN_TRUE if bits at positions \a (row-1,col-1), \a (row,col),
+ * \a (row+1,col-1), \a (row,col-1), \a (row,col+1), \a (row-1,col),
+ * \a (row+1,col-1), \a (row+1,col) and \a (row+1,col+1) are all set
+ * to one (true), or #GAN_FALSE otherwise.
+ */
+Gan_Bool
+ gan_image_bit_get_pix_9group ( const Gan_Image *image,
+                                unsigned row, unsigned col )
+{
+   /* consistency check */
+   assert ( image->format == GAN_GREY_LEVEL_IMAGE && image->type == GAN_BOOL );
+
+   gan_assert ( row > 0 && row < image->height-1 &&
+                col > 0 && col < image->width-1,
+                "illegal pixel position in gan_image_bit_get_pix_9group()" );
+
+   /* get pixel */
+   return ((gan_bit_array_get_bit ( &image->ba[row-1], col-1 ) &&
+            gan_bit_array_get_bit ( &image->ba[row-1], col   ) &&
+            gan_bit_array_get_bit ( &image->ba[row-1], col+1 ) &&
+            gan_bit_array_get_bit ( &image->ba[row],   col-1 ) &&
+            gan_bit_array_get_bit ( &image->ba[row],   col   ) &&
+            gan_bit_array_get_bit ( &image->ba[row],   col+1 ) &&
+            gan_bit_array_get_bit ( &image->ba[row+1], col-1 ) &&
+            gan_bit_array_get_bit ( &image->ba[row+1], col   ) &&
+            gan_bit_array_get_bit ( &image->ba[row+1], col+1 ))
+           ? GAN_TRUE : GAN_FALSE);
+}
+
 /**
  * \brief Test local group of three binary pixels in a row.
  *
@@ -231,7 +274,7 @@ Gan_Bool
  * \a (row,col+1) are all set to one (true), or #GAN_FALSE otherwise.
  */
 Gan_Bool
- gan_image_bit_get_pix_3group_horiz ( Gan_Image *image,
+ gan_image_bit_get_pix_3group_horiz ( const Gan_Image *image,
                                       unsigned row, unsigned col )
 {
    /* consistency check */
@@ -255,7 +298,7 @@ Gan_Bool
  * \a (row+1,col) are all set to one (true), or #GAN_FALSE otherwise.
  */
 Gan_Bool
- gan_image_bit_get_pix_3group_vert ( Gan_Image *image,
+ gan_image_bit_get_pix_3group_vert ( const Gan_Image *image,
                                      unsigned row, unsigned col )
 {
    /* consistency check */
@@ -283,11 +326,11 @@ Gan_Bool
  * only inside the given subwindow.
  */
 int
- gan_image_get_pixel_count_b ( Gan_Image *image, 
+ gan_image_get_pixel_count_b ( const Gan_Image *image, 
                                Gan_Bool val,
-                               Gan_ImageWindow *subwin )
+                               const Gan_ImageWindow *psubwin )
 {
-   Gan_ImageWindow new_subwin;
+   Gan_ImageWindow subwin;
    unsigned int max_row, row;
    unsigned int max_col, col;
    int pixel_count = 0;
@@ -301,13 +344,10 @@ int
       return -1;
    }
 
-   if(subwin == NULL)
+   if(psubwin == NULL)
    {
-      subwin = &new_subwin;
-
       success = gan_image_get_active_subwindow_b(image, GAN_BYTE_ALIGNMENT,
-                                                 subwin);
-      
+                                                 &subwin);
       if(!success)
       {
          gan_err_flush_trace();
@@ -317,12 +357,14 @@ int
          return -1;
       }
    }
-   
-   max_row = subwin->r0 + subwin->height;
-   max_col = subwin->c0 + subwin->width;
+   else
+      subwin = *psubwin;
 
-   for(row = subwin->r0; row < max_row; row++)
-      for(col = subwin->c0; col < max_col; col++)
+   max_row = subwin.r0 + subwin.height;
+   max_col = subwin.c0 + subwin.width;
+
+   for(row = subwin.r0; row < max_row; row++)
+      for(col = subwin.c0; col < max_col; col++)
          if(gan_image_get_pix_b( image, row, col ))
             pixel_count++;
 
@@ -653,6 +695,330 @@ Gan_Bool
    return GAN_TRUE;
 }
 
+/**
+ * \brief Dilate binary image horizontally
+ * \param image Binary image to dilate
+ * \param no_pixels Number of pixels to dilate image by
+ * \param restrict_mask Mask to restrict dilation or \c NULL
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Dilate binary image \a image horizontally by given number of pixels
+ * \a no_pixels, optionally specifying a mask \a restrict_mask to restrict
+ * the dilation.
+ */
+Gan_Bool
+ gan_image_bit_dilate_horiz ( Gan_Image *image, int no_pixels,
+                              Gan_Image *restrict_mask )
+{
+   Gan_ImageWindow subwin;
+   int row, row_max;
+
+   /* return immediately if zero expansion is requested */
+   if ( no_pixels == 0 ) return GAN_TRUE;
+
+   if ( !gan_image_get_active_subwindow_b( image, GAN_BIT_ALIGNMENT, &subwin) )
+   {
+      gan_err_register ( "gan_image_bit_dilate_horiz", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   if ( subwin.width == 0 || subwin.height == 0 )
+      return GAN_TRUE;
+
+   row_max = (int)(subwin.r0 + subwin.height);
+
+   /* expand horizontally */
+   if ( restrict_mask == NULL )
+      for ( row = (int)subwin.r0; row < row_max; row++ )
+      {
+         if ( !gan_bit_array_dilate_i ( &image->ba[row],
+                                        (unsigned)no_pixels ) )
+         {
+            gan_err_register ( "gan_image_bit_dilate_horiz", GAN_ERROR_FAILURE,
+                               "" );
+            return GAN_FALSE;
+         }
+      }
+   else
+   {
+      Gan_BitArray *bit_array=NULL;
+
+      bit_array = gan_bit_array_alloc(image->width);
+      if ( bit_array == NULL )
+      {
+         gan_err_register ( "gan_image_bit_dilate_horiz", GAN_ERROR_FAILURE,
+                            "" );
+         return GAN_FALSE;
+      }
+      
+      for ( row = (int)subwin.r0; row < row_max; row++ )
+      {
+         if ( !gan_bit_array_dilate_q ( &image->ba[row], (unsigned)no_pixels,
+                                        bit_array ) ||
+              !gan_bit_array_and_i ( bit_array, &restrict_mask->ba[row] ) ||
+              !gan_bit_array_or_i ( &image->ba[row], bit_array ) )
+         {
+            gan_err_register ( "gan_image_bit_dilate_horiz", GAN_ERROR_FAILURE,
+                               "" );
+            return GAN_FALSE;
+         }
+      }
+
+      gan_bit_array_free(bit_array);
+   }
+
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Dilate binary image vertically
+ * \param image Binary image to dilate
+ * \param no_pixels Number of pixels to dilate image by
+ * \param restrict_mask Mask to restrict dilation or \c NULL
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ *
+ * Dilate binary image \a image vertically by given number of pixels
+ * \a no_pixels, optionally specifying a mask \a restrict_mask to restrict
+ * the dilation.
+ */
+Gan_Bool
+ gan_image_bit_dilate_vert ( Gan_Image *image, int no_pixels,
+                             Gan_Image *restrict_mask )
+{
+   Gan_ImageWindow subwin;
+   int row, row_max, pixel;
+   Gan_BitArray *bit_array=NULL, **array=NULL;
+
+   /* return immediately if zero expansion is requested */
+   if ( no_pixels == 0 ) return GAN_TRUE;
+
+   if ( !gan_image_get_active_subwindow_b( image, GAN_BIT_ALIGNMENT, &subwin) )
+   {
+      gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   if ( subwin.width == 0 || subwin.height == 0 )
+      return GAN_TRUE;
+
+   bit_array = gan_bit_array_alloc(image->width);
+   if ( bit_array == NULL )
+   {
+      gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   /* expand vertically */
+   array = gan_malloc_array ( Gan_BitArray *, no_pixels+1 );
+   if ( array == NULL )
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_MALLOC_FAILED,
+                         "" );
+      return GAN_FALSE;
+   }
+
+   gan_fill_array_p ( (void **)array, no_pixels+1, 1, NULL );
+
+   if ( (int)subwin.r0 < no_pixels )
+   {
+      subwin.height += subwin.r0;
+      subwin.r0 = 0;
+   }
+   else
+   {
+      subwin.r0 -= (unsigned)no_pixels;
+      subwin.height += (unsigned)no_pixels;
+   }
+
+   subwin.height += (unsigned)no_pixels;
+   if ( subwin.r0+subwin.height > (int)image->height )
+      subwin.height = (int)image->height-subwin.r0;
+
+   row_max = (int)(subwin.r0 + subwin.height);
+   if ( restrict_mask == NULL )
+      for ( row = (int)subwin.r0; row < row_max; row++ )
+      {
+         Gan_BitArray *ptmp;
+
+         /* shift array of bit arrays */
+         ptmp = array[0];
+         for ( pixel = 0; pixel < no_pixels; pixel++ )
+            array[pixel] = array[pixel+1];
+
+         array[no_pixels] = ptmp;
+
+         /* copy this row */
+         if ( array[no_pixels] == NULL )
+         {
+            array[no_pixels] = gan_bit_array_alloc(image->width);
+            if ( array[no_pixels] == NULL )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+         }
+
+         if ( !gan_bit_array_copy_q ( &image->ba[row], array[no_pixels] ) )
+         {
+            gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_FAILURE,
+                               "" );
+            return GAN_FALSE;
+         }
+
+         /* compute results for this row */
+         for ( pixel = no_pixels-1; pixel >= 0; pixel-- )
+         {
+            if ( array[pixel] != NULL &&
+                 !gan_bit_array_or_i ( &image->ba[row], array[pixel] ) )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+
+            if ( row+pixel+1 < (int)image->height &&
+                 !gan_bit_array_or_i ( &image->ba[row],
+                                       &image->ba[row+pixel+1] ) )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+         }
+      }
+   else
+      for ( row = (int)subwin.r0; row < row_max; row++ )
+      {
+         Gan_BitArray *ptmp;
+
+         /* shift array of bit arrays */
+         ptmp = array[0];
+         for ( pixel = 0; pixel < no_pixels; pixel++ )
+            array[pixel] = array[pixel+1];
+
+         array[no_pixels] = ptmp;
+
+         /* copy this row */
+         if ( array[no_pixels] == NULL )
+         {
+            array[no_pixels] = gan_bit_array_alloc(image->width);
+            if ( array[no_pixels] == NULL )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+         }
+
+         if ( !gan_bit_array_copy_q ( &image->ba[row], array[no_pixels] ) ||
+              !gan_bit_array_copy_q ( &image->ba[row], bit_array ) )
+         {
+            gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_FAILURE,
+                               "" );
+            return GAN_FALSE;
+         }
+
+         /* compute results for this row */
+         for ( pixel = no_pixels-1; pixel >= 0; pixel-- )
+         {
+            if ( array[pixel] != NULL &&
+                 !gan_bit_array_or_i ( bit_array, array[pixel] ) )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+
+            if ( row+pixel+1 < (int)image->height &&
+                 !gan_bit_array_or_i ( bit_array, &image->ba[row+pixel+1] ) )
+            {
+               gan_err_register ( "gan_image_bit_dilate_vert",
+                                  GAN_ERROR_FAILURE, "" );
+               return GAN_FALSE;
+            }
+         }
+
+         /* apply restriction mask */
+         if ( !gan_bit_array_and_i ( bit_array, &restrict_mask->ba[row] ) ||
+              !gan_bit_array_or_i ( &image->ba[row], bit_array ) )
+         {
+            gan_err_register ( "gan_image_bit_dilate_vert", GAN_ERROR_FAILURE,
+                               "" );
+            return GAN_FALSE;
+         }
+      }
+
+   /* success */
+   for ( pixel = no_pixels; pixel >= 0; pixel-- )
+      if ( array[pixel] != NULL )
+         gan_bit_array_free(array[pixel]);
+
+   free(array);
+   gan_bit_array_free(bit_array);
+         
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Shift binary image horizontally and vertically
+ * \param image Binary image to shift
+ * \param vshift Vertical shift
+ * \param hshift Horizontal shift
+ * \param result_image Binary result image
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+Gan_Bool gan_image_bit_shift ( const Gan_Image *image, int vshift, int hshift,
+                               Gan_Image *result_image )
+{
+   Gan_ImageWindow subwin;
+   int row, row_max, rheight = result_image->height;
+
+   if(image->format != GAN_GREY_LEVEL_IMAGE || image->type != GAN_BOOL ||
+      result_image->format != GAN_GREY_LEVEL_IMAGE ||
+      result_image->type != GAN_BOOL)
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_image_bit_shift", GAN_ERROR_INCOMPATIBLE, "" );
+      return GAN_FALSE;
+   }
+
+   // can't do in-place shift
+   if(image == result_image)
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "gan_image_bit_shift", GAN_ERROR_INCOMPATIBLE, "" );
+      return GAN_FALSE;
+   }
+
+   // fill result image with zeros
+   gan_image_fill_const_b(result_image, GAN_FALSE);
+
+   // get subwindow
+   if ( !gan_image_get_active_subwindow_b( image, GAN_WORD_ALIGNMENT, &subwin) )
+   {
+      gan_err_register ( "gan_image_bit_shift", GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
+
+   // shift each row in turn
+   row_max = (int)(subwin.r0+subwin.height);
+   for(row = (int)subwin.r0; row < row_max; row++)
+   {
+      if(row+vshift >= 0 && row+vshift<rheight)
+      {
+         if(!gan_bit_array_shift_q( &image->ba[row], hshift, &result_image->ba[row+vshift] ))
+         {
+            gan_err_register ( "gan_image_bit_shift", GAN_ERROR_FAILURE, "" );
+            return GAN_FALSE;
+         }
+      }
+   }
+
+   return GAN_TRUE;
+}
+
 /**
  * \}
  */
