@@ -3,8 +3,8 @@
  * Module:        Image definitions and general functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.45 $
- * Last edited:   $Date: 2003/05/19 15:57:29 $
+ * Revision:      $Revision: 1.54 $
+ * Last edited:   $Date: 2005/04/12 13:06:44 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -148,7 +148,7 @@ typedef struct Gan_Image
    Gan_BitArray *ba;
 
    /// user-defined offsets
-   unsigned int offset_x, offset_y;
+   int offset_x, offset_y;
 
    /// flag indicating whether row pointer array was dynamically allocated
    Gan_Bool row_data_alloc;
@@ -280,96 +280,96 @@ typedef struct Gan_Image
       /// grey-level formats
       union
       {
-         unsigned char  (*uc) (struct Gan_Image *, unsigned row, unsigned col);
-         short          (*s)  (struct Gan_Image *, unsigned row, unsigned col);
-         unsigned short (*us) (struct Gan_Image *, unsigned row, unsigned col);
-         int            (*i)  (struct Gan_Image *, unsigned row, unsigned col);
-         unsigned int   (*ui) (struct Gan_Image *, unsigned row, unsigned col);
-         float          (*f)  (struct Gan_Image *, unsigned row, unsigned col);
-         double         (*d)  (struct Gan_Image *, unsigned row, unsigned col);
-         Gan_Bool       (*b)  (struct Gan_Image *, unsigned row, unsigned col);
-         void *         (*p)  (struct Gan_Image *, unsigned row, unsigned col);
+         unsigned char  (*uc) (const struct Gan_Image *, unsigned row, unsigned col);
+         short          (*s)  (const struct Gan_Image *, unsigned row, unsigned col);
+         unsigned short (*us) (const struct Gan_Image *, unsigned row, unsigned col);
+         int            (*i)  (const struct Gan_Image *, unsigned row, unsigned col);
+         unsigned int   (*ui) (const struct Gan_Image *, unsigned row, unsigned col);
+         float          (*f)  (const struct Gan_Image *, unsigned row, unsigned col);
+         double         (*d)  (const struct Gan_Image *, unsigned row, unsigned col);
+         Gan_Bool       (*b)  (const struct Gan_Image *, unsigned row, unsigned col);
+         void *         (*p)  (const struct Gan_Image *, unsigned row, unsigned col);
       } gl;
 
       /// grey-level formats with alpha channels
       union
       {
-         Gan_GLAPixel_uc (*uc)( struct Gan_Image *,
+         Gan_GLAPixel_uc (*uc)( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_GLAPixel_s  (*s)( struct Gan_Image *,
+         Gan_GLAPixel_s  (*s)( const struct Gan_Image *,
                                unsigned row, unsigned col );
-         Gan_GLAPixel_us (*us)( struct Gan_Image *,
+         Gan_GLAPixel_us (*us)( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_GLAPixel_i  (*i)( struct Gan_Image *,
+         Gan_GLAPixel_i  (*i)( const struct Gan_Image *,
                                unsigned row, unsigned col );
-         Gan_GLAPixel_ui (*ui)( struct Gan_Image *,
+         Gan_GLAPixel_ui (*ui)( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_GLAPixel_f  (*f)( struct Gan_Image *,
+         Gan_GLAPixel_f  (*f)( const struct Gan_Image *,
                                unsigned row, unsigned col );
-         Gan_GLAPixel_d  (*d)( struct Gan_Image *,
+         Gan_GLAPixel_d  (*d)( const struct Gan_Image *,
                                unsigned row, unsigned col );
       } gla;
 
       /// RGB colour formats
       union
       {
-         Gan_RGBPixel_uc (*uc) ( struct Gan_Image *,
+         Gan_RGBPixel_uc (*uc) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBPixel_s  (*s) ( struct Gan_Image *,
+         Gan_RGBPixel_s  (*s) ( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_RGBPixel_us (*us) ( struct Gan_Image *,
+         Gan_RGBPixel_us (*us) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBPixel_i  (*i) ( struct Gan_Image *,
+         Gan_RGBPixel_i  (*i) ( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_RGBPixel_ui (*ui) ( struct Gan_Image *,
+         Gan_RGBPixel_ui (*ui) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBPixel_f  (*f) ( struct Gan_Image *,
+         Gan_RGBPixel_f  (*f) ( const struct Gan_Image *,
                                 unsigned row, unsigned col );
-         Gan_RGBPixel_d  (*d) ( struct Gan_Image *,
+         Gan_RGBPixel_d  (*d) ( const struct Gan_Image *,
                                 unsigned row, unsigned col );
       } rgb;
 
       /// RGB colour formats with alpha channels
       union
       {
-         Gan_RGBAPixel_uc (*uc) ( struct Gan_Image *,
+         Gan_RGBAPixel_uc (*uc) ( const struct Gan_Image *,
                                   unsigned row, unsigned col );
-         Gan_RGBAPixel_s  (*s) ( struct Gan_Image *,
+         Gan_RGBAPixel_s  (*s) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBAPixel_us (*us) ( struct Gan_Image *,
+         Gan_RGBAPixel_us (*us) ( const struct Gan_Image *,
                                   unsigned row, unsigned col );
-         Gan_RGBAPixel_i  (*i) ( struct Gan_Image *,
+         Gan_RGBAPixel_i  (*i) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBAPixel_ui (*ui) ( struct Gan_Image *,
+         Gan_RGBAPixel_ui (*ui) ( const struct Gan_Image *,
                                   unsigned row, unsigned col );
-         Gan_RGBAPixel_f  (*f) ( struct Gan_Image *,
+         Gan_RGBAPixel_f  (*f) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
-         Gan_RGBAPixel_d  (*d) ( struct Gan_Image *,
+         Gan_RGBAPixel_d  (*d) ( const struct Gan_Image *,
                                  unsigned row, unsigned col );
       } rgba;
 
       /// 2D vector field formats
       union
       {
-         Gan_Vector2_f (*f) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector2   (*d) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector2_s (*s) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector2_i (*i) ( struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector2_f (*f) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector2   (*d) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector2_s (*s) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector2_i (*i) ( const struct Gan_Image *, unsigned row, unsigned col );
       } vfield2D;
 
       /// 3D vector field formats
       union
       {
-         Gan_Vector3_f (*f) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector3   (*d) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector3_s (*s) ( struct Gan_Image *, unsigned row, unsigned col );
-         Gan_Vector3_i (*i) ( struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector3_f (*f) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector3   (*d) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector3_s (*s) ( const struct Gan_Image *, unsigned row, unsigned col );
+         Gan_Vector3_i (*i) ( const struct Gan_Image *, unsigned row, unsigned col );
       } vfield3D;
    } get_pix;
 
 #endif /* #ifndef NDEBUG */
 
-   /// fill image with constant value
+   /// fill image with constant value in a mask
    union
    {
       /// grey-level formats
@@ -442,7 +442,7 @@ typedef struct Gan_Image
    } fill_const;
 
    /// copy image
-   struct Gan_Image * (*copy) ( struct Gan_Image *, struct Gan_Image * );
+   struct Gan_Image * (*copy) ( const struct Gan_Image *, struct Gan_Image * );
 } Gan_Image;
 
 #define GAN_IMAGE_STRUCT_DEFINED
@@ -451,10 +451,10 @@ typedef struct Gan_Image
 typedef struct Gan_ImageWindow
 {
    /// offset of window relative to top-left corner of the image
-   unsigned c0, r0;
+   int c0, r0;
 
    /// dimensions of window
-   unsigned width, height;
+   int width, height;
 } Gan_ImageWindow;
 
 /**
@@ -594,7 +594,7 @@ void gan_image_free ( Gan_Image *img );
  * Copy image \a img_s to image \a img_d, and return \a img_d.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Image *gan_image_copy_q ( Gan_Image *img_s, Gan_Image *img_d );
+Gan_Image *gan_image_copy_q ( const Gan_Image *img_s, Gan_Image *img_d );
 #else
 #define gan_image_copy_q(is,id) ((is)->copy(is,id))
 #endif
@@ -606,7 +606,7 @@ Gan_Image *gan_image_copy_q ( Gan_Image *img_s, Gan_Image *img_d );
  * Allocate and return a copy of image \a img.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Image *gan_image_copy_s ( Gan_Image *img );
+Gan_Image *gan_image_copy_s ( const Gan_Image *img );
 #else
 #define gan_image_copy_s(img) ((img)->copy(img,NULL))
 #endif
@@ -698,7 +698,7 @@ Gan_Image *gan_image_set_dims ( Gan_Image *img,
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_image_test_format ( Gan_Image *img, Gan_ImageFormat format );
+Gan_Bool gan_image_test_format ( const Gan_Image *img, Gan_ImageFormat format );
 #else
 #define gan_image_test_format(img,f) ((img)->format == f)
 #endif
@@ -710,7 +710,7 @@ Gan_Bool gan_image_test_format ( Gan_Image *img, Gan_ImageFormat format );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_image_test_type ( Gan_Image *img, Gan_Type type );
+Gan_Bool gan_image_test_type ( const Gan_Image *img, Gan_Type type );
 #else
 #define gan_image_test_type(img,t) ((img)->type == t)
 #endif
@@ -722,7 +722,7 @@ Gan_Bool gan_image_test_type ( Gan_Image *img, Gan_Type type );
  * \a img1 are the same as those of \a img2, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_image_test_dims ( Gan_Image *img1, Gan_Image *img2 );
+Gan_Bool gan_image_test_dims ( const Gan_Image *img1, const Gan_Image *img2 );
 #else
 #define gan_image_test_dims(img1,img2) ((img1)->width == (img2)->width && (img1)->height == (img2)->height)
 #endif
@@ -749,10 +749,26 @@ Gan_Bool gan_image_fill_zero_window ( Gan_Image *img,
            gan_image_fill_const_window(img,r0,c0,h,w,NULL)
 #endif
 
+/**
+ * \brief Macro: Fill mask in image with zero.
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+#ifdef GAN_GENERATE_DOCUMENTATION
+Gan_Bool gan_image_fill_zero_mask ( Gan_Image *img, Gan_Image *mask );
+#else
+#define gan_image_fill_zero_mask(img,mask) \
+           gan_image_fill_const_mask(img,mask,NULL)
+#endif
+
 /**
  * \}
  */
 
+/**
+ * \addtogroup ImageAllocate
+ * \{
+ */
+
 Gan_Image *gan_image_form_gen ( Gan_Image *img,
                                 Gan_ImageFormat format, Gan_Type type,
                                 unsigned long height, unsigned long width,
@@ -761,28 +777,120 @@ Gan_Image *gan_image_form_gen ( Gan_Image *img,
                                 void *row_data, size_t row_data_size );
 Gan_Bool gan_image_set_data_free_func ( Gan_Image *img,
                                         void (*data_free_func)(void *) );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageSize
+ * \{
+ */
+
 size_t gan_image_pixel_size ( Gan_ImageFormat format, Gan_Type type );
 size_t gan_image_min_stride ( Gan_ImageFormat format, Gan_Type type,
                               unsigned long width, size_t alignment );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageFill
+ * \{
+ */
+
 Gan_Bool gan_image_fill_const ( Gan_Image *img, Gan_Pixel *pixel );
 Gan_Bool gan_image_fill_zero ( Gan_Image *img );
 Gan_Bool gan_image_fill_const_window ( Gan_Image *img,
                                        unsigned r0,     unsigned c0,
                                        unsigned height, unsigned width,
                                        Gan_Pixel *pixel );
-Gan_Pixel gan_image_get_pix ( Gan_Image *img, unsigned row, unsigned col );
-void     *gan_image_get_pixptr ( Gan_Image *img, unsigned row, unsigned col );
-Gan_Bool gan_image_get_pix_zero ( Gan_Image *img, unsigned row, unsigned col );
+Gan_Bool gan_image_fill_const_mask ( Gan_Image *img, Gan_Image *mask,
+                                     Gan_Pixel *pixel );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageAccessPixel
+ * \{
+ */
+
+Gan_Pixel gan_image_get_pix ( const Gan_Image *img, unsigned row, unsigned col );
+void     *gan_image_get_pixptr ( const Gan_Image *img, unsigned row, unsigned col );
+Gan_Bool gan_image_get_pix_zero ( const Gan_Image *img, unsigned row, unsigned col );
 Gan_Bool gan_image_set_pix ( Gan_Image *img, unsigned row, unsigned col,
                              Gan_Pixel *pix );
 Gan_Bool gan_image_set_pix_zero ( Gan_Image *img, unsigned row, unsigned col );
-Gan_Bool gan_image_get_active_subwindow ( Gan_Image *image,
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageExtract
+ * \{
+ */
+
+Gan_Bool gan_image_get_active_subwindow ( const Gan_Image *image,
                                           Gan_ImageWindow *subwin );
+
+void            gan_image_intersect_subwindows_q ( const Gan_ImageWindow *window1, const Gan_ImageWindow *window2,
+                                                   Gan_ImageWindow *resultwindow );
+Gan_ImageWindow gan_image_intersect_subwindows_s ( const Gan_ImageWindow *window1, const Gan_ImageWindow *window2 );
+void            gan_image_intersect_subwindows_i ( Gan_ImageWindow *sourcedestwindow, const Gan_ImageWindow *window );
+void            gan_image_fit_subwindow ( Gan_ImageWindow *subwin, const Gan_Image *image );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageFill
+ * \{
+ */
+
 Gan_Bool gan_image_mask_window ( Gan_Image *image,
                                  unsigned r0,     unsigned c0,
                                  unsigned height, unsigned width );
+Gan_Bool gan_image_clear_window ( Gan_Image *image,
+                                  unsigned r0,     unsigned c0,
+                                  unsigned height, unsigned width );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageMinMax
+ * \{
+ */
+
+Gan_Bool gan_image_get_minimum_pixel ( Gan_Image *image, Gan_Image *mask, Gan_Pixel *minpix );
+Gan_Bool gan_image_get_maximum_pixel ( Gan_Image *image, Gan_Image *mask, Gan_Pixel *maxpix );
+
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageAllocate
+ * \{
+ */
+
 void gan_image_free_va ( Gan_Image *img, ... );
 
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup ImageSet
+ * \{
+ */
+
 /* not user functions: use macros instead */
 Gan_Image *gan_image_set_format_type_dims_gen  ( Gan_Image *img,
                                                  Gan_ImageFormat format,
@@ -796,6 +904,10 @@ Gan_Image *gan_image_set_format_type_dims_gen  ( Gan_Image *img,
  * \}
  */
 
+/**
+ * \}
+ */
+
 #ifdef __cplusplus
 }
 #endif
