@@ -3,9 +3,9 @@
  * Module:        Display images using Open GL
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.28 $
- * Last edited:   $Date: 2003/08/26 15:11:45 $
- * Author:        $Author: pm $
+ * Revision:      $Revision: 1.30 $
+ * Last edited:   $Date: 2004/07/14 15:08:40 $
+ * Author:        $Author: jps $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
 
@@ -49,9 +49,15 @@
         #include <windows.h>
 #endif
 
+#ifdef GCC_MAC
+#include <OpenGL/gl.h>
+#include <OpenGL/glu.h>
+#include <GLUT/glut.h>
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <GL/glut.h>
+#endif
 
 /**
  * \addtogroup ImagePackage
@@ -65,10 +71,8 @@
 
 /* display binary images */
 static Gan_Bool
- gan_image_display_b ( Gan_Image *img )
+ gan_image_display_b ( const Gan_Image *img )
 {
-   Gan_Bool copied = GAN_FALSE;
-
    switch ( img->format )
    {
       case GAN_GREY_LEVEL_IMAGE:
@@ -91,8 +95,6 @@ static Gan_Bool
         break;
    }
 
-   if ( copied ) gan_image_free(img);
-
    /* success */
    return GAN_TRUE;
 }
@@ -109,7 +111,7 @@ static Gan_Bool
  * \sa gan_image_display_new_window().
  */
 Gan_Bool
- gan_image_display ( Gan_Image *img )
+ gan_image_display ( const Gan_Image *img )
 {
    Gan_Bool copied = GAN_FALSE;
 
@@ -236,7 +238,7 @@ Gan_Bool
         break;
    }
 
-   if ( copied ) gan_image_free(img);
+   if ( copied ) gan_image_free((Gan_Image*)img);
 
    /* success */
    return GAN_TRUE;
@@ -536,7 +538,7 @@ void
  * \sa gan_image_display().
  */
 Gan_Bool
- gan_image_display_new_window ( Gan_Image *img, double zoom, char *name,
+ gan_image_display_new_window ( const Gan_Image *img, double zoom, char *name,
                                 int offset_r, int offset_c, int *window_idp )
                                    
 {
@@ -589,7 +591,7 @@ Gan_Bool
    }
 
    assoc->window_id = window_id;
-   assoc->img       = img;
+   assoc->img       = (Gan_Image*)img;
    gan_list_insert_first ( list, assoc );
 
    /* success */
