@@ -3,9 +3,9 @@
  * Module:        Extract/fill selected channel(s) of image
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.2 $
- * Last edited:   $Date: 2003/03/30 03:02:08 $
- * Author:        $Author: pm $
+ * Revision:      $Revision: 1.3 $
+ * Last edited:   $Date: 2004/07/13 17:03:11 $
+ * Author:        $Author: ndm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
 
@@ -113,8 +113,8 @@ Gan_Image *
    }
    else if ( source != dest )
    {
-      dest = gan_image_set_format_type_dims ( dest, dest->format, dest->type,
-                                              dest->height, dest->width );
+      dest = gan_image_set_format_type_dims ( dest, source->format, source->type,
+                                              source->height, source->width );
       if ( dest == NULL )
       {
          gan_err_register ( "gan_image_flip_q", GAN_ERROR_FAILURE, "" );
