@@ -3,8 +3,8 @@
  * Module:        Grey level image functions
  * Part of:       Gandalf Library 
  *
- * Revision:      $Revision: 1.28 $
- * Last edited:   $Date: 2003/08/08 16:34:03 $
+ * Revision:      $Revision: 1.39 $
+ * Last edited:   $Date: 2005/05/21 22:11:18 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -74,7 +74,7 @@ static Gan_Bool
 }
 
 static Gan_Bool
- image_get_pix ( Gan_Image *img, unsigned row, unsigned col )
+ image_get_pix ( const Gan_Image *img, unsigned row, unsigned col )
 {
    /* consistency check */
    assert ( img->format == GAN_PIXFMT && img->type == GAN_PIXTYPE );
@@ -128,7 +128,7 @@ static Gan_Bool
 }
 
 static GAN_PIXEL
- image_get_pix ( Gan_Image *img, unsigned row, unsigned col )
+ image_get_pix ( const Gan_Image *img, unsigned row, unsigned col )
 
 {
    /* consistency check */
@@ -175,7 +175,7 @@ static Gan_Bool
 
 /* copy image */
 static Gan_Image *
- image_copy ( Gan_Image *img_s, Gan_Image *img_d )
+ image_copy ( const Gan_Image *img_s, Gan_Image *img_d )
 {
    /* consistency check */
    assert ( img_s->format == GAN_PIXFMT && img_s->type == GAN_PIXTYPE );
@@ -185,25 +185,44 @@ static Gan_Image *
 
    /* allocate image img_d if necessary */
    if ( img_d == NULL )
+   {
       img_d = GAN_IMAGE_FORM_GEN ( NULL, img_s->height, img_s->width,
                                    gan_image_min_stride(GAN_PIXFMT,
                                                         GAN_PIXTYPE,
                                                         img_s->width,
                                                         0),
                                    GAN_TRUE, NULL, 0, NULL, 0 );
+      if ( img_d == NULL )
+      {
+         gan_err_register("image_copy", GAN_ERROR_FAILURE, "");
+         return NULL;
+      }
+   }
    else
    {
       if ( img_d->format != img_s->format || img_d->type != img_s->type )
+      {
          /* make formats and types compatible */
-         GAN_IMAGE_SET_GEN ( img_d, img_s->height, img_s->width,
-                             gan_image_min_stride(GAN_PIXFMT, GAN_PIXTYPE,
-                                                  img_s->width, 0),
-                             GAN_TRUE );
+         img_d = GAN_IMAGE_SET_GEN ( img_d, img_s->height, img_s->width,
+                                     gan_image_min_stride(GAN_PIXFMT, GAN_PIXTYPE,
+                                                          img_s->width, 0),
+                                     GAN_TRUE );
+         if ( img_d == NULL )
+         {
+            gan_err_register("image_copy", GAN_ERROR_FAILURE, "");
+            return NULL;
+         }
+      }
       else
+      {
          /* reallocate image if necessary */
-         image_realloc ( img_d, img_s->height, img_s->width,
-                         gan_image_min_stride(GAN_PIXFMT, GAN_PIXTYPE,
-                                              img_s->width, 0), GAN_TRUE );
+         if ( !image_realloc ( img_d, img_s->height, img_s->width,
+                               gan_image_min_stride(GAN_PIXFMT, GAN_PIXTYPE, img_s->width, 0), GAN_TRUE ) )
+         {
+            gan_err_register("image_copy", GAN_ERROR_FAILURE, "");
+            return NULL;
+         }
+      }
    }
 
    /* if the image has no pixels, return immediately */
@@ -264,12 +283,12 @@ static Gan_Image *
  * \a subwin structure will be set to zero.
  */
 Gan_Bool
- GAN_IMAGE_GET_ACTIVE_SUBWINDOW ( Gan_Image *image,
+ GAN_IMAGE_GET_ACTIVE_SUBWINDOW ( const Gan_Image *image,
                                   Gan_Alignment alignment,
                                   Gan_ImageWindow *subwin )
 {
    unsigned int first_row, last_row, first_col, last_col;
-   unsigned int tmp_first_col, tmp_last_col, words_per_row;
+   unsigned int tmp_first_col=0, tmp_last_col=0, words_per_row;
    int ctr_row, ctr_col;
    Gan_Bool last_col_set, last_row_set, non_zero_row;
    Gan_BitWord *w;
@@ -297,6 +316,14 @@ Gan_Bool
       return GAN_FALSE;
    }
 
+   if ( image->width == 0 || image->height == 0 )
+   {
+      /* no set pixels found: set window to zero size */
+      subwin->c0    = subwin->r0 =
+      subwin->width = subwin->height = 0;
+      return GAN_TRUE;
+   }
+      
    /* Get bitarray length */
    words_per_row = GAN_NO_BITWORDS(image->width);
 
@@ -361,17 +388,17 @@ Gan_Bool
    }
 
    /* fill in subwindow with result */
-   subwin->c0 = first_col*GAN_BITWORD_SIZE;
-   subwin->r0 = first_row;
-   subwin->width = (last_col - first_col + 1)*GAN_BITWORD_SIZE;
-   subwin->height = (last_row - first_row)+1;
+   subwin->c0 = (int)first_col*GAN_BITWORD_SIZE;
+   subwin->r0 = (int)first_row;
+   subwin->width = (int)(last_col - first_col + 1)*GAN_BITWORD_SIZE;
+   subwin->height = (int)(last_row - first_row)+1;
 
    /* if word alignment was requested, return immediately */
    if ( alignment == GAN_WORD_ALIGNMENT )
    {
       /* adjust subwindow to make sure it doesn't go outside image */
-      if ( subwin->c0 + subwin->width > image->width )
-         subwin->width = image->width - subwin->c0;
+      if ( subwin->c0 + subwin->width > (int)image->width )
+         subwin->width = (int)image->width - subwin->c0;
 
       return GAN_TRUE;
    }
@@ -381,7 +408,7 @@ Gan_Bool
 
    /* search for first non-zero byte */
    first_byte = INT_MAX;
-   for ( ctr_row = (int)subwin->r0; ctr_row < row_limit; ctr_row++ )
+   for ( ctr_row = subwin->r0; ctr_row < row_limit; ctr_row++ )
    {
       if ( image->row_data.gl.b[ctr_row][first_col] != 0 )
       {
@@ -413,12 +440,12 @@ Gan_Bool
                        "" );
 
    /* adjust start of row */
-   subwin->c0 += (unsigned)first_byte*8;
-   subwin->width -= (unsigned)first_byte*8;
+   subwin->c0 += first_byte*8;
+   subwin->width -= first_byte*8;
 
    /* now check end of row */
    last_byte = INT_MIN;
-   for ( ctr_row = (int)subwin->r0; ctr_row < row_limit; ctr_row++ )
+   for ( ctr_row = subwin->r0; ctr_row < row_limit; ctr_row++ )
    {
       if ( image->row_data.gl.b[ctr_row][last_col] != 0 )
       {
@@ -449,14 +476,14 @@ Gan_Bool
                        GAN_ERROR_FAILURE, "" );
 
    /* adjust end of row */
-   subwin->width -= (GAN_BITWORD_SIZE - ((unsigned)last_byte+1)*8);
+   subwin->width -= (GAN_BITWORD_SIZE - (last_byte+1)*8);
    
    /* if byte alignment was requested, return immediately */
    if ( alignment == GAN_BYTE_ALIGNMENT )
    {
       /* adjust subwindow to make sure it doesn't go outside image */
-      if ( subwin->c0 + subwin->width > image->width )
-         subwin->width = image->width - subwin->c0;
+      if ( subwin->c0 + subwin->width > (int)image->width )
+         subwin->width = (int)image->width - subwin->c0;
 
       return GAN_TRUE;
    }
@@ -468,7 +495,7 @@ Gan_Bool
 
    /* compute bit alignment for start of row */
    bit = INT_MAX;
-   for ( ctr_row = (int)subwin->r0; ctr_row < row_limit; ctr_row++ )
+   for ( ctr_row = subwin->r0; ctr_row < row_limit; ctr_row++ )
    {
       int byte;
       unsigned char ucval;
@@ -516,12 +543,12 @@ Gan_Bool
                        GAN_ERROR_FAILURE, "" );
 
    /* adjust start of row */
-   subwin->c0 += (unsigned)bit;
-   subwin->width -= (unsigned)bit;
+   subwin->c0 += bit;
+   subwin->width -= bit;
    
    /* compute bit alignment for end of row */
    bit = INT_MIN;
-   for ( ctr_row = (int)subwin->r0; ctr_row < row_limit; ctr_row++ )
+   for ( ctr_row = subwin->r0; ctr_row < row_limit; ctr_row++ )
    {
       int byte;
       unsigned char ucval;
@@ -572,8 +599,8 @@ Gan_Bool
    subwin->width -= (8-1-bit);
    
    /* adjust subwindow to make sure it doesn't go outside image */
-   if ( subwin->c0 + subwin->width > image->width )
-      subwin->width = image->width - subwin->c0;
+   if ( subwin->c0 + subwin->width > (int)image->width )
+      subwin->width = (int)image->width - subwin->c0;
 
    /* success */
    return GAN_TRUE;
@@ -600,7 +627,7 @@ Gan_Bool
  *
  */
 Gan_Bool
- GAN_IMAGE_GET_ACTIVE_SUBWINDOW ( Gan_Image *image,
+ GAN_IMAGE_GET_ACTIVE_SUBWINDOW ( const Gan_Image *image,
                                   Gan_ImageWindow *subwin )
 {
    int row, col;
@@ -658,8 +685,8 @@ Gan_Bool
    else
    {
       /* set window */
-      subwin->c0 = (unsigned)xmin; subwin->width  = (unsigned)(xmax-xmin+1);
-      subwin->r0 = (unsigned)ymin; subwin->height = (unsigned)(ymax-ymin+1);
+      subwin->c0 = xmin; subwin->width  = xmax-xmin+1;
+      subwin->r0 = ymin; subwin->height = ymax-ymin+1;
    }
       
    /* success */
@@ -683,7 +710,7 @@ Gan_Bool
  * Returns the pointer to the pixel at position \a row, \a col in
  * GAN_PIXEL_FORMAT \c GAN_PIXEL_TYPE image \a img.
  */
-GAN_PIXEL *GAN_IMAGE_GET_PIXPTR ( Gan_Image *img,
+GAN_PIXEL *GAN_IMAGE_GET_PIXPTR ( const Gan_Image *img,
                                   unsigned row, unsigned col )
 {
    /* consistency check */
@@ -707,7 +734,7 @@ GAN_PIXEL *GAN_IMAGE_GET_PIXPTR ( Gan_Image *img,
  * Returns the pixel array from the GAN_PIXEL_FORMAT \c GAN_PIXEL_TYPE image
  * \a img.
  */
-GAN_PIXEL **GAN_IMAGE_GET_PIXARR ( Gan_Image *img )
+GAN_PIXEL **GAN_IMAGE_GET_PIXARR ( const Gan_Image *img )
 {
    /* consistency check */
    gan_err_test_ptr ( img->format == GAN_PIXFMT && img->type == GAN_PIXTYPE,
@@ -737,35 +764,35 @@ GAN_PIXEL **GAN_IMAGE_GET_PIXARR ( Gan_Image *img )
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- GAN_IMAGE_MASK_WINDOW ( Gan_Image *pImage,
+ GAN_IMAGE_MASK_WINDOW ( Gan_Image *img,
                          unsigned r0,     unsigned c0,
                          unsigned height, unsigned width )
 {
    unsigned r;
 
    /* make sure that region is inside image */
-   gan_err_test_bool ( r0+height <= pImage->height &&
-                       c0+width <= pImage->width,
+   gan_err_test_bool ( r0+height <= img->height &&
+                       c0+width <= img->width,
                        "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "" );
 
    for ( r = 0; r < r0; r++ )
-      if ( !gan_image_bit_fill_row ( pImage, r, 0, pImage->width, GAN_FALSE ) )
+      if ( !gan_image_bit_fill_row ( img, r, 0, img->width, GAN_FALSE ) )
       {
          gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "" );
          return GAN_FALSE;
       }
 
    for ( ; r < r0+height; r++ )
-      if ( !gan_image_bit_fill_row ( pImage, r, 0, c0, GAN_FALSE ) ||
-           !gan_image_bit_fill_row ( pImage, r, c0+width,
-                                     pImage->width-c0-width, GAN_FALSE ) )
+      if ( !gan_image_bit_fill_row ( img, r, 0, c0, GAN_FALSE ) ||
+           !gan_image_bit_fill_row ( img, r, c0+width,
+                                     img->width-c0-width, GAN_FALSE ) )
       {
          gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "" );
          return GAN_FALSE;
       }
       
-   for ( ; r < pImage->height; r++ )
-      if ( !gan_image_bit_fill_row ( pImage, r, 0, pImage->width, GAN_FALSE ) )
+   for ( ; r < img->height; r++ )
+      if ( !gan_image_bit_fill_row ( img, r, 0, img->width, GAN_FALSE ) )
       {
          gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "" );
          return GAN_FALSE;
@@ -775,6 +802,33 @@ Gan_Bool
    return GAN_TRUE;
 }
 
+/**
+ * \brief Clear binary image in specified rectangular region.
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+Gan_Bool
+ GAN_IMAGE_CLEAR_WINDOW ( Gan_Image *img,
+                          unsigned r0,     unsigned c0,
+                          unsigned height, unsigned width )
+{
+   unsigned r;
+
+   /* make sure that region is inside image */
+   gan_err_test_bool ( r0+height <= img->height &&
+                       c0+width <= img->width,
+                       "GAN_IMAGE_CLEAR_WINDOW", GAN_ERROR_FAILURE, "" );
+
+   for ( r = r0; r < r0+height; r++ )
+      if ( !gan_image_bit_fill_row ( img, r, c0, width, GAN_FALSE ) )
+      {
+         gan_err_register ( "GAN_IMAGE_CLEAR_WINDOW", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+
+   /* success */
+   return GAN_TRUE;
+}
+
 #else
 
 /**
@@ -782,49 +836,52 @@ Gan_Bool
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- GAN_IMAGE_MASK_WINDOW ( Gan_Image *pImage,
+ GAN_IMAGE_MASK_WINDOW ( Gan_Image *img,
                          unsigned r0,     unsigned c0,
                          unsigned height, unsigned width )
 {
-   unsigned r, c;
+   unsigned r;
+   unsigned last_bit = img->width - c0 - width;
 
    /* make sure that region is inside image */
-   gan_err_test_bool ( r0+height <= pImage->height &&
-                       c0+width <= pImage->width,
+   gan_err_test_bool ( r0+height <= img->height &&
+                       c0+width <= img->width,
                        "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "" );
 
    for ( r = 0; r < r0; r++ )
-      for ( c = 0; c < pImage->width; c++ )
-         if ( !GAN_IMAGE_SET_PIX ( pImage, r, c, GAN_IMAGE_PIXEL_ZERO_VAL ) )
-         {
-            gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "");
-            return GAN_FALSE;
-         }
+      GAN_FILL_ARRAY ( img->row_data.gl.GAN_IMTYPE[r], img->width, 1, GAN_IMAGE_PIXEL_ZERO_VAL );
 
    for ( ; r < r0+height; r++ )
    {
-      for ( c = 0; c < c0; c++ )
-         if ( !GAN_IMAGE_SET_PIX ( pImage, r, c, GAN_IMAGE_PIXEL_ZERO_VAL ) )
-         {
-            gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "");
-            return GAN_FALSE;
-         }
-
-      for ( c = c0+width; c < pImage->width; c++ )
-         if ( !GAN_IMAGE_SET_PIX ( pImage, r, c, GAN_IMAGE_PIXEL_ZERO_VAL ) )
-         {
-            gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "");
-            return GAN_FALSE;
-         }
+      GAN_FILL_ARRAY ( &img->row_data.gl.GAN_IMTYPE[r][0],        c0,       1, GAN_IMAGE_PIXEL_ZERO_VAL );
+      GAN_FILL_ARRAY ( &img->row_data.gl.GAN_IMTYPE[r][c0+width], last_bit, 1, GAN_IMAGE_PIXEL_ZERO_VAL );
    }
 
-   for ( ; r < pImage->height; r++ )
-      for ( c = 0; c < pImage->width; c++ )
-         if ( !GAN_IMAGE_SET_PIX ( pImage, r, c, GAN_IMAGE_PIXEL_ZERO_VAL ) )
-         {
-            gan_err_register ( "GAN_IMAGE_MASK_WINDOW", GAN_ERROR_FAILURE, "");
-            return GAN_FALSE;
-         }
+   for ( ; r < img->height; r++ )
+      GAN_FILL_ARRAY ( img->row_data.gl.GAN_IMTYPE[r], img->width, 1, GAN_IMAGE_PIXEL_ZERO_VAL );
+
+   /* success */
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Clear image in specified rectangular region.
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+Gan_Bool
+ GAN_IMAGE_CLEAR_WINDOW ( Gan_Image *img,
+                          unsigned r0,     unsigned c0,
+                          unsigned height, unsigned width )
+{
+   unsigned r;
+
+   /* make sure that region is inside image */
+   gan_err_test_bool ( r0+height <= img->height &&
+                       c0+width <= img->width,
+                       "GAN_IMAGE_CLEAR_WINDOW", GAN_ERROR_FAILURE, "" );
+
+   for ( r = r0; r < r0+height; r++ )
+      GAN_FILL_ARRAY ( &img->row_data.gl.GAN_IMTYPE[r][c0], width, 1, GAN_IMAGE_PIXEL_ZERO_VAL );
 
    /* success */
    return GAN_TRUE;
@@ -836,6 +893,156 @@ Gan_Bool
  * \}
  */
 
+#ifdef GAN_IMAGE_GET_MINIMUM_PIXEL
+#ifndef GAN_BITMAP
+
+/**
+ * \addtogroup ImageMinMax
+ * \{
+ */
+
+/**
+ * \brief Calculates the minimum pixel value in an image
+ *
+ * Calculates the minimum pixel value in \a pImage over the mask \a pMask,
+ * which can be passed as \c NULL to consider all the pixels. The result is
+ * returned in \a minval.
+ *
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+Gan_Bool
+ GAN_IMAGE_GET_MINIMUM_PIXEL ( const Gan_Image *pImage, const Gan_Image *pMask, GAN_PIXEL *minval )
+{
+   GAN_PIXEL min_pixel = GAN_IMAGE_PIXEL_MAX_VAL;
+
+   gan_err_test_bool ( pMask == NULL ||
+                       (pMask->width == pImage->width && pMask->height == pImage->height),
+                       "GAN_IMAGE_GET_MINIMUM_PIXEL", GAN_ERROR_INCOMPATIBLE, "" );
+
+   if(pImage->width == 0 || pImage->height == 0)
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "GAN_IMAGE_GET_MINIMUM_PIXEL", GAN_ERROR_NO_DATA, "" );
+      return GAN_FALSE;
+   }
+
+   if(pMask == NULL)
+   {
+      int r, c;
+
+      for(r = (int)pImage->height-1; r >= 0; r--)
+         for(c = (int)pImage->width-1; c >= 0; c--)
+            if(GAN_IMAGE_GET_PIX(pImage, r, c) < min_pixel)
+               min_pixel = GAN_IMAGE_GET_PIX(pImage, r, c);
+   }
+   else
+   {
+      Gan_ImageWindow window;
+      unsigned int row_max, col_max, r, c;
+
+      if(!gan_image_get_active_subwindow_b ( pMask, GAN_BIT_ALIGNMENT, &window ))
+      {
+         gan_err_register ( "GAN_IMAGE_GET_MINIMUM_PIXEL", GAN_ERROR_FAILURE, "" );
+         return GAN_FALSE;
+      }
+
+      if(window.width == 0 || window.height == 0)
+      {
+         gan_err_flush_trace();
+         gan_err_register ( "GAN_IMAGE_GET_MINIMUM_PIXEL", GAN_ERROR_NO_DATA, "" );
+         return GAN_FALSE;
+      }
+
+      col_max = window.c0 + window.width;
+      row_max = window.r0 + window.height;
+      for ( r = window.r0; r < row_max; r++ )
+         for ( c = window.c0; c < col_max; c++ )
+            if(GAN_IMAGE_GET_PIX(pImage, r, c) < min_pixel)
+               min_pixel = GAN_IMAGE_GET_PIX(pImage, r, c);
+   }
+
+   *minval = min_pixel;
+   return GAN_TRUE;
+}
+
+/**
+ * \brief Calculates the maximum pixel value in an image
+ *
+ * Calculates the maximum pixel value in \a pImage over the mask \a pMask,
+ * which can be passed as \c NULL to consider all the pixels. The result is
+ * returned in \a maxval.
+ *
+ * \return #GAN_TRUE on success, #GAN_FALSE on failure.
+ */
+Gan_Bool
+ GAN_IMAGE_GET_MAXIMUM_PIXEL ( const Gan_Image *pImage, const Gan_Image *pMask, GAN_PIXEL *maxval )
+{
+   GAN_PIXEL max_pixel = GAN_IMAGE_PIXEL_MIN_VAL;
+
+   gan_err_test_bool ( pMask == NULL ||
+                       (pMask->width == pImage->width &&
+                        pMask->height == pImage->height),
+                       "GAN_IMAGE_GET_MAXIMUM_PIXEL", GAN_ERROR_INCOMPATIBLE,
+                       "" );
+
+   if(pImage->width == 0 || pImage->height == 0)
+   {
+      gan_err_flush_trace();
+      gan_err_register ( "GAN_IMAGE_GET_MAXIMUM_PIXEL", GAN_ERROR_NO_DATA,
+                         "" );
+      return GAN_FALSE;
+   }
+
+   if(pMask == NULL)
+   {
+      int r, c;
+
+      for(r = (int)pImage->height-1; r >= 0; r--)
+         for(c = (int)pImage->width-1; c >= 0; c--)
+            if(GAN_IMAGE_GET_PIX(pImage, r, c) > max_pixel)
+               max_pixel = GAN_IMAGE_GET_PIX(pImage, r, c);
+   }
+   else
+   {
+      Gan_ImageWindow window;
+      unsigned int row_max, col_max, r, c;
+
+      if(!gan_image_get_active_subwindow_b ( pMask, GAN_BIT_ALIGNMENT,
+                                             &window ))
+      {
+         gan_err_register ( "GAN_IMAGE_GET_MAXIMUM_PIXEL", GAN_ERROR_FAILURE,
+                            "" );
+         return GAN_FALSE;
+      }
+
+      if(window.width == 0 || window.height == 0)
+      {
+         gan_err_flush_trace();
+         gan_err_register ( "GAN_IMAGE_GET_MAXIMUM_PIXEL", GAN_ERROR_NO_DATA,
+                            "" );
+         return GAN_FALSE;
+      }
+
+      col_max = window.c0 + window.width;
+      row_max = window.r0 + window.height;
+      for ( r = window.r0; r < row_max; r++ )
+         for ( c = window.c0; c < col_max; c++ )
+            if(gan_image_get_pix_b(pMask, r, c) &&
+               GAN_IMAGE_GET_PIX(pImage, r, c) > max_pixel)
+               max_pixel = GAN_IMAGE_GET_PIX(pImage, r, c);
+   }
+
+   *maxval = max_pixel;
+   return GAN_TRUE;
+}
+
+/**
+ * \}
+ */
+
+#endif /* #ifndef GAN_BITMAP */
+#endif /* #ifdef GAN_IMAGE_GET_MINIMUM_PIXEL */
+
 /**
  * \}
  */
