@@ -3,8 +3,8 @@
  * Module:        Image package test program
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.39 $
- * Last edited:   $Date: 2003/08/11 14:10:14 $
+ * Revision:      $Revision: 1.41 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -243,6 +243,92 @@ static Gan_Bool ramp_image ( Gan_Image *img )
    return GAN_TRUE;
 }
 
+static Gan_Bool
+ image_bit_dilate_horiz_safe ( Gan_Image *image, int no_pixels,
+                               Gan_Image *restrict_mask )
+{
+   Gan_Image *image_copy, *copy2;
+   int row, col, pixel;
+
+   image_copy = gan_image_copy_s(image);
+   copy2 = gan_image_copy_s(image);
+   cu_assert(image_copy != NULL && copy2 != NULL);
+   for ( pixel = 0; pixel < no_pixels; pixel++ )
+   {
+      for ( row = (int)image->height-1; row >= 0; row-- )
+         for ( col = (int)image->width-1; col >= 0; col-- )
+            if ( gan_image_get_pix_b ( copy2, row, col ) )
+            {
+               if ( col > 0 )
+                  cu_assert ( gan_image_set_pix_b ( image_copy, row, col-1,
+                                                    GAN_TRUE ) );
+
+               if ( col < (int)image->width-1 )
+                  cu_assert ( gan_image_set_pix_b ( image_copy, row, col+1,
+                                                    GAN_TRUE ) );
+            }
+
+      cu_assert ( gan_image_copy_q ( image_copy, copy2 ) != NULL );
+   }
+
+   if ( restrict_mask == NULL )
+   {
+      cu_assert ( gan_image_copy_q ( copy2, image ) != NULL );
+   }
+   else
+   {
+      cu_assert ( gan_image_bit_and_i ( copy2, restrict_mask ) );
+      cu_assert ( gan_image_bit_or_i ( image, copy2 ) );
+   }
+
+   gan_image_free(copy2);
+   gan_image_free(image_copy);
+   return GAN_TRUE;
+}
+
+static Gan_Bool
+ image_bit_dilate_vert_safe ( Gan_Image *image, int no_pixels,
+                              Gan_Image *restrict_mask )
+{
+   Gan_Image *image_copy, *copy2;
+   int row, col, pixel;
+
+   image_copy = gan_image_copy_s(image);
+   copy2 = gan_image_copy_s(image);
+   cu_assert(image_copy != NULL && copy2 != NULL);
+   for ( pixel = 0; pixel < no_pixels; pixel++ )
+   {
+      for ( row = (int)image->height-1; row >= 0; row-- )
+         for ( col = (int)image->width-1; col >= 0; col-- )
+            if ( gan_image_get_pix_b ( copy2, row, col ) )
+            {
+               if ( row > 0 )
+                  cu_assert ( gan_image_set_pix_b ( image_copy, row-1, col,
+                                                    GAN_TRUE ) );
+
+               if ( row < (int)image->height-1 )
+                  cu_assert ( gan_image_set_pix_b ( image_copy, row+1, col,
+                                                    GAN_TRUE ) );
+            }
+
+      cu_assert ( gan_image_copy_q ( image_copy, copy2 ) != NULL );
+   }
+
+   if ( restrict_mask == NULL )
+   {
+      cu_assert ( gan_image_copy_q ( copy2, image ) != NULL );
+   }
+   else
+   {
+      cu_assert ( gan_image_bit_and_i ( copy2, restrict_mask ) );
+      cu_assert ( gan_image_bit_or_i ( image, copy2 ) );
+   }
+
+   gan_image_free(copy2);
+   gan_image_free(image_copy);
+   return GAN_TRUE;
+}
+
 static Gan_Bool
  gan_no_bits_for_type ( Gan_Type type, unsigned *no_bits )
 {
@@ -263,13 +349,13 @@ static Gan_Bool
 #define NO_TESTS 10
 
 /* precompiled test values */
-static gan_ui8 val_ui8[NO_TESTS] =
+static gan_uint8 val_ui8[NO_TESTS] =
 { 0u, 26u, 59u, 83u, 111u, 137u, 163u, 189u, 223u, 255u };
 
-static gan_ui16 val_ui16[NO_TESTS] =
+static gan_uint16 val_ui16[NO_TESTS] =
 { 0u, 6866u, 15258u, 21347u, 28538u, 35228u, 41953u, 48583u, 57278u, 65535u };
 
-static gan_ui32 val_ui32[NO_TESTS] =
+static gan_uint32 val_ui32[NO_TESTS] =
 { 0u, 450000000u, 1000000000u, 1399034893u, 1870283845u, 2308758392u,
   2749485920u, 3183948561u, 3753829104u, 4294967295u };
 
@@ -622,7 +708,7 @@ static Gan_Bool
    
    {
       /* test grey-level 8-bit unsigned integer images */
-      gan_ui8 *ptr, **arr;
+      gan_uint8 *ptr, **arr;
 
       img = gan_image_alloc_gl_ui8 ( 10, 12 );
       cu_assert ( img != NULL );
@@ -633,7 +719,7 @@ static Gan_Bool
    
    {
       /* test grey-level 16-bit unsigned integer images */
-      gan_ui16 *ptr, **arr;
+      gan_uint16 *ptr, **arr;
 
       img = gan_image_alloc_gl_ui16 ( 10, 12 );
       cu_assert ( img != NULL );
@@ -644,7 +730,7 @@ static Gan_Bool
    
    {
       /* test grey-level 32-bit unsigned integer images */
-      gan_ui32 *ptr, **arr;
+      gan_uint32 *ptr, **arr;
 
       img = gan_image_alloc_gl_ui32 ( 10, 12 );
       cu_assert ( img != NULL );
@@ -1134,9 +1220,11 @@ static Gan_Bool teardown_test(void)
 /* Runs the image test functions */
 static Gan_Bool run_test(void)
 {
-   Gan_Image *img, *img2, *img3;
+   Gan_Image *img, *img2, *img3=NULL;
    int pix[12], i, j, count, it;
 
+   if(img3!=NULL)gan_image_write("",GAN_PNG_FORMAT,NULL);
+
    /* build image */
    for ( i = 11; i >= 0; i-- )
       pix[i] = i;
@@ -1430,9 +1518,85 @@ static Gan_Bool run_test(void)
       cu_assert ( subwin.r0 == ymin && subwin.height == ymax - ymin + 1 );
    }
 
+   /* test binary image dilation */
+   for ( it = 0; it < 100; it++ )
+   {
+      unsigned width = 1+(gan_random()%256), height = 1+(gan_random()%256);
+      int hdilate = 1+(gan_random()%7), vdilate = 1+(gan_random()%7);
+      unsigned row, col;
+      
+      img  = gan_image_set_b ( img, height, width );
+      cu_assert ( img != NULL );
+
+      /* build bit image with a random number of bits */
+      gan_image_fill_zero ( img );
+      for ( count = gan_random()%32; count >= 0; count-- )
+      {
+         row = gan_random()%img->height;
+         col = gan_random()%img->width;
+         cu_assert ( gan_image_set_pix_b ( img, row, col, GAN_TRUE ) );
+      }
+
+      cu_assert ( gan_image_copy_q ( img, img2 ) != NULL );
+      cu_assert ( gan_image_bit_dilate_horiz ( img, hdilate, NULL ) );
+      cu_assert ( image_bit_dilate_horiz_safe ( img2, hdilate, NULL ) );
+      cu_assert ( gan_image_bit_dilate_vert ( img, vdilate, NULL ) );
+      cu_assert ( image_bit_dilate_vert_safe ( img2, vdilate, NULL ) );
+
+      /* test result */
+      for ( i = (int)img->height-1; i >= 0; i-- )
+         for ( j = (int)img->width-1; j >= 0; j-- )
+         {
+            Gan_Bool bit, bit2;
+
+            bit = gan_image_get_pix_b ( img, i, j );
+            bit2 = gan_image_get_pix_b ( img2, i, j );
+            cu_assert ( bit == bit2 );
+         }
+
+      /* build bit image with a random number of bits */
+      gan_image_fill_zero ( img );
+      for ( count = gan_random()%32; count >= 0; count-- )
+      {
+         row = gan_random()%img->height;
+         col = gan_random()%img->width;
+         cu_assert ( gan_image_set_pix_b ( img, row, col, GAN_TRUE ) );
+      }
+
+      /* build restriction mask */
+      img3 = gan_image_set_b ( img3, height, width );
+      cu_assert ( img3 != NULL );
+
+      /* build bit image with a random number of bits */
+      gan_image_fill_zero ( img3 );
+      for ( count = gan_random()%1000; count >= 0; count-- )
+      {
+         row = gan_random()%img3->height;
+         col = gan_random()%img3->width;
+         cu_assert ( gan_image_set_pix_b ( img3, row, col, GAN_TRUE ) );
+      }
+
+      cu_assert ( gan_image_copy_q ( img, img2 ) != NULL );
+      cu_assert ( gan_image_bit_dilate_horiz ( img, hdilate, img3 ) );
+      cu_assert ( image_bit_dilate_horiz_safe ( img2, hdilate, img3 ) );
+      cu_assert ( gan_image_bit_dilate_vert ( img, vdilate, img3 ) );
+      cu_assert ( image_bit_dilate_vert_safe ( img2, vdilate, img3 ) );
+
+      /* test result */
+      for ( i = (int)img->height-1; i >= 0; i-- )
+         for ( j = (int)img->width-1; j >= 0; j-- )
+         {
+            Gan_Bool bit, bit2;
+
+            bit = gan_image_get_pix_b ( img, i, j );
+            bit2 = gan_image_get_pix_b ( img2, i, j );
+            cu_assert ( bit == bit2 );
+         }
+   }
+
    /* free all images and exit */
    gan_image_free_va ( img3, img2, img, NULL );
-        
+
    /* test pixel/image conversions */
    from_image = gan_image_alloc ( GAN_GREY_LEVEL_IMAGE, GAN_UINT8, 1, 1 );
    to_image = gan_image_alloc ( GAN_GREY_LEVEL_IMAGE, GAN_UINT8, 1, 1 );
