@@ -1,3 +1,5 @@
+#include <gandalf/image/io/cineon_io.h>
+#include <gandalf/image/io/dpx_io.h>
 #include <gandalf/image/io/image_io.h>
 #include <gandalf/image/io/jpeg_io.h>
 #include <gandalf/image/io/movie.h>
