@@ -3,8 +3,8 @@
  * Module:        Image pixel definitions and functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.37 $
- * Last edited:   $Date: 2003/09/08 17:30:45 $
+ * Revision:      $Revision: 1.46 $
+ * Last edited:   $Date: 2005/02/20 04:12:45 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -51,15 +51,16 @@ extern "C" {
 /**
  * \brief Different formats of image pixel.
  */
-typedef enum { GAN_GREY_LEVEL_IMAGE,       /**< grey-level images */
-               GAN_GREY_LEVEL_ALPHA_IMAGE, /**< grey-level images with alpha
-                                                channel */
-               GAN_RGB_COLOUR_IMAGE,       /**< RGB colour images */
-               GAN_RGB_COLOUR_ALPHA_IMAGE, /**< RGB colour images with alpha
-                                                channel */
-               GAN_VECTOR_FIELD_2D,        /**< 2D image of 2D vectors */
-               GAN_VECTOR_FIELD_3D }       /**< 2D image of 3D vectors */
-  Gan_ImageFormat;
+typedef enum
+{
+   GAN_GREY_LEVEL_IMAGE,       /**< grey-level images */
+   GAN_GREY_LEVEL_ALPHA_IMAGE, /**< grey-level images with alpha channel */
+   GAN_RGB_COLOUR_IMAGE,       /**< RGB colour images */
+   GAN_RGB_COLOUR_ALPHA_IMAGE, /**< RGB colour images with alpha channel */
+   GAN_VECTOR_FIELD_2D,        /**< 2D image of 2D vectors */
+   GAN_VECTOR_FIELD_3D,       /**< 2D image of 3D vectors */
+   GAN_UNDEFINED_IMAGE_FORMAT
+} Gan_ImageFormat;
 
 /**
  * \brief Structure defining grey-level unsigned character pixel with alpha channel.
@@ -339,6 +340,52 @@ typedef struct Gan_RGBAPixel_ui Gan_RGBAPixel_ui32;
 #endif /* #if (SIZEOF_INT == 4) */
 #endif /* #ifdef GAN_UINT32 */
 
+#ifdef GAN_FLOAT32
+#if (SIZEOF_FLOAT == 4)
+/**
+ * \brief Structure defining grey-level 32-bit float pixel with
+ *        alpha channel.
+ */
+typedef struct Gan_GLAPixel_f Gan_GLAPixel_f32;
+
+/**
+ * \brief Structure defining RGB colour 32-bit float pixel.
+ */
+typedef struct Gan_RGBPixel_f Gan_RGBPixel_f32;
+
+/**
+ * \brief Structure defining RGB colour 32-bit float pixel with
+ *        alpha channel.
+ */
+typedef struct Gan_RGBAPixel_f Gan_RGBAPixel_f32;
+#else
+#error "32-bit float type conflict"
+#endif /* #if (SIZEOF_FLOAT == 4) */
+#endif /* #ifdef GAN_FLOAT32 */
+
+#ifdef GAN_FLOAT64
+#if (SIZEOF_DOUBLE == 8)
+/**
+ * \brief Structure defining grey-level 64-bit float pixel with
+ *        alpha channel.
+ */
+typedef struct Gan_GLAPixel_d Gan_GLAPixel_f64;
+
+/**
+ * \brief Structure defining RGB colour 64-bit float pixel.
+ */
+typedef struct Gan_RGBPixel_d Gan_RGBPixel_f64;
+
+/**
+ * \brief Structure defining RGB colour 64-bit float pixel with
+ *        alpha channel.
+ */
+typedef struct Gan_RGBAPixel_d Gan_RGBAPixel_f64;
+#else
+#error "64-bit float type conflict"
+#endif /* #if (SIZEOF_DOUBLE == 8) */
+#endif /* #ifdef GAN_FLOAT64 */
+
 /**
  * \brief Structure definition for image pixel of any format or type.
  */
@@ -367,13 +414,13 @@ typedef struct Gan_Pixel
          void          *p;
 
 #ifdef GAN_UINT8
-         gan_ui8 ui8;
+         gan_uint8 ui8;
 #endif
 #ifdef GAN_UINT16
-         gan_ui16 ui16;
+         gan_uint16 ui16;
 #endif
 #ifdef GAN_UINT32
-         gan_ui32 ui32;
+         gan_uint32 ui32;
 #endif
       } gl;
 
@@ -463,22 +510,22 @@ typedef struct Gan_Pixel
    } data;
 } Gan_Pixel;
 
-unsigned char  gan_pixel_rgbuc_to_yuc ( Gan_RGBPixel_uc *rgb );
-unsigned short gan_pixel_rgbuc_to_yus ( Gan_RGBPixel_uc *rgb );
-unsigned int   gan_pixel_rgbuc_to_yui ( Gan_RGBPixel_uc *rgb );
-unsigned char  gan_pixel_rgbus_to_yuc ( Gan_RGBPixel_us *rgb );
-unsigned short gan_pixel_rgbus_to_yus ( Gan_RGBPixel_us *rgb );
-float          gan_pixel_rgbui_to_yf  ( Gan_RGBPixel_ui *rgb );
-float          gan_pixel_rgbf_to_yf   ( Gan_RGBPixel_f  *rgb );
-double         gan_pixel_rgbd_to_yd   ( Gan_RGBPixel_d  *rgb );
-unsigned char  gan_pixel_rgbauc_to_yuc ( Gan_RGBAPixel_uc *rgba );
-unsigned short gan_pixel_rgbauc_to_yus ( Gan_RGBAPixel_uc *rgba );
-unsigned int   gan_pixel_rgbauc_to_yui ( Gan_RGBAPixel_uc *rgba );
-unsigned char  gan_pixel_rgbaus_to_yuc ( Gan_RGBAPixel_us *rgba );
-unsigned short gan_pixel_rgbaus_to_yus ( Gan_RGBAPixel_us *rgba );
-float          gan_pixel_rgbaui_to_yf  ( Gan_RGBAPixel_ui *rgba );
-float          gan_pixel_rgbaf_to_yf   ( Gan_RGBAPixel_f  *rgba );
-double         gan_pixel_rgbad_to_yd   ( Gan_RGBAPixel_d  *rgba );
+unsigned char  gan_pixel_rgbuc_to_yuc ( const Gan_RGBPixel_uc *rgb );
+unsigned short gan_pixel_rgbuc_to_yus ( const Gan_RGBPixel_uc *rgb );
+unsigned int   gan_pixel_rgbuc_to_yui ( const Gan_RGBPixel_uc *rgb );
+unsigned char  gan_pixel_rgbus_to_yuc ( const Gan_RGBPixel_us *rgb );
+unsigned short gan_pixel_rgbus_to_yus ( const Gan_RGBPixel_us *rgb );
+float          gan_pixel_rgbui_to_yf  ( const Gan_RGBPixel_ui *rgb );
+float          gan_pixel_rgbf_to_yf   ( const Gan_RGBPixel_f  *rgb );
+double         gan_pixel_rgbd_to_yd   ( const Gan_RGBPixel_d  *rgb );
+unsigned char  gan_pixel_rgbauc_to_yuc ( const Gan_RGBAPixel_uc *rgba );
+unsigned short gan_pixel_rgbauc_to_yus ( const Gan_RGBAPixel_uc *rgba );
+unsigned int   gan_pixel_rgbauc_to_yui ( const Gan_RGBAPixel_uc *rgba );
+unsigned char  gan_pixel_rgbaus_to_yuc ( const Gan_RGBAPixel_us *rgba );
+unsigned short gan_pixel_rgbaus_to_yus ( const Gan_RGBAPixel_us *rgba );
+float          gan_pixel_rgbaui_to_yf  ( const Gan_RGBAPixel_ui *rgba );
+float          gan_pixel_rgbaf_to_yf   ( const Gan_RGBAPixel_f  *rgba );
+double         gan_pixel_rgbad_to_yd   ( const Gan_RGBAPixel_d  *rgba );
 
 Gan_Bool gan_image_convert_pixel ( Gan_Pixel *inpix,
                                    Gan_ImageFormat format, Gan_Type type,
@@ -681,13 +728,15 @@ unsigned int gan_pixel_d_to_ui ( double x );
                                : (unsigned) UINT_MAX))
 #endif
 
-/**
+extern float lookup_table_8bit[];
+
+/*
  * \brief Macro: Convert a pixel field from \c unsigned \c char to \c float type.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 float gan_pixel_uc_to_f ( unsigned char x );
 #else
-#define gan_pixel_uc_to_f(x) ((float)(x)/GAN_UCHAR_MAXF)
+#define gan_pixel_uc_to_f(x) (lookup_table_8bit[x])
 #endif
 
 /**
@@ -828,7 +877,106 @@ double gan_pixel_i_to_d ( int x );
 #else
 #define gan_pixel_i_to_d(x) ((double)(x)/GAN_INT_MAXD)
 #endif
-   
+
+// hard-wired 8 bit conversions
+#if (SIZEOF_CHAR == 1)
+
+/// Macro: Convert a pixel field from \c float to unsigned 8 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_f_to_ui8 ( float x );
+#else
+#define gan_pixel_f_to_ui8(x) gan_pixel_f_to_uc(x)
+#endif
+
+/// Macro: Convert a pixel field from \c double to unsigned 8 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_d_to_ui8 ( double x );
+#else
+#define gan_pixel_d_to_ui8(x) gan_pixel_d_to_uc(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 8 bit integer to \c float type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+float gan_pixel_ui8_to_f ( gan_uint8 x );
+#else
+#define gan_pixel_ui8_to_f(x) gan_pixel_uc_to_f(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 8 bit integer to \c double type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+double gan_pixel_ui8_to_d ( gan_uint8 x );
+#else
+#define gan_pixel_ui8_to_d(x) gan_pixel_uc_to_d(x)
+#endif
+
+#endif /* (SIZEOF_CHAR == 1) */
+
+// hard-wired 16 bit conversions
+#if (SIZEOF_SHORT == 2)
+
+/// Macro: Convert a pixel field from \c float to unsigned 16 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_f_to_ui16 ( float x );
+#else
+#define gan_pixel_f_to_ui16(x) gan_pixel_f_to_us(x)
+#endif
+
+/// Macro: Convert a pixel field from \c double to unsigned 16 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_d_to_ui16 ( double x );
+#else
+#define gan_pixel_d_to_ui16(x) gan_pixel_d_to_us(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 16 bit integer to \c float type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+float gan_pixel_ui16_to_f ( gan_uint16 x );
+#else
+#define gan_pixel_ui16_to_f(x) gan_pixel_us_to_f(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 16 bit integer to \c double type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+double gan_pixel_ui16_to_d ( gan_uint16 x );
+#else
+#define gan_pixel_ui16_to_d(x) gan_pixel_us_to_d(x)
+#endif
+
+#endif /* (SIZEOF_SHORT == 2) */
+
+// hard-wired 32 bit conversions
+#if (SIZEOF_INT == 4)
+
+/// Macro: Convert a pixel field from \c float to unsigned 32 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_f_to_ui32 ( float x );
+#else
+#define gan_pixel_f_to_ui32(x) gan_pixel_f_to_ui(x)
+#endif
+
+/// Macro: Convert a pixel field from \c double to unsigned 32 bit integer type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+int gan_pixel_d_to_ui32 ( double x );
+#else
+#define gan_pixel_d_to_ui32(x) gan_pixel_d_to_ui(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 32 bit integer to \c float type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+float gan_pixel_ui32_to_f ( gan_uint32 x );
+#else
+#define gan_pixel_ui32_to_f(x) gan_pixel_ui_to_f(x)
+#endif
+
+/// Macro: Convert a pixel field from unsigned 32 bit integer to \c double type.
+#ifdef GAN_GENERATE_DOCUMENTATION
+double gan_pixel_ui32_to_d ( gan_uint32 x );
+#else
+#define gan_pixel_ui32_to_d(x) gan_pixel_ui_to_d(x)
+#endif
+
+#endif /* (SIZEOF_INT == 4) */
+
 /**
  * \brief Macro: Convert pixel.
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
