@@ -3,8 +3,8 @@
  * Module:        2x2 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.22 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.25 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -72,7 +72,7 @@ Gan_Matrix22 *gan_mat22_fill_q ( Gan_Matrix22 *A, double XX, double XY,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat22_get_rows_q ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
+void gan_mat22_get_rows_q ( const Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
 #else
 #define gan_mat22_get_rows_q(A,p,q)\
            (GAN_TEST_OP3(A,p,q,Gan_Matrix22,Gan_Vector2,Gan_Vector2),\
@@ -92,7 +92,7 @@ void gan_mat22_get_rows_q ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat22_get_cols_q ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
+void gan_mat22_get_cols_q ( const Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
 #else
 #define gan_mat22_get_cols_q(A,p,q)\
            (GAN_TEST_OP3(A,p,q,Gan_Matrix22,Gan_Vector2,Gan_Vector2),\
@@ -108,11 +108,11 @@ void gan_mat22_get_cols_q ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
 
 Gan_Matrix22 gan_mat22_fill_s ( double XX, double XY,
                                 double YX, double YY );
-void gan_mat22_get_rows_s ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
-void gan_mat22_get_cols_s ( Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
-Gan_Matrix22 *gan_mat22_from_mat_q ( Gan_Matrix *A, Gan_Matrix22 *B );
-Gan_Matrix22 *gan_mat22_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix22 *B );
-Gan_SquMatrix22 *gan_squmat22_from_squmat_q ( Gan_SquMatrix *A,
+void gan_mat22_get_rows_s ( const Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
+void gan_mat22_get_cols_s ( const Gan_Matrix22 *A, Gan_Vector2 *p, Gan_Vector2 *q );
+Gan_Matrix22 *gan_mat22_from_mat_q ( const Gan_Matrix *A, Gan_Matrix22 *B );
+Gan_Matrix22 *gan_mat22_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix22 *B );
+Gan_SquMatrix22 *gan_squmat22_from_squmat_q ( const Gan_SquMatrix *A,
                                               Gan_SquMatrix22 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -125,6 +125,8 @@ Gan_SquMatrix22 *gan_squmat22_from_squmat_q ( Gan_SquMatrix *A,
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector2
 #define GAN_VECTYPE2 Gan_Vector2
 #define GAN_MATTYPEL Gan_Matrix22
