@@ -3,8 +3,8 @@
  * Module:        2x2 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.17 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.20 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -72,7 +72,7 @@ Gan_Matrix22_f *gan_mat22f_fill_q ( Gan_Matrix22_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat22f_get_rows_q ( Gan_Matrix22_f *A,
+void gan_mat22f_get_rows_q ( const Gan_Matrix22_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q );
 #else
 #define gan_mat22f_get_rows_q(A,p,q)\
@@ -93,7 +93,7 @@ void gan_mat22f_get_rows_q ( Gan_Matrix22_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat22f_get_cols_q ( Gan_Matrix22_f *A,
+void gan_mat22f_get_cols_q ( const Gan_Matrix22_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q );
 #else
 #define gan_mat22f_get_cols_q(A,p,q)\
@@ -109,14 +109,14 @@ void gan_mat22f_get_cols_q ( Gan_Matrix22_f *A,
 #ifndef GAN_GENERATE_DOCUMENTATION
 
 Gan_Matrix22_f gan_mat22f_fill_s ( float XX, float XY, float YX, float YY );
-void gan_mat22f_get_rows_s ( Gan_Matrix22_f *A,
+void gan_mat22f_get_rows_s ( const Gan_Matrix22_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q );
-void gan_mat22f_get_cols_s ( Gan_Matrix22_f *A,
+void gan_mat22f_get_cols_s ( const Gan_Matrix22_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q );
-Gan_Matrix22_f *gan_mat22f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix22_f *B );
-Gan_Matrix22_f *gan_mat22f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_Matrix22_f *gan_mat22f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix22_f *B );
+Gan_Matrix22_f *gan_mat22f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                             Gan_Matrix22_f *B );
-Gan_SquMatrix22_f *gan_squmat22f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_SquMatrix22_f *gan_squmat22f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                                   Gan_SquMatrix22_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -129,6 +129,8 @@ Gan_SquMatrix22_f *gan_squmat22f_from_squmatf_q ( Gan_SquMatrix_f *A,
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector2_f
 #define GAN_VECTYPE2 Gan_Vector2_f
 #define GAN_MATTYPEL Gan_Matrix22_f
