@@ -3,8 +3,8 @@
  * Module:        2x3 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.20 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.23 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -75,7 +75,7 @@ Gan_Matrix23 *gan_mat23_fill_q ( Gan_Matrix23 *A,
  *
  * \return Pointer to the result matrix \a C.
  */
-Gan_Matrix22 *gan_mat23_rmultm23T_q ( Gan_Matrix23 *A, Gan_Matrix23 *B,
+Gan_Matrix22 *gan_mat23_rmultm23T_q ( const Gan_Matrix23 *A, const Gan_Matrix23 *B,
                                       Gan_Matrix22 *C );
 #endif
 
@@ -101,7 +101,7 @@ Gan_Matrix22 *gan_mat23_rmultm23T_q ( Gan_Matrix23 *A, Gan_Matrix23 *B,
  *
  * \return Pointer to the result vector \a q.
  */
-Gan_Vector2 *gan_mat23_multv2h_q ( Gan_Matrix23 *A, Gan_Vector2 *p, double h,
+Gan_Vector2 *gan_mat23_multv2h_q ( const Gan_Matrix23 *A, const Gan_Vector2 *p, double h,
                                    Gan_Vector2 *q );
 #endif
 
@@ -126,7 +126,7 @@ Gan_Vector2 *gan_mat23_multv2h_q ( Gan_Matrix23 *A, Gan_Vector2 *p, double h,
  *
  * \return No value.
  */
-void gan_mat23_get_rows_q ( Gan_Matrix23 *A, Gan_Vector3 *p, Gan_Vector3 *q );
+void gan_mat23_get_rows_q ( const Gan_Matrix23 *A, Gan_Vector3 *p, Gan_Vector3 *q );
 #else
 #define gan_mat23_get_rows_q(A,p,q)\
            (GAN_TEST_OP3(A,p,q,Gan_Matrix23,Gan_Vector3,Gan_Vector3),\
@@ -146,7 +146,7 @@ void gan_mat23_get_rows_q ( Gan_Matrix23 *A, Gan_Vector3 *p, Gan_Vector3 *q );
  *
  * \return No value.
  */
-void gan_mat23_get_cols_q ( Gan_Matrix23 *A,
+void gan_mat23_get_cols_q ( const Gan_Matrix23 *A,
                             Gan_Vector2 *p, Gan_Vector2 *q, Gan_Vector2 *r );
 #else
 #define gan_mat23_get_cols_q(A,p,q,r)\
@@ -164,7 +164,7 @@ void gan_mat23_get_cols_q ( Gan_Matrix23 *A,
  *
  * \return Pointer to result 2x2 matrix \a B.
  */
-Gan_Matrix22 *gan_mat23_get_m22l_q ( Gan_Matrix23 *A, Gan_Matrix22 *B );
+Gan_Matrix22 *gan_mat23_get_m22l_q ( const Gan_Matrix23 *A, Gan_Matrix22 *B );
 #else
 #define gan_mat23_get_m22l_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix23,Gan_Matrix22),\
@@ -180,7 +180,7 @@ Gan_Matrix22 *gan_mat23_get_m22l_q ( Gan_Matrix23 *A, Gan_Matrix22 *B );
  *
  * \return Pointer to result 2-vector \a p.
  */
-Gan_Vector2 *gan_mat23_get_v2r_q ( Gan_Matrix23 *A, Gan_Vector2 *p );
+Gan_Vector2 *gan_mat23_get_v2r_q ( const Gan_Matrix23 *A, Gan_Vector2 *p );
 #else
 #define gan_mat23_get_v2r_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix23,Gan_Vector2),\
@@ -210,7 +210,7 @@ Gan_Vector2 *gan_mat23_get_v2r_q ( Gan_Matrix23 *A, Gan_Vector2 *p );
  * \return Pointer to result 2x3 matrix \a A.
  */
 Gan_Matrix23 *gan_mat23_set_parts_q ( Gan_Matrix23 *A,
-                                      Gan_Matrix22 *B, Gan_Vector2 *p );
+                                      const Gan_Matrix22 *B, const Gan_Vector2 *p );
 #else
 #define gan_mat23_set_parts_q(A,B,p)\
            (GAN_TEST_OP3(A,B,p,\
@@ -227,19 +227,19 @@ Gan_Matrix23 *gan_mat23_set_parts_q ( Gan_Matrix23 *A,
 
 Gan_Matrix23 gan_mat23_fill_s ( double XX, double XY, double XZ,
                                 double YX, double YY, double YZ );
-Gan_Matrix22 gan_mat23_rmultm23T_s ( Gan_Matrix23 *A, Gan_Matrix23 *B );
-Gan_Vector2 gan_mat23_multv2h_s ( Gan_Matrix23 *A,
-                                  Gan_Vector2 *p, double h );
-Gan_Bool gan_mat23h_invert_q ( Gan_Matrix23 *A, double a,
+Gan_Matrix22 gan_mat23_rmultm23T_s ( const Gan_Matrix23 *A, const Gan_Matrix23 *B );
+Gan_Vector2 gan_mat23_multv2h_s ( const Gan_Matrix23 *A,
+                                  const Gan_Vector2 *p, double h );
+Gan_Bool gan_mat23h_invert_q ( const Gan_Matrix23 *A, double a,
                                Gan_Matrix23 *B, double *b );
-Gan_Matrix23 gan_mat23h_invert_s ( Gan_Matrix23 *A, double a, double *b );
-void gan_mat23_get_rows_s ( Gan_Matrix23 *A, Gan_Vector3 *p, Gan_Vector3 *q );
-void gan_mat23_get_cols_s ( Gan_Matrix23 *A,
+Gan_Matrix23 gan_mat23h_invert_s ( const Gan_Matrix23 *A, double a, double *b );
+void gan_mat23_get_rows_s ( const Gan_Matrix23 *A, Gan_Vector3 *p, Gan_Vector3 *q );
+void gan_mat23_get_cols_s ( const Gan_Matrix23 *A,
                             Gan_Vector2 *p, Gan_Vector2 *q, Gan_Vector2 *r );
-Gan_Matrix22 gan_mat23_get_m22l_s ( Gan_Matrix23 *A );
-Gan_Vector2  gan_mat23_get_v2r_s  ( Gan_Matrix23 *A );
-Gan_Matrix23 gan_mat23_set_parts_s ( Gan_Matrix22 *B, Gan_Vector2 *p );
-Gan_Matrix23 *gan_mat23_from_mat_q ( Gan_Matrix *A, Gan_Matrix23 *B );
+Gan_Matrix22 gan_mat23_get_m22l_s ( const Gan_Matrix23 *A );
+Gan_Vector2  gan_mat23_get_v2r_s  ( const Gan_Matrix23 *A );
+Gan_Matrix23 gan_mat23_set_parts_s ( const Gan_Matrix22 *B, const Gan_Vector2 *p );
+Gan_Matrix23 *gan_mat23_from_mat_q ( const Gan_Matrix *A, Gan_Matrix23 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -251,6 +251,8 @@ Gan_Matrix23 *gan_mat23_from_mat_q ( Gan_Matrix *A, Gan_Matrix23 *B );
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector2
 #define GAN_VECTYPE2 Gan_Vector3
 #define GAN_MATTYPEL Gan_Matrix22
