@@ -3,8 +3,8 @@
  * Module:        2x3 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.16 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.19 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -75,7 +75,7 @@ Gan_Matrix23_f *gan_mat23f_fill_q ( Gan_Matrix23_f *A,
  *
  * \return Pointer to the result matrix \a C.
  */
-Gan_Matrix22_f *gan_mat23f_rmultm23T_q ( Gan_Matrix23_f *A, Gan_Matrix23_f *B,
+Gan_Matrix22_f *gan_mat23f_rmultm23T_q ( const Gan_Matrix23_f *A, const Gan_Matrix23_f *B,
                                          Gan_Matrix22_f *C );
 #endif
 
@@ -101,8 +101,8 @@ Gan_Matrix22_f *gan_mat23f_rmultm23T_q ( Gan_Matrix23_f *A, Gan_Matrix23_f *B,
  *
  * \return Pointer to the result vector \a q.
  */
-Gan_Vector2_f *gan_mat23f_multv2h_q ( Gan_Matrix23_f *A,
-                                      Gan_Vector2_f *p, float h,
+Gan_Vector2_f *gan_mat23f_multv2h_q ( const Gan_Matrix23_f *A,
+                                      const Gan_Vector2_f *p, float h,
                                       Gan_Vector2_f *q );
 #endif
 
@@ -127,7 +127,7 @@ Gan_Vector2_f *gan_mat23f_multv2h_q ( Gan_Matrix23_f *A,
  *
  * \return No value.
  */
-void gan_mat23f_get_rows_q ( Gan_Matrix23_f *A,
+void gan_mat23f_get_rows_q ( const Gan_Matrix23_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q );
 #else
 #define gan_mat23f_get_rows_q(A,p,q)\
@@ -148,7 +148,7 @@ void gan_mat23f_get_rows_q ( Gan_Matrix23_f *A,
  *
  * \return No value.
  */
-void gan_mat23f_get_cols_q ( Gan_Matrix23_f *A,
+void gan_mat23f_get_cols_q ( const Gan_Matrix23_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q,
                              Gan_Vector2_f *r );
 #else
@@ -167,7 +167,7 @@ void gan_mat23f_get_cols_q ( Gan_Matrix23_f *A,
  *
  * \return Pointer to result 2x2 matrix \a B.
  */
-Gan_Matrix22_f *gan_mat23f_get_m22l_q ( Gan_Matrix23_f *A, Gan_Matrix22_f *B );
+Gan_Matrix22_f *gan_mat23f_get_m22l_q ( const Gan_Matrix23_f *A, Gan_Matrix22_f *B );
 #else
 #define gan_mat23f_get_m22l_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix23_f,Gan_Matrix22_f),\
@@ -183,7 +183,7 @@ Gan_Matrix22_f *gan_mat23f_get_m22l_q ( Gan_Matrix23_f *A, Gan_Matrix22_f *B );
  *
  * \return Pointer to result 2-vector \a p.
  */
-Gan_Vector2_f *gan_mat23f_get_v2r_q ( Gan_Matrix23_f *A, Gan_Vector2_f *p );
+Gan_Vector2_f *gan_mat23f_get_v2r_q ( const Gan_Matrix23_f *A, Gan_Vector2_f *p );
 #else
 #define gan_mat23f_get_v2r_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix23_f,Gan_Vector2_f),\
@@ -213,7 +213,7 @@ Gan_Vector2_f *gan_mat23f_get_v2r_q ( Gan_Matrix23_f *A, Gan_Vector2_f *p );
  * \return Pointer to result 2x3 matrix \a A.
  */
 Gan_Matrix23_f *gan_mat23f_set_parts_q ( Gan_Matrix23_f *A,
-                                         Gan_Matrix22_f *B, Gan_Vector2_f *p );
+                                         const Gan_Matrix22_f *B, const Gan_Vector2_f *p );
 #else
 #define gan_mat23f_set_parts_q(A,B,p)\
            (GAN_TEST_OP3(A,B,p,\
@@ -230,21 +230,21 @@ Gan_Matrix23_f *gan_mat23f_set_parts_q ( Gan_Matrix23_f *A,
 
 Gan_Matrix23_f gan_mat23f_fill_s ( float XX, float XY, float XZ,
                                    float YX, float YY, float YZ );
-Gan_Matrix22_f gan_mat23f_rmultm23T_s ( Gan_Matrix23_f *A, Gan_Matrix23_f *B );
-Gan_Vector2_f gan_mat23f_multv2h_s ( Gan_Matrix23_f *A,
-                                     Gan_Vector2_f *p, float h );
-Gan_Bool gan_mat23hf_invert_q ( Gan_Matrix23_f *A, float a,
+Gan_Matrix22_f gan_mat23f_rmultm23T_s ( const Gan_Matrix23_f *A, const Gan_Matrix23_f *B );
+Gan_Vector2_f gan_mat23f_multv2h_s ( const Gan_Matrix23_f *A,
+                                     const Gan_Vector2_f *p, float h );
+Gan_Bool gan_mat23hf_invert_q ( const Gan_Matrix23_f *A, float a,
                                 Gan_Matrix23_f *B, float *b );
-Gan_Matrix23_f gan_mat23hf_invert_s ( Gan_Matrix23_f *A, float a, float *b );
-void gan_mat23f_get_rows_s ( Gan_Matrix23_f *A,
+Gan_Matrix23_f gan_mat23hf_invert_s ( const Gan_Matrix23_f *A, float a, float *b );
+void gan_mat23f_get_rows_s ( const Gan_Matrix23_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q );
-void gan_mat23f_get_cols_s ( Gan_Matrix23_f *A,
+void gan_mat23f_get_cols_s ( const Gan_Matrix23_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q,
                              Gan_Vector2_f *r );
-Gan_Matrix22_f gan_mat23f_get_m22l_s ( Gan_Matrix23_f *A );
-Gan_Vector2_f  gan_mat23f_get_v2r_s  ( Gan_Matrix23_f *A );
-Gan_Matrix23_f gan_mat23f_set_parts_s ( Gan_Matrix22_f *B, Gan_Vector2_f *p );
-Gan_Matrix23_f *gan_mat23f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix23_f *B );
+Gan_Matrix22_f gan_mat23f_get_m22l_s ( const Gan_Matrix23_f *A );
+Gan_Vector2_f  gan_mat23f_get_v2r_s  ( const Gan_Matrix23_f *A );
+Gan_Matrix23_f gan_mat23f_set_parts_s ( const Gan_Matrix22_f *B, const Gan_Vector2_f *p );
+Gan_Matrix23_f *gan_mat23f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix23_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -256,6 +256,8 @@ Gan_Matrix23_f *gan_mat23f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix23_f *B );
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector2_f
 #define GAN_VECTYPE2 Gan_Vector3_f
 #define GAN_MATTYPEL Gan_Matrix22_f
