@@ -3,8 +3,8 @@
  * Module:        2x4 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.20 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.23 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -76,7 +76,7 @@ Gan_Matrix24 *gan_mat24_fill_q ( Gan_Matrix24 *A,
  * \return Pointer to the result 2-vector \a q.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector2 *gan_mat24_multv3h_q ( Gan_Matrix24 *A, Gan_Vector3 *p, double h,
+Gan_Vector2 *gan_mat24_multv3h_q ( const Gan_Matrix24 *A, const Gan_Vector3 *p, double h,
                                    Gan_Vector2 *q );
 #else
 #define gan_mat24_multv3h_q(A,p,h,q)\
@@ -106,7 +106,7 @@ Gan_Vector2 *gan_mat24_multv3h_q ( Gan_Matrix24 *A, Gan_Vector3 *p, double h,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat24_get_rows_q ( Gan_Matrix24 *A, Gan_Vector4 *p, Gan_Vector4 *q );
+void gan_mat24_get_rows_q ( const Gan_Matrix24 *A, Gan_Vector4 *p, Gan_Vector4 *q );
 #else
 #define gan_mat24_get_rows_q(A,p,q)\
            (GAN_TEST_OP3(A,p,q,Gan_Matrix24,Gan_Vector4,Gan_Vector4),\
@@ -126,8 +126,8 @@ void gan_mat24_get_rows_q ( Gan_Matrix24 *A, Gan_Vector4 *p, Gan_Vector4 *q );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat24_get_cols_q ( Gan_Matrix24 *A, Gan_Vector2 *p, Gan_Vector2 *q,
-                                             Gan_Vector2 *r, Gan_Vector2 *s );
+void gan_mat24_get_cols_q ( const Gan_Matrix24 *A, Gan_Vector2 *p, Gan_Vector2 *q,
+                                                   Gan_Vector2 *r, Gan_Vector2 *s );
 #else
 #define gan_mat24_get_cols_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,Gan_Matrix24,\
@@ -145,12 +145,12 @@ void gan_mat24_get_cols_q ( Gan_Matrix24 *A, Gan_Vector2 *p, Gan_Vector2 *q,
 Gan_Matrix24 gan_mat24_fill_s (
                           double XX, double XY, double XZ, double XW,
                           double YX, double YY, double YZ, double YW );
-Gan_Vector2 gan_mat24_multv3h_s ( Gan_Matrix24 *A,
-                                  Gan_Vector3 *p, double h );
-void gan_mat24_get_rows_s ( Gan_Matrix24 *A, Gan_Vector4 *p, Gan_Vector4 *q );
-void gan_mat24_get_cols_s ( Gan_Matrix24 *A, Gan_Vector2 *p, Gan_Vector2 *q,
-                                             Gan_Vector2 *r, Gan_Vector2 *s );
-Gan_Matrix24 *gan_mat24_from_mat_q ( Gan_Matrix *A, Gan_Matrix24 *B );
+Gan_Vector2 gan_mat24_multv3h_s ( const Gan_Matrix24 *A,
+                                  const Gan_Vector3 *p, double h );
+void gan_mat24_get_rows_s ( const Gan_Matrix24 *A, Gan_Vector4 *p, Gan_Vector4 *q );
+void gan_mat24_get_cols_s ( const Gan_Matrix24 *A, Gan_Vector2 *p, Gan_Vector2 *q,
+                                                   Gan_Vector2 *r, Gan_Vector2 *s );
+Gan_Matrix24 *gan_mat24_from_mat_q ( const Gan_Matrix *A, Gan_Matrix24 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -162,6 +162,8 @@ Gan_Matrix24 *gan_mat24_from_mat_q ( Gan_Matrix *A, Gan_Matrix24 *B );
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector2
 #define GAN_VECTYPE2 Gan_Vector4
 #define GAN_MATTYPEL Gan_Matrix22
