@@ -3,8 +3,8 @@
  * Module:        2x4 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.15 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.18 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -76,8 +76,8 @@ Gan_Matrix24_f *gan_mat24f_fill_q ( Gan_Matrix24_f *A,
  * \return Pointer to the result 2-vector \a q.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector2_f *gan_mat24f_multv3h_q ( Gan_Matrix24_f *A,
-                                      Gan_Vector3_f *p, float h,
+Gan_Vector2_f *gan_mat24f_multv3h_q ( const Gan_Matrix24_f *A,
+                                      const Gan_Vector3_f *p, float h,
                                       Gan_Vector2_f *q );
 #else
 #define gan_mat24f_multv3h_q(A,p,h,q)\
@@ -107,7 +107,7 @@ Gan_Vector2_f *gan_mat24f_multv3h_q ( Gan_Matrix24_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat24f_get_rows_q ( Gan_Matrix24_f *A,
+void gan_mat24f_get_rows_q ( const Gan_Matrix24_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q );
 #else
 #define gan_mat24f_get_rows_q(A,p,q)\
@@ -128,7 +128,7 @@ void gan_mat24f_get_rows_q ( Gan_Matrix24_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat24f_get_cols_q ( Gan_Matrix24_f *A,
+void gan_mat24f_get_cols_q ( const Gan_Matrix24_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q,
                              Gan_Vector2_f *r, Gan_Vector2_f *s );
 #else
@@ -147,14 +147,14 @@ void gan_mat24f_get_cols_q ( Gan_Matrix24_f *A,
 
 Gan_Matrix24_f gan_mat24f_fill_s ( float XX, float XY, float XZ, float XW,
                                    float YX, float YY, float YZ, float YW );
-Gan_Vector2_f gan_mat24f_multv3h_s ( Gan_Matrix24_f *A,
-                                     Gan_Vector3_f *p, float h );
-void gan_mat24f_get_rows_s ( Gan_Matrix24_f *A,
+Gan_Vector2_f gan_mat24f_multv3h_s ( const Gan_Matrix24_f *A,
+                                     const Gan_Vector3_f *p, float h );
+void gan_mat24f_get_rows_s ( const Gan_Matrix24_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q );
-void gan_mat24f_get_cols_s ( Gan_Matrix24_f *A,
+void gan_mat24f_get_cols_s ( const Gan_Matrix24_f *A,
                              Gan_Vector2_f *p, Gan_Vector2_f *q,
                              Gan_Vector2_f *r, Gan_Vector2_f *s );
-Gan_Matrix24_f *gan_mat24f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix24_f *B );
+Gan_Matrix24_f *gan_mat24f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix24_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -166,6 +166,8 @@ Gan_Matrix24_f *gan_mat24f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix24_f *B );
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector2_f
 #define GAN_VECTYPE2 Gan_Vector4_f
 #define GAN_MATTYPEL Gan_Matrix22_f
