@@ -3,8 +3,8 @@
  * Module:        3x3 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.20 $
- * Last edited:   $Date: 2003/02/24 10:06:35 $
+ * Revision:      $Revision: 1.21 $
+ * Last edited:   $Date: 2004/03/18 18:26:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -40,7 +40,7 @@
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat33_fprint ( FILE *fp, Gan_Matrix33 *A,
+ gan_mat33_fprint ( FILE *fp, const Gan_Matrix33 *A,
                     const char *prefix, int indent, const char *fmt )
 {
    int i;
@@ -200,7 +200,7 @@ Gan_Matrix33
  * \return Pointer to result 2-vector \a q.
  */
 Gan_Vector2 *
- gan_mat33_multv2h_q ( Gan_Matrix33 *A, Gan_Vector2 *p, double h,
+ gan_mat33_multv2h_q ( const Gan_Matrix33 *A, const Gan_Vector2 *p, double h,
                        Gan_Vector2 *q )
 {
    Gan_Vector3 p3;
@@ -230,7 +230,7 @@ Gan_Vector2 *
  * \return Result 2-vector \a q.
  */
 Gan_Vector2
- gan_mat33_multv2h_s ( Gan_Matrix33 *A, Gan_Vector2 *p, double h )
+ gan_mat33_multv2h_s ( const Gan_Matrix33 *A, const Gan_Vector2 *p, double h )
 {
    Gan_Vector2 q;
 
@@ -256,7 +256,7 @@ Gan_Vector2
  * \return Result matrix \f$ [p]_{\times} \f$.
  */
 Gan_Matrix33
- gan_mat33_cross_s ( Gan_Vector3 *p )
+ gan_mat33_cross_s ( const Gan_Vector3 *p )
 {
    Gan_Matrix33 A;
 
@@ -374,7 +374,7 @@ Gan_Matrix33 *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix33 *
- gan_mat33_adjoint_q ( Gan_Matrix33 *A, Gan_Matrix33 *B )
+ gan_mat33_adjoint_q ( const Gan_Matrix33 *A, Gan_Matrix33 *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat33_adjoint_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -399,7 +399,7 @@ Gan_Matrix33 *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix33 *
- gan_mat33_adjointT_q ( Gan_Matrix33 *A, Gan_Matrix33 *B )
+ gan_mat33_adjointT_q ( const Gan_Matrix33 *A, Gan_Matrix33 *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat33_adjointT_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -434,7 +434,7 @@ Gan_Matrix33 *
  * \return No value.
  */
 void
- gan_mat33_get_rows_s ( Gan_Matrix33 *A,
+ gan_mat33_get_rows_s ( const Gan_Matrix33 *A,
                         Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r )
 {
    gan_mat33_get_rows_q ( A, p, q, r );
@@ -452,7 +452,7 @@ void
  * \return No value.
  */
 void
- gan_mat33_get_cols_s ( Gan_Matrix33 *A,
+ gan_mat33_get_cols_s ( const Gan_Matrix33 *A,
                         Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r )
 {
    gan_mat33_get_cols_q ( A, p, q, r );
@@ -466,7 +466,7 @@ void
  * \return Result 2x2 matrix part.
  */
 Gan_Matrix22
- gan_mat33_get_m22tl_s ( Gan_Matrix33 *A )
+ gan_mat33_get_m22tl_s ( const Gan_Matrix33 *A )
 {
    Gan_Matrix22 B;
 
@@ -482,7 +482,7 @@ Gan_Matrix22
  * \return Result 2-vector part.
  */
 Gan_Vector2
- gan_mat33_get_v2tr_s ( Gan_Matrix33 *A )
+ gan_mat33_get_v2tr_s ( const Gan_Matrix33 *A )
 {
    Gan_Vector2 p;
 
@@ -504,7 +504,7 @@ Gan_Vector2
  * \return No value.
  */
 void
- gan_mat33_get_parts_s ( Gan_Matrix33 *A, Gan_Matrix22 *B,
+ gan_mat33_get_parts_s ( const Gan_Matrix33 *A, Gan_Matrix22 *B,
                          Gan_Vector2 *p, Gan_Vector2 *q, double *s )
 {
    gan_mat33_get_parts_q ( A, B, p, q, s );
@@ -535,8 +535,8 @@ void
  * \return Pointer to filled matrix \a A.
  */
 Gan_Matrix33
- gan_mat33_set_parts_s ( Gan_Matrix22 *B,
-                         Gan_Vector2 *p, Gan_Vector2 *q, double s )
+ gan_mat33_set_parts_s ( const Gan_Matrix22 *B,
+                         const Gan_Vector2 *p, const Gan_Vector2 *q, double s )
 {
    Gan_Matrix33 A;
    Gan_Vector2 zerov = { 0.0, 0.0 };
@@ -558,7 +558,7 @@ Gan_Matrix33
  * \return Filled matrix.
  */
 Gan_Matrix33
- gan_mat33_set_cols_s ( Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r )
+ gan_mat33_set_cols_s ( const Gan_Vector3 *p, const Gan_Vector3 *q, const Gan_Vector3 *r )
 {
    Gan_Matrix33 A;
 
@@ -579,7 +579,7 @@ Gan_Matrix33
  * \return Filled matrix.
  */
 Gan_Matrix33
- gan_mat33_set_rows_s ( Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r )
+ gan_mat33_set_rows_s ( const Gan_Vector3 *p, const Gan_Vector3 *q, const Gan_Vector3 *r )
 {
    Gan_Matrix33 A;
 
@@ -610,7 +610,7 @@ Gan_Matrix33
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix33 *
- gan_mat33_from_mat_q ( Gan_Matrix *A, Gan_Matrix33 *B )
+ gan_mat33_from_mat_q ( const Gan_Matrix *A, Gan_Matrix33 *B )
 {
    if ( !gan_mat_read_va ( A, 3, 3, &B->xx, &B->xy, &B->xz,
                                     &B->yx, &B->yy, &B->yz,
@@ -631,7 +631,7 @@ Gan_Matrix33 *
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix33 *
- gan_mat33_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix33 *B )
+ gan_mat33_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix33 *B )
 {
    switch ( A->type )
    {
@@ -712,7 +712,7 @@ Gan_Matrix33 *
  * \return Pointer to result matrix \a B.
  */
 Gan_SquMatrix33 *
- gan_squmat33_from_squmat_q ( Gan_SquMatrix *A, Gan_SquMatrix33 *B )
+ gan_squmat33_from_squmat_q ( const Gan_SquMatrix *A, Gan_SquMatrix33 *B )
 {
    switch ( A->type )
    {
