@@ -3,8 +3,8 @@
  * Module:        3x3 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.23 $
- * Last edited:   $Date: 2003/02/24 10:06:35 $
+ * Revision:      $Revision: 1.26 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -80,7 +80,7 @@ Gan_Matrix33 *gan_mat33_fill_q ( Gan_Matrix33 *A,
  * \return Pointer to result matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33 *gan_mat33_cross_q ( Gan_Vector3 *p, Gan_Matrix33 *A );
+Gan_Matrix33 *gan_mat33_cross_q ( const Gan_Vector3 *p, const Gan_Matrix33 *A );
 #else
 #define gan_mat33_cross_q(p,A)\
            (GAN_TEST_OP2(p,A,Gan_Vector3,Gan_Matrix33),\
@@ -109,7 +109,7 @@ Gan_Matrix33 *gan_mat33_cross_q ( Gan_Vector3 *p, Gan_Matrix33 *A );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33_get_rows_q ( Gan_Matrix33 *A,
+void gan_mat33_get_rows_q ( const Gan_Matrix33 *A,
                             Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r );
 #else
 #define gan_mat33_get_rows_q(A,p,q,r)\
@@ -132,7 +132,7 @@ void gan_mat33_get_rows_q ( Gan_Matrix33 *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33_get_cols_q ( Gan_Matrix33 *A,
+void gan_mat33_get_cols_q ( const Gan_Matrix33 *A,
                             Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r );
 #else
 #define gan_mat33_get_cols_q(A,p,q,r)\
@@ -151,7 +151,7 @@ void gan_mat33_get_cols_q ( Gan_Matrix33 *A,
  * \return Pointer to 2x2 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix22 *gan_mat33_get_m22tl_q ( Gan_Matrix33 *A, Gan_Matrix22 *B );
+Gan_Matrix22 *gan_mat33_get_m22tl_q ( const Gan_Matrix33 *A, Gan_Matrix22 *B );
 #else
 #define gan_mat33_get_m22tl_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix33,Gan_Matrix22),\
@@ -167,7 +167,7 @@ Gan_Matrix22 *gan_mat33_get_m22tl_q ( Gan_Matrix33 *A, Gan_Matrix22 *B );
  * \return Pointer to result 2-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector2 *gan_mat33_get_v2tr_q ( Gan_Matrix33 *A, Gan_Vector2 *p );
+Gan_Vector2 *gan_mat33_get_v2tr_q ( const Gan_Matrix33 *A, Gan_Vector2 *p );
 #else
 #define gan_mat33_get_v2tr_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix33,Gan_Vector2),\
@@ -188,7 +188,7 @@ Gan_Vector2 *gan_mat33_get_v2tr_q ( Gan_Matrix33 *A, Gan_Vector2 *p );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33_get_parts_q ( Gan_Matrix33 *A, Gan_Matrix22 *B,
+void gan_mat33_get_parts_q ( const Gan_Matrix33 *A, Gan_Matrix22 *B,
                              Gan_Vector2 *p, Gan_Vector2 *q, double *s );
 #else
 #define gan_mat33_get_parts_q(A,B,p,q,s)\
@@ -222,8 +222,8 @@ void gan_mat33_get_parts_q ( Gan_Matrix33 *A, Gan_Matrix22 *B,
  * \return Pointer to filled matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33 *gan_mat33_set_parts_q ( Gan_Matrix33 *A, Gan_Matrix22 *B,
-                                      Gan_Vector2 *p, Gan_Vector2 *q,
+Gan_Matrix33 *gan_mat33_set_parts_q ( Gan_Matrix33 *A, const Gan_Matrix22 *B,
+                                      const Gan_Vector2 *p, const Gan_Vector2 *q,
                                       double s );
 #else
 #define gan_mat33_set_parts_q(A,B,p,q,s)\
@@ -247,8 +247,8 @@ Gan_Matrix33 *gan_mat33_set_parts_q ( Gan_Matrix33 *A, Gan_Matrix22 *B,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix33 *gan_mat33_set_cols_q ( Gan_Matrix33 *A,
-                                     Gan_Vector3 *p, Gan_Vector3 *q,
-                                     Gan_Vector3 *r );
+                                     const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                     const Gan_Vector3 *r );
 #else
 #define gan_mat33_set_cols_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -272,8 +272,8 @@ Gan_Matrix33 *gan_mat33_set_cols_q ( Gan_Matrix33 *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix33 *gan_mat33_set_rows_q ( Gan_Matrix33 *A,
-                                     Gan_Vector3 *p, Gan_Vector3 *q,
-                                     Gan_Vector3 *r );
+                                     const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                     const Gan_Vector3 *r );
 #else
 #define gan_mat33_set_rows_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -292,23 +292,23 @@ Gan_Matrix33 *gan_mat33_set_rows_q ( Gan_Matrix33 *A,
 Gan_Matrix33 gan_mat33_fill_s ( double XX, double XY, double XZ,
                                 double YX, double YY, double YZ,
                                 double ZX, double ZY, double ZZ );
-Gan_Vector2 *gan_mat33_multv2h_q ( Gan_Matrix33 *A, Gan_Vector2 *p, double h,
+Gan_Vector2 *gan_mat33_multv2h_q ( const Gan_Matrix33 *A, const Gan_Vector2 *p, double h,
                                    Gan_Vector2 *q );
-Gan_Vector2 gan_mat33_multv2h_s ( Gan_Matrix33 *A, Gan_Vector2 *p, double h);
-Gan_Matrix33 gan_mat33_cross_s ( Gan_Vector3 *p );
-void gan_mat33_get_rows_s ( Gan_Matrix33 *A,
+Gan_Vector2 gan_mat33_multv2h_s ( const Gan_Matrix33 *A, const Gan_Vector2 *p, double h);
+Gan_Matrix33 gan_mat33_cross_s ( const Gan_Vector3 *p );
+void gan_mat33_get_rows_s ( const Gan_Matrix33 *A,
                             Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r );
-void gan_mat33_get_cols_s ( Gan_Matrix33 *A,
+void gan_mat33_get_cols_s ( const Gan_Matrix33 *A,
                             Gan_Vector3 *p, Gan_Vector3 *q, Gan_Vector3 *r );
-Gan_Matrix33 gan_mat33_set_parts_s ( Gan_Matrix22 *B,
-                                     Gan_Vector2 *p, Gan_Vector2 *q, double s);
-Gan_Matrix33 gan_mat33_set_cols_s ( Gan_Vector3 *p, Gan_Vector3 *q,
-                                    Gan_Vector3 *r );
-Gan_Matrix33 gan_mat33_set_rows_s ( Gan_Vector3 *p, Gan_Vector3 *q,
-                                    Gan_Vector3 *r );
-Gan_Matrix33 *gan_mat33_from_mat_q ( Gan_Matrix *A, Gan_Matrix33 *B );
-Gan_Matrix33 *gan_mat33_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix33 *B );
-Gan_SquMatrix33 *gan_squmat33_from_squmat_q ( Gan_SquMatrix *A,
+Gan_Matrix33 gan_mat33_set_parts_s ( const Gan_Matrix22 *B,
+                                     const Gan_Vector2 *p, const Gan_Vector2 *q, double s);
+Gan_Matrix33 gan_mat33_set_cols_s ( const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                    const Gan_Vector3 *r );
+Gan_Matrix33 gan_mat33_set_rows_s ( const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                    const Gan_Vector3 *r );
+Gan_Matrix33 *gan_mat33_from_mat_q ( const Gan_Matrix *A, Gan_Matrix33 *B );
+Gan_Matrix33 *gan_mat33_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix33 *B );
+Gan_SquMatrix33 *gan_squmat33_from_squmat_q ( const Gan_SquMatrix *A,
                                               Gan_SquMatrix33 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -321,6 +321,8 @@ Gan_SquMatrix33 *gan_squmat33_from_squmat_q ( Gan_SquMatrix *A,
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector3
 #define GAN_VECTYPE2 Gan_Vector3
 #define GAN_MATTYPEL Gan_Matrix33
