@@ -3,8 +3,8 @@
  * Module:        3x3 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.12 $
- * Last edited:   $Date: 2003/02/24 10:06:35 $
+ * Revision:      $Revision: 1.13 $
+ * Last edited:   $Date: 2004/03/18 18:26:18 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -40,7 +40,7 @@
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat33f_fprint ( FILE *fp, Gan_Matrix33_f *A,
+ gan_mat33f_fprint ( FILE *fp, const Gan_Matrix33_f *A,
                      const char *prefix, int indent, const char *fmt )
 {
    int i;
@@ -200,7 +200,7 @@ Gan_Matrix33_f
  * \return Pointer to result 2-vector \a q.
  */
 Gan_Vector2_f *
- gan_mat33f_multv2h_q ( Gan_Matrix33_f *A, Gan_Vector2_f *p, float h,
+ gan_mat33f_multv2h_q ( const Gan_Matrix33_f *A, const Gan_Vector2_f *p, float h,
                         Gan_Vector2_f *q )
 {
    Gan_Vector3_f p3;
@@ -230,7 +230,7 @@ Gan_Vector2_f *
  * \return Result 2-vector \a q.
  */
 Gan_Vector2_f
- gan_mat33f_multv2h_s ( Gan_Matrix33_f *A, Gan_Vector2_f *p, float h )
+ gan_mat33f_multv2h_s ( const Gan_Matrix33_f *A, const Gan_Vector2_f *p, float h )
 {
    Gan_Vector2_f q;
 
@@ -256,7 +256,7 @@ Gan_Vector2_f
  * \return Result matrix \f$ [p]_{\times} \f$.
  */
 Gan_Matrix33_f
- gan_mat33f_cross_s ( Gan_Vector3_f *p )
+ gan_mat33f_cross_s ( const Gan_Vector3_f *p )
 {
    Gan_Matrix33_f A;
 
@@ -374,7 +374,7 @@ Gan_Matrix33_f *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix33_f *
- gan_mat33f_adjoint_q ( Gan_Matrix33_f *A, Gan_Matrix33_f *B )
+ gan_mat33f_adjoint_q ( const Gan_Matrix33_f *A, Gan_Matrix33_f *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat33f_adjoint_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -399,7 +399,7 @@ Gan_Matrix33_f *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix33_f *
- gan_mat33f_adjointT_q ( Gan_Matrix33_f *A, Gan_Matrix33_f *B )
+ gan_mat33f_adjointT_q ( const Gan_Matrix33_f *A, Gan_Matrix33_f *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat33f_adjointT_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -434,7 +434,7 @@ Gan_Matrix33_f *
  * \return No value.
  */
 void
- gan_mat33f_get_rows_s ( Gan_Matrix33_f *A,
+ gan_mat33f_get_rows_s ( const Gan_Matrix33_f *A,
                          Gan_Vector3_f *p, Gan_Vector3_f *q, Gan_Vector3_f *r )
 {
    gan_mat33f_get_rows_q ( A, p, q, r );
@@ -452,7 +452,7 @@ void
  * \return No value.
  */
 void
- gan_mat33f_get_cols_s ( Gan_Matrix33_f *A,
+ gan_mat33f_get_cols_s ( const Gan_Matrix33_f *A,
                          Gan_Vector3_f *p, Gan_Vector3_f *q, Gan_Vector3_f *r )
 {
    gan_mat33f_get_cols_q ( A, p, q, r );
@@ -466,7 +466,7 @@ void
  * \return Result 2x2 matrix part.
  */
 Gan_Matrix22_f
- gan_mat33f_get_m22tl_s ( Gan_Matrix33_f *A )
+ gan_mat33f_get_m22tl_s ( const Gan_Matrix33_f *A )
 {
    Gan_Matrix22_f B;
 
@@ -482,7 +482,7 @@ Gan_Matrix22_f
  * \return Result 2-vector part.
  */
 Gan_Vector2_f
- gan_mat33f_get_v2tr_s ( Gan_Matrix33_f *A )
+ gan_mat33f_get_v2tr_s ( const Gan_Matrix33_f *A )
 {
    Gan_Vector2_f p;
 
@@ -504,7 +504,7 @@ Gan_Vector2_f
  * \return No value.
  */
 void
- gan_mat33f_get_parts_s ( Gan_Matrix33_f *A, Gan_Matrix22_f *B,
+ gan_mat33f_get_parts_s ( const Gan_Matrix33_f *A, Gan_Matrix22_f *B,
                           Gan_Vector2_f *p, Gan_Vector2_f *q, float *s )
 {
    gan_mat33f_get_parts_q ( A, B, p, q, s );
@@ -536,7 +536,7 @@ void
  */
 Gan_Matrix33_f
  gan_mat33f_set_parts_s ( Gan_Matrix22_f *B,
-                          Gan_Vector2_f *p, Gan_Vector2_f *q, float s )
+                          const Gan_Vector2_f *p, const Gan_Vector2_f *q, float s )
 {
    Gan_Matrix33_f A;
    Gan_Vector2_f zerov = { 0.0F, 0.0F };
@@ -558,7 +558,7 @@ Gan_Matrix33_f
  * \return Filled matrix.
  */
 Gan_Matrix33_f
- gan_mat33f_set_cols_s ( Gan_Vector3_f *p, Gan_Vector3_f *q, Gan_Vector3_f *r )
+ gan_mat33f_set_cols_s ( const Gan_Vector3_f *p, const Gan_Vector3_f *q, const Gan_Vector3_f *r )
 {
    Gan_Matrix33_f A;
 
@@ -579,7 +579,7 @@ Gan_Matrix33_f
  * \return Filled matrix.
  */
 Gan_Matrix33_f
- gan_mat33f_set_rows_s ( Gan_Vector3_f *p, Gan_Vector3_f *q, Gan_Vector3_f *r )
+ gan_mat33f_set_rows_s ( const Gan_Vector3_f *p, const Gan_Vector3_f *q, const Gan_Vector3_f *r )
 {
    Gan_Matrix33_f A;
 
@@ -610,7 +610,7 @@ Gan_Matrix33_f
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix33_f *
- gan_mat33f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix33_f *B )
+ gan_mat33f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix33_f *B )
 {
    if ( !gan_matf_read_va ( A, 3, 3, &B->xx, &B->xy, &B->xz,
                                      &B->yx, &B->yy, &B->yz,
@@ -631,7 +631,7 @@ Gan_Matrix33_f *
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix33_f *
- gan_mat33f_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_Matrix33_f *B )
+ gan_mat33f_from_squmatf_q ( const Gan_SquMatrix_f *A, Gan_Matrix33_f *B )
 {
    switch ( A->type )
    {
@@ -713,7 +713,7 @@ Gan_Matrix33_f *
  * \return Pointer to result matrix \a B.
  */
 Gan_SquMatrix33_f *
- gan_squmat33f_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_SquMatrix33_f *B )
+ gan_squmat33f_from_squmatf_q ( const Gan_SquMatrix_f *A, Gan_SquMatrix33_f *B )
 {
    switch ( A->type )
    {
