@@ -3,8 +3,8 @@
  * Module:        3x3 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.19 $
- * Last edited:   $Date: 2003/02/24 10:06:35 $
+ * Revision:      $Revision: 1.22 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -80,7 +80,7 @@ Gan_Matrix33_f *gan_mat33f_fill_q ( Gan_Matrix33_f *A,
  * \return Pointer to result matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33_f *gan_mat33f_cross_q ( Gan_Vector3_f *p, Gan_Matrix33_f *A );
+Gan_Matrix33_f *gan_mat33f_cross_q ( const Gan_Vector3_f *p, Gan_Matrix33_f *A );
 #else
 #define gan_mat33f_cross_q(p,A)\
            (GAN_TEST_OP2(p,A,Gan_Vector3_f,Gan_Matrix33_f),\
@@ -109,7 +109,7 @@ Gan_Matrix33_f *gan_mat33f_cross_q ( Gan_Vector3_f *p, Gan_Matrix33_f *A );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33f_get_rows_q ( Gan_Matrix33_f *A,
+void gan_mat33f_get_rows_q ( const Gan_Matrix33_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r );
 #else
@@ -133,7 +133,7 @@ void gan_mat33f_get_rows_q ( Gan_Matrix33_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33f_get_cols_q ( Gan_Matrix33_f *A,
+void gan_mat33f_get_cols_q ( const Gan_Matrix33_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r );
 #else
@@ -153,7 +153,7 @@ void gan_mat33f_get_cols_q ( Gan_Matrix33_f *A,
  * \return Pointer to 2x2 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix22_f *gan_mat33f_get_m22tl_q ( Gan_Matrix33_f *A,
+Gan_Matrix22_f *gan_mat33f_get_m22tl_q ( const Gan_Matrix33_f *A,
                                          Gan_Matrix22_f *B );
 #else
 #define gan_mat33f_get_m22tl_q(A,B)\
@@ -170,7 +170,7 @@ Gan_Matrix22_f *gan_mat33f_get_m22tl_q ( Gan_Matrix33_f *A,
  * \return Pointer to result 2-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector2_f *gan_mat33f_get_v2tr_q ( Gan_Matrix33_f *A, Gan_Vector2_f *p );
+Gan_Vector2_f *gan_mat33f_get_v2tr_q ( const Gan_Matrix33_f *A, Gan_Vector2_f *p );
 #else
 #define gan_mat33f_get_v2tr_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix33_f,Gan_Vector2_f),\
@@ -191,7 +191,7 @@ Gan_Vector2_f *gan_mat33f_get_v2tr_q ( Gan_Matrix33_f *A, Gan_Vector2_f *p );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat33f_get_parts_q ( Gan_Matrix33_f *A, Gan_Matrix22_f *B,
+void gan_mat33f_get_parts_q ( const Gan_Matrix33_f *A, Gan_Matrix22_f *B,
                               Gan_Vector2_f *p, Gan_Vector2_f *q, float *s );
 #else
 #define gan_mat33f_get_parts_q(A,B,p,q,s)\
@@ -225,8 +225,8 @@ void gan_mat33f_get_parts_q ( Gan_Matrix33_f *A, Gan_Matrix22_f *B,
  * \return Pointer to filled matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33_f *gan_mat33f_set_parts_q ( Gan_Matrix33_f *A, Gan_Matrix22_f *B,
-                                         Gan_Vector2_f *p, Gan_Vector2_f *q,
+Gan_Matrix33_f *gan_mat33f_set_parts_q ( Gan_Matrix33_f *A, const Gan_Matrix22_f *B,
+                                         const Gan_Vector2_f *p, const Gan_Vector2_f *q,
                                          float s );
 #else
 #define gan_mat33f_set_parts_q(A,B,p,q,s)\
@@ -250,8 +250,8 @@ Gan_Matrix33_f *gan_mat33f_set_parts_q ( Gan_Matrix33_f *A, Gan_Matrix22_f *B,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix33_f *gan_mat33f_set_cols_q ( Gan_Matrix33_f *A,
-                                        Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                        Gan_Vector3_f *r );
+                                        const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                        const Gan_Vector3_f *r );
 #else
 #define gan_mat33f_set_cols_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -275,8 +275,8 @@ Gan_Matrix33_f *gan_mat33f_set_cols_q ( Gan_Matrix33_f *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix33_f *gan_mat33f_set_rows_q ( Gan_Matrix33_f *A,
-                                        Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                        Gan_Vector3_f *r );
+                                        const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                        const Gan_Vector3_f *r );
 #else
 #define gan_mat33f_set_rows_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -295,30 +295,30 @@ Gan_Matrix33_f *gan_mat33f_set_rows_q ( Gan_Matrix33_f *A,
 Gan_Matrix33_f gan_mat33f_fill_s ( float XX, float XY, float XZ,
                                    float YX, float YY, float YZ,
                                    float ZX, float ZY, float ZZ );
-Gan_Vector2_f *gan_mat33f_multv2h_q ( Gan_Matrix33_f *A,
-                                      Gan_Vector2_f *p, float h,
+Gan_Vector2_f *gan_mat33f_multv2h_q ( const Gan_Matrix33_f *A,
+                                      const Gan_Vector2_f *p, float h,
                                       Gan_Vector2_f *q );
-Gan_Vector2_f gan_mat33f_multv2h_s ( Gan_Matrix33_f *A,
-                                     Gan_Vector2_f *p, float h);
-Gan_Matrix33_f gan_mat33f_cross_s ( Gan_Vector3_f *p );
-void gan_mat33f_get_rows_s ( Gan_Matrix33_f *A,
+Gan_Vector2_f gan_mat33f_multv2h_s ( const Gan_Matrix33_f *A,
+                                     const Gan_Vector2_f *p, float h);
+Gan_Matrix33_f gan_mat33f_cross_s ( const Gan_Vector3_f *p );
+void gan_mat33f_get_rows_s ( const Gan_Matrix33_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r );
-void gan_mat33f_get_cols_s ( Gan_Matrix33_f *A,
+void gan_mat33f_get_cols_s ( const Gan_Matrix33_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r );
 Gan_Matrix33_f gan_mat33f_set_parts_s ( Gan_Matrix22_f *B,
-                                        Gan_Vector2_f *p, Gan_Vector2_f *q,
+                                        const Gan_Vector2_f *p, const Gan_Vector2_f *q,
                                         float s);
-Gan_Matrix33_f gan_mat33f_set_cols_s ( Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                       Gan_Vector3_f *r );
-Gan_Matrix33_f gan_mat33f_set_rows_s ( Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                       Gan_Vector3_f *r );
-Gan_Matrix33_f *gan_mat33f_from_matf_q ( Gan_Matrix_f *A,
+Gan_Matrix33_f gan_mat33f_set_cols_s ( const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                       const Gan_Vector3_f *r );
+Gan_Matrix33_f gan_mat33f_set_rows_s ( const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                       const Gan_Vector3_f *r );
+Gan_Matrix33_f *gan_mat33f_from_matf_q ( const Gan_Matrix_f *A,
                                          Gan_Matrix33_f *B );
-Gan_Matrix33_f *gan_mat33f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_Matrix33_f *gan_mat33f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                             Gan_Matrix33_f *B );
-Gan_SquMatrix33_f *gan_squmat33f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_SquMatrix33_f *gan_squmat33f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                                   Gan_SquMatrix33_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -331,6 +331,8 @@ Gan_SquMatrix33_f *gan_squmat33f_from_squmatf_q ( Gan_SquMatrix_f *A,
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector3_f
 #define GAN_VECTYPE2 Gan_Vector3_f
 #define GAN_MATTYPEL Gan_Matrix33_f
