@@ -3,8 +3,8 @@
  * Module:        3x4 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.21 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.24 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -76,7 +76,7 @@ Gan_Matrix34 *gan_mat34_fill_q ( Gan_Matrix34 *A,
  *
  * \return Pointer to the result matrix \a C.
  */
-Gan_Matrix33 *gan_mat34_rmultm34T_q ( Gan_Matrix34 *A, Gan_Matrix34 *B,
+Gan_Matrix33 *gan_mat34_rmultm34T_q ( const Gan_Matrix34 *A, const Gan_Matrix34 *B,
                                       Gan_Matrix33 *C );
 #endif
 
@@ -102,7 +102,7 @@ Gan_Matrix33 *gan_mat34_rmultm34T_q ( Gan_Matrix34 *A, Gan_Matrix34 *B,
  *
  * \return Pointer to the result vector \a q.
  */
-Gan_Vector3 *gan_mat34_multv3h_q ( Gan_Matrix34 *A, Gan_Vector3 *p, double h,
+Gan_Vector3 *gan_mat34_multv3h_q ( const Gan_Matrix34 *A, const Gan_Vector3 *p, double h,
                                    Gan_Vector3 *q );
 #endif
 
@@ -128,7 +128,7 @@ Gan_Vector3 *gan_mat34_multv3h_q ( Gan_Matrix34 *A, Gan_Vector3 *p, double h,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat34_get_rows_q ( Gan_Matrix34 *A,
+void gan_mat34_get_rows_q ( const Gan_Matrix34 *A,
                             Gan_Vector4 *p, Gan_Vector4 *q, Gan_Vector4 *r );
 #else
 #define gan_mat34_get_rows_q(A,p,q,r)\
@@ -151,8 +151,8 @@ void gan_mat34_get_rows_q ( Gan_Matrix34 *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat34_get_cols_q ( Gan_Matrix34 *A, Gan_Vector3 *p, Gan_Vector3 *q,
-                                             Gan_Vector3 *r, Gan_Vector3 *s );
+void gan_mat34_get_cols_q ( const Gan_Matrix34 *A, Gan_Vector3 *p, Gan_Vector3 *q,
+                                                   Gan_Vector3 *r, Gan_Vector3 *s );
 #else
 #define gan_mat34_get_cols_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,Gan_Matrix34,\
@@ -170,7 +170,7 @@ void gan_mat34_get_cols_q ( Gan_Matrix34 *A, Gan_Vector3 *p, Gan_Vector3 *q,
  * \return Pointer to result 3x3 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33 *gan_mat34_get_m33l_q ( Gan_Matrix34 *A, Gan_Matrix33 *B );
+Gan_Matrix33 *gan_mat34_get_m33l_q ( const Gan_Matrix34 *A, Gan_Matrix33 *B );
 #else
 #define gan_mat34_get_m33l_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix34,Gan_Matrix33),\
@@ -187,7 +187,7 @@ Gan_Matrix33 *gan_mat34_get_m33l_q ( Gan_Matrix34 *A, Gan_Matrix33 *B );
  * \return Pointer to result 3-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector3 *gan_mat34_get_v3r_q ( Gan_Matrix34 *A, Gan_Vector3 *p );
+Gan_Vector3 *gan_mat34_get_v3r_q ( const Gan_Matrix34 *A, Gan_Vector3 *p );
 #else
 #define gan_mat34_get_v3r_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix34,Gan_Vector3),\
@@ -217,7 +217,7 @@ Gan_Vector3 *gan_mat34_get_v3r_q ( Gan_Matrix34 *A, Gan_Vector3 *p );
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34 *gan_mat34_set_parts_q ( Gan_Matrix34 *A,
-                                      Gan_Matrix33 *B, Gan_Vector3 *p );
+                                      const Gan_Matrix33 *B, const Gan_Vector3 *p );
 #else
 #define gan_mat34_set_parts_q(A,B,p)\
            (GAN_TEST_OP3(A,B,p,\
@@ -240,8 +240,8 @@ Gan_Matrix34 *gan_mat34_set_parts_q ( Gan_Matrix34 *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34 *gan_mat34_set_cols_q ( Gan_Matrix34 *A,
-                                     Gan_Vector3 *p, Gan_Vector3 *q,
-                                     Gan_Vector3 *r, Gan_Vector3 *s );
+                                     const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                     const Gan_Vector3 *r, const Gan_Vector3 *s );
 #else
 #define gan_mat34_set_cols_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,\
@@ -265,8 +265,8 @@ Gan_Matrix34 *gan_mat34_set_cols_q ( Gan_Matrix34 *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34 *gan_mat34_set_rows_q ( Gan_Matrix34 *A,
-                                     Gan_Vector4 *p, Gan_Vector4 *q,
-                                     Gan_Vector4 *r );
+                                     const Gan_Vector4 *p, const Gan_Vector4 *q,
+                                     const Gan_Vector4 *r );
 #else
 #define gan_mat34_set_rows_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -286,23 +286,23 @@ Gan_Matrix34 gan_mat34_fill_s (
                           double XX, double XY, double XZ, double XW,
                           double YX, double YY, double YZ, double YW,
                           double ZX, double ZY, double ZZ, double ZW );
-Gan_Matrix33 gan_mat34_rmultm34T_s ( Gan_Matrix34 *A, Gan_Matrix34 *B );
-Gan_Vector3 gan_mat34_multv3h_s ( Gan_Matrix34 *A, Gan_Vector3 *p, double h );
-Gan_Bool gan_mat34h_invert_q ( Gan_Matrix34 *A, double a,
+Gan_Matrix33 gan_mat34_rmultm34T_s ( const Gan_Matrix34 *A, const Gan_Matrix34 *B );
+Gan_Vector3 gan_mat34_multv3h_s ( const Gan_Matrix34 *A, const Gan_Vector3 *p, double h );
+Gan_Bool gan_mat34h_invert_q ( const Gan_Matrix34 *A, double a,
                                Gan_Matrix34 *B, double *b );
-Gan_Matrix34 gan_mat34h_invert_s ( Gan_Matrix34 *A, double a, double *b );
-void gan_mat34_get_rows_s ( Gan_Matrix34 *A,
+Gan_Matrix34 gan_mat34h_invert_s ( const Gan_Matrix34 *A, double a, double *b );
+void gan_mat34_get_rows_s ( const Gan_Matrix34 *A,
                             Gan_Vector4 *p, Gan_Vector4 *q, Gan_Vector4 *r );
-void gan_mat34_get_cols_s ( Gan_Matrix34 *A, Gan_Vector3 *p, Gan_Vector3 *q,
-                                             Gan_Vector3 *r, Gan_Vector3 *s );
-Gan_Matrix33 gan_mat34_get_m33l_s ( Gan_Matrix34 *A );
-Gan_Vector3  gan_mat34_get_v3r_s  ( Gan_Matrix34 *A );
-Gan_Matrix34 gan_mat34_set_parts_s ( Gan_Matrix33 *B, Gan_Vector3 *p );
-Gan_Matrix34 gan_mat34_set_cols_s ( Gan_Vector3 *p, Gan_Vector3 *q,
-                                    Gan_Vector3 *r, Gan_Vector3 *s );
-Gan_Matrix34 gan_mat34_set_rows_s ( Gan_Vector4 *p, Gan_Vector4 *q,
-                                    Gan_Vector4 *r );
-Gan_Matrix34 *gan_mat34_from_mat_q ( Gan_Matrix *A, Gan_Matrix34 *B );
+void gan_mat34_get_cols_s ( const Gan_Matrix34 *A, Gan_Vector3 *p, Gan_Vector3 *q,
+                                                   Gan_Vector3 *r, Gan_Vector3 *s );
+Gan_Matrix33 gan_mat34_get_m33l_s ( const Gan_Matrix34 *A );
+Gan_Vector3  gan_mat34_get_v3r_s  ( const Gan_Matrix34 *A );
+Gan_Matrix34 gan_mat34_set_parts_s ( const Gan_Matrix33 *B, const Gan_Vector3 *p );
+Gan_Matrix34 gan_mat34_set_cols_s ( const Gan_Vector3 *p, const Gan_Vector3 *q,
+                                    const Gan_Vector3 *r, const Gan_Vector3 *s );
+Gan_Matrix34 gan_mat34_set_rows_s ( const Gan_Vector4 *p, const Gan_Vector4 *q,
+                                    const Gan_Vector4 *r );
+Gan_Matrix34 *gan_mat34_from_mat_q ( const Gan_Matrix *A, Gan_Matrix34 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -314,6 +314,8 @@ Gan_Matrix34 *gan_mat34_from_mat_q ( Gan_Matrix *A, Gan_Matrix34 *B );
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector3
 #define GAN_VECTYPE2 Gan_Vector4
 #define GAN_MATTYPEL Gan_Matrix33
