@@ -3,8 +3,8 @@
  * Module:        3x4 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.17 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.20 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -76,7 +76,7 @@ Gan_Matrix34_f *gan_mat34f_fill_q ( Gan_Matrix34_f *A,
  *
  * \return Pointer to the result matrix \a C.
  */
-Gan_Matrix33_f *gan_mat34f_rmultm34T_q ( Gan_Matrix34_f *A, Gan_Matrix34_f *B,
+Gan_Matrix33_f *gan_mat34f_rmultm34T_q ( const Gan_Matrix34_f *A, const Gan_Matrix34_f *B,
                                          Gan_Matrix33_f *C );
 #endif
 
@@ -102,7 +102,7 @@ Gan_Matrix33_f *gan_mat34f_rmultm34T_q ( Gan_Matrix34_f *A, Gan_Matrix34_f *B,
  *
  * \return Pointer to the result vector \a q.
  */
-Gan_Vector3_f *gan_mat34_multv3h_q ( Gan_Matrix34_f *A, Gan_Vector3_f *p,
+Gan_Vector3_f *gan_mat34_multv3h_q ( const Gan_Matrix34_f *A, const Gan_Vector3_f *p,
                                      float h, Gan_Vector3_f *q );
 #endif
 
@@ -128,7 +128,7 @@ Gan_Vector3_f *gan_mat34_multv3h_q ( Gan_Matrix34_f *A, Gan_Vector3_f *p,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat34f_get_rows_q ( Gan_Matrix34_f *A,
+void gan_mat34f_get_rows_q ( const Gan_Matrix34_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r );
 #else
@@ -152,7 +152,7 @@ void gan_mat34f_get_rows_q ( Gan_Matrix34_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat34f_get_cols_q ( Gan_Matrix34_f *A,
+void gan_mat34f_get_cols_q ( const Gan_Matrix34_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r, Gan_Vector3_f *s );
 #else
@@ -172,7 +172,7 @@ void gan_mat34f_get_cols_q ( Gan_Matrix34_f *A,
  * \return Pointer to result 3x3 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33_f *gan_mat34f_get_m33l_q ( Gan_Matrix34_f *A, Gan_Matrix33_f *B );
+Gan_Matrix33_f *gan_mat34f_get_m33l_q ( const Gan_Matrix34_f *A, Gan_Matrix33_f *B );
 #else
 #define gan_mat34f_get_m33l_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix34_f,Gan_Matrix33_f),\
@@ -189,7 +189,7 @@ Gan_Matrix33_f *gan_mat34f_get_m33l_q ( Gan_Matrix34_f *A, Gan_Matrix33_f *B );
  * \return Pointer to result 3-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector3_f *gan_mat34f_get_v3r_q ( Gan_Matrix34_f *A, Gan_Vector3_f *p );
+Gan_Vector3_f *gan_mat34f_get_v3r_q ( const Gan_Matrix34_f *A, Gan_Vector3_f *p );
 #else
 #define gan_mat34f_get_v3r_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix34_f,Gan_Vector3_f),\
@@ -219,7 +219,7 @@ Gan_Vector3_f *gan_mat34f_get_v3r_q ( Gan_Matrix34_f *A, Gan_Vector3_f *p );
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34_f *gan_mat34f_set_parts_q ( Gan_Matrix34_f *A,
-                                         Gan_Matrix33_f *B, Gan_Vector3_f *p );
+                                         const Gan_Matrix33_f *B, const Gan_Vector3_f *p );
 #else
 #define gan_mat34f_set_parts_q(A,B,p)\
            (GAN_TEST_OP3(A,B,p,\
@@ -242,8 +242,8 @@ Gan_Matrix34_f *gan_mat34f_set_parts_q ( Gan_Matrix34_f *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34_f *gan_mat34f_set_cols_q ( Gan_Matrix34_f *A,
-                                        Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                        Gan_Vector3_f *r, Gan_Vector3_f *s );
+                                        const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                        const Gan_Vector3_f *r, const Gan_Vector3_f *s );
 #else
 #define gan_mat34f_set_cols_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,\
@@ -268,8 +268,8 @@ Gan_Matrix34_f *gan_mat34f_set_cols_q ( Gan_Matrix34_f *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix34_f *gan_mat34f_set_rows_q ( Gan_Matrix34_f *A,
-                                        Gan_Vector4_f *p, Gan_Vector4_f *q,
-                                        Gan_Vector4_f *r );
+                                        const Gan_Vector4_f *p, const Gan_Vector4_f *q,
+                                        const Gan_Vector4_f *r );
 #else
 #define gan_mat34f_set_rows_q(A,p,q,r)\
            (GAN_TEST_OP4(A,p,q,r,\
@@ -289,25 +289,25 @@ Gan_Matrix34_f gan_mat34f_fill_s (
                           float XX, float XY, float XZ, float XW,
                           float YX, float YY, float YZ, float YW,
                           float ZX, float ZY, float ZZ, float ZW );
-Gan_Vector3_f gan_mat34f_multv3h_s ( Gan_Matrix34_f *A,
-                                     Gan_Vector3_f *p, float h);
-Gan_Bool gan_mat34hf_invert_q ( Gan_Matrix34_f *A, float a,
+Gan_Vector3_f gan_mat34f_multv3h_s ( const Gan_Matrix34_f *A,
+                                     const Gan_Vector3_f *p, float h);
+Gan_Bool gan_mat34hf_invert_q ( const Gan_Matrix34_f *A, float a,
                                 Gan_Matrix34_f *B, float *b );
-Gan_Matrix34_f gan_mat34hf_invert_s ( Gan_Matrix34_f *A, float a, float *b );
-void gan_mat34f_get_rows_s ( Gan_Matrix34_f *A,
+Gan_Matrix34_f gan_mat34hf_invert_s ( const Gan_Matrix34_f *A, float a, float *b );
+void gan_mat34f_get_rows_s ( const Gan_Matrix34_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r );
-void gan_mat34f_get_cols_s ( Gan_Matrix34_f *A,
+void gan_mat34f_get_cols_s ( const Gan_Matrix34_f *A,
                              Gan_Vector3_f *p, Gan_Vector3_f *q,
                              Gan_Vector3_f *r, Gan_Vector3_f *s );
-Gan_Matrix33_f gan_mat34f_get_m33l_s ( Gan_Matrix34_f *A );
-Gan_Vector3_f  gan_mat34f_get_v3r_s  ( Gan_Matrix34_f *A );
-Gan_Matrix34_f gan_mat34f_set_parts_s ( Gan_Matrix33_f *B, Gan_Vector3_f *p );
-Gan_Matrix34_f gan_mat34f_set_cols_s ( Gan_Vector3_f *p, Gan_Vector3_f *q,
-                                       Gan_Vector3_f *r, Gan_Vector3_f *s );
-Gan_Matrix34_f gan_mat34f_set_rows_s ( Gan_Vector4_f *p, Gan_Vector4_f *q,
-                                       Gan_Vector4_f *r );
-Gan_Matrix34_f *gan_mat34f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix34_f *B );
+Gan_Matrix33_f gan_mat34f_get_m33l_s ( const Gan_Matrix34_f *A );
+Gan_Vector3_f  gan_mat34f_get_v3r_s  ( const Gan_Matrix34_f *A );
+Gan_Matrix34_f gan_mat34f_set_parts_s ( const Gan_Matrix33_f *B, const Gan_Vector3_f *p );
+Gan_Matrix34_f gan_mat34f_set_cols_s ( const Gan_Vector3_f *p, const Gan_Vector3_f *q,
+                                       const Gan_Vector3_f *r, const Gan_Vector3_f *s );
+Gan_Matrix34_f gan_mat34f_set_rows_s ( const Gan_Vector4_f *p, const Gan_Vector4_f *q,
+                                       const Gan_Vector4_f *r );
+Gan_Matrix34_f *gan_mat34f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix34_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
 
@@ -319,6 +319,8 @@ Gan_Matrix34_f *gan_mat34f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix34_f *B );
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector3_f
 #define GAN_VECTYPE2 Gan_Vector4_f
 #define GAN_MATTYPEL Gan_Matrix33_f
