@@ -3,8 +3,8 @@
  * Module:        4x4 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.21 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.22 $
+ * Last edited:   $Date: 2004/03/18 18:26:21 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -40,7 +40,7 @@
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat44_fprint ( FILE *fp, Gan_Matrix44 *A,
+ gan_mat44_fprint ( FILE *fp, const Gan_Matrix44 *A,
                     const char *prefix, int indent, const char *fmt )
 {
    int i;
@@ -243,7 +243,7 @@ Gan_Matrix44 *
  * \return Pointer to result 3-vector \a q.
  */
 Gan_Vector3 *
- gan_mat44_multv3h_q ( Gan_Matrix44 *A, Gan_Vector3 *p, double h,
+ gan_mat44_multv3h_q ( const Gan_Matrix44 *A, const Gan_Vector3 *p, double h,
                        Gan_Vector3 *q )
 {
    Gan_Vector4 p4;
@@ -272,7 +272,7 @@ Gan_Vector3 *
  * \return Result 3-vector \a q.
  */
 Gan_Vector3
- gan_mat44_multv3h_s ( Gan_Matrix44 *A, Gan_Vector3 *p, double h )
+ gan_mat44_multv3h_s ( const Gan_Matrix44 *A, const Gan_Vector3 *p, double h )
 {
    Gan_Vector3 q;
 
@@ -289,7 +289,7 @@ Gan_Vector3
 #if 0
 /* LU decomposition of 4x4 general matrix */
 static void
- gan_mat44_LUdecomp_q ( Gan_Matrix44 *A,
+ gan_mat44_LUdecomp_q ( const Gan_Matrix44 *A,
                         Gan_SquMatrix44 *L, Gan_SquMatrix44 *U )
 {
 #ifndef NDEBUG
@@ -323,7 +323,7 @@ static void
 
 /* right-multiply lower triangular matrix by upper triangular matrix */
 static void
- gan_ltmat44_rmultl44T_q ( Gan_SquMatrix44 *L, Gan_SquMatrix44 *U,
+ gan_ltmat44_rmultl44T_q ( const Gan_SquMatrix44 *L, Gan_SquMatrix44 *U,
                            Gan_Matrix44 *A )
 {
 #ifndef NDEBUG
@@ -351,7 +351,7 @@ static void
 
 /* left-multiply lower triangular matrix by upper triangular matrix */
 static void
- gan_ltmat44_lmultl44T_q ( Gan_SquMatrix44 *L, Gan_SquMatrix44 *U,
+ gan_ltmat44_lmultl44T_q ( const Gan_SquMatrix44 *L, Gan_SquMatrix44 *U,
                            Gan_Matrix44 *A )
 {
 #ifndef NDEBUG
@@ -488,7 +488,7 @@ Gan_Matrix44 *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix44 *
- gan_mat44_adjoint_q ( Gan_Matrix44 *A, Gan_Matrix44 *B )
+ gan_mat44_adjoint_q ( const Gan_Matrix44 *A, Gan_Matrix44 *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat44_adjoint_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -508,7 +508,7 @@ Gan_Matrix44 *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix44 *
- gan_mat44_adjointT_q ( Gan_Matrix44 *A, Gan_Matrix44 *B )
+ gan_mat44_adjointT_q ( const Gan_Matrix44 *A, Gan_Matrix44 *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat44_adjointT_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -538,8 +538,8 @@ Gan_Matrix44 *
  * \return No value.
  */
 void
- gan_mat44_get_rows_s ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                         Gan_Vector4 *r, Gan_Vector4 *s )
+ gan_mat44_get_rows_s ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                               Gan_Vector4 *r, Gan_Vector4 *s )
 {
    gan_mat44_get_rows_q ( A, p, q, r, s );
 }
@@ -556,8 +556,8 @@ void
  * \return No value.
  */
 void
- gan_mat44_get_cols_s ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                             Gan_Vector4 *r, Gan_Vector4 *s )
+ gan_mat44_get_cols_s ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                               Gan_Vector4 *r, Gan_Vector4 *s )
 {
    gan_mat44_get_cols_q ( A, p, q, r, s );
 }
@@ -570,7 +570,7 @@ void
  * \return Result 3x3 matrix part.
  */
 Gan_Matrix33
- gan_mat44_get_m33tl_s ( Gan_Matrix44 *A )
+ gan_mat44_get_m33tl_s ( const Gan_Matrix44 *A )
 {
    Gan_Matrix33 B;
 
@@ -586,7 +586,7 @@ Gan_Matrix33
  * \return Result 3-vector part.
  */
 Gan_Vector3
- gan_mat44_get_v3tr_s ( Gan_Matrix44 *A )
+ gan_mat44_get_v3tr_s ( const Gan_Matrix44 *A )
 {
    Gan_Vector3 p;
 
@@ -608,7 +608,7 @@ Gan_Vector3
  * \return No value.
  */
 void
- gan_mat44_get_parts_s ( Gan_Matrix44 *A, Gan_Matrix33 *B,
+ gan_mat44_get_parts_s ( const Gan_Matrix44 *A, Gan_Matrix33 *B,
                          Gan_Vector3 *p, Gan_Vector3 *q, double *s )
 {
    gan_mat44_get_parts_q ( A, B, p, q, s );
@@ -626,7 +626,7 @@ void
  *
  * \return No value.
  */
-void gan_mat44_get_blocks_s ( Gan_Matrix44 *M,
+void gan_mat44_get_blocks_s ( const Gan_Matrix44 *M,
                               Gan_Matrix22 *A, Gan_Matrix22 *B,
                               Gan_Matrix22 *C, Gan_Matrix22 *D )
 {
@@ -659,7 +659,7 @@ void gan_mat44_get_blocks_s ( Gan_Matrix44 *M,
  */
 Gan_Matrix44
  gan_mat44_set_parts_s ( Gan_Matrix33 *B,
-                         Gan_Vector3 *p, Gan_Vector3 *q, double s )
+                         const Gan_Vector3 *p, const Gan_Vector3 *q, double s )
 {
    Gan_Matrix44 A;
    Gan_Vector3 zerov = { 0.0, 0.0, 0.0 };
@@ -685,16 +685,16 @@ Gan_Matrix44
  * \return Filled matrix.
  */
 Gan_Matrix44
- gan_mat44_set_blocks_s ( Gan_Matrix22 *A, Gan_Matrix22 *B,
-                          Gan_Matrix22 *C, Gan_Matrix22 *D )
+ gan_mat44_set_blocks_s ( const Gan_Matrix22 *A, const Gan_Matrix22 *B,
+                          const Gan_Matrix22 *C, const Gan_Matrix22 *D )
 {
    Gan_Matrix44 M;
    Gan_Matrix22 zerob = { 0.0, 0.0,
                           0.0, 0.0 };
 
    (void)gan_mat44_set_blocks_q ( &M,
-                              (A==NULL) ? &zerob : A, (B==NULL) ? &zerob : B,
-                              (C==NULL) ? &zerob : C, (D==NULL) ? &zerob : D );
+                                  (A==NULL) ? &zerob : A, (B==NULL) ? &zerob : B,
+                                  (C==NULL) ? &zerob : C, (D==NULL) ? &zerob : D );
    return M;
 }
 
@@ -721,7 +721,7 @@ Gan_Matrix44
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix44 *
- gan_mat44_from_mat_q ( Gan_Matrix *A, Gan_Matrix44 *B )
+ gan_mat44_from_mat_q ( const Gan_Matrix *A, Gan_Matrix44 *B )
 {
    if ( !gan_mat_read_va ( A, 4, 4, &B->xx, &B->xy, &B->xz, &B->xw,
                                     &B->yx, &B->yy, &B->yz, &B->yw,
@@ -743,7 +743,7 @@ Gan_Matrix44 *
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix44 *
- gan_mat44_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix44 *B )
+ gan_mat44_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix44 *B )
 {
    switch ( A->type )
    {
@@ -830,7 +830,7 @@ Gan_Matrix44 *
  * \return Pointer to result matrix \a B.
  */
 Gan_SquMatrix44 *
- gan_squmat44_from_squmat_q ( Gan_SquMatrix *A, Gan_SquMatrix44 *B )
+ gan_squmat44_from_squmat_q ( const Gan_SquMatrix *A, Gan_SquMatrix44 *B )
 {
    switch ( A->type )
    {
