@@ -3,8 +3,8 @@
  * Module:        4x4 matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.22 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.25 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -78,8 +78,8 @@ Gan_Matrix44 *gan_mat44_fill_q ( Gan_Matrix44 *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44_get_rows_q ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                             Gan_Vector4 *r, Gan_Vector4 *s );
+void gan_mat44_get_rows_q ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                                   Gan_Vector4 *r, Gan_Vector4 *s );
 #else
 #define gan_mat44_get_rows_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,Gan_Matrix44,\
@@ -102,8 +102,8 @@ void gan_mat44_get_rows_q ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44_get_cols_q ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                             Gan_Vector4 *r, Gan_Vector4 *s );
+void gan_mat44_get_cols_q ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                                   Gan_Vector4 *r, Gan_Vector4 *s );
 #else
 #define gan_mat44_get_cols_q(A,p,q,r,s)\
            (GAN_TEST_OP5(A,p,q,r,s,Gan_Matrix44,\
@@ -122,7 +122,7 @@ void gan_mat44_get_cols_q ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
  * \return Pointer to 3x3 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33 *gan_mat44_get_m33tl_q ( Gan_Matrix44 *A, Gan_Matrix33 *B );
+Gan_Matrix33 *gan_mat44_get_m33tl_q ( const Gan_Matrix44 *A, Gan_Matrix33 *B );
 #else
 #define gan_mat44_get_m33tl_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix44,Gan_Matrix33),\
@@ -139,7 +139,7 @@ Gan_Matrix33 *gan_mat44_get_m33tl_q ( Gan_Matrix44 *A, Gan_Matrix33 *B );
  * \return Pointer to result 3-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector3 *gan_mat44_get_v3tr_q ( Gan_Matrix44 *A, Gan_Vector3 *p );
+Gan_Vector3 *gan_mat44_get_v3tr_q ( const Gan_Matrix44 *A, Gan_Vector3 *p );
 #else
 #define gan_mat44_get_v3tr_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix44,Gan_Vector3),\
@@ -160,7 +160,7 @@ Gan_Vector3 *gan_mat44_get_v3tr_q ( Gan_Matrix44 *A, Gan_Vector3 *p );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44_get_parts_q ( Gan_Matrix44 *A, Gan_Matrix33 *B,
+void gan_mat44_get_parts_q ( const Gan_Matrix44 *A, Gan_Matrix33 *B,
                              Gan_Vector3 *p, Gan_Vector3 *q, double *s );
 #else
 #define gan_mat44_get_parts_q(A,B,p,q,s)\
@@ -185,7 +185,7 @@ void gan_mat44_get_parts_q ( Gan_Matrix44 *A, Gan_Matrix33 *B,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44_get_blocks_q ( Gan_Matrix44 *M,
+void gan_mat44_get_blocks_q ( const Gan_Matrix44 *M,
                               Gan_Matrix22 *A, Gan_Matrix22 *B,
                               Gan_Matrix22 *C, Gan_Matrix22 *D );
 #else
@@ -223,8 +223,8 @@ void gan_mat44_get_blocks_q ( Gan_Matrix44 *M,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix44 *gan_mat44_set_parts_q ( Gan_Matrix44 *A,
-                                      Gan_Matrix33 *B, Gan_Vector3 *p,
-                                      Gan_Vector3 *q, double s );
+                                      const Gan_Matrix33 *B, const Gan_Vector3 *p,
+                                      const Gan_Vector3 *q, double s );
 #else
 #define gan_mat44_set_parts_q(A,B,p,q,s)\
            (GAN_TEST_OP4(A,B,p,q,\
@@ -250,8 +250,8 @@ Gan_Matrix44 *gan_mat44_set_parts_q ( Gan_Matrix44 *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix44 *gan_mat44_set_blocks_q ( Gan_Matrix44 *M,
-                                       Gan_Matrix22 *A, Gan_Matrix22 *B,
-                                       Gan_Matrix22 *C, Gan_Matrix22 *D );
+                                       const Gan_Matrix22 *A, const Gan_Matrix22 *B,
+                                       const Gan_Matrix22 *C, const Gan_Matrix22 *D );
 #else
 #define gan_mat44_set_blocks_q(M,A,B,C,D)\
            (GAN_TEST_OP5(M,A,B,C,D,\
@@ -273,28 +273,28 @@ Gan_Matrix44 gan_mat44_fill_s ( double XX, double XY, double XZ, double XW,
                                 double YX, double YY, double YZ, double YW,
                                 double ZX, double ZY, double ZZ, double ZW,
                                 double WX, double WY, double WZ, double WW );
-Gan_Vector3 *gan_mat44_multv3h_q ( Gan_Matrix44 *A, Gan_Vector3 *p, double h,
+Gan_Vector3 *gan_mat44_multv3h_q ( const Gan_Matrix44 *A, const Gan_Vector3 *p, double h,
                                    Gan_Vector3 *q );
-Gan_Vector3 gan_mat44_multv3h_s ( Gan_Matrix44 *A, Gan_Vector3 *p, double h);
-void gan_mat44_get_rows_s ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                             Gan_Vector4 *r, Gan_Vector4 *s );
-void gan_mat44_get_cols_s ( Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
-                                             Gan_Vector4 *r, Gan_Vector4 *s );
-Gan_Matrix33 gan_mat44_get_m33tl_s ( Gan_Matrix44 *A );
-Gan_Vector3  gan_mat44_get_v3tr_s  ( Gan_Matrix44 *A );
-void gan_mat44_get_parts_s ( Gan_Matrix44 *A, Gan_Matrix33 *B,
+Gan_Vector3 gan_mat44_multv3h_s ( const Gan_Matrix44 *A, const Gan_Vector3 *p, double h);
+void gan_mat44_get_rows_s ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                                   Gan_Vector4 *r, Gan_Vector4 *s );
+void gan_mat44_get_cols_s ( const Gan_Matrix44 *A, Gan_Vector4 *p, Gan_Vector4 *q,
+                                                   Gan_Vector4 *r, Gan_Vector4 *s );
+Gan_Matrix33 gan_mat44_get_m33tl_s ( const Gan_Matrix44 *A );
+Gan_Vector3  gan_mat44_get_v3tr_s  ( const Gan_Matrix44 *A );
+void gan_mat44_get_parts_s ( const Gan_Matrix44 *A, Gan_Matrix33 *B,
                              Gan_Vector3 *p, Gan_Vector3 *q, double *s );
-void gan_mat44_get_blocks_s ( Gan_Matrix44 *M,
+void gan_mat44_get_blocks_s ( const Gan_Matrix44 *M,
                               Gan_Matrix22 *A, Gan_Matrix22 *B,
                               Gan_Matrix22 *C, Gan_Matrix22 *D );
 Gan_Matrix44 gan_mat44_set_parts_s ( Gan_Matrix33 *B,
-                                     Gan_Vector3 *p, Gan_Vector3 *q,
+                                     const Gan_Vector3 *p, const Gan_Vector3 *q,
                                      double s );
-Gan_Matrix44 gan_mat44_set_blocks_s ( Gan_Matrix22 *A, Gan_Matrix22 *B,
-                                      Gan_Matrix22 *C, Gan_Matrix22 *D );
-Gan_Matrix44 *gan_mat44_from_mat_q ( Gan_Matrix *A, Gan_Matrix44 *B );
-Gan_Matrix44 *gan_mat44_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix44 *B );
-Gan_SquMatrix44 *gan_squmat44_from_squmat_q ( Gan_SquMatrix *A,
+Gan_Matrix44 gan_mat44_set_blocks_s ( const Gan_Matrix22 *A, const Gan_Matrix22 *B,
+                                      const Gan_Matrix22 *C, const Gan_Matrix22 *D );
+Gan_Matrix44 *gan_mat44_from_mat_q ( const Gan_Matrix *A, Gan_Matrix44 *B );
+Gan_Matrix44 *gan_mat44_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix44 *B );
+Gan_SquMatrix44 *gan_squmat44_from_squmat_q ( const Gan_SquMatrix *A,
                                               Gan_SquMatrix44 *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -307,6 +307,8 @@ Gan_SquMatrix44 *gan_squmat44_from_squmat_q ( Gan_SquMatrix *A,
 #define GAN_VECTOR_TYPE Gan_Vector
 #define GAN_MAT_ELEMENT_TYPE GAN_DOUBLE
 #define GAN_REALTYPE double
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f64
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f64
 #define GAN_VECTYPE1 Gan_Vector4
 #define GAN_VECTYPE2 Gan_Vector4
 #define GAN_MATTYPEL Gan_Matrix44
