@@ -3,8 +3,8 @@
  * Module:        4x4 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.13 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.14 $
+ * Last edited:   $Date: 2004/03/18 18:26:22 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -40,7 +40,7 @@
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat44f_fprint ( FILE *fp, Gan_Matrix44_f *A,
+ gan_mat44f_fprint ( FILE *fp, const Gan_Matrix44_f *A,
                      const char *prefix, int indent, const char *fmt )
 {
    int i;
@@ -243,7 +243,7 @@ Gan_Matrix44_f *
  * \return Pointer to result 3-vector \a q.
  */
 Gan_Vector3_f *
- gan_mat44f_multv3h_q ( Gan_Matrix44_f *A, Gan_Vector3_f *p, float h,
+ gan_mat44f_multv3h_q ( const Gan_Matrix44_f *A, const Gan_Vector3_f *p, float h,
                         Gan_Vector3_f *q )
 {
    Gan_Vector4_f p4;
@@ -272,7 +272,7 @@ Gan_Vector3_f *
  * \return Result 3-vector \a q.
  */
 Gan_Vector3_f
- gan_mat44f_multv3h_s ( Gan_Matrix44_f *A, Gan_Vector3_f *p, float h )
+ gan_mat44f_multv3h_s ( const Gan_Matrix44_f *A, const Gan_Vector3_f *p, float h )
 {
    Gan_Vector3_f q;
 
@@ -289,8 +289,8 @@ Gan_Vector3_f
 #if 0
 /* LU decomposition of 4x4 general matrix */
 static void
- gan_mat44f_LUdecomp_q ( Gan_Matrix44_f *A,
-                        Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U )
+ gan_mat44f_LUdecomp_q ( const Gan_Matrix44_f *A,
+                         Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U )
 {
 #ifndef NDEBUG
    L->type = GAN_LOWER_TRI_MATRIX44_F;
@@ -323,8 +323,8 @@ static void
 
 /* right-multiply lower triangular matrix by upper triangular matrix */
 static void
- gan_ltmat44f_rmultl44T_q ( Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U,
-                           Gan_Matrix44_f *A )
+ gan_ltmat44f_rmultl44T_q ( const Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U,
+                            Gan_Matrix44_f *A )
 {
 #ifndef NDEBUG
    assert ( L->type == GAN_LOWER_TRI_MATRIX44_F &&
@@ -351,8 +351,8 @@ static void
 
 /* left-multiply lower triangular matrix by upper triangular matrix */
 static void
- gan_ltmat44f_lmultl44T_q ( Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U,
-                           Gan_Matrix44_f *A )
+ gan_ltmat44f_lmultl44T_q ( const Gan_SquMatrix44_f *L, Gan_SquMatrix44_f *U,
+                            Gan_Matrix44_f *A )
 {
 #ifndef NDEBUG
    assert ( L->type == GAN_LOWER_TRI_MATRIX44_F &&
@@ -489,7 +489,7 @@ Gan_Matrix44_f *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix44_f *
- gan_mat44f_adjoint_q ( Gan_Matrix44_f *A, Gan_Matrix44_f *B )
+ gan_mat44f_adjoint_q ( const Gan_Matrix44_f *A, Gan_Matrix44_f *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat44f_adjoint_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -509,7 +509,7 @@ Gan_Matrix44_f *
  * \return Result matrix \a B on success, \c NULL on error.
  */
 Gan_Matrix44_f *
- gan_mat44f_adjointT_q ( Gan_Matrix44_f *A, Gan_Matrix44_f *B )
+ gan_mat44f_adjointT_q ( const Gan_Matrix44_f *A, Gan_Matrix44_f *B )
 {
    gan_err_test_ptr ( A != B, "gan_mat44f_adjointT_q",
                       GAN_ERROR_ILLEGAL_ARGUMENT, "can't do adjoint in place");
@@ -539,8 +539,8 @@ Gan_Matrix44_f *
  * \return No value.
  */
 void
- gan_mat44f_get_rows_s ( Gan_Matrix44_f *A, Gan_Vector4_f *p, Gan_Vector4_f *q,
-                                            Gan_Vector4_f *r, Gan_Vector4_f *s)
+ gan_mat44f_get_rows_s ( const Gan_Matrix44_f *A, Gan_Vector4_f *p, Gan_Vector4_f *q,
+                                                  Gan_Vector4_f *r, Gan_Vector4_f *s)
 {
    gan_mat44f_get_rows_q ( A, p, q, r, s );
 }
@@ -557,8 +557,8 @@ void
  * \return No value.
  */
 void
- gan_mat44f_get_cols_s ( Gan_Matrix44_f *A, Gan_Vector4_f *p, Gan_Vector4_f *q,
-                                            Gan_Vector4_f *r, Gan_Vector4_f *s)
+ gan_mat44f_get_cols_s ( const Gan_Matrix44_f *A, Gan_Vector4_f *p, Gan_Vector4_f *q,
+                                                  Gan_Vector4_f *r, Gan_Vector4_f *s)
 {
    gan_mat44f_get_cols_q ( A, p, q, r, s );
 }
@@ -571,7 +571,7 @@ void
  * \return Result 3x3 matrix part.
  */
 Gan_Matrix33_f
- gan_mat44f_get_m33tl_s ( Gan_Matrix44_f *A )
+ gan_mat44f_get_m33tl_s ( const Gan_Matrix44_f *A )
 {
    Gan_Matrix33_f B;
 
@@ -587,7 +587,7 @@ Gan_Matrix33_f
  * \return Result 3-vector part.
  */
 Gan_Vector3_f
- gan_mat44f_get_v3tr_s ( Gan_Matrix44_f *A )
+ gan_mat44f_get_v3tr_s ( const Gan_Matrix44_f *A )
 {
    Gan_Vector3_f p;
 
@@ -609,7 +609,7 @@ Gan_Vector3_f
  * \return No value.
  */
 void
- gan_mat44f_get_parts_s ( Gan_Matrix44_f *A, Gan_Matrix33_f *B,
+ gan_mat44f_get_parts_s ( const Gan_Matrix44_f *A, Gan_Matrix33_f *B,
                           Gan_Vector3_f *p, Gan_Vector3_f *q, float *s )
 {
    gan_mat44f_get_parts_q ( A, B, p, q, s );
@@ -627,7 +627,7 @@ void
  *
  * \return No value.
  */
-void gan_mat44f_get_blocks_s ( Gan_Matrix44_f *M,
+void gan_mat44f_get_blocks_s ( const Gan_Matrix44_f *M,
                                Gan_Matrix22_f *A, Gan_Matrix22_f *B,
                                Gan_Matrix22_f *C, Gan_Matrix22_f *D )
 {
@@ -660,7 +660,7 @@ void gan_mat44f_get_blocks_s ( Gan_Matrix44_f *M,
  */
 Gan_Matrix44_f
  gan_mat44f_set_parts_s ( Gan_Matrix33_f *B,
-                          Gan_Vector3_f *p, Gan_Vector3_f *q, float s )
+                          const Gan_Vector3_f *p, const Gan_Vector3_f *q, float s )
 {
    Gan_Matrix44_f A;
    Gan_Vector3_f zerov = { 0.0F, 0.0F, 0.0F };
@@ -686,8 +686,8 @@ Gan_Matrix44_f
  * \return Filled matrix.
  */
 Gan_Matrix44_f
- gan_mat44f_set_blocks_s ( Gan_Matrix22_f *A, Gan_Matrix22_f *B,
-                           Gan_Matrix22_f *C, Gan_Matrix22_f *D )
+ gan_mat44f_set_blocks_s ( const Gan_Matrix22_f *A, const Gan_Matrix22_f *B,
+                           const Gan_Matrix22_f *C, const Gan_Matrix22_f *D )
 {
    Gan_Matrix44_f M;
    Gan_Matrix22_f zerob = { 0.0F, 0.0F,
@@ -722,7 +722,7 @@ Gan_Matrix44_f
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix44_f *
- gan_mat44f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix44_f *B )
+ gan_mat44f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix44_f *B )
 {
    if ( !gan_matf_read_va ( A, 4, 4, &B->xx, &B->xy, &B->xz, &B->xw,
                                      &B->yx, &B->yy, &B->yz, &B->yw,
@@ -744,7 +744,7 @@ Gan_Matrix44_f *
  * \return Pointer to result matrix \a B.
  */
 Gan_Matrix44_f *
- gan_mat44f_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_Matrix44_f *B )
+ gan_mat44f_from_squmatf_q ( const Gan_SquMatrix_f *A, Gan_Matrix44_f *B )
 {
    switch ( A->type )
    {
@@ -831,7 +831,7 @@ Gan_Matrix44_f *
  * \return Pointer to result matrix \a B.
  */
 Gan_SquMatrix44_f *
- gan_squmat44f_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_SquMatrix44_f *B )
+ gan_squmat44f_from_squmatf_q ( const Gan_SquMatrix_f *A, Gan_SquMatrix44_f *B )
 {
    switch ( A->type )
    {
