@@ -3,8 +3,8 @@
  * Module:        4x4 matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.17 $
- * Last edited:   $Date: 2003/01/31 18:57:10 $
+ * Revision:      $Revision: 1.20 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -78,7 +78,7 @@ Gan_Matrix44_f *gan_mat44f_fill_q ( Gan_Matrix44_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44f_get_rows_q ( Gan_Matrix44_f *A,
+void gan_mat44f_get_rows_q ( const Gan_Matrix44_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r, Gan_Vector4_f *s );
 #else
@@ -103,7 +103,7 @@ void gan_mat44f_get_rows_q ( Gan_Matrix44_f *A,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44f_get_cols_q ( Gan_Matrix44_f *A,
+void gan_mat44f_get_cols_q ( const Gan_Matrix44_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r, Gan_Vector4_f *s );
 #else
@@ -124,7 +124,7 @@ void gan_mat44f_get_cols_q ( Gan_Matrix44_f *A,
  * \return Pointer to 3x3 matrix \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix33_f *gan_mat44f_get_m33tl_q ( Gan_Matrix44_f *A, Gan_Matrix33_f *B );
+Gan_Matrix33_f *gan_mat44f_get_m33tl_q ( const Gan_Matrix44_f *A, Gan_Matrix33_f *B );
 #else
 #define gan_mat44f_get_m33tl_q(A,B)\
            (GAN_TEST_OP2(A,B,Gan_Matrix44_f,Gan_Matrix33_f),\
@@ -141,7 +141,7 @@ Gan_Matrix33_f *gan_mat44f_get_m33tl_q ( Gan_Matrix44_f *A, Gan_Matrix33_f *B );
  * \return Pointer to result 3-vector \a p.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector3_f *gan_mat44f_get_v3tr_q ( Gan_Matrix44_f *A, Gan_Vector3_f *p );
+Gan_Vector3_f *gan_mat44f_get_v3tr_q ( const Gan_Matrix44_f *A, Gan_Vector3_f *p );
 #else
 #define gan_mat44f_get_v3tr_q(A,p)\
            (GAN_TEST_OP2(A,p,Gan_Matrix44_f,Gan_Vector3_f),\
@@ -162,7 +162,7 @@ Gan_Vector3_f *gan_mat44f_get_v3tr_q ( Gan_Matrix44_f *A, Gan_Vector3_f *p );
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44f_get_parts_q ( Gan_Matrix44_f *A, Gan_Matrix33_f *B,
+void gan_mat44f_get_parts_q ( const Gan_Matrix44_f *A, Gan_Matrix33_f *B,
                               Gan_Vector3_f *p, Gan_Vector3_f *q, float *s );
 #else
 #define gan_mat44f_get_parts_q(A,B,p,q,s)\
@@ -187,7 +187,7 @@ void gan_mat44f_get_parts_q ( Gan_Matrix44_f *A, Gan_Matrix33_f *B,
  * \return No value.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-void gan_mat44f_get_blocks_q ( Gan_Matrix44_f *M,
+void gan_mat44f_get_blocks_q ( const Gan_Matrix44_f *M,
                                Gan_Matrix22_f *A, Gan_Matrix22_f *B,
                                Gan_Matrix22_f *C, Gan_Matrix22_f *D );
 #else
@@ -225,8 +225,8 @@ void gan_mat44f_get_blocks_q ( Gan_Matrix44_f *M,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix44_f *gan_mat44f_set_parts_q ( Gan_Matrix44_f *A,
-                                         Gan_Matrix33_f *B, Gan_Vector3_f *p,
-                                         Gan_Vector3_f *q, float s );
+                                         const Gan_Matrix33_f *B, const Gan_Vector3_f *p,
+                                         const Gan_Vector3_f *q, float s );
 #else
 #define gan_mat44f_set_parts_q(A,B,p,q,s)\
            (GAN_TEST_OP4(A,B,p,q,\
@@ -252,8 +252,8 @@ Gan_Matrix44_f *gan_mat44f_set_parts_q ( Gan_Matrix44_f *A,
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix44_f *gan_mat44f_set_blocks_q ( Gan_Matrix44_f *M,
-                                          Gan_Matrix22_f *A, Gan_Matrix22_f *B,
-                                          Gan_Matrix22_f *C, Gan_Matrix22_f *D );
+                                          const Gan_Matrix22_f *A, const Gan_Matrix22_f *B,
+                                          const Gan_Matrix22_f *C, const Gan_Matrix22_f *D );
 #else
 #define gan_mat44f_set_blocks_q(M,A,B,C,D)\
            (GAN_TEST_OP5(M,A,B,C,D,\
@@ -275,33 +275,33 @@ Gan_Matrix44_f gan_mat44f_fill_s ( float XX, float XY, float XZ, float XW,
                                    float YX, float YY, float YZ, float YW,
                                    float ZX, float ZY, float ZZ, float ZW,
                                    float WX, float WY, float WZ, float WW );
-Gan_Vector3_f *gan_mat44f_multv3h_q ( Gan_Matrix44_f *A,
-                                      Gan_Vector3_f *p, float h,
+Gan_Vector3_f *gan_mat44f_multv3h_q ( const Gan_Matrix44_f *A,
+                                      const Gan_Vector3_f *p, float h,
                                       Gan_Vector3_f *q );
-Gan_Vector3_f gan_mat44f_multv3h_s ( Gan_Matrix44_f *A,
-                                     Gan_Vector3_f *p, float h);
-void gan_mat44f_get_rows_s ( Gan_Matrix44_f *A,
+Gan_Vector3_f gan_mat44f_multv3h_s ( const Gan_Matrix44_f *A,
+                                     const Gan_Vector3_f *p, float h);
+void gan_mat44f_get_rows_s ( const Gan_Matrix44_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r, Gan_Vector4_f *s );
-void gan_mat44f_get_cols_s ( Gan_Matrix44_f *A,
+void gan_mat44f_get_cols_s ( const Gan_Matrix44_f *A,
                              Gan_Vector4_f *p, Gan_Vector4_f *q,
                              Gan_Vector4_f *r, Gan_Vector4_f *s );
-Gan_Matrix33_f gan_mat44f_get_m33tl_s ( Gan_Matrix44_f *A );
-Gan_Vector3_f  gan_mat44f_get_v3tr_s  ( Gan_Matrix44_f *A );
-void gan_mat44f_get_parts_s ( Gan_Matrix44_f *A, Gan_Matrix33_f *B,
+Gan_Matrix33_f gan_mat44f_get_m33tl_s ( const Gan_Matrix44_f *A );
+Gan_Vector3_f  gan_mat44f_get_v3tr_s  ( const Gan_Matrix44_f *A );
+void gan_mat44f_get_parts_s ( const Gan_Matrix44_f *A, Gan_Matrix33_f *B,
                               Gan_Vector3_f *p, Gan_Vector3_f *q, float *s );
-void gan_mat44f_get_blocks_s ( Gan_Matrix44_f *M,
+void gan_mat44f_get_blocks_s ( const Gan_Matrix44_f *M,
                                Gan_Matrix22_f *A, Gan_Matrix22_f *B,
                                Gan_Matrix22_f *C, Gan_Matrix22_f *D );
 Gan_Matrix44_f gan_mat44f_set_parts_s ( Gan_Matrix33_f *B,
-                                        Gan_Vector3_f *p, Gan_Vector3_f *q,
+                                        const Gan_Vector3_f *p, const Gan_Vector3_f *q,
                                         float s );
-Gan_Matrix44_f gan_mat44f_set_blocks_s ( Gan_Matrix22_f *A, Gan_Matrix22_f *B,
-                                         Gan_Matrix22_f *C, Gan_Matrix22_f *D);
-Gan_Matrix44_f *gan_mat44f_from_matf_q ( Gan_Matrix_f *A, Gan_Matrix44_f *B );
-Gan_Matrix44_f *gan_mat44f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_Matrix44_f gan_mat44f_set_blocks_s ( const Gan_Matrix22_f *A, const Gan_Matrix22_f *B,
+                                         const Gan_Matrix22_f *C, const Gan_Matrix22_f *D);
+Gan_Matrix44_f *gan_mat44f_from_matf_q ( const Gan_Matrix_f *A, Gan_Matrix44_f *B );
+Gan_Matrix44_f *gan_mat44f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                             Gan_Matrix44_f *B );
-Gan_SquMatrix44_f *gan_squmat44f_from_squmatf_q ( Gan_SquMatrix_f *A,
+Gan_SquMatrix44_f *gan_squmat44f_from_squmatf_q ( const Gan_SquMatrix_f *A,
                                                   Gan_SquMatrix44_f *B );
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -314,6 +314,8 @@ Gan_SquMatrix44_f *gan_squmat44f_from_squmatf_q ( Gan_SquMatrix_f *A,
 #define GAN_VECTOR_TYPE Gan_Vector_f
 #define GAN_MAT_ELEMENT_TYPE GAN_FLOAT
 #define GAN_REALTYPE float
+#define GAN_FWRITE_LENDIAN gan_fwrite_lendian_f32
+#define GAN_FREAD_LENDIAN  gan_fread_lendian_f32
 #define GAN_VECTYPE1 Gan_Vector4_f
 #define GAN_VECTYPE2 Gan_Vector4_f
 #define GAN_MATTYPEL Gan_Matrix44_f
