@@ -4,8 +4,8 @@
 * Module:        BLAS function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.3 $
-* Last edited:   $Date: 2001/10/04 16:27:00 $
+* Revision:      $Revision: 1.4 $
+* Last edited:   $Date: 2005/02/25 09:30:20 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -22,7 +22,7 @@
 #include <gandalf/common/misc_error.h>
 
 /* only define this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 
 Gan_Bool
  gan_dcopy ( long n, double *dx, long incx, double *dy, long incy )
@@ -48,4 +48,4 @@ Gan_Bool
    return GAN_TRUE;
 }
 
-#endif /* #ifndef HAVE_LAPACK */
+#endif /* #if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK) */
