@@ -4,8 +4,8 @@
 * Module:        BLAS function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.1 $
-* Last edited:   $Date: 2003/01/31 18:57:26 $
+* Revision:      $Revision: 1.3 $
+* Last edited:   $Date: 2005/02/25 09:30:20 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2003 Imagineer Systems Limited
 *
@@ -26,9 +26,10 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
-Gan_Bool gan_dtrsm ( Gan_SideFlag side, Gan_UpLoFlag uplo,
-                     Gan_TposeFlag transa, Gan_UnitFlag diag, long m, long n,
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
+Gan_Bool gan_dtrsm ( Gan_LapackSideFlag side, Gan_LapackUpLoFlag uplo,
+                     Gan_TposeFlag transa, Gan_LapackUnitFlag diag,
+                     long m, long n,
                      double alpha, double *a, long lda, double *b, long ldb );
 #endif
 
