@@ -4,8 +4,8 @@
 * Module:        BLAS function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.4 $
-* Last edited:   $Date: 2001/10/04 16:27:00 $
+* Revision:      $Revision: 1.5 $
+* Last edited:   $Date: 2005/02/25 09:30:20 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -22,7 +22,7 @@
 #include <gandalf/common/misc_error.h>
 
 /* only define this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 
 float
  gan_sdot ( long n, float *dx, long incx, float *dy, long incy )
@@ -45,4 +45,4 @@ float
    return total;
 }
 
-#endif /* #ifndef HAVE_LAPACK */
+#endif /* #if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK) */
