@@ -4,8 +4,8 @@
 * Module:        BLAS function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.2 $
-* Last edited:   $Date: 2001/10/04 16:27:00 $
+* Revision:      $Revision: 1.3 $
+* Last edited:   $Date: 2005/02/25 09:30:20 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -26,7 +26,7 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 Gan_Bool gan_sgemv ( Gan_TposeFlag trans, long m, long n, float alpha,
                      float *a, long lda, float *x, long incx, 
                      float beta, float *y, long incy );
