@@ -4,8 +4,8 @@
 * Module:        BLAS function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.2 $
-* Last edited:   $Date: 2001/10/04 16:27:01 $
+* Revision:      $Revision: 1.4 $
+* Last edited:   $Date: 2005/02/25 09:30:20 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -26,9 +26,9 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
-Gan_Bool gan_stpmv ( Gan_UpLoFlag upper, Gan_TposeFlag trans,
-                     Gan_UnitFlag unit, long n, float *ap,
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
+Gan_Bool gan_stpmv ( Gan_LapackUpLoFlag upper, Gan_TposeFlag trans,
+                     Gan_LapackUnitFlag unit, long n, float *ap,
                      float *x, long incx );
 #endif
 
