@@ -4,8 +4,8 @@
 * Module:        CLAPACK function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.3 $
-* Last edited:   $Date: 2003/01/31 18:57:28 $
+* Revision:      $Revision: 1.4 $
+* Last edited:   $Date: 2005/02/25 09:30:23 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -26,7 +26,7 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 Gan_Bool gan_dgelq2 ( long m, long n, double *a, long lda, double *tau,
                       double *work, long *info );
 #endif
