@@ -4,8 +4,8 @@
 * Module:        CLAPACK function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.1 $
-* Last edited:   $Date: 2003/01/31 18:57:29 $
+* Revision:      $Revision: 1.2 $
+* Last edited:   $Date: 2005/02/25 09:30:24 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -25,13 +25,13 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 typedef long ftnlen;
 
 long gan_ilaenv ( long ispec, char *name, char *opts,
                   long n1, long n2, long n3, long n4,
                   ftnlen name_len, ftnlen opts_len );
-#endif
+#endif /* #if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK) */
 
 #ifdef __cplusplus
 }
