@@ -4,8 +4,8 @@
 * Module:        CLAPACK function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.6 $
-* Last edited:   $Date: 2003/01/31 18:57:29 $
+* Revision:      $Revision: 1.7 $
+* Last edited:   $Date: 2005/02/25 09:30:24 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -23,7 +23,7 @@
 #include <gandalf/common/compare.h>
 
 /* only define this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 
 /* modified from CLAPACK source */
 float
@@ -47,4 +47,4 @@ float
    return ret_val;
 }
 
-#endif /* #ifndef HAVE_LAPACK */
+#endif /* #if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK) */
