@@ -4,8 +4,8 @@
 * Module:        CLAPACK function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.1 $
-* Last edited:   $Date: 2003/01/31 18:57:29 $
+* Revision:      $Revision: 1.2 $
+* Last edited:   $Date: 2005/02/25 09:30:25 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -25,7 +25,7 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
 Gan_Bool gan_slaswp ( long n, float *a, long lda, long k1, long k2,
                       long *ipiv, long incx );
 #endif
