@@ -4,8 +4,8 @@
 * Module:        CLAPACK function
 * Part of:       Gandalf Library
 *
-* Revision:      $Revision: 1.3 $
-* Last edited:   $Date: 2003/01/31 18:57:29 $
+* Revision:      $Revision: 1.5 $
+* Last edited:   $Date: 2005/02/25 09:30:25 $
 * Author:        $Author: pm $
 * Copyright:     Modifications (c) 2000 Imagineer Software Limited
 *
@@ -26,8 +26,9 @@ extern "C" {
 #endif
 
 /* only declare this function locally if there is no LAPACK installed */
-#ifndef HAVE_LAPACK
-Gan_Bool gan_spptrf ( Gan_UpLoFlag upper, long n, float *ap, long *info );
+#if !defined(HAVE_LAPACK) || defined(FORCE_LOCAL_LAPACK)
+Gan_Bool gan_spptrf ( Gan_LapackUpLoFlag upper, long n, float *ap,
+                      long *info );
 #endif
 
 #ifdef __cplusplus
