@@ -3,8 +3,8 @@
  * Module:        Diagonal matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.26 $
- * Last edited:   $Date: 2002/12/06 17:48:54 $
+ * Revision:      $Revision: 1.27 $
+ * Last edited:   $Date: 2004/03/18 18:26:23 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -71,7 +71,7 @@ static Gan_Bool
 
 /* function to return element i,j of diagonal matrix */
 static double
- diagonal_matrix_get_el ( Gan_SquMatrix *A, unsigned i, unsigned j )
+ diagonal_matrix_get_el ( const Gan_SquMatrix *A, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( A->type == GAN_DIAGONAL_MATRIX );
@@ -125,7 +125,7 @@ static Gan_Bool
 
 /* function to print diagonal matrix to file pointer */
 static Gan_Bool
- diagonal_matrix_fprint ( FILE *fp, Gan_SquMatrix *A, const char *prefix,
+ diagonal_matrix_fprint ( FILE *fp, const Gan_SquMatrix *A, const char *prefix,
                           unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -160,7 +160,7 @@ static Gan_Bool
 
 /* function to copy diagonal matrix A into another matrix B */
 static Gan_SquMatrix *
- diagonal_matrix_copy ( Gan_SquMatrix *A, Gan_SquMatrix *B )
+ diagonal_matrix_copy ( const Gan_SquMatrix *A, Gan_SquMatrix *B )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -204,11 +204,11 @@ static Gan_SquMatrix *
 
 #ifdef HAVE_LAPACK
    /* copy and scale matrix */
-   dcopy_ ( (long *)&A->size, A->data, &onei, B->data, &onei );
+   if(A!=B) dcopy_ ( (long *)&A->size, A->data, &onei, B->data, &onei );
    dscal_ ( (long *)&A->size, &a, B->data, &onei );
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   gan_dcopy ( A->size, A->data, 1, B->data, 1 );
+   if(A!=B) gan_dcopy ( A->size, A->data, 1, B->data, 1 );
    gan_dscal ( A->size, a, B->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
 
@@ -393,7 +393,7 @@ static Gan_SquMatrix *
 /* function to compute matrix/vector product A*x = y, returning y */
 static Gan_Vector *
  diagonal_matrix_multv (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector *x, Gan_Vector *y )
 {
    long i;
@@ -460,8 +460,8 @@ static Gan_Vector *
  */
 static Gan_Matrix *
  diagonal_matrix_rmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
    unsigned long i, j;
@@ -550,8 +550,8 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  diagonal_matrix_lmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    unsigned long i, j;
@@ -711,7 +711,7 @@ static Gan_SquMatrix *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix *
- diagonal_matrix_lrmult ( Gan_SquMatrix *A, Gan_Matrix *B, Gan_TposeFlag B_tr,
+ diagonal_matrix_lrmult ( const Gan_SquMatrix *A, const Gan_Matrix *B, Gan_TposeFlag B_tr,
                           Gan_Matrix *C, Gan_SquMatrix *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -745,8 +745,8 @@ static Gan_SquMatrix *
    }
 
    /* perform first multiplication step A*B(^T) */
-   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT, B,
-                     B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C );
+   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
+                     (Gan_Matrix *)B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C );
 
    /* perform second multiplication step B*(A*B^T) or B^T*(A*B) */
    gan_mat_rmult_sym_gen ( B, B_tr, C, GAN_NOTRANSPOSE, D );
@@ -836,7 +836,7 @@ static Gan_SquMatrix *
 /* Compute squared Frobenius norm of diagonal matrix A.
  */
 static double
- diagonal_matrix_sumsqr ( Gan_SquMatrix *A )
+ diagonal_matrix_sumsqr ( const Gan_SquMatrix *A )
 {
    long i;
    double total = 0.0;
@@ -853,7 +853,7 @@ static double
 /* Compute Frobenius norm of diagonal matrix A.
  */
 static double
- diagonal_matrix_Fnorm ( Gan_SquMatrix *A )
+ diagonal_matrix_Fnorm ( const Gan_SquMatrix *A )
 {
    long i;
    double total = 0.0;
@@ -869,7 +869,7 @@ static double
 
 /* Convert diagonal matrix to generic matrix */
 static Gan_Matrix *
- diagonal_matrix_matrix ( Gan_SquMatrix *A, struct Gan_Matrix *B )
+ diagonal_matrix_matrix ( const Gan_SquMatrix *A, struct Gan_Matrix *B )
 {
    long i;
 
@@ -1054,7 +1054,7 @@ Gan_SquMatrix *
  * The \a size argument must match the size of the matrix.
  */
 Gan_Bool
- gan_diagmat_read_va ( Gan_SquMatrix *A, unsigned long size, ... )
+ gan_diagmat_read_va ( const Gan_SquMatrix *A, unsigned long size, ... )
 {
    unsigned int i;
    va_list ap;
@@ -1219,7 +1219,7 @@ Gan_SquMatrix *
  * in-place.
  */
 Gan_Matrix *
- gan_diagmatI_lmult_inf_q ( Gan_SquMatrix *A, Gan_Matrix *B, Gan_Matrix *C )
+ gan_diagmatI_lmult_inf_q ( const Gan_SquMatrix *A, Gan_Matrix *B, Gan_Matrix *C )
 {
    unsigned long i, j;
    double *Aptr = A->data, *Bptr = B->data, *Cptr = C->data;
