@@ -3,8 +3,8 @@
  * Module:        Generic rectangular matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.40 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.43 $
+ * Last edited:   $Date: 2005/04/11 16:04:39 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -95,6 +95,31 @@ void
    if ( A->struct_alloc ) free(A);
 }
 
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup GeneralSizeMatrixTest
+ * \{
+ */
+
+/**
+ * \brief Checks whether a matrix is zero
+ * \return Returns #GAN_TRUE if all the elements of \a A are zero, #GAN_FALSE otherwise
+ */
+Gan_Bool gan_mat_is_zero ( Gan_Matrix *A )
+{
+   int i;
+
+   for ( i = (int)(A->rows*A->cols)-1; i >= 0; i-- )
+      if(A->data[i] != 0.0)
+         return GAN_FALSE;
+
+   /* all zero */
+   return GAN_TRUE;
+}
+
 /**
  * \}
  */
@@ -151,7 +176,7 @@ Gan_Matrix *
  * (sum of squares of matrix elements).
  */
 double
- gan_mat_sumsqr ( Gan_Matrix *A )
+ gan_mat_sumsqr ( const Gan_Matrix *A )
 {
    long i;
    double total = 0.0;
@@ -171,7 +196,7 @@ double
  * (square-root of sum of squares of matrix elements).
  */
 double
- gan_mat_Fnorm ( Gan_Matrix *A )
+ gan_mat_Fnorm ( const Gan_Matrix *A )
 {
    long i;
    double total = 0.0;
@@ -202,7 +227,7 @@ double
  * \sa gan_mat_invert_s().
  */
 Gan_Matrix *
- gan_mat_invert_q ( Gan_Matrix *A, Gan_Matrix *B )
+ gan_mat_invert_q ( const Gan_Matrix *A, Gan_Matrix *B )
 {
    long *ipiv;
    double *work;
@@ -401,7 +426,7 @@ Gan_Matrix *
  * \brief Read elements of matrix from variable argument list.
  */
 Gan_Bool
- gan_mat_read_va ( Gan_Matrix *A, unsigned long rows, unsigned long cols, ... )
+ gan_mat_read_va ( const Gan_Matrix *A, unsigned long rows, unsigned long cols, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -437,7 +462,7 @@ Gan_Bool
  * Copy matrix \a A to matrix \a B, and return \a B.
  */
 Gan_Matrix *
- gan_mat_copy_q ( Gan_Matrix *A, Gan_Matrix *B )
+ gan_mat_copy_q ( const Gan_Matrix *A, Gan_Matrix *B )
 {
    /* allocate matrix B if necessary */
    if ( B == NULL )
@@ -812,8 +837,8 @@ Gan_Matrix *
  */
 Gan_SquMatrix *
  gan_mat_add_sym_gen (
-     Gan_Matrix *A, Gan_TposeFlag A_tr,
-     Gan_Matrix *B, Gan_TposeFlag B_tr, struct Gan_SquMatrix *C )
+     const Gan_Matrix *A, Gan_TposeFlag A_tr,
+     const Gan_Matrix *B, Gan_TposeFlag B_tr, struct Gan_SquMatrix *C )
 {
    /* result matrix dimensions */
    unsigned long Csize = (A_tr ? A->cols : A->rows);
@@ -1021,7 +1046,7 @@ Gan_Matrix *
  * \return Pointer to result vector \a y.
  */
 Gan_Vector *
- gan_mat_multv_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr, Gan_Vector *x,
+ gan_mat_multv_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr, const Gan_Vector *x,
                      Gan_Vector *y )
 {
    unsigned long yrows = (A_tr ? A->cols : A->rows);
@@ -1072,8 +1097,8 @@ Gan_Vector *
  * Not a user function
  */
 Gan_Matrix *
- gan_mat_rmult_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
-                     Gan_Matrix *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+ gan_mat_rmult_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                     const Gan_Matrix *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    /* result matrix dimensions */
    unsigned long Crows = (A_tr ? A->cols : A->rows),
@@ -1137,8 +1162,8 @@ Gan_Matrix *
  * Not a user function.
  */
 Gan_SquMatrix *
- gan_mat_rmult_sym_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
-                         Gan_Matrix *B, Gan_TposeFlag B_tr,
+ gan_mat_rmult_sym_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                         const Gan_Matrix *B, Gan_TposeFlag B_tr,
                          struct Gan_SquMatrix *C )
 {
    /* result matrix dimensions */
@@ -1234,8 +1259,8 @@ Gan_SquMatrix *
  */
 Gan_Matrix *
  gan_mat_insert_gen ( Gan_Matrix *A, unsigned long rA, unsigned long cA,
-                      Gan_Matrix *B, Gan_TposeFlag B_tr,
-                                     unsigned long rB, unsigned long cB,
+                      const Gan_Matrix *B, Gan_TposeFlag B_tr,
+                                           unsigned long rB, unsigned long cB,
                       unsigned long rows , unsigned long cols )
 {
    long i, j;
@@ -1277,7 +1302,7 @@ Gan_Matrix *
  */
 Gan_Matrix *
  gan_mat_insertv_gen ( Gan_Matrix *A, unsigned long rA, unsigned long cA,
-                       Gan_Vector *x, Gan_TposeFlag x_tr, unsigned long rx,
+                       const Gan_Vector *x, Gan_TposeFlag x_tr, unsigned long rx,
                        unsigned long size )
 {
    long i;
@@ -1317,7 +1342,7 @@ Gan_Matrix *
  */
 Gan_Matrix *
  gan_mat_insertsym ( Gan_Matrix    *A, unsigned long rA, unsigned long cA,
-                     Gan_SquMatrix *B, unsigned long rB, unsigned long cB,
+                     const Gan_SquMatrix *B, unsigned long rB, unsigned long cB,
                      unsigned long rows , unsigned long cols )
 {
    long i, j;
@@ -1489,7 +1514,7 @@ Gan_Matrix *
  * \sa gan_mat_extract_s(), gan_mat_insert().
  */
 Gan_Matrix *
- gan_mat_extract_q ( Gan_Matrix *A,
+ gan_mat_extract_q ( const Gan_Matrix *A,
                      unsigned long r0,   unsigned long c0,
                      unsigned long rows, unsigned long cols,
                      Gan_Matrix *B )
@@ -1538,7 +1563,7 @@ Gan_Matrix *
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat_fprint ( FILE *fp, Gan_Matrix *A, const char *prefix,
+ gan_mat_fprint ( FILE *fp, const Gan_Matrix *A, const char *prefix,
                   unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -1683,12 +1708,12 @@ Gan_Matrix *
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_mat_fwrite ( FILE *fp, Gan_Matrix *A, gan_ui32 magic_number )
+ gan_mat_fwrite ( FILE *fp, const Gan_Matrix *A, gan_uint32 magic_number )
 {
    if ( !gan_fwrite_lendian_ui32 ( fp, &magic_number, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &A->rows, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &A->cols, 1 ) ||
-        !gan_fwrite_lendian_d ( fp, A->data, A->rows*A->cols ) )
+        !gan_fwrite_lendian_f64 ( fp, A->data, A->rows*A->cols ) )
    {
       gan_err_register ( "gan_mat_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -1709,7 +1734,7 @@ Gan_Bool
  * \sa gan_vecf_fread_s()
  */
 Gan_Matrix *
- gan_mat_fread_q ( FILE *fp, Gan_Matrix *A, gan_ui32 *magic_number )
+ gan_mat_fread_q ( FILE *fp, Gan_Matrix *A, gan_uint32 *magic_number )
 {
    unsigned long rows, cols;
 
@@ -1741,7 +1766,7 @@ Gan_Matrix *
    }
 
    /* read matrix data */
-   if ( !gan_fread_lendian_d ( fp, A->data, A->rows*A->cols ) )
+   if ( !gan_fread_lendian_f64 ( fp, A->data, A->rows*A->cols ) )
    {
       gan_err_register ( "gan_mat_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -1759,7 +1784,7 @@ Gan_Matrix *
 /* for use in a debugger: use gan_mat_print() or gan_mat_fprint in
  * your program!
  */
-void gan_mat_db ( Gan_Matrix *A )
+void gan_mat_db ( const Gan_Matrix *A )
 {
    gan_mat_print ( A, "", 0, "%+g" );
 }
