@@ -3,8 +3,8 @@
  * Module:        Generic rectangular matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.27 $
- * Last edited:   $Date: 2002/04/18 14:56:24 $
+ * Revision:      $Revision: 1.31 $
+ * Last edited:   $Date: 2005/05/05 14:19:56 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -92,32 +92,33 @@ Gan_Matrix *gan_mat_fill_vap ( Gan_Matrix *A,
 Gan_Matrix *gan_mat_fill_const_q ( Gan_Matrix *A,
                                    unsigned long rows, unsigned long cols,
                                    double value );
-Gan_Bool gan_mat_read_va ( Gan_Matrix *A,
+Gan_Bool gan_mat_read_va ( const Gan_Matrix *A,
                            unsigned long rows, unsigned long cols, ... );
-Gan_Matrix *gan_mat_copy_q ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_mat_copy_q ( const Gan_Matrix *A, Gan_Matrix *B );
 Gan_Matrix *gan_mat_scale_q ( Gan_Matrix *A, double a, Gan_Matrix *B );
 Gan_Matrix *gan_mat_divide_q ( Gan_Matrix *A, double a, Gan_Matrix *B );
 Gan_Matrix *gan_mat_tpose_q ( Gan_Matrix *A, Gan_Matrix *B );
 Gan_Matrix *gan_mat_insertsym ( Gan_Matrix *A,
                                 unsigned long rA, unsigned long cA,
-                                struct Gan_SquMatrix *B,
+                                const struct Gan_SquMatrix *B,
                                 unsigned long rB, unsigned long cB,
                                 unsigned long rows , unsigned long cols );
+Gan_Bool gan_mat_is_zero ( Gan_Matrix *A );
 void gan_mat_free_va ( Gan_Matrix *A, ... );
-Gan_Matrix *gan_mat_extract_q ( Gan_Matrix *A,
+Gan_Matrix *gan_mat_extract_q ( const Gan_Matrix *A,
                                 unsigned long r0,   unsigned long c0,
                                 unsigned long rows, unsigned long cols,
                                 Gan_Matrix *B );
-double gan_mat_sumsqr ( Gan_Matrix *A );
-double gan_mat_Fnorm ( Gan_Matrix *A );
-Gan_Matrix *gan_mat_invert_q ( Gan_Matrix *A, Gan_Matrix *B );
-Gan_Bool gan_mat_fprint ( FILE *fp, Gan_Matrix *A, const char *prefix,
+double gan_mat_sumsqr ( const Gan_Matrix *A );
+double gan_mat_Fnorm ( const Gan_Matrix *A );
+Gan_Matrix *gan_mat_invert_q ( const Gan_Matrix *A, Gan_Matrix *B );
+Gan_Bool gan_mat_fprint ( FILE *fp, const Gan_Matrix *A, const char *prefix,
                           unsigned indent, const char *fmt );
 Gan_Matrix *gan_mat_fscanf_q ( FILE *fp, Gan_Matrix *A,
                                char *prefix, int prefix_len );
-Gan_Bool gan_mat_fwrite ( FILE *fp, Gan_Matrix *A, gan_ui32 magic_number );
+Gan_Bool gan_mat_fwrite ( FILE *fp, const Gan_Matrix *A, gan_uint32 magic_number );
 Gan_Matrix *gan_mat_fread_q ( FILE *fp, Gan_Matrix *A,
-                              gan_ui32 *magic_number );
+                              gan_uint32 *magic_number );
 
 #ifdef GAN_GENERATE_DOCUMENTATION
 #define Gan_SquMatrix struct Gan_SquMatrix
@@ -177,7 +178,7 @@ Gan_Matrix *gan_mat_form ( Gan_Matrix *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix *gan_mat_form_data ( Gan_Matrix *A,
                                 unsigned long rows, unsigned long cols,
-                                void *data, size_t data_size );
+                                double *data, size_t data_size );
 #else
 #define gan_mat_form_data(A,rows,cols,data,data_size)\
             gan_mat_form_gen(A,rows,cols,data,data_size)
@@ -221,7 +222,7 @@ Gan_Bool gan_mat_set_el ( Gan_Matrix *A, unsigned i, unsigned j, double value);
  * Return element \a i,\a j of generic matrix, or \c DBL_MAX on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_mat_get_el ( Gan_Matrix *A, unsigned i, unsigned j );
+double gan_mat_get_el ( const Gan_Matrix *A, unsigned i, unsigned j );
 #else
 #ifdef NDEBUG
 #define gan_mat_get_el(A,i,j) (A)->data[(j)*(A)->rows+(i)]
@@ -299,7 +300,7 @@ Gan_Bool gan_mat_dec_el ( Gan_Matrix *A, unsigned i, unsigned j, double value);
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_mat_print ( Gan_Matrix *A,
+Gan_Bool gan_mat_print ( const Gan_Matrix *A,
                          const char *prefix, int indent, const char *fmt );
 #else
 #define gan_mat_print(A,p,i,f) (gan_mat_fprint(stdout,A,p,i,f))
@@ -382,7 +383,7 @@ Gan_Matrix *gan_mat_fill_zero_s ( unsigned long rows, unsigned cols );
  * Allocate and return a copy of matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_copy_s ( Gan_Matrix *A );
+Gan_Matrix *gan_mat_copy_s ( const Gan_Matrix *A );
 #else
 #define gan_mat_copy_s(A) (gan_mat_copy_q(A,NULL))
 #endif
@@ -618,7 +619,7 @@ Gan_Matrix *gan_mat_incrementT ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_add_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_mat_add_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                    Gan_SquMatrix *C );
 #else
 #define gan_mat_add_sym_q(A,B,C) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -632,7 +633,7 @@ Gan_SquMatrix *gan_mat_add_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_add_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_mat_add_sym_s ( const Gan_Matrix *A, Gan_Matrix *B );
 #else
 #define gan_mat_add_sym_s(A,B) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -645,7 +646,7 @@ Gan_SquMatrix *gan_mat_add_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_addT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_mat_addT_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                     Gan_SquMatrix *C );
 #else
 #define gan_mat_addT_sym_q(A,B,C) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -659,7 +660,7 @@ Gan_SquMatrix *gan_mat_addT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_addT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_mat_addT_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_addT_sym_s(A,B) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -672,7 +673,7 @@ Gan_SquMatrix *gan_mat_addT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_add_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_matT_add_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                     Gan_SquMatrix *C );
 #else
 #define gan_matT_add_sym_q(A,B,C) (gan_mat_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -686,7 +687,7 @@ Gan_SquMatrix *gan_matT_add_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_add_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_matT_add_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_add_sym_s(A,B) (gan_mat_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -713,7 +714,7 @@ Gan_SquMatrix *gan_matT_addT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_addT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_matT_addT_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_addT_sym_s(A,B) (gan_mat_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -725,7 +726,7 @@ Gan_SquMatrix *gan_matT_addT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * square symmetric matrix \f$ B = A + A^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_saddT_sym_q ( Gan_Matrix *A, Gan_SquMatrix *B );
+Gan_SquMatrix *gan_mat_saddT_sym_q ( const Gan_Matrix *A, const Gan_SquMatrix *B );
 #else
 #define gan_mat_saddT_sym_q(A,B) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,B))
 #endif
@@ -737,7 +738,7 @@ Gan_SquMatrix *gan_mat_saddT_sym_q ( Gan_Matrix *A, Gan_SquMatrix *B );
  * a new square symmetric matrix \f$ A + A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_saddT_sym_s ( Gan_Matrix *A );
+Gan_SquMatrix *gan_mat_saddT_sym_s ( const Gan_Matrix *A );
 #else
 #define gan_mat_saddT_sym_s(A) (gan_mat_add_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,NULL))
 #endif
@@ -749,7 +750,7 @@ Gan_SquMatrix *gan_mat_saddT_sym_s ( Gan_Matrix *A );
  * square symmetric matrix \f$ B = A^{\top} + A \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_sadd_sym_q ( Gan_Matrix *A, Gan_SquMatrix *B );
+Gan_SquMatrix *gan_matT_sadd_sym_q ( const Gan_Matrix *A, Gan_SquMatrix *B );
 #else
 #define gan_matT_sadd_sym_q(A,B) (gan_mat_add_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,B))
 #endif
@@ -761,7 +762,7 @@ Gan_SquMatrix *gan_matT_sadd_sym_q ( Gan_Matrix *A, Gan_SquMatrix *B );
  * a new square symmetric matrix \f$ A^{\top} + A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_sadd_sym_s ( Gan_Matrix *A );
+Gan_SquMatrix *gan_matT_sadd_sym_s ( const Gan_Matrix *A );
 #else
 #define gan_matT_sadd_sym_s(A) (gan_mat_add_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -888,7 +889,7 @@ Gan_Matrix *gan_mat_decrementT ( Gan_Matrix *A, Gan_Matrix *B );
  * \f$ y = A x \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_mat_multv_q ( Gan_Matrix *A, Gan_Vector *x,
+Gan_Vector *gan_mat_multv_q ( const Gan_Matrix *A, const Gan_Vector *x,
                               Gan_Vector *y );
 #else
 #define gan_mat_multv_q(A,x,y) (gan_mat_multv_gen(A,GAN_NOTRANSPOSE,x,y))
@@ -901,7 +902,7 @@ Gan_Vector *gan_mat_multv_q ( Gan_Matrix *A, Gan_Vector *x,
  * result \f$ A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_mat_multv_s ( Gan_Matrix *A, Gan_Vector *x );
+Gan_Vector *gan_mat_multv_s ( const Gan_Matrix *A, const Gan_Vector *x );
 #else
 #define gan_mat_multv_s(A,x) (gan_mat_multv_gen(A,GAN_NOTRANSPOSE,x,NULL))
 #endif
@@ -913,7 +914,7 @@ Gan_Vector *gan_mat_multv_s ( Gan_Matrix *A, Gan_Vector *x );
  * \f$ y = A^{\top} x \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_matT_multv_q ( Gan_Matrix *A, Gan_Vector *x,
+Gan_Vector *gan_matT_multv_q ( const Gan_Matrix *A, const Gan_Vector *x,
                                Gan_Vector *y );
 #else
 #define gan_matT_multv_q(A,x,y) (gan_mat_multv_gen(A,GAN_TRANSPOSE,x,y))
@@ -926,7 +927,7 @@ Gan_Vector *gan_matT_multv_q ( Gan_Matrix *A, Gan_Vector *x,
  * returning the result \f$ A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_matT_multv_s ( Gan_Matrix *A, Gan_Vector *x );
+Gan_Vector *gan_matT_multv_s ( const Gan_Matrix *A, const Gan_Vector *x );
 #else
 #define gan_matT_multv_s(A,x) (gan_mat_multv_gen(A,GAN_TRANSPOSE,x,NULL))
 #endif
@@ -947,7 +948,7 @@ Gan_Vector *gan_matT_multv_s ( Gan_Matrix *A, Gan_Vector *x );
  * \f$ C = A B \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_rmult_q ( Gan_Matrix *A, Gan_Matrix *B, Gan_Matrix *C );
+Gan_Matrix *gan_mat_rmult_q ( const Gan_Matrix *A, const Gan_Matrix *B, Gan_Matrix *C );
 #else
 #define gan_mat_rmult_q(A,B,C) (gan_mat_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
 #endif
@@ -959,7 +960,7 @@ Gan_Matrix *gan_mat_rmult_q ( Gan_Matrix *A, Gan_Matrix *B, Gan_Matrix *C );
  * \f$ A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_rmult_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_mat_rmult_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_rmult_s(A,B) (gan_mat_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -971,7 +972,7 @@ Gan_Matrix *gan_mat_rmult_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \f$ C = A B^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_rmultT_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_mat_rmultT_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                Gan_Matrix *C );
 #else
 #define gan_mat_rmultT_q(A,B,C) (gan_mat_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -984,7 +985,7 @@ Gan_Matrix *gan_mat_rmultT_q ( Gan_Matrix *A, Gan_Matrix *B,
  * result as a new matrix \f$ A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_rmultT_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_mat_rmultT_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_rmultT_s(A,B) (gan_mat_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -996,7 +997,7 @@ Gan_Matrix *gan_mat_rmultT_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \f$ C = A^{\top} B \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_matT_rmult_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_matT_rmult_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                Gan_Matrix *C );
 #else
 #define gan_matT_rmult_q(A,B,C) (gan_mat_rmult_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1009,7 +1010,7 @@ Gan_Matrix *gan_matT_rmult_q ( Gan_Matrix *A, Gan_Matrix *B,
  * result as a new matrix \f$ A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_matT_rmult_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_matT_rmult_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_rmult_s(A,B) (gan_mat_rmult_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1021,7 +1022,7 @@ Gan_Matrix *gan_matT_rmult_s ( Gan_Matrix *A, Gan_Matrix *B );
  * and write result into \f$ C = A^{\top} B^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_matT_rmultT_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_matT_rmultT_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                 Gan_Matrix *C );
 #else
 #define gan_matT_rmultT_q(A,B,C) (gan_mat_rmult_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1034,7 +1035,7 @@ Gan_Matrix *gan_matT_rmultT_q ( Gan_Matrix *A, Gan_Matrix *B,
  * and return the result as a new matrix \f$ A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_matT_rmultT_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_matT_rmultT_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_rmultT_s(A,B) (gan_mat_rmult_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1047,7 +1048,7 @@ Gan_Matrix *gan_matT_rmultT_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_rmult_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_mat_rmult_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                      Gan_SquMatrix *C );
 #else
 #define gan_mat_rmult_sym_q(A,B,C) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1061,7 +1062,7 @@ Gan_SquMatrix *gan_mat_rmult_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_rmult_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_mat_rmult_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_rmult_sym_s(A,B) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1074,7 +1075,7 @@ Gan_SquMatrix *gan_mat_rmult_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_rmultT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_mat_rmultT_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                       Gan_SquMatrix *C );
 #else
 #define gan_mat_rmultT_sym_q(A,B,C) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1088,7 +1089,7 @@ Gan_SquMatrix *gan_mat_rmultT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_rmultT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_mat_rmultT_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_rmultT_sym_s(A,B) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1101,7 +1102,7 @@ Gan_SquMatrix *gan_mat_rmultT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_rmult_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_matT_rmult_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                       Gan_SquMatrix *C );
 #else
 #define gan_matT_rmult_sym_q(A,B,C) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1115,7 +1116,7 @@ Gan_SquMatrix *gan_matT_rmult_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_rmult_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_matT_rmult_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_rmult_sym_s(A,B) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1128,7 +1129,7 @@ Gan_SquMatrix *gan_matT_rmult_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_rmultT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
+Gan_SquMatrix *gan_matT_rmultT_sym_q ( const Gan_Matrix *A, const Gan_Matrix *B,
                                        Gan_SquMatrix *C );
 #else
 #define gan_matT_rmultT_sym_q(A,B,C) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1142,7 +1143,7 @@ Gan_SquMatrix *gan_matT_rmultT_sym_q ( Gan_Matrix *A, Gan_Matrix *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_matT_rmultT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_SquMatrix *gan_matT_rmultT_sym_s ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_matT_rmultT_sym_s(A,B) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1154,7 +1155,7 @@ Gan_SquMatrix *gan_matT_rmultT_sym_s ( Gan_Matrix *A, Gan_Matrix *B );
  * square symmetric matrix \f$ B = A A^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_srmultT_q ( Gan_Matrix *A, Gan_SquMatrix *B );
+Gan_SquMatrix *gan_mat_srmultT_q ( const Gan_Matrix *A, const Gan_SquMatrix *B );
 #else
 #define gan_mat_srmultT_q(A,B) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,B))
 #endif
@@ -1166,7 +1167,7 @@ Gan_SquMatrix *gan_mat_srmultT_q ( Gan_Matrix *A, Gan_SquMatrix *B );
  * a new square symmetric matrix \f$ A A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_srmultT_s ( Gan_Matrix *A );
+Gan_SquMatrix *gan_mat_srmultT_s ( const Gan_Matrix *A );
 #else
 #define gan_mat_srmultT_s(A) (gan_mat_rmult_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,NULL))
 #endif
@@ -1178,7 +1179,7 @@ Gan_SquMatrix *gan_mat_srmultT_s ( Gan_Matrix *A );
  * square symmetric matrix \f$ B = A^{\top} A \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_slmultT_q ( Gan_Matrix *A, Gan_SquMatrix *B );
+Gan_SquMatrix *gan_mat_slmultT_q ( const Gan_Matrix *A, Gan_SquMatrix *B );
 #else
 #define gan_mat_slmultT_q(A,B) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,B))
 #endif
@@ -1190,7 +1191,7 @@ Gan_SquMatrix *gan_mat_slmultT_q ( Gan_Matrix *A, Gan_SquMatrix *B );
  * a new square symmetric matrix \f$ A^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_mat_slmultT_s ( Gan_Matrix *A );
+Gan_SquMatrix *gan_mat_slmultT_s ( const Gan_Matrix *A );
 #else
 #define gan_mat_slmultT_s(A) (gan_mat_rmult_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1218,7 +1219,7 @@ Gan_SquMatrix *gan_mat_slmultT_s ( Gan_Matrix *A );
  * \sa gan_mat_invert_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_invert_s ( Gan_Matrix *A );
+Gan_Matrix *gan_mat_invert_s ( const Gan_Matrix *A );
 #else
 #define gan_mat_invert_s(A) gan_mat_invert_q(A,NULL)
 #endif
@@ -1241,7 +1242,7 @@ Gan_Matrix *gan_mat_invert_s ( Gan_Matrix *A );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_mat_same_dims ( Gan_Matrix *A, Gan_Matrix *B );
+Gan_Bool gan_mat_same_dims ( const Gan_Matrix *A, const Gan_Matrix *B );
 #else
 #define gan_mat_same_dims(A,B) ((A)->rows == (B)->rows && (A)->cols == (B)->cols)
 #endif
@@ -1266,7 +1267,7 @@ Gan_Bool gan_mat_same_dims ( Gan_Matrix *A, Gan_Matrix *B );
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix *gan_mat_insert ( Gan_Matrix *A, unsigned long rA, unsigned long cA,
-                             Gan_Matrix *B,
+                             const Gan_Matrix *B,
                              unsigned long rB,   unsigned long cB,
                              unsigned long rows, unsigned long cols );
 #else
@@ -1285,7 +1286,7 @@ Gan_Matrix *gan_mat_insert ( Gan_Matrix *A, unsigned long rA, unsigned long cA,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix *gan_mat_insertT ( Gan_Matrix *A,
                               unsigned long rA, unsigned long cA,
-                              Gan_Matrix *B,
+                              const Gan_Matrix *B,
                               unsigned long rB,   unsigned long cB,
                               unsigned long rows, unsigned long cols );
 #else
@@ -1302,7 +1303,7 @@ Gan_Matrix *gan_mat_insertT ( Gan_Matrix *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix *gan_mat_insertv ( Gan_Matrix *A,
                               unsigned long rA, unsigned long cA,
-                              Gan_Vector *x, unsigned long rx,
+                              const Gan_Vector *x, unsigned long rx,
                               unsigned long rows );
 #else
 #define gan_mat_insertv(A,rA,cA,x,rx,rows) (gan_mat_insertv_gen(A,rA,cA,x,GAN_NOTRANSPOSE,rx,rows))
@@ -1321,7 +1322,7 @@ Gan_Matrix *gan_mat_insertv ( Gan_Matrix *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix *gan_mat_insertvT ( Gan_Matrix *A,
                                unsigned long rA, unsigned long cA,
-                               Gan_Vector *x, unsigned long rx,
+                               const Gan_Vector *x, unsigned long rx,
                                unsigned long cols );
 #else
 #define gan_mat_insertvT(A,rA,cA,x,rx,cols) (gan_mat_insertv_gen(A,rA,cA,x,GAN_TRANSPOSE,rx,cols))
@@ -1343,7 +1344,7 @@ Gan_Matrix *gan_mat_insertvT ( Gan_Matrix *A,
  * \a cols, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_mat_test_dims ( Gan_Matrix *A, unsigned long rows,
+Gan_Bool gan_mat_test_dims ( const Gan_Matrix *A, unsigned long rows,
                                             unsigned long cols );
 #else
 #define gan_mat_test_dims(A,rs,cs) (((A)->rows==(rs))&&((A)->cols==(cs)))
@@ -1395,7 +1396,7 @@ size_t gan_mat_data_size ( unsigned long rows, unsigned long cols );
  * \sa gan_mat_extract_q(), gan_mat_insert().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_extract_s ( Gan_Matrix *A,
+Gan_Matrix *gan_mat_extract_s ( const Gan_Matrix *A,
                                 unsigned long r0,   unsigned long c0,
                                 unsigned long rows, unsigned long cols );
 #else
@@ -1441,7 +1442,7 @@ Gan_Matrix *gan_mat_fscanf_s ( FILE *fp, const char *prefix, int prefix_len );
  * \sa gan_mat_fread_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_Matrix *gan_mat_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_mat_fread_s(fp,ml) gan_mat_fread_q(fp,NULL,m)
 #endif
@@ -1463,32 +1464,32 @@ Gan_Bool gan_mat_realloc_gen ( Gan_Matrix *A,
 Gan_Matrix *gan_mat_add_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
                               Gan_Matrix *B, Gan_TposeFlag B_tr,
                               Gan_Matrix *C );
-struct Gan_SquMatrix *gan_mat_add_sym_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
-                                            Gan_Matrix *B, Gan_TposeFlag B_tr,
+struct Gan_SquMatrix *gan_mat_add_sym_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                                            const Gan_Matrix *B, Gan_TposeFlag B_tr,
                                             struct Gan_SquMatrix *C );
 Gan_Matrix *gan_mat_sub_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
                               Gan_Matrix *B, Gan_TposeFlag B_tr,
                               Gan_Matrix *C );
-Gan_Vector *gan_mat_multv_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
-                                Gan_Vector *x, Gan_Vector *y );
-Gan_Matrix *gan_mat_rmult_gen ( Gan_Matrix *A, Gan_TposeFlag A_tr,
-                                Gan_Matrix *B, Gan_TposeFlag B_tr,
+Gan_Vector *gan_mat_multv_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                                const Gan_Vector *x, Gan_Vector *y );
+Gan_Matrix *gan_mat_rmult_gen ( const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                                const Gan_Matrix *B, Gan_TposeFlag B_tr,
                                 Gan_Matrix *C );
-struct Gan_SquMatrix *gan_mat_rmult_sym_gen (Gan_Matrix *A, Gan_TposeFlag A_tr,
-                                             Gan_Matrix *B, Gan_TposeFlag B_tr,
+struct Gan_SquMatrix *gan_mat_rmult_sym_gen (const Gan_Matrix *A, Gan_TposeFlag A_tr,
+                                             const Gan_Matrix *B, Gan_TposeFlag B_tr,
                                              struct Gan_SquMatrix *C );
 Gan_Matrix *gan_mat_insert_gen ( Gan_Matrix *A,
                                  unsigned long rA, unsigned long cA,
-                                 Gan_Matrix *B, Gan_TposeFlag B_tr,
+                                 const Gan_Matrix *B, Gan_TposeFlag B_tr,
                                  unsigned long rB, unsigned long cB,
                                  unsigned long rows , unsigned long cols );
 Gan_Matrix *gan_mat_insertv_gen ( Gan_Matrix *A,
                                   unsigned long rA, unsigned long cA,
-                                  struct Gan_Vector *x, Gan_TposeFlag x_tr,
+                                  const struct Gan_Vector *x, Gan_TposeFlag x_tr,
                                   unsigned long rx, unsigned long size );
 
 #ifndef NDEBUG
-void gan_mat_db ( Gan_Matrix *A ); /* for use in a debugger */
+void gan_mat_db ( const Gan_Matrix *A ); /* for use in a debugger */
 #endif
 
 #define gan_mat_realloc(A,rows,cols) gan_mat_realloc_gen(A,rows,cols)
