@@ -3,8 +3,8 @@
  * Module:        Scaled identity matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.24 $
- * Last edited:   $Date: 2002/04/22 13:42:22 $
+ * Revision:      $Revision: 1.25 $
+ * Last edited:   $Date: 2004/03/18 18:26:25 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -61,7 +61,7 @@ static Gan_Bool
 
 /* function to return element i,j of multiple of identity */
 static double
- scaledI_matrix_get_el ( Gan_SquMatrix *A, unsigned i, unsigned j )
+ scaledI_matrix_get_el ( const Gan_SquMatrix *A, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( A->type == GAN_SCALED_IDENT_MATRIX );
@@ -97,7 +97,7 @@ static Gan_Bool
 
 /* function to print multiple of identity to file pointer */
 static Gan_Bool
- scaledI_matrix_fprint ( FILE *fp, Gan_SquMatrix *A, const char *prefix,
+ scaledI_matrix_fprint ( FILE *fp, const Gan_SquMatrix *A, const char *prefix,
                          unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -131,7 +131,7 @@ static Gan_Bool
 
 /* function to copy multiple of identity A into another matrix B */
 static Gan_SquMatrix *
- scaledI_matrix_copy ( Gan_SquMatrix *A, Gan_SquMatrix *B )
+ scaledI_matrix_copy ( const Gan_SquMatrix *A, Gan_SquMatrix *B )
 {
    /* consistency check */
    assert ( A->type == GAN_SCALED_IDENT_MATRIX );
@@ -231,7 +231,7 @@ static Gan_SquMatrix *
 /* function to compute matrix/vector product A*x = y, returning y */
 static Gan_Vector *
  scaledI_matrix_multv (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector *x, Gan_Vector *y )
 {
    long i;
@@ -287,7 +287,7 @@ static Gan_Vector *
  */
 static Gan_Matrix *
  scaledI_matrix_rmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
@@ -379,8 +379,8 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  scaledI_matrix_lmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    unsigned long i, j;
@@ -531,7 +531,7 @@ static Gan_SquMatrix *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix *
- scaledI_matrix_lrmult ( Gan_SquMatrix *A, Gan_Matrix *B, Gan_TposeFlag B_tr,
+ scaledI_matrix_lrmult ( const Gan_SquMatrix *A, const Gan_Matrix *B, Gan_TposeFlag B_tr,
                          Gan_Matrix *C, Gan_SquMatrix *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -565,8 +565,8 @@ static Gan_SquMatrix *
       D = gan_symmat_set_size ( D, Dsize );
 
    /* perform first multiplication step A*B(^T) */
-   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT, B,
-                     (Gan_TposeFlag) !B_tr, C );
+   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
+                     (Gan_Matrix *)B, (Gan_TposeFlag) !B_tr, C );
 
    /* perform second multiplication step B*(A*B^T) or B^T*(A*B) */
    gan_mat_rmult_sym_gen ( B, B_tr, C, GAN_NOTRANSPOSE, D );
@@ -640,7 +640,7 @@ static Gan_SquMatrix *
 /* Compute squared Frobenius norm of multiple of identity matrix A.
  */
 static double
- scaledI_matrix_sumsqr ( Gan_SquMatrix *A )
+ scaledI_matrix_sumsqr ( const Gan_SquMatrix *A )
 {
    return ((double)A->size)*gan_sqr(A->data[0]);
 }
@@ -648,14 +648,14 @@ static double
 /* Compute Frobenius norm of multiple of identity matrix A.
  */
 static double
- scaledI_matrix_Fnorm ( Gan_SquMatrix *A )
+ scaledI_matrix_Fnorm ( const Gan_SquMatrix *A )
 {
    return sqrt(((double)A->size)*gan_sqr(A->data[0]));
 }
 
 /* Convert multiple of identity to generic matrix */
 static Gan_Matrix *
- scaledI_matrix_matrix ( Gan_SquMatrix *A, struct Gan_Matrix *B )
+ scaledI_matrix_matrix ( const Gan_SquMatrix *A, struct Gan_Matrix *B )
 {
    long i;
    double Aval = A->data[0];
@@ -835,7 +835,7 @@ Gan_SquMatrix *
  * The \a size argument must match the size of the matrix.
  */
 Gan_Bool
- gan_scalImat_read_va ( Gan_SquMatrix *A, unsigned long size, ... )
+ gan_scalImat_read_va ( const Gan_SquMatrix *A, unsigned long size, ... )
 {
    va_list ap;
 
