@@ -3,8 +3,8 @@
  * Module:        Square matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.25 $
- * Last edited:   $Date: 2002/04/22 13:42:22 $
+ * Revision:      $Revision: 1.27 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -685,7 +685,7 @@ Gan_SquMatrix *
  * \sa gan_squmatf_fread_q().
  */
 Gan_Bool
- gan_squmat_fwrite ( FILE *fp, Gan_SquMatrix *A, gan_ui32 magic_number )
+ gan_squmat_fwrite ( FILE *fp, const Gan_SquMatrix *A, gan_uint32 magic_number )
 {
    char type_char;
 
@@ -718,8 +718,8 @@ Gan_Bool
    }
 
    if ( !gan_fwrite_lendian_ul ( fp, &A->size, 1 ) ||
-        !gan_fwrite_lendian_d ( fp, A->data,
-                                gan_squmat_data_size ( A->type, A->size ) ) )
+        !gan_fwrite_lendian_f64 ( fp, A->data,
+                                  gan_squmat_data_size ( A->type, A->size ) ) )
    {
       gan_err_register ( "gan_squmat_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -739,7 +739,7 @@ Gan_Bool
  * \sa gan_squmatf_fread_s().
  */
 Gan_SquMatrix *
- gan_squmat_fread_q ( FILE *fp, Gan_SquMatrix *A, gan_ui32 *magic_number )
+ gan_squmat_fread_q ( FILE *fp, Gan_SquMatrix *A, gan_uint32 *magic_number )
 {
    unsigned long size;
    char type_char;
@@ -795,8 +795,8 @@ Gan_SquMatrix *
    }
 
    /* read matrix data */
-   if ( !gan_fread_lendian_d ( fp, A->data,
-                               gan_squmat_data_size ( type, size ) ) )
+   if ( !gan_fread_lendian_f64 ( fp, A->data,
+                                 gan_squmat_data_size ( type, size ) ) )
    {
       gan_err_register ( "gan_squmat_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -862,7 +862,7 @@ Gan_SquMatrix *
 /* for use in a debugger: use gan_squmat_print() or gan_squmat_fprint in
  * your program!
  */
-void gan_squmat_db ( Gan_SquMatrix *A )
+void gan_squmat_db ( const Gan_SquMatrix *A )
 {
    gan_squmat_print ( A, "", 0, "%+g" );
 }
