@@ -3,8 +3,8 @@
  * Module:        Square matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.26 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.28 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -60,14 +60,14 @@ typedef struct Gan_SquMatrixFuncs
 {
    Gan_Bool (*set_el) ( struct Gan_SquMatrix *A, unsigned i, unsigned j,
                         double value );
-   double (*get_el) ( struct Gan_SquMatrix *A, unsigned i, unsigned j );
+   double (*get_el) ( const struct Gan_SquMatrix *A, unsigned i, unsigned j );
    Gan_Bool (*inc_el) ( struct Gan_SquMatrix *A, unsigned i, unsigned j,
                         double value );
    Gan_Bool (*dec_el) ( struct Gan_SquMatrix *A, unsigned i, unsigned j,
                         double value );
-   Gan_Bool (*fprint) ( FILE *fp, struct Gan_SquMatrix *A, const char *prefix,
+   Gan_Bool (*fprint) ( FILE *fp, const struct Gan_SquMatrix *A, const char *prefix,
                         unsigned indent, const char *fmt );
-   struct Gan_SquMatrix *(*copy) ( struct Gan_SquMatrix *A,
+   struct Gan_SquMatrix *(*copy) ( const struct Gan_SquMatrix *A,
                                    struct Gan_SquMatrix *B );
    struct Gan_SquMatrix *(*scale) ( struct Gan_SquMatrix *A, double a,
                                     struct Gan_SquMatrix *B );
@@ -77,14 +77,14 @@ typedef struct Gan_SquMatrixFuncs
    struct Gan_SquMatrix *(*sub) ( struct Gan_SquMatrix *A, Gan_TposeFlag A_tr,
                                   struct Gan_SquMatrix *B, Gan_TposeFlag B_tr,
                                   struct Gan_SquMatrix *C );
-   struct Gan_Vector *(*multv) ( struct Gan_SquMatrix *A,
+   struct Gan_Vector *(*multv) ( const struct Gan_SquMatrix *A,
                                  Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
                                  struct Gan_Vector *x, struct Gan_Vector *y );
-   struct Gan_Matrix *(*rmult) ( struct Gan_SquMatrix *A,
+   struct Gan_Matrix *(*rmult) ( const struct Gan_SquMatrix *A,
                                  Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
                                  struct Gan_Matrix *B, Gan_TposeFlag B_tr,
                                  struct Gan_Matrix *C );
-   struct Gan_Matrix *(*lmult) ( struct Gan_SquMatrix *A,
+   struct Gan_Matrix *(*lmult) ( const struct Gan_SquMatrix *A,
                                  Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
                                  struct Gan_Matrix *B, Gan_TposeFlag B_tr,
                                  struct Gan_Matrix *C );
@@ -95,8 +95,8 @@ typedef struct Gan_SquMatrixFuncs
                                         Gan_TposeFlag B_tr,
                                         Gan_InvertFlag B_inv,
                                         struct Gan_SquMatrix *C );
-   struct Gan_SquMatrix *(*lrmult) ( struct Gan_SquMatrix *A,
-                                     struct Gan_Matrix *B, Gan_TposeFlag B_tr,
+   struct Gan_SquMatrix *(*lrmult) ( const struct Gan_SquMatrix *A,
+                                     const struct Gan_Matrix *B, Gan_TposeFlag B_tr,
                                      struct Gan_Matrix *C,
                                      struct Gan_SquMatrix *D );
    struct Gan_SquMatrix *(*invert) ( struct Gan_SquMatrix *A,
@@ -104,9 +104,9 @@ typedef struct Gan_SquMatrixFuncs
    struct Gan_SquMatrix *(*cholesky) ( struct Gan_SquMatrix *S,
                                        struct Gan_SquMatrix *L,
                                        int *error_code );
-   double (*sumsqr) ( struct Gan_SquMatrix *A );
-   double (*Fnorm)  ( struct Gan_SquMatrix *A );
-   struct Gan_Matrix *(*matrix) ( struct Gan_SquMatrix *A,
+   double (*sumsqr) ( const struct Gan_SquMatrix *A );
+   double (*Fnorm)  ( const struct Gan_SquMatrix *A );
+   struct Gan_Matrix *(*matrix) ( const struct Gan_SquMatrix *A,
                                   struct Gan_Matrix *B );
 } Gan_SquMatrixFuncs;
 
@@ -228,7 +228,7 @@ Gan_Bool gan_squmat_set_el ( Gan_SquMatrix *A, unsigned row, unsigned col,
  * failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_squmat_get_el ( Gan_SquMatrix *A, unsigned row, unsigned col );
+double gan_squmat_get_el ( const Gan_SquMatrix *A, unsigned row, unsigned col );
 #else
 #define gan_squmat_get_el(A,r,c) ((A)->funcs->get_el(A,r,c))
 #endif
@@ -276,7 +276,7 @@ Gan_Bool gan_squmat_dec_el ( Gan_SquMatrix *A, unsigned row, unsigned col,
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_fprint ( FILE *fp, Gan_SquMatrix *A,
+Gan_Bool gan_squmat_fprint ( FILE *fp, const Gan_SquMatrix *A,
                              const char *prefix, int indent, const char *fmt );
 #else
 #define gan_squmat_fprint(fp,A,p,i,f) ((A)->funcs->fprint(fp,A,p,i,f))
@@ -290,7 +290,7 @@ Gan_Bool gan_squmat_fprint ( FILE *fp, Gan_SquMatrix *A,
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_print ( Gan_SquMatrix *A,
+Gan_Bool gan_squmat_print ( const Gan_SquMatrix *A,
                             const char *prefix, int indent, const char *fmt );
 #else
 #define gan_squmat_print(A,p,i,f) ((A)->funcs->fprint(stdout,A,p,i,f))
@@ -378,7 +378,7 @@ Gan_SquMatrix *
  * Copy square matrix \a A to square matrix \a B, and return \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_squmat_copy_q ( Gan_SquMatrix *A, Gan_SquMatrix *B );
+Gan_SquMatrix *gan_squmat_copy_q ( const Gan_SquMatrix *A, Gan_SquMatrix *B );
 #else
 #define gan_squmat_copy_q(A,B) ((A)->funcs->copy(A,B))
 #endif
@@ -389,7 +389,7 @@ Gan_SquMatrix *gan_squmat_copy_q ( Gan_SquMatrix *A, Gan_SquMatrix *B );
  * Allocate and return a copy of square matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_squmat_copy_s ( Gan_SquMatrix *A );
+Gan_SquMatrix *gan_squmat_copy_s ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_copy_s(A) ((A)->funcs->copy(A,NULL))
 #endif
@@ -763,7 +763,7 @@ Gan_SquMatrix *gan_squmat_decrementT ( Gan_SquMatrix *A, Gan_SquMatrix *B );
  * \f$ y = A * x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmat_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
+Gan_Vector *gan_squmat_multv_q ( const Gan_SquMatrix *A, Gan_Vector *x,
                                  Gan_Vector *y );
 #else
 #define gan_squmat_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,y))
@@ -777,7 +777,7 @@ Gan_Vector *gan_squmat_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
  * result \f$ A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmat_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmat_multv_s ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmat_multv_s(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,NULL))
 #endif
@@ -790,7 +790,7 @@ Gan_Vector *gan_squmat_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
  * the result \f$ x \leftarrow A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmat_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmat_multv_i ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmat_multv_i(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,x))
 #endif
@@ -803,7 +803,7 @@ Gan_Vector *gan_squmat_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
  * into \f$ y = A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatT_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
+Gan_Vector *gan_squmatT_multv_q ( const Gan_SquMatrix *A, Gan_Vector *x,
                                   Gan_Vector *y );
 #else
 #define gan_squmatT_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,y))
@@ -816,7 +816,7 @@ Gan_Vector *gan_squmatT_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
  * returning the result \f$ A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatT_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatT_multv_s ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatT_multv_s(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,NULL))
 #endif
@@ -829,7 +829,7 @@ Gan_Vector *gan_squmatT_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
  * \a x with the result \f$ x \leftarrow A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatT_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatT_multv_i ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatT_multv_i(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,x))
 #endif
@@ -842,7 +842,7 @@ Gan_Vector *gan_squmatT_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
  * into \f$ y = A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatI_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
+Gan_Vector *gan_squmatI_multv_q ( const Gan_SquMatrix *A, Gan_Vector *x,
                                   Gan_Vector *y );
 #else
 #define gan_squmatI_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,y))
@@ -856,7 +856,7 @@ Gan_Vector *gan_squmatI_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
  * returning the result \f$ A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatI_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatI_multv_s ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatI_multv_s(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,NULL))
 #endif
@@ -869,7 +869,7 @@ Gan_Vector *gan_squmatI_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
  * \a x with the result \f$ x \leftarrow A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatI_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatI_multv_i ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatI_multv_i(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,x))
 #endif
@@ -882,7 +882,7 @@ Gan_Vector *gan_squmatI_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
  * writing result into \f$ y = A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatIT_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
+Gan_Vector *gan_squmatIT_multv_q ( const Gan_SquMatrix *A, Gan_Vector *x,
                                    Gan_Vector *y );
 #else
 #define gan_squmatIT_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,y))
@@ -896,7 +896,7 @@ Gan_Vector *gan_squmatIT_multv_q ( Gan_SquMatrix *A, Gan_Vector *x,
  * creating and returning the result \f$ A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatIT_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatIT_multv_s ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatIT_multv_s(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,NULL))
 #endif
@@ -909,7 +909,7 @@ Gan_Vector *gan_squmatIT_multv_s ( Gan_SquMatrix *A, Gan_Vector *x );
  * overwriting \a x with the result \f$ x \leftarrow A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_squmatIT_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
+Gan_Vector *gan_squmatIT_multv_i ( const Gan_SquMatrix *A, Gan_Vector *x );
 #else
 #define gan_squmatIT_multv_i(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,x))
 #endif
@@ -931,7 +931,7 @@ Gan_Vector *gan_squmatIT_multv_i ( Gan_SquMatrix *A, Gan_Vector *x );
  * result into generic matrix \f$ C = A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmat_rmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                  Gan_Matrix *C );
 #else
 #define gan_squmat_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -945,7 +945,7 @@ Gan_Matrix *gan_squmat_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * as a new matrix \f$ A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_rmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -958,7 +958,7 @@ Gan_Matrix *gan_squmat_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * with the result: \f$ B \leftarrow A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_rmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -971,7 +971,7 @@ Gan_Matrix *gan_squmat_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * and write result into \f$ C = A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmat_rmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmat_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -985,7 +985,7 @@ Gan_Matrix *gan_squmat_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * and return the result as a new matrix \f$ A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_rmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -998,7 +998,7 @@ Gan_Matrix *gan_squmat_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * write result into \f$ C = A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatT_rmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmatT_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1012,7 +1012,7 @@ Gan_Matrix *gan_squmatT_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * return the result as a new matrix \f$ A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_rmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1025,7 +1025,7 @@ Gan_Matrix *gan_squmatT_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_rmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1038,7 +1038,7 @@ Gan_Matrix *gan_squmatT_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * matrix \a B and write result into \f$ C = A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatT_rmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatT_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1052,7 +1052,7 @@ Gan_Matrix *gan_squmatT_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * matrix \a B and return the result as a new matrix \f$ A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_rmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1065,7 +1065,7 @@ Gan_Matrix *gan_squmatT_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * and write result into generic matrix \f$ C = A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatI_rmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmatI_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1079,7 +1079,7 @@ Gan_Matrix *gan_squmatI_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * the result as a new matrix \f$ A^{-1} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_rmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1092,7 +1092,7 @@ Gan_Matrix *gan_squmatI_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{-1} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_rmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1105,7 +1105,7 @@ Gan_Matrix *gan_squmatI_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * \a B and write result into \f$ C = A^{-1} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatI_rmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatI_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1119,7 +1119,7 @@ Gan_Matrix *gan_squmatI_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \a B and return the result as a new matrix \f$ A^{-1} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_rmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1132,7 +1132,7 @@ Gan_Matrix *gan_squmatI_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * \a B and write result into \f$ C = A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatIT_rmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatIT_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1146,7 +1146,7 @@ Gan_Matrix *gan_squmatIT_rmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \a B and return the result as a new matrix \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_rmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1159,7 +1159,7 @@ Gan_Matrix *gan_squmatIT_rmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_rmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1172,7 +1172,7 @@ Gan_Matrix *gan_squmatIT_rmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * generic matrix \a B and write result into \f$ C = A^{-{\top}} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatIT_rmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                     Gan_Matrix *C );
 #else
 #define gan_squmatIT_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1187,7 +1187,7 @@ Gan_Matrix *gan_squmatIT_rmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \f$ A^{-{\top}} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_rmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1200,7 +1200,7 @@ Gan_Matrix *gan_squmatIT_rmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * result into generic matrix \f$ C = B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmat_lmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                  Gan_Matrix *C );
 #else
 #define gan_squmat_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1214,7 +1214,7 @@ Gan_Matrix *gan_squmat_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * as a new matrix \f$ B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_lmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1227,7 +1227,7 @@ Gan_Matrix *gan_squmat_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * with the result \f$ B \leftarrow B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_lmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1240,7 +1240,7 @@ Gan_Matrix *gan_squmat_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * and write result into \f$ C = B^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmat_lmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmat_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1254,7 +1254,7 @@ Gan_Matrix *gan_squmat_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * and return the result as a new matrix \f$ B^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmat_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmat_lmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmat_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1267,7 +1267,7 @@ Gan_Matrix *gan_squmat_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * write result into \f$ C = B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatT_lmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmatT_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1281,7 +1281,7 @@ Gan_Matrix *gan_squmatT_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * return the result as a new matrix \f$ B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_lmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1294,7 +1294,7 @@ Gan_Matrix *gan_squmatT_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ B \leftarrow B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_lmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1307,7 +1307,7 @@ Gan_Matrix *gan_squmatT_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * matrix \a A, and write result into \f$ C = B^{\top} A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatT_lmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatT_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1322,7 +1322,7 @@ Gan_Matrix *gan_squmatT_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \f$ B^{\top} A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatT_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatT_lmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatT_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1335,7 +1335,7 @@ Gan_Matrix *gan_squmatT_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * and write result into generic matrix \f$ C = B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatI_lmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                   Gan_Matrix *C );
 #else
 #define gan_squmatI_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1349,7 +1349,7 @@ Gan_Matrix *gan_squmatI_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * return the result as a new matrix \f$ B A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_lmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1362,7 +1362,7 @@ Gan_Matrix *gan_squmatI_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ B A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_lmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1375,7 +1375,7 @@ Gan_Matrix *gan_squmatI_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * matrix \a A, and write result into \f$ C = B^{\top} A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatI_lmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatI_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1389,7 +1389,7 @@ Gan_Matrix *gan_squmatI_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * matrix \a A, and return the result as a new matrix \f$ B^{\top} A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatI_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatI_lmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatI_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1402,7 +1402,7 @@ Gan_Matrix *gan_squmatI_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * matrix \a A, and write result into \f$ C = B A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatIT_lmult_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                    Gan_Matrix *C );
 #else
 #define gan_squmatIT_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1416,7 +1416,7 @@ Gan_Matrix *gan_squmatIT_lmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \a B and return the result as a new matrix \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_lmult_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1429,7 +1429,7 @@ Gan_Matrix *gan_squmatIT_lmult_s ( Gan_SquMatrix *A, Gan_Matrix *B );
  * overwriting \a B with the result \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_lmult_i ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1443,7 +1443,7 @@ Gan_Matrix *gan_squmatIT_lmult_i ( Gan_SquMatrix *A, Gan_Matrix *B );
  * \f$ C = B^{\top} A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
+Gan_Matrix *gan_squmatIT_lmultT_q ( const Gan_SquMatrix *A, Gan_Matrix *B,
                                     Gan_Matrix *C );
 #else
 #define gan_squmatIT_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1458,7 +1458,7 @@ Gan_Matrix *gan_squmatIT_lmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * \f$ B^{\top} A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_squmatIT_lmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_squmatIT_lmultT_s ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_squmatIT_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1641,8 +1641,8 @@ Gan_SquMatrix *gan_squmatIT_srmultI_squ_i ( Gan_SquMatrix *A );
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_symmat_lrmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
-                                     Gan_Matrix *C, Gan_SquMatrix *D );
+Gan_SquMatrix *gan_symmat_lrmult_q ( Gan_SquMatrix *A, const Gan_Matrix *B,
+                                     const Gan_Matrix *C, Gan_SquMatrix *D );
 #else
 #define gan_symmat_lrmult_q(A,B,C,D) (assert(gan_squmat_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_NOTRANSPOSE,C,D))
 #endif
@@ -1658,8 +1658,8 @@ Gan_SquMatrix *gan_symmat_lrmult_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_symmat_lrmult_s ( Gan_SquMatrix *A, Gan_Matrix *B,
-                                     Gan_Matrix *C );
+Gan_SquMatrix *gan_symmat_lrmult_s ( Gan_SquMatrix *A, const Gan_Matrix *B,
+                                     const Gan_Matrix *C );
 #else
 #define gan_symmat_lrmult_s(A,B,C) (assert(gan_squmat_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_NOTRANSPOSE,C,NULL))
 #endif
@@ -1675,8 +1675,8 @@ Gan_SquMatrix *gan_symmat_lrmult_s ( Gan_SquMatrix *A, Gan_Matrix *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_symmat_lrmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
-                                      Gan_Matrix *C, Gan_SquMatrix *D );
+Gan_SquMatrix *gan_symmat_lrmultT_q ( Gan_SquMatrix *A, const Gan_Matrix *B,
+                                      const Gan_Matrix *C, Gan_SquMatrix *D );
 #else
 #define gan_symmat_lrmultT_q(A,B,C,D) (assert(gan_squmat_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_TRANSPOSE,C,D))
 #endif
@@ -1692,8 +1692,8 @@ Gan_SquMatrix *gan_symmat_lrmultT_q ( Gan_SquMatrix *A, Gan_Matrix *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_symmat_lrmultT_s ( Gan_SquMatrix *A, Gan_Matrix *B,
-                                      Gan_Matrix *C );
+Gan_SquMatrix *gan_symmat_lrmultT_s ( Gan_SquMatrix *A, const Gan_Matrix *B,
+                                      const Gan_Matrix *C );
 #else
 #define gan_symmat_lrmultT_s(A,B,C) (assert(gan_squmat_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_TRANSPOSE,C,NULL))
 #endif
@@ -1811,7 +1811,7 @@ Gan_SquMatrix *gan_squmat_cholesky_i ( Gan_SquMatrix *A );
  * the squared elements of \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_squmat_sumsqr ( Gan_SquMatrix *A );
+double gan_squmat_sumsqr ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_sumsqr(A) ((A)->funcs->sumsqr(A))
 #endif
@@ -1822,7 +1822,7 @@ double gan_squmat_sumsqr ( Gan_SquMatrix *A );
  * Return Frobenius norm of square matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_squmat_Fnorm ( Gan_SquMatrix *A );
+double gan_squmat_Fnorm ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_Fnorm(A) ((A)->funcs->Fnorm(A))
 #endif
@@ -1845,7 +1845,7 @@ double gan_squmat_Fnorm ( Gan_SquMatrix *A );
  * \sa gan_mat_from_squmat_s().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix *B );
+Gan_Matrix *gan_mat_from_squmat_q ( const Gan_SquMatrix *A, Gan_Matrix *B );
 #else
 #define gan_mat_from_squmat_q(A,B) ((A)->funcs->matrix(A,B))
 #endif
@@ -1859,7 +1859,7 @@ Gan_Matrix *gan_mat_from_squmat_q ( Gan_SquMatrix *A, Gan_Matrix *B );
  * \sa gan_mat_from_squmat_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_mat_from_squmat_s ( Gan_SquMatrix *A );
+Gan_Matrix *gan_mat_from_squmat_s ( const Gan_SquMatrix *A );
 #else
 #define gan_mat_from_squmat_s(A) ((A)->funcs->matrix(A,NULL))
 #endif
@@ -1880,7 +1880,7 @@ Gan_Matrix *gan_mat_from_squmat_s ( Gan_SquMatrix *A );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_same_type ( Gan_SquMatrix *A, Gan_SquMatrix *B );
+Gan_Bool gan_squmat_same_type ( const Gan_SquMatrix *A, const Gan_SquMatrix *B );
 #else
 #define gan_squmat_same_type(A,B) ((A)->type == (B)->type)
 #endif
@@ -1892,7 +1892,7 @@ Gan_Bool gan_squmat_same_type ( Gan_SquMatrix *A, Gan_SquMatrix *B );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_same_size ( Gan_SquMatrix *A, Gan_SquMatrix *B );
+Gan_Bool gan_squmat_same_size ( const Gan_SquMatrix *A, const Gan_SquMatrix *B );
 #else
 #define gan_squmat_same_size(A,B) ((A)->size == (B)->size)
 #endif
@@ -1904,7 +1904,7 @@ Gan_Bool gan_squmat_same_size ( Gan_SquMatrix *A, Gan_SquMatrix *B );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_same_type_size ( Gan_SquMatrix *A, Gan_SquMatrix *B );
+Gan_Bool gan_squmat_same_type_size ( const Gan_SquMatrix *A, const Gan_SquMatrix *B );
 #else
 #define gan_squmat_same_type_size(A,B) ((A)->type == (B)->type && (A)->size == (B)->size)
 #endif
@@ -1963,7 +1963,7 @@ Gan_SquMatrix *gan_squmat_set_size ( Gan_SquMatrix *A, unsigned long size );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_test_size ( Gan_Matrix *A, unsigned long size );
+Gan_Bool gan_squmat_test_size ( const Gan_Matrix *A, unsigned long size );
 #else
 #define gan_squmat_test_size(A,s) ((A)->size==s)
 #endif
@@ -1975,7 +1975,7 @@ Gan_Bool gan_squmat_test_size ( Gan_Matrix *A, unsigned long size );
  * #GAN_SYMMETRIC_MATRIX, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_symmetric_type ( Gan_SquMatrix *A );
+Gan_Bool gan_squmat_symmetric_type ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_symmetric_type(A) ((A)->type==GAN_SYMMETRIC_MATRIX)
 #endif
@@ -1987,7 +1987,7 @@ Gan_Bool gan_squmat_symmetric_type ( Gan_SquMatrix *A );
  * #GAN_DIAGONAL_MATRIX, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_diagonal_type ( Gan_SquMatrix *A );
+Gan_Bool gan_squmat_diagonal_type ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_diagonal_type(A) ((A)->type==GAN_DIAGONAL_MATRIX)
 #endif
@@ -2001,7 +2001,7 @@ Gan_Bool gan_squmat_diagonal_type ( Gan_SquMatrix *A );
  * #GAN_FALSE is returned if \a A is not one of these faults.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_symmetric ( Gan_SquMatrix *A );
+Gan_Bool gan_squmat_symmetric ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_symmetric(A) ((A)->type==GAN_SYMMETRIC_MATRIX || (A)->type==GAN_DIAGONAL_MATRIX || (A)->type==GAN_SCALED_IDENT_MATRIX)
 #endif
@@ -2015,7 +2015,7 @@ Gan_Bool gan_squmat_symmetric ( Gan_SquMatrix *A );
  * #GAN_FALSE is returned if \a A is not one of these faults.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmat_diagonal ( Gan_SquMatrix *A );
+Gan_Bool gan_squmat_diagonal ( const Gan_SquMatrix *A );
 #else
 #define gan_squmat_diagonal(A) ((A)->type==GAN_DIAGONAL_MATRIX || (A)->type==GAN_SCALED_IDENT_MATRIX)
 #endif
@@ -2057,7 +2057,7 @@ Gan_SquMatrix *gan_squmat_fscanf_s ( FILE *fp,
  * \sa: gan_squmat_fread_q()
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_squmat_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_SquMatrix *gan_squmat_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_squmat_fread_s(fp,m) gan_squmat_fread_q(fp,NULL,m)
 #endif
@@ -2079,10 +2079,10 @@ Gan_SquMatrix *gan_squmat_fill_const_q ( Gan_SquMatrix *A,
 size_t gan_squmat_data_size ( Gan_SquMatrixType type, unsigned long size );
 Gan_SquMatrix *gan_squmat_fscanf_q ( FILE *fp, Gan_SquMatrix *A,
                                      char *prefix, int prefix_len );
-Gan_Bool gan_squmat_fwrite ( FILE *fp, Gan_SquMatrix *A,
-                             gan_ui32 magic_number );
+Gan_Bool gan_squmat_fwrite ( FILE *fp, const Gan_SquMatrix *A,
+                             gan_uint32 magic_number );
 Gan_SquMatrix *gan_squmat_fread_q ( FILE *fp, Gan_SquMatrix *A,
-                                    gan_ui32 *magic_number );
+                                    gan_uint32 *magic_number );
 
 /* not user functions */
 Gan_SquMatrix *gan_squmat_form_gen ( Gan_SquMatrix *A,
@@ -2090,7 +2090,7 @@ Gan_SquMatrix *gan_squmat_form_gen ( Gan_SquMatrix *A,
                                      unsigned long size,
                                      double *data, size_t data_size );
 #ifndef NDEBUG
-void gan_squmat_db ( Gan_SquMatrix *A ); /* for use in a debugger */
+void gan_squmat_db ( const Gan_SquMatrix *A ); /* for use in a debugger */
 #endif
 
 /**
