@@ -3,8 +3,8 @@
  * Module:        Symmetric matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.37 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.41 $
+ * Last edited:   $Date: 2005/02/25 09:29:54 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -35,6 +35,7 @@
 #include <gandalf/linalg/cblas.h>
 #include <gandalf/common/numerics.h>
 #include <gandalf/common/allocate.h>
+#include <gandalf/common/array.h>
 #include <gandalf/common/misc_error.h>
 
 /**
@@ -77,7 +78,7 @@ static Gan_Bool
 
 /* function to return element i,j of symmetric matrix */
 static double
- symmetric_matrix_get_el ( Gan_SquMatrix *A, unsigned i, unsigned j )
+ symmetric_matrix_get_el ( const Gan_SquMatrix *A, unsigned i, unsigned j )
 {
    /* consistency check */
    gan_err_test_double ( A->type == GAN_SYMMETRIC_MATRIX,
@@ -144,7 +145,7 @@ static Gan_Bool
 
 /* function to print symmetric matrix to file pointer */
 static Gan_Bool
- symmetric_matrix_fprint ( FILE *fp, Gan_SquMatrix *A, const char *prefix,
+ symmetric_matrix_fprint ( FILE *fp, const Gan_SquMatrix *A, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -180,7 +181,7 @@ static Gan_Bool
 
 /* function to copy symmetric matrix A into another matrix B */
 static Gan_SquMatrix *
- symmetric_matrix_copy ( Gan_SquMatrix *A, Gan_SquMatrix *B )
+ symmetric_matrix_copy ( const Gan_SquMatrix *A, Gan_SquMatrix *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_SYMMETRIC_MATRIX,
@@ -242,11 +243,11 @@ static Gan_SquMatrix *
    {
       long nel = A->size*(A->size+1)/2, onei = 1;
 
-      dcopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) dcopy_ ( &nel, A->data, &onei, B->data, &onei );
       dscal_ ( &nel, &a, B->data, &onei );
    }
 #else
-   if ( !gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B && !gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_dscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -475,7 +476,7 @@ static Gan_SquMatrix *
 /* function to compute matrix/vector product A*x = y */
 static Gan_Vector *
  symmetric_matrix_multv (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector *x, Gan_Vector *y )
 {
    /* consistency check */
@@ -520,7 +521,7 @@ static Gan_Vector *
    }
 #else
    /* compute matrix/vector product */
-   if ( !gan_dspmv ( GAN_UPPER, A->size, 1.0, A->data, x->data, 1, 0.0,
+   if ( !gan_dspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0, A->data, x->data, 1, 0.0,
                      y->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -538,7 +539,7 @@ static Gan_Vector *
  */
 static Gan_Matrix *
  symmetric_matrix_rmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
@@ -598,7 +599,7 @@ static Gan_Matrix *
    {
       /* multiply matrix data C = A*B^T */
       for ( j = 0; j < Ccols; j++ )
-         if ( !gan_dspmv ( GAN_UPPER, A->size, 1.0, A->data, &B->data[j],
+         if ( !gan_dspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0, A->data, &B->data[j],
                            B->rows, 0.0, &C->data[j*C->rows], 1 ) )
          {
             gan_err_flush_trace();
@@ -611,7 +612,7 @@ static Gan_Matrix *
    {
       /* multiply matrix data C = A*B */
       for ( j = 0; j < Ccols; j++ )
-         if ( !gan_dspmv ( GAN_UPPER, A->size, 1.0, A->data,
+         if ( !gan_dspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0, A->data,
                            &B->data[j*B->rows], 1, 0.0,
                            &C->data[j*C->rows], 1 ) )
          {
@@ -632,7 +633,7 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  symmetric_matrix_lmult (
-     Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
@@ -692,7 +693,7 @@ static Gan_Matrix *
    {
       /* multiply matrix data C = B^T*A */
       for ( i = 0; i < Crows; i++ )
-         if ( !gan_dspmv ( GAN_UPPER, A->size, 1.0, A->data,
+         if ( !gan_dspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0, A->data,
                            &B->data[i*B->rows], 1, 0.0, &C->data[i], C->rows ))
          {
             gan_err_flush_trace();
@@ -705,7 +706,7 @@ static Gan_Matrix *
    {
       /* multiply matrix data C = B*A */
       for ( i = 0; i < Crows; i++ )
-         if ( !gan_dspmv ( GAN_UPPER, A->size, 1.0, A->data, &B->data[i],
+         if ( !gan_dspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0, A->data, &B->data[i],
                            B->rows, 0.0, &C->data[i], C->rows ) )
          {
             gan_err_flush_trace();
@@ -746,7 +747,7 @@ static Gan_SquMatrix *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix *
- symmetric_matrix_lrmult ( Gan_SquMatrix *A, Gan_Matrix *B, Gan_TposeFlag B_tr,
+ symmetric_matrix_lrmult ( const Gan_SquMatrix *A, const Gan_Matrix *B, Gan_TposeFlag B_tr,
                            Gan_Matrix *C, Gan_SquMatrix *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -783,7 +784,7 @@ static Gan_SquMatrix *
 
    /* perform first multiplication step A*B(^T) */
    if ( A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
-                          B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C )
+                          (Gan_Matrix *)B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C )
         == NULL )
    {
       gan_err_register ( "symmetric_matrix_lrmult", GAN_ERROR_FAILURE, "" );
@@ -947,7 +948,7 @@ static Gan_SquMatrix *
    }
    
    /* compute Cholesky factorisation */
-   if ( !gan_dpptrf ( GAN_UPPER, B->size, B->data, &info ) )
+   if ( !gan_dpptrf ( GAN_MATRIXPART_UPPER, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -963,7 +964,7 @@ static Gan_SquMatrix *
    }
 
    /* compute inverse from Cholesky factorisation */
-   if ( !gan_dpptri ( GAN_UPPER, B->size, B->data, &info ) )
+   if ( !gan_dpptri ( GAN_MATRIXPART_UPPER, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -1065,7 +1066,7 @@ static Gan_SquMatrix *
 /* Compute squared Frobenius norm of symmetric matrix A.
  */
 static double
- symmetric_matrix_sumsqr ( Gan_SquMatrix *A )
+ symmetric_matrix_sumsqr ( const Gan_SquMatrix *A )
 {
    long i, j;
    double total = 0.0;
@@ -1087,7 +1088,7 @@ static double
 /* Compute Frobenius norm of symmetric matrix A.
  */
 static double
- symmetric_matrix_Fnorm ( Gan_SquMatrix *A )
+ symmetric_matrix_Fnorm ( const Gan_SquMatrix *A )
 {
    long i, j;
    double total = 0.0;
@@ -1108,7 +1109,7 @@ static double
 
 /* Convert symmetric matrix to generic matrix */
 static Gan_Matrix *
- symmetric_matrix_matrix ( Gan_SquMatrix *A, struct Gan_Matrix *B )
+ symmetric_matrix_matrix ( const Gan_SquMatrix *A, struct Gan_Matrix *B )
 {
    long i, j;
 
@@ -1296,7 +1297,7 @@ Gan_SquMatrix *
  * The \a size argument must match the size of the matrix \a A.
  */
 Gan_Bool
- gan_symmat_read_va ( Gan_SquMatrix *A, unsigned long size, ... )
+ gan_symmat_read_va ( const Gan_SquMatrix *A, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -1332,7 +1333,7 @@ Gan_Bool
 Gan_SquMatrix *
  gan_symmat_insert_gen ( Gan_SquMatrix *S,
                          unsigned long rS,   unsigned long cS,
-                         Gan_Matrix *B, Gan_TposeFlag B_tr,
+                         const Gan_Matrix *B, Gan_TposeFlag B_tr,
                          unsigned long rB,   unsigned long cB,
                          unsigned long rows, unsigned long cols )
 {
@@ -1393,7 +1394,7 @@ Gan_SquMatrix *
 Gan_SquMatrix *
  gan_symmat_insertv_gen ( Gan_SquMatrix *S,
                           unsigned long rS, unsigned long cS,
-                          Gan_Vector *x, Gan_TposeFlag x_tr,
+                          const Gan_Vector *x, Gan_TposeFlag x_tr,
                           unsigned long rx,
                           unsigned long size )
 {
@@ -1443,7 +1444,7 @@ Gan_SquMatrix *
  */
 Gan_SquMatrix *
  gan_symmat_insertsym ( Gan_SquMatrix *A, unsigned long rA,
-                        Gan_SquMatrix *B, unsigned long rB,
+                        const Gan_SquMatrix *B, unsigned long rB,
                         unsigned long size )
 {
    long i, j;
@@ -1506,7 +1507,7 @@ Gan_SquMatrix *
 Gan_SquMatrix *
  gan_symmat_insertsym_od ( Gan_SquMatrix *A,
                            unsigned long rA, unsigned long cA,
-                           Gan_SquMatrix *B,
+                           const Gan_SquMatrix *B,
                            unsigned long rB, unsigned long cB,
                            unsigned long rows,
                            unsigned long cols )
@@ -1680,8 +1681,23 @@ Gan_Bool
          dspev_ ( "N", "U", (long *)&A->size, A->data, W->data, NULL, &onei,
                   work, &info );
       else
+      {
+#if 0
+         extern Gan_Bool gan_dspev(Gan_LapackEvecType jobz, Gan_LapackUpLoFlag upper,
+                                   long n, double *ap, double *w, double *z, long ldz,
+                                   double *work, long *info);
+         Gan_SquMatrix* Acopy = gan_squmat_copy_s(A);
+         gan_fill_array_d(work, work_size, 1, 0.0);
+#endif
          dspev_ ( "V", "U", (long *)&A->size, A->data, W->data,
                   Z->data, (long *)&Z->rows, work, &info );
+#if 0
+         gan_fill_array_d(work, work_size, 1, 0.0);
+         gan_dspev ( GAN_EVECTYPE_EVECSORIGINAL, GAN_MATRIXPART_UPPER, A->size, Acopy->data, W->data,
+                     Z->data, Z->rows, work, &info );
+         gan_squmat_free(Acopy);
+#endif
+      }
 
       if ( info != 0 )
       {
@@ -1699,6 +1715,7 @@ Gan_Bool
       }
    }
 #else
+#if 1
    /* use CCM package instead */
    if ( Z == NULL )
    {
@@ -1729,6 +1746,46 @@ Gan_Bool
          return GAN_FALSE;
       }
    }
+#else
+   {
+      long info;
+      Gan_Bool result;
+      Gan_SquMatrix* Acopy = gan_squmat_copy_s(A);
+
+      gan_err_test_bool ( work_size >= 3*A->size,
+                          "gan_symmat_eigen", GAN_ERROR_FAILURE,
+                          "workspace too small (at least 3*size of matrix)" );
+
+      if ( Z == NULL )
+      {
+         result = gan_dspev ( GAN_EVECTYPE_EVECSORIGINAL, GAN_MATRIXPART_UPPER,
+                              A->size, A->data, W->data, NULL, 1, work, &info );
+      }
+      else
+      {
+         result = gan_dspev ( GAN_EVECTYPE_EVECSORIGINAL, GAN_MATRIXPART_UPPER,
+                              A->size, A->data, W->data, Z->data, Z->rows, work,
+                              &info );
+      }
+
+      gan_squmat_free(Acopy);
+
+      if (!result)
+      {
+         gan_err_flush_trace();
+         if ( info < 0 )
+         {
+            gan_err_register ( "gan_symmat_eigen",
+                               GAN_ERROR_CLAPACK_ILLEGAL_ARG, "" );
+         }
+         else /* info > 0 */
+            gan_err_register ( "gan_symmat_eigen",
+                               GAN_ERROR_NO_CONVERGENCE, "" );
+
+         return GAN_FALSE;
+      }
+   }
+#endif
 #endif /* #ifdef HAVE_LAPACK */
 
    /* sort eigenvalues & aigenvectors so that eigenvalues are in
