@@ -3,8 +3,8 @@
  * Module:        Lower/upper triangular matrices (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.32 $
- * Last edited:   $Date: 2003/06/05 12:38:12 $
+ * Revision:      $Revision: 1.34 $
+ * Last edited:   $Date: 2005/02/23 13:13:16 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -70,7 +70,7 @@ static Gan_Bool
 
 /* function to return element i,j of lower triangular matrix */
 static double
- lower_tri_matrix_get_el ( Gan_SquMatrix *L, unsigned i, unsigned j )
+ lower_tri_matrix_get_el ( const Gan_SquMatrix *L, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( L->type == GAN_LOWER_TRI_MATRIX );
@@ -141,7 +141,7 @@ static Gan_Bool
 
 /* function to return element i,j of upper triangular matrix */
 static double
- upper_tri_matrix_get_el ( Gan_SquMatrix *U, unsigned i, unsigned j )
+ upper_tri_matrix_get_el ( const Gan_SquMatrix *U, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( U->type == GAN_UPPER_TRI_MATRIX );
@@ -193,7 +193,7 @@ static Gan_Bool
 
 /* function to print lower triangular matrix to file pointer */
 static Gan_Bool
- lower_tri_matrix_fprint ( FILE *fp, Gan_SquMatrix *L, const char *prefix,
+ lower_tri_matrix_fprint ( FILE *fp, const Gan_SquMatrix *L, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -229,7 +229,7 @@ static Gan_Bool
 
 /* function to print upper triangular matrix to file pointer */
 static Gan_Bool
- upper_tri_matrix_fprint ( FILE *fp, Gan_SquMatrix *U, const char *prefix,
+ upper_tri_matrix_fprint ( FILE *fp, const Gan_SquMatrix *U, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -265,7 +265,7 @@ static Gan_Bool
 
 /* function to copy lower triangular matrix A into another matrix B */
 static Gan_SquMatrix *
- lower_tri_matrix_copy ( Gan_SquMatrix *A, Gan_SquMatrix *B )
+ lower_tri_matrix_copy ( const Gan_SquMatrix *A, Gan_SquMatrix *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_LOWER_TRI_MATRIX,
@@ -305,7 +305,7 @@ static Gan_SquMatrix *
 
 /* function to copy upper triangular matrix A into another matrix B */
 static Gan_SquMatrix *
- upper_tri_matrix_copy ( Gan_SquMatrix *A, Gan_SquMatrix *B )
+ upper_tri_matrix_copy ( const Gan_SquMatrix *A, Gan_SquMatrix *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_UPPER_TRI_MATRIX,
@@ -369,12 +369,12 @@ static Gan_SquMatrix *
       long nel = A->size*(A->size+1)/2, onei = 1;
 
       /* copy and scale matrix */
-      dcopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) dcopy_ ( &nel, A->data, &onei, B->data, &onei );
       dscal_ ( &nel, &a, B->data, &onei );
    }
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   if ( !gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B &&!gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_dscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -412,12 +412,12 @@ static Gan_SquMatrix *
       long nel = A->size*(A->size+1)/2, onei = 1;
 
       /* copy and scale matrix */
-      dcopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) dcopy_ ( &nel, A->data, &onei, B->data, &onei );
       dscal_ ( &nel, &a, B->data, &onei );
    }
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   if ( !gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B &&!gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_dscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -780,7 +780,7 @@ static Gan_SquMatrix *
 /* function to compute matrix/vector product L*x = y, returning y */
 static Gan_Vector *
  lower_tri_matrix_multv (
-     Gan_SquMatrix *L, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *L, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector *x, Gan_Vector *y )
 {
    double total;
@@ -832,7 +832,7 @@ static Gan_Vector *
    {
       /* compute matrix/vector product y = L^-1*x */
       if ( x != y ) gan_dcopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, L->data,
+      gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, L->data,
                   y->data, 1 );
    }
 #endif /* #ifdef HAVE_LAPACK */
@@ -857,7 +857,7 @@ static Gan_Vector *
       if ( x != y )
          gan_dcopy ( x->rows, x->data, 1, y->data, 1 );
 
-      gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, L->data,
+      gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, L->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -869,7 +869,7 @@ static Gan_Vector *
 /* function to compute matrix/vector product L*x = y, returning y */
 static Gan_Vector *
  upper_tri_matrix_multv (
-     Gan_SquMatrix *U, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix *U, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector *x, Gan_Vector *y )
 {
    double total;
@@ -918,7 +918,7 @@ static Gan_Vector *
 #else /* !HAVE_LAPACK */
       /* compute matrix/vector product y = U^-1*x */
       if ( x != y ) gan_dcopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, U->data,
+      gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, U->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -941,7 +941,7 @@ static Gan_Vector *
 #else /* !HAVE_LAPACK */
       /* compute matrix/vector product y = U^-T*x */
       if ( x != y ) gan_dcopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, U->data,
+      gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, U->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -955,8 +955,8 @@ static Gan_Vector *
  */
 static Gan_Matrix *
  lower_tri_matrix_rmult (
-     Gan_SquMatrix *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
    long     i, j, k;
@@ -1013,7 +1013,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
                         Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1039,7 +1039,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         L->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1070,7 +1070,7 @@ static Gan_Matrix *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
                         Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1099,7 +1099,7 @@ static Gan_Matrix *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         L->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1113,8 +1113,8 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  upper_tri_matrix_rmult (
-     Gan_SquMatrix *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
    long     i, j, k;
@@ -1170,7 +1170,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1196,7 +1196,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1227,7 +1227,7 @@ static Gan_Matrix *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1256,7 +1256,7 @@ static Gan_Matrix *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1270,8 +1270,8 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  lower_tri_matrix_lmult (
-     Gan_SquMatrix *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    long     i, j, k;
@@ -1328,7 +1328,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         L->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1355,7 +1355,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1385,7 +1385,7 @@ static Gan_Matrix *
          if ( B != C ) gan_dcopy ( Crows*L->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         L->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1413,7 +1413,7 @@ static Gan_Matrix *
          if ( B != C ) gan_dcopy ( Crows*L->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1427,8 +1427,8 @@ static Gan_Matrix *
  */
 static Gan_Matrix *
  upper_tri_matrix_lmult (
-     Gan_SquMatrix *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
-     Gan_Matrix    *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
+     const Gan_SquMatrix *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
+     Gan_Matrix          *B, Gan_TposeFlag B_tr, Gan_Matrix *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    long     i, j, k;
@@ -1485,7 +1485,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1512,7 +1512,7 @@ static Gan_Matrix *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         U->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1544,7 +1544,7 @@ static Gan_Matrix *
 
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1572,7 +1572,7 @@ static Gan_Matrix *
          if ( B != C ) gan_dcopy ( Crows*U->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_dtpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_dtpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         U->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1634,7 +1634,7 @@ static Gan_SquMatrix *
             gan_err_test_ptr ( info == 0, "lower_tri_matrix_rmult_squ",
                                GAN_ERROR_FAILURE, "" );
 #else /* !HAVE_LAPACK */
-            if ( !gan_dpptri ( GAN_UPPER, C->size, C->data, &info ) )
+            if ( !gan_dpptri ( GAN_MATRIXPART_UPPER, C->size, C->data, &info ) )
             {
                gan_err_flush_trace();
                gan_err_register ( "lower_tri_matrix_rmult_squ",
@@ -1775,7 +1775,7 @@ static Gan_SquMatrix *
             gan_err_test_ptr ( info == 0, "upper_tri_matrix_rmult_squ",
                                GAN_ERROR_FAILURE, "" );
 #else /* !HAVE_LAPACK */
-            if ( !gan_dpptri ( GAN_UPPER, C->size, C->data, &info ) )
+            if ( !gan_dpptri ( GAN_MATRIXPART_UPPER, C->size, C->data, &info ) )
             {
                gan_err_flush_trace();
                gan_err_register ( "upper_tri_matrix_rmult_squ",
@@ -1817,7 +1817,7 @@ static Gan_SquMatrix *
 
 static Gan_SquMatrix *
  triangular_matrix_lrmult (
-     Gan_SquMatrix *A, Gan_Matrix *B, Gan_TposeFlag B_tr,
+     const Gan_SquMatrix *A, const Gan_Matrix *B, Gan_TposeFlag B_tr,
      Gan_Matrix *C, Gan_SquMatrix *D )
 {
    gan_err_flush_trace();
@@ -1887,7 +1887,7 @@ static Gan_SquMatrix *
    gan_dcopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 );
 
    /* perform matrix inversion */
-   if ( !gan_dtptri ( GAN_UPPER, GAN_NOUNIT, B->size, B->data, &info ) )
+   if ( !gan_dtptri ( GAN_MATRIXPART_UPPER, GAN_NOUNIT, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -1918,7 +1918,7 @@ static Gan_SquMatrix *
 /* Compute squared Frobenius norm of triangular matrix A.
  */
 static double
- triangular_matrix_sumsqr ( Gan_SquMatrix *A )
+ triangular_matrix_sumsqr ( const Gan_SquMatrix *A )
 {
    long i;
    double total = 0.0;
@@ -1932,7 +1932,7 @@ static double
 /* Compute Frobenius norm of triangular matrix A.
  */
 static double
- triangular_matrix_Fnorm ( Gan_SquMatrix *A )
+ triangular_matrix_Fnorm ( const Gan_SquMatrix *A )
 {
    long i;
    double total = 0.0;
@@ -1945,7 +1945,7 @@ static double
 
 /* Convert lower triangular matrix to generic matrix */
 static Gan_Matrix *
- lower_tri_matrix_matrix ( Gan_SquMatrix *L, struct Gan_Matrix *B )
+ lower_tri_matrix_matrix ( const Gan_SquMatrix *L, struct Gan_Matrix *B )
 {
    long i, j;
 
@@ -1977,7 +1977,7 @@ static Gan_Matrix *
 
 /* Convert upper triangular matrix to generic matrix */
 static Gan_Matrix *
- upper_tri_matrix_matrix ( Gan_SquMatrix *U, struct Gan_Matrix *B )
+ upper_tri_matrix_matrix ( const Gan_SquMatrix *U, struct Gan_Matrix *B )
 {
    unsigned long i, j;
 
@@ -2286,7 +2286,7 @@ Gan_SquMatrix *
  * size of the matrix \a L.
  */
 Gan_Bool
- gan_ltmat_read_va ( Gan_SquMatrix *L, unsigned long size, ... )
+ gan_ltmat_read_va ( const Gan_SquMatrix *L, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -2316,7 +2316,7 @@ Gan_Bool
  * size of the matrix \a U.
  */
 Gan_Bool
- gan_utmat_read_va ( Gan_SquMatrix *U, unsigned long size, ... )
+ gan_utmat_read_va ( const Gan_SquMatrix *U, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
