@@ -3,8 +3,8 @@
  * Module:        Diagonal matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.14 $
- * Last edited:   $Date: 2002/12/06 17:48:54 $
+ * Revision:      $Revision: 1.15 $
+ * Last edited:   $Date: 2004/03/18 18:26:27 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -71,7 +71,7 @@ static Gan_Bool
 
 /* function to return element i,j of diagonal matrix */
 static float
- diagonal_matrix_get_el ( Gan_SquMatrix_f *A, unsigned i, unsigned j )
+ diagonal_matrix_get_el ( const Gan_SquMatrix_f *A, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( A->type == GAN_DIAGONAL_MATRIX );
@@ -125,7 +125,7 @@ static Gan_Bool
 
 /* function to print diagonal matrix to file pointer */
 static Gan_Bool
- diagonal_matrix_fprint ( FILE *fp, Gan_SquMatrix_f *A, const char *prefix,
+ diagonal_matrix_fprint ( FILE *fp, const Gan_SquMatrix_f *A, const char *prefix,
                           unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -160,7 +160,7 @@ static Gan_Bool
 
 /* function to copy diagonal matrix A into another matrix B */
 static Gan_SquMatrix_f *
- diagonal_matrix_copy ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
+ diagonal_matrix_copy ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -204,11 +204,11 @@ static Gan_SquMatrix_f *
 
 #ifdef HAVE_LAPACK
    /* copy and scale matrix */
-   scopy_ ( (long *)&A->size, A->data, &onei, B->data, &onei );
+   if(A!=B) scopy_ ( (long *)&A->size, A->data, &onei, B->data, &onei );
    sscal_ ( (long *)&A->size, &a, B->data, &onei );
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   gan_scopy ( A->size, A->data, 1, B->data, 1 );
+   if(A!=B) gan_scopy ( A->size, A->data, 1, B->data, 1 );
    gan_sscal ( A->size, a, B->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
 
@@ -395,7 +395,7 @@ static Gan_SquMatrix_f *
 /* function to compute matrix/vector product A*x = y, returning y */
 static Gan_Vector_f *
  diagonal_matrix_multv (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector_f *x, Gan_Vector_f *y )
 {
    long i;
@@ -462,7 +462,7 @@ static Gan_Vector_f *
  */
 static Gan_Matrix_f *
  diagonal_matrix_rmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
@@ -552,8 +552,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  diagonal_matrix_lmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    unsigned long i, j;
@@ -714,8 +714,8 @@ static Gan_SquMatrix_f *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix_f *
- diagonal_matrix_lrmult ( Gan_SquMatrix_f *A,
-                          Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+ diagonal_matrix_lrmult ( const Gan_SquMatrix_f *A,
+                          const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                           Gan_Matrix_f *C, Gan_SquMatrix_f *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -749,8 +749,8 @@ static Gan_SquMatrix_f *
    }
 
    /* perform first multiplication step A*B(^T) */
-   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT, B,
-                     B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C );
+   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
+                     (Gan_Matrix_f *)B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C );
 
    /* perform second multiplication step B*(A*B^T) or B^T*(A*B) */
    gan_matf_rmult_sym_gen ( B, B_tr, C, GAN_NOTRANSPOSE, D );
@@ -840,7 +840,7 @@ static Gan_SquMatrix_f *
 /* Compute squared Frobenius norm of diagonal matrix A.
  */
 static float
- diagonal_matrix_sumsqr ( Gan_SquMatrix_f *A )
+ diagonal_matrix_sumsqr ( const Gan_SquMatrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -857,7 +857,7 @@ static float
 /* Compute Frobenius norm of diagonal matrix A.
  */
 static float
- diagonal_matrix_Fnorm ( Gan_SquMatrix_f *A )
+ diagonal_matrix_Fnorm ( const Gan_SquMatrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -873,7 +873,7 @@ static float
 
 /* Convert diagonal matrix to generic matrix */
 static Gan_Matrix_f *
- diagonal_matrix_matrix ( Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
+ diagonal_matrix_matrix ( const Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
 {
    long i;
 
@@ -1057,7 +1057,7 @@ Gan_SquMatrix_f *
  * The \a size argument must match the size of the matrix.
  */
 Gan_Bool
- gan_diagmatf_read_va ( Gan_SquMatrix_f *A, unsigned long size, ... )
+ gan_diagmatf_read_va ( const Gan_SquMatrix_f *A, unsigned long size, ... )
 {
    unsigned int i;
    va_list ap;
@@ -1222,7 +1222,7 @@ Gan_SquMatrix_f *
  * in-place.
  */
 Gan_Matrix_f *
- gan_diagmatIf_lmult_inf_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+ gan_diagmatIf_lmult_inf_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                              Gan_Matrix_f *C )
 {
    unsigned long i, j;
