@@ -3,8 +3,8 @@
  * Module:        Generic rectangular matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.16 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.19 $
+ * Last edited:   $Date: 2005/04/11 16:04:39 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -134,6 +134,31 @@ Gan_Matrix_f *
    return A;
 }
 
+/**
+ * \}
+ */
+
+/**
+ * \addtogroup GeneralSizeMatrixTest
+ * \{
+ */
+
+/**
+ * \brief Checks whether a matrix is zero
+ * \return Returns #GAN_TRUE if all the elements of \a A are zero, #GAN_FALSE otherwise
+ */
+Gan_Bool gan_matf_is_zero ( Gan_Matrix_f *A )
+{
+   int i;
+
+   for ( i = (int)(A->rows*A->cols)-1; i >= 0; i-- )
+      if(A->data[i] != 0.0F)
+         return GAN_FALSE;
+
+   /* all zero */
+   return GAN_TRUE;
+}
+
 /**
  * \}
  */
@@ -152,7 +177,7 @@ Gan_Matrix_f *
  * (sum of squares of matrix elements).
  */
 float
- gan_matf_sumsqr ( Gan_Matrix_f *A )
+ gan_matf_sumsqr ( const Gan_Matrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -172,7 +197,7 @@ float
  * (square-root of sum of squares of matrix elements).
  */
 float
- gan_matf_Fnorm ( Gan_Matrix_f *A )
+ gan_matf_Fnorm ( const Gan_Matrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -203,7 +228,7 @@ float
  * \sa gan_matf_invert_s().
  */
 Gan_Matrix_f *
- gan_matf_invert_q ( Gan_Matrix_f *A, Gan_Matrix_f *B )
+ gan_matf_invert_q ( const Gan_Matrix_f *A, Gan_Matrix_f *B )
 {
    long *ipiv;
    float *work;
@@ -405,8 +430,7 @@ Gan_Matrix_f *
  * \brief Read elements of matrix from variable argument list.
  */
 Gan_Bool
- gan_matf_read_va ( Gan_Matrix_f *A, unsigned long rows, unsigned long cols,
-                    ... )
+ gan_matf_read_va ( const Gan_Matrix_f *A, unsigned long rows, unsigned long cols, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -442,7 +466,7 @@ Gan_Bool
  * Copy matrix \a A to matrix \a B, and return \a B.
  */
 Gan_Matrix_f *
- gan_matf_copy_q ( Gan_Matrix_f *A, Gan_Matrix_f *B )
+ gan_matf_copy_q ( const Gan_Matrix_f *A, Gan_Matrix_f *B )
 {
    /* allocate matrix B if necessary */
    if ( B == NULL )
@@ -822,8 +846,8 @@ Gan_Matrix_f *
  */
 Gan_SquMatrix_f *
  gan_matf_add_sym_gen (
-     Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-     Gan_Matrix_f *B, Gan_TposeFlag B_tr, struct Gan_SquMatrix_f *C )
+     const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+     const Gan_Matrix_f *B, Gan_TposeFlag B_tr, struct Gan_SquMatrix_f *C )
 {
    /* result matrix dimensions */
    unsigned long Csize = (A_tr ? A->cols : A->rows);
@@ -1034,8 +1058,8 @@ Gan_Matrix_f *
  * \return Pointer to result vector \a y.
  */
 Gan_Vector_f *
- gan_matf_multv_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr, Gan_Vector_f *x,
-                     Gan_Vector_f *y )
+ gan_matf_multv_gen ( const Gan_Matrix_f *A, Gan_TposeFlag A_tr, const Gan_Vector_f *x,
+                      Gan_Vector_f *y )
 {
    unsigned long yrows = (A_tr ? A->cols : A->rows);
 
@@ -1085,8 +1109,8 @@ Gan_Vector_f *
  * Not a user function
  */
 Gan_Matrix_f *
- gan_matf_rmult_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-                     Gan_Matrix_f *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+ gan_matf_rmult_gen ( const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+                      const Gan_Matrix_f *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    /* result matrix dimensions */
    unsigned long Crows = (A_tr ? A->cols : A->rows),
@@ -1150,8 +1174,8 @@ Gan_Matrix_f *
  * Not a user function
  */
 Gan_SquMatrix_f *
- gan_matf_rmult_sym_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-                          Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+ gan_matf_rmult_sym_gen ( const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+                          const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                           struct Gan_SquMatrix_f *C )
 {
    /* result matrix dimensions */
@@ -1247,9 +1271,9 @@ Gan_SquMatrix_f *
  */
 Gan_Matrix_f *
  gan_matf_insert_gen ( Gan_Matrix_f *A, unsigned long rA, unsigned long cA,
-                      Gan_Matrix_f *B, Gan_TposeFlag B_tr,
-                                     unsigned long rB, unsigned long cB,
-                      unsigned long rows , unsigned long cols )
+                       const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+                       unsigned long rB, unsigned long cB,
+                       unsigned long rows , unsigned long cols )
 {
    long i, j;
 
@@ -1290,8 +1314,8 @@ Gan_Matrix_f *
  */
 Gan_Matrix_f *
  gan_matf_insertv_gen ( Gan_Matrix_f *A, unsigned long rA, unsigned long cA,
-                       Gan_Vector_f *x, Gan_TposeFlag x_tr, unsigned long rx,
-                       unsigned long size )
+                        const Gan_Vector_f *x, Gan_TposeFlag x_tr, unsigned long rx,
+                        unsigned long size )
 {
    long i;
 
@@ -1330,7 +1354,7 @@ Gan_Matrix_f *
  */
 Gan_Matrix_f *
  gan_matf_insertsym ( Gan_Matrix_f    *A, unsigned long rA, unsigned long cA,
-                      Gan_SquMatrix_f *B, unsigned long rB, unsigned long cB,
+                      const Gan_SquMatrix_f *B, unsigned long rB, unsigned long cB,
                       unsigned long rows , unsigned long cols )
 {
    long i, j;
@@ -1502,7 +1526,7 @@ Gan_Matrix_f *
  * \sa gan_matf_extract_s(), gan_matf_insert().
  */
 Gan_Matrix_f *
- gan_matf_extract_q ( Gan_Matrix_f *A,
+ gan_matf_extract_q ( const Gan_Matrix_f *A,
                       unsigned long r0,   unsigned long c0,
                       unsigned long rows, unsigned long cols,
                       Gan_Matrix_f *B )
@@ -1551,7 +1575,7 @@ Gan_Matrix_f *
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_matf_fprint ( FILE *fp, Gan_Matrix_f *A, const char *prefix,
+ gan_matf_fprint ( FILE *fp, const Gan_Matrix_f *A, const char *prefix,
                   unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -1696,12 +1720,12 @@ Gan_Matrix_f *
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_matf_fwrite ( FILE *fp, Gan_Matrix_f *A, gan_ui32 magic_number )
+ gan_matf_fwrite ( FILE *fp, const Gan_Matrix_f *A, gan_uint32 magic_number )
 {
    if ( !gan_fwrite_lendian_ui32 ( fp, &magic_number, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &A->rows, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &A->cols, 1 ) ||
-        !gan_fwrite_lendian_f ( fp, A->data, A->rows*A->cols ) )
+        !gan_fwrite_lendian_f32 ( fp, A->data, A->rows*A->cols ) )
    {
       gan_err_register ( "gan_matf_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -1722,7 +1746,7 @@ Gan_Bool
  * \sa gan_vecf_fread_s()
  */
 Gan_Matrix_f *
- gan_matf_fread_q ( FILE *fp, Gan_Matrix_f *A, gan_ui32 *magic_number )
+ gan_matf_fread_q ( FILE *fp, Gan_Matrix_f *A, gan_uint32 *magic_number )
 {
    unsigned long rows, cols;
 
@@ -1754,7 +1778,7 @@ Gan_Matrix_f *
    }
 
    /* read matrix data */
-   if ( !gan_fread_lendian_f ( fp, A->data, A->rows*A->cols ) )
+   if ( !gan_fread_lendian_f32 ( fp, A->data, A->rows*A->cols ) )
    {
       gan_err_register ( "gan_matf_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -1772,7 +1796,7 @@ Gan_Matrix_f *
 /* for use in a debugger: use gan_matf_print() or gan_matf_fprint in
  * your program!
  */
-void gan_matf_db ( Gan_Matrix_f *A )
+void gan_matf_db ( const Gan_Matrix_f *A )
 {
    gan_matf_print ( A, "", 0, "%+g" );
 }
