@@ -3,8 +3,8 @@
  * Module:        Generic rectangular matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.11 $
- * Last edited:   $Date: 2002/04/18 14:56:24 $
+ * Revision:      $Revision: 1.15 $
+ * Last edited:   $Date: 2005/05/05 14:19:56 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -92,32 +92,33 @@ Gan_Matrix_f *gan_matf_fill_vap ( Gan_Matrix_f *A,
 Gan_Matrix_f *gan_matf_fill_const_q ( Gan_Matrix_f *A,
                                       unsigned long rows, unsigned long cols,
                                       float value );
-Gan_Bool gan_matf_read_va ( Gan_Matrix_f *A,
+Gan_Bool gan_matf_read_va ( const Gan_Matrix_f *A,
                             unsigned long rows, unsigned long cols, ... );
-Gan_Matrix_f *gan_matf_copy_q ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matf_copy_q ( const Gan_Matrix_f *A, Gan_Matrix_f *B );
 Gan_Matrix_f *gan_matf_scale_q ( Gan_Matrix_f *A, float a, Gan_Matrix_f *B );
 Gan_Matrix_f *gan_matf_divide_q ( Gan_Matrix_f *A, float a, Gan_Matrix_f *B );
 Gan_Matrix_f *gan_matf_tpose_q ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Bool gan_matf_is_zero ( Gan_Matrix_f *A );
 Gan_Matrix_f *gan_matf_insertsym ( Gan_Matrix_f *A,
                                    unsigned long rA, unsigned long cA,
-                                   struct Gan_SquMatrix_f *B,
+                                   const struct Gan_SquMatrix_f *B,
                                    unsigned long rB, unsigned long cB,
                                    unsigned long rows , unsigned long cols );
 void gan_matf_free_va ( Gan_Matrix_f *A, ... );
-Gan_Matrix_f *gan_matf_extract_q ( Gan_Matrix_f *A,
+Gan_Matrix_f *gan_matf_extract_q ( const Gan_Matrix_f *A,
                                    unsigned long r0,   unsigned long c0,
                                    unsigned long rows, unsigned long cols,
                                    Gan_Matrix_f *B );
-float gan_matf_sumsqr ( Gan_Matrix_f *A );
-float gan_matf_Fnorm ( Gan_Matrix_f *A );
-Gan_Matrix_f *gan_matf_invert_q ( Gan_Matrix_f *A, Gan_Matrix_f *B );
-Gan_Bool gan_matf_fprint ( FILE *fp, Gan_Matrix_f *A, const char *prefix,
+float gan_matf_sumsqr ( const Gan_Matrix_f *A );
+float gan_matf_Fnorm ( const Gan_Matrix_f *A );
+Gan_Matrix_f *gan_matf_invert_q ( const Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Bool gan_matf_fprint ( FILE *fp, const Gan_Matrix_f *A, const char *prefix,
                            unsigned indent, const char *fmt );
 Gan_Matrix_f *gan_matf_fscanf_q ( FILE *fp, Gan_Matrix_f *A,
                                   char *prefix, int prefix_len );
-Gan_Bool gan_matf_fwrite ( FILE *fp, Gan_Matrix_f *A, gan_ui32 magic_number );
+Gan_Bool gan_matf_fwrite ( FILE *fp, const Gan_Matrix_f *A, gan_uint32 magic_number );
 Gan_Matrix_f *gan_matf_fread_q ( FILE *fp, Gan_Matrix_f *A,
-                                 gan_ui32 *magic_number );
+                                 gan_uint32 *magic_number );
 
 #ifdef GAN_GENERATE_DOCUMENTATION
 #define Gan_SquMatrix_f struct Gan_SquMatrix_f
@@ -168,7 +169,7 @@ Gan_Matrix_f *gan_matf_form ( Gan_Matrix_f *A,
  * is filled; otherwise \a A is passed as \c NULL and the matrix is
  * dynamically allocated. Likewise if the provided \a data pointer is \c NULL,
  * the matrix data is dynamically allocated. Otherwise, \a data_size is
- * the size of the \a data array in terms of \c double values.
+ * the size of the \a data array in terms of \c float values.
  *
  * Implemented as a macro call to gan_matf_form_gen().
  *
@@ -177,7 +178,7 @@ Gan_Matrix_f *gan_matf_form ( Gan_Matrix_f *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix_f *gan_matf_form_data ( Gan_Matrix_f *A,
                                    unsigned long rows, unsigned long cols,
-                                   void *data, size_t data_size );
+                                   float *data, size_t data_size );
 #else
 #define gan_matf_form_data(A,rows,cols,data,data_size)\
             gan_matf_form_gen(A,rows,cols,data,data_size)
@@ -222,7 +223,7 @@ Gan_Bool gan_matf_set_el ( Gan_Matrix_f *A,
  * Return element \a i,\a j of generic matrix, or \c FLT_MAX on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-float gan_matf_get_el ( Gan_Matrix_f *A, unsigned i, unsigned j );
+float gan_matf_get_el ( const Gan_Matrix_f *A, unsigned i, unsigned j );
 #else
 #ifdef NDEBUG
 #define gan_matf_get_el(A,i,j) (A)->data[(j)*(A)->rows+(i)]
@@ -302,7 +303,7 @@ Gan_Bool gan_matf_dec_el ( Gan_Matrix_f *A,
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_matf_print ( Gan_Matrix_f *A,
+Gan_Bool gan_matf_print ( const Gan_Matrix_f *A,
                           const char *prefix, int indent, const char *fmt );
 #else
 #define gan_matf_print(A,p,i,f) (gan_matf_fprint(stdout,A,p,i,f))
@@ -385,7 +386,7 @@ Gan_Matrix_f *gan_matf_fill_zero_s ( unsigned long rows, unsigned cols );
  * Allocate and return a copy of matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_copy_s ( Gan_Matrix_f *A );
+Gan_Matrix_f *gan_matf_copy_s ( const Gan_Matrix_f *A );
 #else
 #define gan_matf_copy_s(A) (gan_matf_copy_q(A,NULL))
 #endif
@@ -623,7 +624,7 @@ Gan_Matrix_f *gan_matf_incrementT ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_add_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matf_add_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                       Gan_SquMatrix_f *C );
 #else
 #define gan_matf_add_sym_q(A,B,C) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -637,7 +638,7 @@ Gan_SquMatrix_f *gan_matf_add_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_add_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matf_add_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_add_sym_s(A,B) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -650,7 +651,7 @@ Gan_SquMatrix_f *gan_matf_add_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_addT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matf_addT_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                        Gan_SquMatrix_f *C );
 #else
 #define gan_matf_addT_sym_q(A,B,C) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -664,7 +665,7 @@ Gan_SquMatrix_f *gan_matf_addT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_addT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matf_addT_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_addT_sym_s(A,B) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -677,7 +678,7 @@ Gan_SquMatrix_f *gan_matf_addT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_add_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matTf_add_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                        Gan_SquMatrix_f *C );
 #else
 #define gan_matTf_add_sym_q(A,B,C) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -691,7 +692,7 @@ Gan_SquMatrix_f *gan_matTf_add_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_add_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matTf_add_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_add_sym_s(A,B) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -704,7 +705,7 @@ Gan_SquMatrix_f *gan_matTf_add_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_addT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matTf_addT_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                         Gan_SquMatrix_f *C );
 #else
 #define gan_matTf_addT_sym_q(A,B,C) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -718,7 +719,7 @@ Gan_SquMatrix_f *gan_matTf_addT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_addT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matTf_addT_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_addT_sym_s(A,B) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -730,7 +731,7 @@ Gan_SquMatrix_f *gan_matTf_addT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * square symmetric matrix \f$ B = A + A^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_saddT_sym_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
+Gan_SquMatrix_f *gan_matf_saddT_sym_q ( const Gan_Matrix_f *A, Gan_SquMatrix_f *B );
 #else
 #define gan_matf_saddT_sym_q(A,B) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,B))
 #endif
@@ -742,7 +743,7 @@ Gan_SquMatrix_f *gan_matf_saddT_sym_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
  * a new square symmetric matrix \f$ A + A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_saddT_sym_s ( Gan_Matrix_f *A );
+Gan_SquMatrix_f *gan_matf_saddT_sym_s ( const Gan_Matrix_f *A );
 #else
 #define gan_matf_saddT_sym_s(A) (gan_matf_add_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,NULL))
 #endif
@@ -754,7 +755,7 @@ Gan_SquMatrix_f *gan_matf_saddT_sym_s ( Gan_Matrix_f *A );
  * square symmetric matrix \f$ B = A^{\top} + A \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_sadd_sym_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
+Gan_SquMatrix_f *gan_matTf_sadd_sym_q ( const Gan_Matrix_f *A, Gan_SquMatrix_f *B );
 #else
 #define gan_matTf_sadd_sym_q(A,B) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,B))
 #endif
@@ -766,7 +767,7 @@ Gan_SquMatrix_f *gan_matTf_sadd_sym_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
  * a new square symmetric matrix \f$ A^{\top} + A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_sadd_sym_s ( Gan_Matrix_f *A );
+Gan_SquMatrix_f *gan_matTf_sadd_sym_s ( const Gan_Matrix_f *A );
 #else
 #define gan_matTf_sadd_sym_s(A) (gan_matf_add_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -893,7 +894,7 @@ Gan_Matrix_f *gan_matf_decrementT ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \f$ y = A x \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_matf_multv_q ( Gan_Matrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_matf_multv_q ( const Gan_Matrix_f *A, const Gan_Vector_f *x,
                                  Gan_Vector_f *y );
 #else
 #define gan_matf_multv_q(A,x,y) (gan_matf_multv_gen(A,GAN_NOTRANSPOSE,x,y))
@@ -906,7 +907,7 @@ Gan_Vector_f *gan_matf_multv_q ( Gan_Matrix_f *A, Gan_Vector_f *x,
  * result \f$ A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_matf_multv_s ( Gan_Matrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_matf_multv_s ( const Gan_Matrix_f *A, const Gan_Vector_f *x );
 #else
 #define gan_matf_multv_s(A,x) (gan_matf_multv_gen(A,GAN_NOTRANSPOSE,x,NULL))
 #endif
@@ -918,7 +919,7 @@ Gan_Vector_f *gan_matf_multv_s ( Gan_Matrix_f *A, Gan_Vector_f *x );
  * \f$ y = A^{\top} x \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_matTf_multv_q ( Gan_Matrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_matTf_multv_q ( const Gan_Matrix_f *A, const Gan_Vector_f *x,
                                   Gan_Vector_f *y );
 #else
 #define gan_matTf_multv_q(A,x,y) (gan_matf_multv_gen(A,GAN_TRANSPOSE,x,y))
@@ -931,7 +932,7 @@ Gan_Vector_f *gan_matTf_multv_q ( Gan_Matrix_f *A, Gan_Vector_f *x,
  * returning the result \f$ A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_matTf_multv_s ( Gan_Matrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_matTf_multv_s ( const Gan_Matrix_f *A, const Gan_Vector_f *x );
 #else
 #define gan_matTf_multv_s(A,x) (gan_matf_multv_gen(A,GAN_TRANSPOSE,x,NULL))
 #endif
@@ -952,7 +953,7 @@ Gan_Vector_f *gan_matTf_multv_s ( Gan_Matrix_f *A, Gan_Vector_f *x );
  * \f$ C = A B \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_rmult_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_matf_rmult_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                  Gan_Matrix_f *C );
 #else
 #define gan_matf_rmult_q(A,B,C) (gan_matf_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -965,7 +966,7 @@ Gan_Matrix_f *gan_matf_rmult_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \f$ A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_rmult_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matf_rmult_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_rmult_s(A,B) (gan_matf_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -977,7 +978,7 @@ Gan_Matrix_f *gan_matf_rmult_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \f$ C = A B^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_rmultT_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_matf_rmultT_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                   Gan_Matrix_f *C );
 #else
 #define gan_matf_rmultT_q(A,B,C) (gan_matf_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -990,7 +991,7 @@ Gan_Matrix_f *gan_matf_rmultT_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * result as a new matrix \f$ A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_rmultT_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matf_rmultT_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_rmultT_s(A,B) (gan_matf_rmult_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1002,7 +1003,7 @@ Gan_Matrix_f *gan_matf_rmultT_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \f$ C = A^{\top} B \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matTf_rmult_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_matTf_rmult_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                   Gan_Matrix_f *C );
 #else
 #define gan_matTf_rmult_q(A,B,C) (gan_matf_rmult_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1015,7 +1016,7 @@ Gan_Matrix_f *gan_matTf_rmult_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * result as a new matrix \f$ A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matTf_rmult_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matTf_rmult_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_rmult_s(A,B) (gan_matf_rmult_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1027,7 +1028,7 @@ Gan_Matrix_f *gan_matTf_rmult_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * and write result into \f$ C = A^{\top} B^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matTf_rmultT_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_matTf_rmultT_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                    Gan_Matrix_f *C );
 #else
 #define gan_matTf_rmultT_q(A,B,C) (gan_matf_rmult_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1040,7 +1041,7 @@ Gan_Matrix_f *gan_matTf_rmultT_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * and return the result as a new matrix \f$ A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matTf_rmultT_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matTf_rmultT_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_rmultT_s(A,B) (gan_matf_rmult_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1053,7 +1054,7 @@ Gan_Matrix_f *gan_matTf_rmultT_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_rmult_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matf_rmult_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                         Gan_SquMatrix_f *C );
 #else
 #define gan_matf_rmult_sym_q(A,B,C) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1067,7 +1068,7 @@ Gan_SquMatrix_f *gan_matf_rmult_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_rmult_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matf_rmult_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_rmult_sym_s(A,B) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1080,7 +1081,7 @@ Gan_SquMatrix_f *gan_matf_rmult_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_rmultT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matf_rmultT_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                          Gan_SquMatrix_f *C );
 #else
 #define gan_matf_rmultT_sym_q(A,B,C) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1094,7 +1095,7 @@ Gan_SquMatrix_f *gan_matf_rmultT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_rmultT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matf_rmultT_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_rmultT_sym_s(A,B) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1107,7 +1108,7 @@ Gan_SquMatrix_f *gan_matf_rmultT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_rmult_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matTf_rmult_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                          Gan_SquMatrix_f *C );
 #else
 #define gan_matTf_rmult_sym_q(A,B,C) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,C))
@@ -1121,7 +1122,7 @@ Gan_SquMatrix_f *gan_matTf_rmult_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_rmult_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matTf_rmult_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_rmult_sym_s(A,B) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1134,7 +1135,7 @@ Gan_SquMatrix_f *gan_matTf_rmult_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_rmultT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
+Gan_SquMatrix_f *gan_matTf_rmultT_sym_q ( const Gan_Matrix_f *A, const Gan_Matrix_f *B,
                                           Gan_SquMatrix_f *C );
 #else
 #define gan_matTf_rmultT_sym_q(A,B,C) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,C))
@@ -1148,7 +1149,7 @@ Gan_SquMatrix_f *gan_matTf_rmultT_sym_q ( Gan_Matrix_f *A, Gan_Matrix_f *B,
  * \em assumed to be symmetric. No checking of this assumption is done.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matTf_rmultT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+Gan_SquMatrix_f *gan_matTf_rmultT_sym_s ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matTf_rmultT_sym_s(A,B) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1160,7 +1161,7 @@ Gan_SquMatrix_f *gan_matTf_rmultT_sym_s ( Gan_Matrix_f *A, Gan_Matrix_f *B );
  * square symmetric matrix \f$ B = A A^{\top} \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_srmultT_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
+Gan_SquMatrix_f *gan_matf_srmultT_q ( const Gan_Matrix_f *A, Gan_SquMatrix_f *B );
 #else
 #define gan_matf_srmultT_q(A,B) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,B))
 #endif
@@ -1172,7 +1173,7 @@ Gan_SquMatrix_f *gan_matf_srmultT_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
  * a new square symmetric matrix \f$ A A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_srmultT_s ( Gan_Matrix_f *A );
+Gan_SquMatrix_f *gan_matf_srmultT_s ( const Gan_Matrix_f *A );
 #else
 #define gan_matf_srmultT_s(A) (gan_matf_rmult_sym_gen(A,GAN_NOTRANSPOSE,A,GAN_TRANSPOSE,NULL))
 #endif
@@ -1184,7 +1185,7 @@ Gan_SquMatrix_f *gan_matf_srmultT_s ( Gan_Matrix_f *A );
  * square symmetric matrix \f$ B = A^{\top} A \f$, which is returned.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_slmultT_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
+Gan_SquMatrix_f *gan_matf_slmultT_q ( const Gan_Matrix_f *A, Gan_SquMatrix_f *B );
 #else
 #define gan_matf_slmultT_q(A,B) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,B))
 #endif
@@ -1196,7 +1197,7 @@ Gan_SquMatrix_f *gan_matf_slmultT_q ( Gan_Matrix_f *A, Gan_SquMatrix_f *B );
  * a new square symmetric matrix \f$ A^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_matf_slmultT_s ( Gan_Matrix_f *A );
+Gan_SquMatrix_f *gan_matf_slmultT_s ( const Gan_Matrix_f *A );
 #else
 #define gan_matf_slmultT_s(A) (gan_matf_rmult_sym_gen(A,GAN_TRANSPOSE,A,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1224,7 +1225,7 @@ Gan_SquMatrix_f *gan_matf_slmultT_s ( Gan_Matrix_f *A );
  * \sa gan_matf_invert_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_invert_s ( Gan_Matrix_f *A ) /* input matrix */;
+Gan_Matrix_f *gan_matf_invert_s ( const Gan_Matrix_f *A ) /* input matrix */;
 #else
 #define gan_matf_invert_s(A) gan_matf_invert_q(A,NULL)
 #endif
@@ -1247,7 +1248,7 @@ Gan_Matrix_f *gan_matf_invert_s ( Gan_Matrix_f *A ) /* input matrix */;
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-   Gan_Bool gan_matf_same_dims ( Gan_Matrix_f *A, Gan_Matrix_f *B );
+   Gan_Bool gan_matf_same_dims ( const Gan_Matrix_f *A, const Gan_Matrix_f *B );
 #else
 #define gan_matf_same_dims(A,B) ((A)->rows == (B)->rows && (A)->cols == (B)->cols)
 #endif
@@ -1273,7 +1274,7 @@ Gan_Matrix_f *gan_matf_invert_s ( Gan_Matrix_f *A ) /* input matrix */;
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix_f *gan_matf_insert ( Gan_Matrix_f *A,
                                 unsigned long rA, unsigned long cA,
-                                Gan_Matrix_f *B,
+                                const Gan_Matrix_f *B,
                                 unsigned long rB,   unsigned long cB,
                                 unsigned long rows, unsigned long cols )
 ;
@@ -1293,7 +1294,7 @@ Gan_Matrix_f *gan_matf_insert ( Gan_Matrix_f *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix_f *gan_matf_insertT ( Gan_Matrix_f *A,
                                  unsigned long rA, unsigned long cA,
-                                 Gan_Matrix_f *B,
+                                 const Gan_Matrix_f *B,
                                  unsigned long rB,   unsigned long cB,
                                  unsigned long rows, unsigned long cols );
 #else
@@ -1310,7 +1311,7 @@ Gan_Matrix_f *gan_matf_insertT ( Gan_Matrix_f *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix_f *gan_matf_insertv ( Gan_Matrix_f *A,
                                  unsigned long rA, unsigned long cA,
-                                 Gan_Vector_f *x, unsigned long rx,
+                                 const Gan_Vector_f *x, unsigned long rx,
                                  unsigned long rows );
 #else
 #define gan_matf_insertv(A,rA,cA,x,rx,rows) (gan_matf_insertv_gen(A,rA,cA,x,GAN_NOTRANSPOSE,rx,rows))
@@ -1329,7 +1330,7 @@ Gan_Matrix_f *gan_matf_insertv ( Gan_Matrix_f *A,
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Matrix_f *gan_matf_insertvT ( Gan_Matrix_f *A,
                                   unsigned long rA, unsigned long cA,
-                                  Gan_Vector_f *x, unsigned long rx,
+                                  const Gan_Vector_f *x, unsigned long rx,
                                   unsigned long cols );
 #else
 #define gan_matf_insertvT(A,rA,cA,x,rx,cols) (gan_matf_insertv_gen(A,rA,cA,x,GAN_TRANSPOSE,rx,cols))
@@ -1351,7 +1352,7 @@ Gan_Matrix_f *gan_matf_insertvT ( Gan_Matrix_f *A,
  * \a cols, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_matf_test_dims ( Gan_Matrix_f *A, unsigned long rows,
+Gan_Bool gan_matf_test_dims ( const Gan_Matrix_f *A, unsigned long rows,
                                                unsigned long cols );
 #else
 #define gan_matf_test_dims(A,rs,cs) (((A)->rows==(rs))&&((A)->cols==(cs)))
@@ -1403,7 +1404,7 @@ size_t gan_matf_data_size ( unsigned long rows, unsigned long cols );
  * \sa gan_matf_extract_q(), gan_matf_insert().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_extract_s ( Gan_Matrix_f *A,
+Gan_Matrix_f *gan_matf_extract_s ( const Gan_Matrix_f *A,
                                    unsigned long r0,   unsigned long c0,
                                    unsigned long rows, unsigned long cols );
 #else
@@ -1450,7 +1451,7 @@ Gan_Matrix_f *gan_matf_fscanf_s ( FILE *fp, const char *prefix,
  * \sa gan_matf_fread_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_Matrix_f *gan_matf_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_matf_fread_s(fp,ml) gan_matf_fread_q(fp,NULL,m)
 #endif
@@ -1473,34 +1474,34 @@ Gan_Matrix_f *gan_matf_add_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
                                  Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                                  Gan_Matrix_f *C );
 struct Gan_SquMatrix_f *gan_matf_add_sym_gen (
-    Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-    Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+    const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+    const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
     struct Gan_SquMatrix_f *C );
 Gan_Matrix_f *gan_matf_sub_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
                                  Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                                  Gan_Matrix_f *C );
-Gan_Vector_f *gan_matf_multv_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-                                   Gan_Vector_f *x, Gan_Vector_f *y );
-Gan_Matrix_f *gan_matf_rmult_gen ( Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-                                   Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+Gan_Vector_f *gan_matf_multv_gen ( const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+                                   const Gan_Vector_f *x, Gan_Vector_f *y );
+Gan_Matrix_f *gan_matf_rmult_gen ( const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+                                   const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                                    Gan_Matrix_f *C );
 struct Gan_SquMatrix_f *gan_matf_rmult_sym_gen (
-    Gan_Matrix_f *A, Gan_TposeFlag A_tr,
-    Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+    const Gan_Matrix_f *A, Gan_TposeFlag A_tr,
+    const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
     struct Gan_SquMatrix_f *C );
 Gan_Matrix_f *gan_matf_insert_gen ( Gan_Matrix_f *A,
                                     unsigned long rA, unsigned long cA,
-                                    Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+                                    const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                                     unsigned long rB, unsigned long cB,
                                     unsigned long rows , unsigned long cols );
 Gan_Matrix_f *gan_matf_insertv_gen ( Gan_Matrix_f *A,
                                      unsigned long rA, unsigned long cA,
-                                     struct Gan_Vector_f *x,
+                                     const struct Gan_Vector_f *x,
                                      Gan_TposeFlag x_tr,
                                      unsigned long rx, unsigned long size );
 
 #ifndef NDEBUG
-void gan_matf_db ( Gan_Matrix_f *A ); /* for use in a debugger */
+void gan_matf_db ( const Gan_Matrix_f *A ); /* for use in a debugger */
 #endif
 
 #define gan_matf_realloc(A,rows,cols) gan_matf_realloc_gen(A,rows,cols)
