@@ -3,8 +3,8 @@
  * Module:        Scaled identity matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.10 $
- * Last edited:   $Date: 2002/04/22 13:42:22 $
+ * Revision:      $Revision: 1.11 $
+ * Last edited:   $Date: 2004/03/18 18:26:28 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -61,7 +61,7 @@ static Gan_Bool
 
 /* function to return element i,j of multiple of identity */
 static float
- scaledI_matrix_get_el ( Gan_SquMatrix_f *A, unsigned i, unsigned j )
+ scaledI_matrix_get_el ( const Gan_SquMatrix_f *A, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( A->type == GAN_SCALED_IDENT_MATRIX );
@@ -97,7 +97,7 @@ static Gan_Bool
 
 /* function to print multiple of identity to file pointer */
 static Gan_Bool
- scaledI_matrix_fprint ( FILE *fp, Gan_SquMatrix_f *A, const char *prefix,
+ scaledI_matrix_fprint ( FILE *fp, const Gan_SquMatrix_f *A, const char *prefix,
                          unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -131,7 +131,7 @@ static Gan_Bool
 
 /* function to copy multiple of identity A into another matrix B */
 static Gan_SquMatrix_f *
- scaledI_matrix_copy ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
+ scaledI_matrix_copy ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
 {
    /* consistency check */
    assert ( A->type == GAN_SCALED_IDENT_MATRIX );
@@ -231,7 +231,7 @@ static Gan_SquMatrix_f *
 /* function to compute matrix/vector product A*x = y, returning y */
 static Gan_Vector_f *
  scaledI_matrix_multv (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector_f *x, Gan_Vector_f *y )
 {
    long i;
@@ -287,7 +287,7 @@ static Gan_Vector_f *
  */
 static Gan_Matrix_f *
  scaledI_matrix_rmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
@@ -379,8 +379,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  scaledI_matrix_lmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    unsigned long i, j;
@@ -531,8 +531,8 @@ static Gan_SquMatrix_f *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix_f *
- scaledI_matrix_lrmult ( Gan_SquMatrix_f *A,
-                         Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+ scaledI_matrix_lrmult ( const Gan_SquMatrix_f *A,
+                         const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                          Gan_Matrix_f *C, Gan_SquMatrix_f *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -566,8 +566,8 @@ static Gan_SquMatrix_f *
       D = gan_symmatf_set_size ( D, Dsize );
 
    /* perform first multiplication step A*B(^T) */
-   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT, B,
-                     (Gan_TposeFlag) !B_tr, C );
+   A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
+                     (Gan_Matrix_f *)B, (Gan_TposeFlag) !B_tr, C );
 
    /* perform second multiplication step B*(A*B^T) or B^T*(A*B) */
    gan_matf_rmult_sym_gen ( B, B_tr, C, GAN_NOTRANSPOSE, D );
@@ -641,7 +641,7 @@ static Gan_SquMatrix_f *
 /* Compute squared Frobenius norm of multiple of identity matrix A.
  */
 static float
- scaledI_matrix_sumsqr ( Gan_SquMatrix_f *A )
+ scaledI_matrix_sumsqr ( const Gan_SquMatrix_f *A )
 {
    return ((float)A->size)*gan_sqr(A->data[0]);
 }
@@ -649,14 +649,14 @@ static float
 /* Compute Frobenius norm of multiple of identity matrix A.
  */
 static float
- scaledI_matrix_Fnorm ( Gan_SquMatrix_f *A )
+ scaledI_matrix_Fnorm ( const Gan_SquMatrix_f *A )
 {
    return (float)sqrt(((double)A->size)*(double)gan_sqr(A->data[0]));
 }
 
 /* Convert multiple of identity to generic matrix */
 static Gan_Matrix_f *
- scaledI_matrix_matrix ( Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
+ scaledI_matrix_matrix ( const Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
 {
    long i;
    float Aval = A->data[0];
@@ -836,7 +836,7 @@ Gan_SquMatrix_f *
  * The \a size argument must match the size of the matrix.
  */
 Gan_Bool
- gan_scalImatf_read_va ( Gan_SquMatrix_f *A, unsigned long size, ... )
+ gan_scalImatf_read_va ( const Gan_SquMatrix_f *A, unsigned long size, ... )
 {
    va_list ap;
 
