@@ -3,8 +3,8 @@
  * Module:        Square matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.12 $
- * Last edited:   $Date: 2002/04/22 13:42:22 $
+ * Revision:      $Revision: 1.14 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -685,7 +685,7 @@ Gan_SquMatrix_f *
  * \sa gan_squmatf_fread_q().
  */
 Gan_Bool
- gan_squmatf_fwrite ( FILE *fp, Gan_SquMatrix_f *A, gan_ui32 magic_number )
+ gan_squmatf_fwrite ( FILE *fp, const Gan_SquMatrix_f *A, gan_uint32 magic_number )
 {
    char type_char;
 
@@ -717,8 +717,8 @@ Gan_Bool
    }
 
    if ( !gan_fwrite_lendian_ul ( fp, &A->size, 1 ) ||
-        !gan_fwrite_lendian_f ( fp, A->data,
-                                gan_squmatf_data_size ( A->type, A->size ) ) )
+        !gan_fwrite_lendian_f32 ( fp, A->data,
+                                  gan_squmatf_data_size ( A->type, A->size ) ) )
    {
       gan_err_register ( "gan_squmatf_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -738,7 +738,7 @@ Gan_Bool
  * \sa gan_squmatf_fread_s().
  */
 Gan_SquMatrix_f *
- gan_squmatf_fread_q ( FILE *fp, Gan_SquMatrix_f *A, gan_ui32 *magic_number )
+ gan_squmatf_fread_q ( FILE *fp, Gan_SquMatrix_f *A, gan_uint32 *magic_number )
 {
    unsigned long size;
    char type_char;
@@ -794,8 +794,8 @@ Gan_SquMatrix_f *
    }
 
    /* read matrix data */
-   if ( !gan_fread_lendian_f ( fp, A->data,
-                               gan_squmatf_data_size ( type, size ) ) )
+   if ( !gan_fread_lendian_f32 ( fp, A->data,
+                                 gan_squmatf_data_size ( type, size ) ) )
    {
       gan_err_register ( "gan_squmatf_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -861,7 +861,7 @@ Gan_SquMatrix_f *
 /* for use in a debugger: use gan_squmatf_print() or gan_squmatf_fprint in
  * your program!
  */
-void gan_squmatf_db ( Gan_SquMatrix_f *A )
+void gan_squmatf_db ( const Gan_SquMatrix_f *A )
 {
    gan_squmatf_print ( A, "", 0, "%+g" );
 }
