@@ -3,8 +3,8 @@
  * Module:        Square matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.9 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.11 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -60,17 +60,17 @@ typedef struct Gan_SquMatrixFFuncs
 {
    Gan_Bool (*set_el) ( struct Gan_SquMatrix_f *A, unsigned i, unsigned j,
                         float value );
-   float (*get_el) ( struct Gan_SquMatrix_f *A, unsigned i, unsigned j );
+   float (*get_el) ( const struct Gan_SquMatrix_f *A, unsigned i, unsigned j );
    Gan_Bool (*inc_el) ( struct Gan_SquMatrix_f *A, unsigned i, unsigned j,
                         float value );
    Gan_Bool (*dec_el) ( struct Gan_SquMatrix_f *A, unsigned i, unsigned j,
                         float value );
-   Gan_Bool (*fprint) ( FILE *fp, struct Gan_SquMatrix_f *A,
+   Gan_Bool (*fprint) ( FILE *fp, const struct Gan_SquMatrix_f *A,
                         const char *prefix, unsigned indent, const char *fmt );
-   struct Gan_SquMatrix_f *(*copy) ( struct Gan_SquMatrix_f *A,
+   struct Gan_SquMatrix_f *(*copy) ( const struct Gan_SquMatrix_f *A,
                                      struct Gan_SquMatrix_f *B );
    struct Gan_SquMatrix_f *(*scale) ( struct Gan_SquMatrix_f *A, float a,
-                                    struct Gan_SquMatrix_f *B );
+                                      struct Gan_SquMatrix_f *B );
    struct Gan_SquMatrix_f *(*add) (
        struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr,
        struct Gan_SquMatrix_f *B, Gan_TposeFlag B_tr,
@@ -80,29 +80,29 @@ typedef struct Gan_SquMatrixFFuncs
        struct Gan_SquMatrix_f *B, Gan_TposeFlag B_tr,
        struct Gan_SquMatrix_f *C );
    struct Gan_Vector_f *(*multv) (
-       struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+       const struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
        struct Gan_Vector_f *x, struct Gan_Vector_f *y );
    struct Gan_Matrix_f *(*rmult) (
-       struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+       const struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
        struct Gan_Matrix_f *B, Gan_TposeFlag B_tr, struct Gan_Matrix_f *C );
    struct Gan_Matrix_f *(*lmult) (
-       struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+       const struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
        struct Gan_Matrix_f *B, Gan_TposeFlag B_tr, struct Gan_Matrix_f *C );
    struct Gan_SquMatrix_f *(*rmult_squ) (
        struct Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
        struct Gan_SquMatrix_f *B, Gan_TposeFlag B_tr, Gan_InvertFlag B_inv,
        struct Gan_SquMatrix_f *C );
    struct Gan_SquMatrix_f *(*lrmult) (
-       struct Gan_SquMatrix_f *A, struct Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+       const struct Gan_SquMatrix_f *A, const struct Gan_Matrix_f *B, Gan_TposeFlag B_tr,
        struct Gan_Matrix_f *C, struct Gan_SquMatrix_f *D );
    struct Gan_SquMatrix_f *(*invert) ( struct Gan_SquMatrix_f *A,
                                        struct Gan_SquMatrix_f *B );
    struct Gan_SquMatrix_f *(*cholesky) ( struct Gan_SquMatrix_f *S,
                                          struct Gan_SquMatrix_f *L,
                                          int *error_code );
-   float (*sumsqr) ( struct Gan_SquMatrix_f *A );
-   float (*Fnorm)  ( struct Gan_SquMatrix_f *A );
-   struct Gan_Matrix_f *(*matrix) ( struct Gan_SquMatrix_f *A,
+   float (*sumsqr) ( const struct Gan_SquMatrix_f *A );
+   float (*Fnorm)  ( const struct Gan_SquMatrix_f *A );
+   struct Gan_Matrix_f *(*matrix) ( const struct Gan_SquMatrix_f *A,
                                     struct Gan_Matrix_f *B );
 } Gan_SquMatrixFFuncs;
 
@@ -225,7 +225,7 @@ Gan_Bool gan_squmatf_set_el ( Gan_SquMatrix_f *A, unsigned row, unsigned col,
  * failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-float gan_squmatf_get_el ( Gan_SquMatrix_f *A, unsigned row, unsigned col );
+float gan_squmatf_get_el ( const Gan_SquMatrix_f *A, unsigned row, unsigned col );
 #else
 #define gan_squmatf_get_el(A,r,c) ((A)->funcs->get_el(A,r,c))
 #endif
@@ -273,7 +273,7 @@ Gan_Bool gan_squmatf_dec_el ( Gan_SquMatrix_f *A, unsigned row, unsigned col,
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_fprint ( FILE *fp, Gan_SquMatrix_f *A,
+Gan_Bool gan_squmatf_fprint ( FILE *fp, const Gan_SquMatrix_f *A,
                               const char *prefix, int indent,
                               const char *fmt );
 #else
@@ -288,7 +288,7 @@ Gan_Bool gan_squmatf_fprint ( FILE *fp, Gan_SquMatrix_f *A,
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_print ( Gan_SquMatrix_f *A,
+Gan_Bool gan_squmatf_print ( const Gan_SquMatrix_f *A,
                              const char *prefix, int indent, const char *fmt );
 #else
 #define gan_squmatf_print(A,p,i,f) ((A)->funcs->fprint(stdout,A,p,i,f))
@@ -376,7 +376,7 @@ Gan_SquMatrix_f *
  * Copy square matrix \a A to square matrix \a B, and return \a B.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_squmatf_copy_q ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
+Gan_SquMatrix_f *gan_squmatf_copy_q ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
 #else
 #define gan_squmatf_copy_q(A,B) ((A)->funcs->copy(A,B))
 #endif
@@ -387,7 +387,7 @@ Gan_SquMatrix_f *gan_squmatf_copy_q ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
  * Allocate and return a copy of square matrix \a A.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_squmatf_copy_s ( Gan_SquMatrix_f *A );
+Gan_SquMatrix_f *gan_squmatf_copy_s ( const Gan_SquMatrix_f *A );
 #else
 #define gan_squmatf_copy_s(A) ((A)->funcs->copy(A,NULL))
 #endif
@@ -765,7 +765,7 @@ Gan_SquMatrix_f *gan_squmatf_decrementT ( Gan_SquMatrix_f *A,
  * \f$ y = A * x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_squmatf_multv_q ( const Gan_SquMatrix_f *A, Gan_Vector_f *x,
                                     Gan_Vector_f *y );
 #else
 #define gan_squmatf_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,y))
@@ -779,7 +779,7 @@ Gan_Vector_f *gan_squmatf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
  * result \f$ A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatf_multv_s ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatf_multv_s(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,NULL))
 #endif
@@ -792,7 +792,7 @@ Gan_Vector_f *gan_squmatf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * the result \f$ x \leftarrow A x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatf_multv_i ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatf_multv_i(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_NOINVERT,x,x))
 #endif
@@ -805,7 +805,7 @@ Gan_Vector_f *gan_squmatf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * into \f$ y = A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatTf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_squmatTf_multv_q ( const Gan_SquMatrix_f *A, Gan_Vector_f *x,
                                      Gan_Vector_f *y );
 #else
 #define gan_squmatTf_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,y))
@@ -818,7 +818,7 @@ Gan_Vector_f *gan_squmatTf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
  * returning the result \f$ A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatTf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatTf_multv_s ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatTf_multv_s(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,NULL))
 #endif
@@ -831,7 +831,7 @@ Gan_Vector_f *gan_squmatTf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * \a x with the result \f$ x \leftarrow A^{\top} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatTf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatTf_multv_i ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatTf_multv_i(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_NOINVERT,x,x))
 #endif
@@ -844,7 +844,7 @@ Gan_Vector_f *gan_squmatTf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * into \f$ y = A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatIf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_squmatIf_multv_q ( const Gan_SquMatrix_f *A, Gan_Vector_f *x,
                                      Gan_Vector_f *y );
 #else
 #define gan_squmatIf_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,y))
@@ -858,7 +858,7 @@ Gan_Vector_f *gan_squmatIf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
  * returning the result \f$ A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatIf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatIf_multv_s ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatIf_multv_s(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,NULL))
 #endif
@@ -871,7 +871,7 @@ Gan_Vector_f *gan_squmatIf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * \a x with the result \f$ x \leftarrow A^{-1} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatIf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatIf_multv_i ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatIf_multv_i(A,x) ((A)->funcs->multv(A,GAN_NOTRANSPOSE,GAN_INVERT,x,x))
 #endif
@@ -884,7 +884,7 @@ Gan_Vector_f *gan_squmatIf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * writing result into \f$ y = A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatITf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
+Gan_Vector_f *gan_squmatITf_multv_q ( const Gan_SquMatrix_f *A, Gan_Vector_f *x,
                                       Gan_Vector_f *y );
 #else
 #define gan_squmatITf_multv_q(A,x,y) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,y))
@@ -898,7 +898,7 @@ Gan_Vector_f *gan_squmatITf_multv_q ( Gan_SquMatrix_f *A, Gan_Vector_f *x,
  * creating and returning the result \f$ A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatITf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatITf_multv_s ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatITf_multv_s(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,NULL))
 #endif
@@ -911,7 +911,7 @@ Gan_Vector_f *gan_squmatITf_multv_s ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * overwriting \a x with the result \f$ x \leftarrow A^{-{\top}} x \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_squmatITf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
+Gan_Vector_f *gan_squmatITf_multv_i ( const Gan_SquMatrix_f *A, Gan_Vector_f *x );
 #else
 #define gan_squmatITf_multv_i(A,x) ((A)->funcs->multv(A,GAN_TRANSPOSE,GAN_INVERT,x,x))
 #endif
@@ -933,7 +933,7 @@ Gan_Vector_f *gan_squmatITf_multv_i ( Gan_SquMatrix_f *A, Gan_Vector_f *x );
  * result into generic matrix \f$ C = A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatf_rmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                     Gan_Matrix_f *C );
 #else
 #define gan_squmatf_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -947,7 +947,7 @@ Gan_Matrix_f *gan_squmatf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * as a new matrix \f$ A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_rmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -960,7 +960,7 @@ Gan_Matrix_f *gan_squmatf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * with the result: \f$ B \leftarrow A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_rmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -973,7 +973,7 @@ Gan_Matrix_f *gan_squmatf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * and write result into \f$ C = A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatf_rmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatf_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -987,7 +987,7 @@ Gan_Matrix_f *gan_squmatf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * and return the result as a new matrix \f$ A B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_rmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1000,7 +1000,7 @@ Gan_Matrix_f *gan_squmatf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * write result into \f$ C = A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatTf_rmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatTf_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1014,7 +1014,7 @@ Gan_Matrix_f *gan_squmatTf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * return the result as a new matrix \f$ A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_rmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1027,7 +1027,7 @@ Gan_Matrix_f *gan_squmatTf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{\top} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_rmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1040,7 +1040,7 @@ Gan_Matrix_f *gan_squmatTf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * matrix \a B and write result into \f$ C = A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatTf_rmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatTf_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1054,7 +1054,7 @@ Gan_Matrix_f *gan_squmatTf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * matrix \a B and return the result as a new matrix \f$ A^{\top} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_rmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1067,7 +1067,7 @@ Gan_Matrix_f *gan_squmatTf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * and write result into generic matrix \f$ C = A B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatIf_rmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatIf_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1081,7 +1081,7 @@ Gan_Matrix_f *gan_squmatIf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * the result as a new matrix \f$ A^{-1} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_rmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1094,7 +1094,7 @@ Gan_Matrix_f *gan_squmatIf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{-1} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_rmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1107,7 +1107,7 @@ Gan_Matrix_f *gan_squmatIf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * \a B and write result into \f$ C = A^{-1} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatIf_rmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatIf_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1121,7 +1121,7 @@ Gan_Matrix_f *gan_squmatIf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \a B and return the result as a new matrix \f$ A^{-1} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_rmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1134,7 +1134,7 @@ Gan_Matrix_f *gan_squmatIf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * \a B and write result into \f$ C = A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatITf_rmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatITf_rmult_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1148,7 +1148,7 @@ Gan_Matrix_f *gan_squmatITf_rmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \a B and return the result as a new matrix \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_rmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_rmult_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1161,7 +1161,7 @@ Gan_Matrix_f *gan_squmatITf_rmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ B \leftarrow A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_rmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_rmult_i(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1174,7 +1174,7 @@ Gan_Matrix_f *gan_squmatITf_rmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * generic matrix \a B and write result into \f$ C = A^{-{\top}} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatITf_rmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                        Gan_Matrix_f *C );
 #else
 #define gan_squmatITf_rmultT_q(A,B,C) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1189,7 +1189,7 @@ Gan_Matrix_f *gan_squmatITf_rmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \f$ A^{-{\top}} B^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_rmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_rmultT_s(A,B) ((A)->funcs->rmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1202,7 +1202,7 @@ Gan_Matrix_f *gan_squmatITf_rmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * result into generic matrix \f$ C = B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatf_lmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                     Gan_Matrix_f *C );
 #else
 #define gan_squmatf_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1216,7 +1216,7 @@ Gan_Matrix_f *gan_squmatf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * as a new matrix \f$ B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_lmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1229,7 +1229,7 @@ Gan_Matrix_f *gan_squmatf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * with the result \f$ B \leftarrow B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_lmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1242,7 +1242,7 @@ Gan_Matrix_f *gan_squmatf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * and write result into \f$ C = B^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatf_lmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatf_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1256,7 +1256,7 @@ Gan_Matrix_f *gan_squmatf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * and return the result as a new matrix \f$ B^{\top} A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatf_lmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatf_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1269,7 +1269,7 @@ Gan_Matrix_f *gan_squmatf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * write result into \f$ C = B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatTf_lmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatTf_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,C))
@@ -1283,7 +1283,7 @@ Gan_Matrix_f *gan_squmatTf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * return the result as a new matrix \f$ B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_lmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1296,7 +1296,7 @@ Gan_Matrix_f *gan_squmatTf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ B \leftarrow B A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_lmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1309,7 +1309,7 @@ Gan_Matrix_f *gan_squmatTf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * matrix \a A, and write result into \f$ C = B^{\top} A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatTf_lmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatTf_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,C))
@@ -1324,7 +1324,7 @@ Gan_Matrix_f *gan_squmatTf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \f$ B^{\top} A^{\top} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatTf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatTf_lmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatTf_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_NOINVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1337,7 +1337,7 @@ Gan_Matrix_f *gan_squmatTf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * and write result into generic matrix \f$ C = B A \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatIf_lmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                      Gan_Matrix_f *C );
 #else
 #define gan_squmatIf_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1351,7 +1351,7 @@ Gan_Matrix_f *gan_squmatIf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * return the result as a new matrix \f$ B A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_lmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1364,7 +1364,7 @@ Gan_Matrix_f *gan_squmatIf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ B A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_lmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1377,7 +1377,7 @@ Gan_Matrix_f *gan_squmatIf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * matrix \a A, and write result into \f$ C = B^{\top} A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatIf_lmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatIf_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1391,7 +1391,7 @@ Gan_Matrix_f *gan_squmatIf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * matrix \a A, and return the result as a new matrix \f$ B^{\top} A^{-1} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatIf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatIf_lmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatIf_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_NOTRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1404,7 +1404,7 @@ Gan_Matrix_f *gan_squmatIf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * matrix \a A, and write result into \f$ C = B A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatITf_lmult_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                       Gan_Matrix_f *C );
 #else
 #define gan_squmatITf_lmult_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,C))
@@ -1418,7 +1418,7 @@ Gan_Matrix_f *gan_squmatITf_lmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \a B and return the result as a new matrix \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_lmult_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_lmult_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,NULL))
 #endif
@@ -1431,7 +1431,7 @@ Gan_Matrix_f *gan_squmatITf_lmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * overwriting \a B with the result \f$ A^{-{\top}} B \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_lmult_i ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_lmult_i(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_NOTRANSPOSE,B))
 #endif
@@ -1445,7 +1445,7 @@ Gan_Matrix_f *gan_squmatITf_lmult_i ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * \f$ C = B^{\top} A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
+Gan_Matrix_f *gan_squmatITf_lmultT_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B,
                                        Gan_Matrix_f *C );
 #else
 #define gan_squmatITf_lmultT_q(A,B,C) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,C))
@@ -1460,7 +1460,7 @@ Gan_Matrix_f *gan_squmatITf_lmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * \f$ B^{\top} A^{-{\top}} \f$.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_squmatITf_lmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_squmatITf_lmultT_s ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_squmatITf_lmultT_s(A,B) ((A)->funcs->lmult(A,GAN_TRANSPOSE,GAN_INVERT,B,GAN_TRANSPOSE,NULL))
 #endif
@@ -1645,8 +1645,8 @@ Gan_SquMatrix_f *gan_squmatITf_srmultI_squ_i ( Gan_SquMatrix_f *A );
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_symmatf_lrmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
-                                        Gan_Matrix_f *C, Gan_SquMatrix_f *D );
+Gan_SquMatrix_f *gan_symmatf_lrmult_q ( Gan_SquMatrix_f *A, const Gan_Matrix_f *B,
+                                        const Gan_Matrix_f *C, Gan_SquMatrix_f *D );
 #else
 #define gan_symmatf_lrmult_q(A,B,C,D) (assert(gan_squmatf_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_NOTRANSPOSE,C,D))
 #endif
@@ -1662,8 +1662,8 @@ Gan_SquMatrix_f *gan_symmatf_lrmult_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_symmatf_lrmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
-                                        Gan_Matrix_f *C );
+Gan_SquMatrix_f *gan_symmatf_lrmult_s ( Gan_SquMatrix_f *A, const Gan_Matrix_f *B,
+                                        const Gan_Matrix_f *C );
 #else
 #define gan_symmatf_lrmult_s(A,B,C) (assert(gan_squmatf_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_NOTRANSPOSE,C,NULL))
 #endif
@@ -1679,8 +1679,8 @@ Gan_SquMatrix_f *gan_symmatf_lrmult_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_symmatf_lrmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
-                                         Gan_Matrix_f *C, Gan_SquMatrix_f *D );
+Gan_SquMatrix_f *gan_symmatf_lrmultT_q ( Gan_SquMatrix_f *A, const Gan_Matrix_f *B,
+                                         const Gan_Matrix_f *C, Gan_SquMatrix_f *D );
 #else
 #define gan_symmatf_lrmultT_q(A,B,C,D) (assert(gan_squmatf_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_TRANSPOSE,C,D))
 #endif
@@ -1696,8 +1696,8 @@ Gan_SquMatrix_f *gan_symmatf_lrmultT_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
  * is allocated and freed inside the routine.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_symmatf_lrmultT_s ( Gan_SquMatrix_f *A, Gan_Matrix_f *B,
-                                         Gan_Matrix_f *C );
+Gan_SquMatrix_f *gan_symmatf_lrmultT_s ( Gan_SquMatrix_f *A, const Gan_Matrix_f *B,
+                                         const Gan_Matrix_f *C );
 #else
 #define gan_symmatf_lrmultT_s(A,B,C) (assert(gan_squmatf_symmetric(A)),(A)->funcs->lrmult(A,B,GAN_TRANSPOSE,C,NULL))
 #endif
@@ -1850,7 +1850,7 @@ float gan_squmatf_Fnorm ( Gan_SquMatrix_f *A );
  * \sa gan_matf_from_squmatf_s().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
+Gan_Matrix_f *gan_matf_from_squmatf_q ( const Gan_SquMatrix_f *A, Gan_Matrix_f *B );
 #else
 #define gan_matf_from_squmatf_q(A,B) ((A)->funcs->matrix(A,B))
 #endif
@@ -1864,7 +1864,7 @@ Gan_Matrix_f *gan_matf_from_squmatf_q ( Gan_SquMatrix_f *A, Gan_Matrix_f *B );
  * \sa gan_matf_from_squmatf_q().
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_matf_from_squmatf_s ( Gan_SquMatrix_f *A );
+Gan_Matrix_f *gan_matf_from_squmatf_s ( const Gan_SquMatrix_f *A );
 #else
 #define gan_matf_from_squmatf_s(A) ((A)->funcs->matrix(A,NULL))
 #endif
@@ -1885,7 +1885,7 @@ Gan_Matrix_f *gan_matf_from_squmatf_s ( Gan_SquMatrix_f *A );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_same_type ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
+Gan_Bool gan_squmatf_same_type ( const Gan_SquMatrix_f *A, const Gan_SquMatrix_f *B );
 #else
 #define gan_squmatf_same_type(A,B) ((A)->type == (B)->type)
 #endif
@@ -1897,7 +1897,7 @@ Gan_Bool gan_squmatf_same_type ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_same_size ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
+Gan_Bool gan_squmatf_same_size ( const Gan_SquMatrix_f *A, const Gan_SquMatrix_f *B );
 #else
 #define gan_squmatf_same_size(A,B) ((A)->size == (B)->size)
 #endif
@@ -1909,7 +1909,7 @@ Gan_Bool gan_squmatf_same_size ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_same_type_size ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B );
+Gan_Bool gan_squmatf_same_type_size ( const Gan_SquMatrix_f *A, const Gan_SquMatrix_f *B );
 #else
 #define gan_squmatf_same_type_size(A,B) ((A)->type == (B)->type && (A)->size == (B)->size)
 #endif
@@ -1969,7 +1969,7 @@ Gan_SquMatrix_f *gan_squmatf_set_size ( Gan_SquMatrix_f *A,
  * otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_test_size ( Gan_Matrix_f *A, unsigned long size );
+Gan_Bool gan_squmatf_test_size ( const Gan_Matrix_f *A, unsigned long size );
 #else
 #define gan_squmatf_test_size(A,s) ((A)->size==s)
 #endif
@@ -1981,7 +1981,7 @@ Gan_Bool gan_squmatf_test_size ( Gan_Matrix_f *A, unsigned long size );
  * #GAN_SYMMETRIC_MATRIX, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_symmetric_type ( Gan_SquMatrix_f *A );
+Gan_Bool gan_squmatf_symmetric_type ( const Gan_SquMatrix_f *A );
 #else
 #define gan_squmatf_symmetric_type(A) ((A)->type==GAN_SYMMETRIC_MATRIX)
 #endif
@@ -1993,7 +1993,7 @@ Gan_Bool gan_squmatf_symmetric_type ( Gan_SquMatrix_f *A );
  * #GAN_DIAGONAL_MATRIX, #GAN_FALSE otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_diagonal_type ( Gan_SquMatrix_f *A );
+Gan_Bool gan_squmatf_diagonal_type ( const Gan_SquMatrix_f *A );
 #else
 #define gan_squmatf_diagonal_type(A) ((A)->type==GAN_DIAGONAL_MATRIX)
 #endif
@@ -2007,7 +2007,7 @@ Gan_Bool gan_squmatf_diagonal_type ( Gan_SquMatrix_f *A );
  * #GAN_FALSE is returned if \a A is not one of these faults.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_symmetric ( Gan_SquMatrix_f *A );
+Gan_Bool gan_squmatf_symmetric ( const Gan_SquMatrix_f *A );
 #else
 #define gan_squmatf_symmetric(A) ((A)->type==GAN_SYMMETRIC_MATRIX || (A)->type==GAN_DIAGONAL_MATRIX || (A)->type==GAN_SCALED_IDENT_MATRIX)
 #endif
@@ -2021,7 +2021,7 @@ Gan_Bool gan_squmatf_symmetric ( Gan_SquMatrix_f *A );
  * #GAN_FALSE is returned if \a A is not one of these faults.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_squmatf_diagonal ( Gan_SquMatrix_f *A );
+Gan_Bool gan_squmatf_diagonal ( const Gan_SquMatrix_f *A );
 #else
 #define gan_squmatf_diagonal(A) ((A)->type==GAN_DIAGONAL_MATRIX || (A)->type==GAN_SCALED_IDENT_MATRIX)
 #endif
@@ -2063,7 +2063,7 @@ Gan_SquMatrix_f *gan_squmatf_fscanf_s ( FILE *fp,
  * \sa: gan_squmatf_fread_q()
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_squmatf_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_SquMatrix_f *gan_squmatf_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_squmatf_fread_s(fp,m) gan_squmatf_fread_q(fp,NULL,m)
 #endif
@@ -2086,10 +2086,10 @@ Gan_SquMatrix_f *gan_squmatf_fill_const_q ( Gan_SquMatrix_f *A,
 size_t gan_squmatf_data_size ( Gan_SquMatrixType type, unsigned long size );
 Gan_SquMatrix_f *gan_squmatf_fscanf_q ( FILE *fp, Gan_SquMatrix_f *A,
                                         char *prefix, int prefix_len );
-Gan_Bool gan_squmatf_fwrite ( FILE *fp, Gan_SquMatrix_f *A,
-                              gan_ui32 magic_number );
+Gan_Bool gan_squmatf_fwrite ( FILE *fp, const Gan_SquMatrix_f *A,
+                              gan_uint32 magic_number );
 Gan_SquMatrix_f *gan_squmatf_fread_q ( FILE *fp, Gan_SquMatrix_f *A,
-                                       gan_ui32 *magic_number );
+                                       gan_uint32 *magic_number );
 
 /* not user functions */
 Gan_SquMatrix_f *gan_squmatf_form_gen ( Gan_SquMatrix_f *A,
@@ -2098,7 +2098,7 @@ Gan_SquMatrix_f *gan_squmatf_form_gen ( Gan_SquMatrix_f *A,
                                         float *data, size_t data_size );
 void gan_squmatf_free_func ( Gan_SquMatrix_f *A );
 #ifndef NDEBUG
-void gan_squmatf_db ( Gan_SquMatrix_f *A ); /* for use in a debugger */
+void gan_squmatf_db ( const Gan_SquMatrix_f *A ); /* for use in a debugger */
 #endif
 
 /**
