@@ -3,8 +3,8 @@
  * Module:        Symmetric matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.18 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.20 $
+ * Last edited:   $Date: 2005/02/23 13:13:16 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -77,7 +77,7 @@ static Gan_Bool
 
 /* function to return element i,j of symmetric matrix */
 static float
- symmetric_matrix_get_el ( Gan_SquMatrix_f *A, unsigned i, unsigned j )
+ symmetric_matrix_get_el ( const Gan_SquMatrix_f *A, unsigned i, unsigned j )
 {
    /* consistency check */
    gan_err_test_float ( A->type == GAN_SYMMETRIC_MATRIX,
@@ -144,7 +144,7 @@ static Gan_Bool
 
 /* function to print symmetric matrix to file pointer */
 static Gan_Bool
- symmetric_matrix_fprint ( FILE *fp, Gan_SquMatrix_f *A, const char *prefix,
+ symmetric_matrix_fprint ( FILE *fp, const Gan_SquMatrix_f *A, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -180,7 +180,7 @@ static Gan_Bool
 
 /* function to copy symmetric matrix A into another matrix B */
 static Gan_SquMatrix_f *
- symmetric_matrix_copy ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
+ symmetric_matrix_copy ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_SYMMETRIC_MATRIX,
@@ -242,11 +242,11 @@ static Gan_SquMatrix_f *
    {
       long nel = A->size*(A->size+1)/2, onei = 1;
 
-      scopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) scopy_ ( &nel, A->data, &onei, B->data, &onei );
       sscal_ ( &nel, &a, B->data, &onei );
    }
 #else
-   if ( !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B && !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_sscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -475,7 +475,7 @@ static Gan_SquMatrix_f *
 /* function to compute matrix/vector product A*x = y */
 static Gan_Vector_f *
  symmetric_matrix_multv (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector_f *x, Gan_Vector_f *y )
 {
    /* consistency check */
@@ -520,7 +520,7 @@ static Gan_Vector_f *
    }
 #else
    /* compute matrix/vector product */
-   if ( !gan_sspmv ( GAN_UPPER, A->size, 1.0F, A->data, x->data, 1, 0.0F,
+   if ( !gan_sspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0F, A->data, x->data, 1, 0.0F,
                      y->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -538,8 +538,8 @@ static Gan_Vector_f *
  */
 static Gan_Matrix_f *
  symmetric_matrix_rmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
    unsigned long j;
@@ -598,7 +598,7 @@ static Gan_Matrix_f *
    {
       /* multiply matrix data C = A*B^T */
       for ( j = 0; j < Ccols; j++ )
-         if ( !gan_sspmv ( GAN_UPPER, A->size, 1.0F, A->data, &B->data[j],
+         if ( !gan_sspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0F, A->data, &B->data[j],
                            B->rows, 0.0F, &C->data[j*C->rows], 1 ) )
          {
             gan_err_flush_trace();
@@ -611,7 +611,7 @@ static Gan_Matrix_f *
    {
       /* multiply matrix data C = A*B */
       for ( j = 0; j < Ccols; j++ )
-         if ( !gan_sspmv ( GAN_UPPER, A->size, 1.0F, A->data,
+         if ( !gan_sspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0F, A->data,
                            &B->data[j*B->rows], 1, 0.0F,
                            &C->data[j*C->rows], 1 ) )
          {
@@ -632,8 +632,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  symmetric_matrix_lmult (
-     Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *A, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    unsigned long i;
@@ -692,7 +692,7 @@ static Gan_Matrix_f *
    {
       /* multiply matrix data C = B^T*A */
       for ( i = 0; i < Crows; i++ )
-         if ( !gan_sspmv ( GAN_UPPER, A->size, 1.0F, A->data,
+         if ( !gan_sspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0F, A->data,
                            &B->data[i*B->rows], 1, 0.0F,
                            &C->data[i], C->rows ))
          {
@@ -706,7 +706,7 @@ static Gan_Matrix_f *
    {
       /* multiply matrix data C = B*A */
       for ( i = 0; i < Crows; i++ )
-         if ( !gan_sspmv ( GAN_UPPER, A->size, 1.0F, A->data, &B->data[i],
+         if ( !gan_sspmv ( GAN_MATRIXPART_UPPER, A->size, 1.0F, A->data, &B->data[i],
                            B->rows, 0.0F, &C->data[i], C->rows ) )
          {
             gan_err_flush_trace();
@@ -747,8 +747,8 @@ static Gan_SquMatrix_f *
  * The result matrix D is returned.
  */
 static Gan_SquMatrix_f *
- symmetric_matrix_lrmult ( Gan_SquMatrix_f *A,
-                           Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+ symmetric_matrix_lrmult ( const Gan_SquMatrix_f *A,
+                           const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                            Gan_Matrix_f *C, Gan_SquMatrix_f *D )
 {
    unsigned long Dsize = B_tr ? B->cols : B->rows;
@@ -785,7 +785,7 @@ static Gan_SquMatrix_f *
 
    /* perform first multiplication step A*B(^T) */
    if ( A->funcs->rmult ( A, GAN_NOTRANSPOSE, GAN_NOINVERT,
-                          B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C )
+                          (Gan_Matrix_f *)B, B_tr ? GAN_NOTRANSPOSE : GAN_TRANSPOSE, C )
         == NULL )
    {
       gan_err_register ( "symmetric_matrix_lrmult", GAN_ERROR_FAILURE, "" );
@@ -949,7 +949,7 @@ static Gan_SquMatrix_f *
    }
    
    /* compute Cholesky factorisation */
-   if ( !gan_spptrf ( GAN_UPPER, B->size, B->data, &info ) )
+   if ( !gan_spptrf ( GAN_MATRIXPART_UPPER, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -965,7 +965,7 @@ static Gan_SquMatrix_f *
    }
 
    /* compute inverse from Cholesky factorisation */
-   if ( !gan_spptri ( GAN_UPPER, B->size, B->data, &info ) )
+   if ( !gan_spptri ( GAN_MATRIXPART_UPPER, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -1067,7 +1067,7 @@ static Gan_SquMatrix_f *
 /* Compute squared Frobenius norm of symmetric matrix A.
  */
 static float
- symmetric_matrix_sumsqr ( Gan_SquMatrix_f *A )
+ symmetric_matrix_sumsqr ( const Gan_SquMatrix_f *A )
 {
    long i, j;
    float total = 0.0F;
@@ -1089,7 +1089,7 @@ static float
 /* Compute Frobenius norm of symmetric matrix A.
  */
 static float
- symmetric_matrix_Fnorm ( Gan_SquMatrix_f *A )
+ symmetric_matrix_Fnorm ( const Gan_SquMatrix_f *A )
 {
    long i, j;
    float total = 0.0F;
@@ -1110,7 +1110,7 @@ static float
 
 /* Convert symmetric matrix to generic matrix */
 static Gan_Matrix_f *
- symmetric_matrix_matrix ( Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
+ symmetric_matrix_matrix ( const Gan_SquMatrix_f *A, struct Gan_Matrix_f *B )
 {
    long i, j;
 
@@ -1299,7 +1299,7 @@ Gan_SquMatrix_f *
  * The \a size argument must match the size of the matrix \a A.
  */
 Gan_Bool
- gan_symmatf_read_va ( Gan_SquMatrix_f *A, unsigned long size, ... )
+ gan_symmatf_read_va ( const Gan_SquMatrix_f *A, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -1335,7 +1335,7 @@ Gan_Bool
 Gan_SquMatrix_f *
  gan_symmatf_insert_gen ( Gan_SquMatrix_f *S,
                           unsigned long rS,   unsigned long cS,
-                          Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+                          const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
                           unsigned long rB,   unsigned long cB,
                           unsigned long rows, unsigned long cols )
 {
@@ -1396,7 +1396,7 @@ Gan_SquMatrix_f *
 Gan_SquMatrix_f *
  gan_symmatf_insertv_gen ( Gan_SquMatrix_f *S,
                            unsigned long rS, unsigned long cS,
-                           Gan_Vector_f *x, Gan_TposeFlag x_tr,
+                           const Gan_Vector_f *x, Gan_TposeFlag x_tr,
                            unsigned long rx,
                            unsigned long size )
 {
@@ -1448,7 +1448,7 @@ Gan_SquMatrix_f *
  */
 Gan_SquMatrix_f *
  gan_symmatf_insertsym ( Gan_SquMatrix_f *A, unsigned long rA,
-                         Gan_SquMatrix_f *B, unsigned long rB,
+                         const Gan_SquMatrix_f *B, unsigned long rB,
                          unsigned long size )
 {
    long i, j;
@@ -1511,7 +1511,7 @@ Gan_SquMatrix_f *
 Gan_SquMatrix_f *
  gan_symmatf_insertsym_od ( Gan_SquMatrix_f *A,
                             unsigned long rA, unsigned long cA,
-                            Gan_SquMatrix_f *B,
+                            const Gan_SquMatrix_f *B,
                             unsigned long rB, unsigned long cB,
                             unsigned long rows,
                             unsigned long cols )
