@@ -3,8 +3,8 @@
  * Module:        Lower/upper triangular matrices (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.14 $
- * Last edited:   $Date: 2003/06/05 12:38:12 $
+ * Revision:      $Revision: 1.16 $
+ * Last edited:   $Date: 2005/02/23 13:13:16 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -70,7 +70,7 @@ static Gan_Bool
 
 /* function to return element i,j of lower triangular matrix */
 static float
- lower_tri_matrix_get_el ( Gan_SquMatrix_f *L, unsigned i, unsigned j )
+ lower_tri_matrix_get_el ( const Gan_SquMatrix_f *L, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( L->type == GAN_LOWER_TRI_MATRIX );
@@ -141,7 +141,7 @@ static Gan_Bool
 
 /* function to return element i,j of upper triangular matrix */
 static float
- upper_tri_matrix_get_el ( Gan_SquMatrix_f *U, unsigned i, unsigned j )
+ upper_tri_matrix_get_el ( const Gan_SquMatrix_f *U, unsigned i, unsigned j )
 {
    /* consistency check */
    assert ( U->type == GAN_UPPER_TRI_MATRIX );
@@ -193,7 +193,7 @@ static Gan_Bool
 
 /* function to print lower triangular matrix to file pointer */
 static Gan_Bool
- lower_tri_matrix_fprint ( FILE *fp, Gan_SquMatrix_f *L, const char *prefix,
+ lower_tri_matrix_fprint ( FILE *fp, const Gan_SquMatrix_f *L, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -229,7 +229,7 @@ static Gan_Bool
 
 /* function to print upper triangular matrix to file pointer */
 static Gan_Bool
- upper_tri_matrix_fprint ( FILE *fp, Gan_SquMatrix_f *U, const char *prefix,
+ upper_tri_matrix_fprint ( FILE *fp, const Gan_SquMatrix_f *U, const char *prefix,
                            unsigned indent, const char *fmt )
 {
    unsigned long i, j;
@@ -265,7 +265,7 @@ static Gan_Bool
 
 /* function to copy lower triangular matrix A into another matrix B */
 static Gan_SquMatrix_f *
- lower_tri_matrix_copy ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
+ lower_tri_matrix_copy ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_LOWER_TRI_MATRIX,
@@ -305,7 +305,7 @@ static Gan_SquMatrix_f *
 
 /* function to copy upper triangular matrix A into another matrix B */
 static Gan_SquMatrix_f *
- upper_tri_matrix_copy ( Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
+ upper_tri_matrix_copy ( const Gan_SquMatrix_f *A, Gan_SquMatrix_f *B )
 {
    /* consistency check */
    gan_err_test_ptr ( A->type == GAN_UPPER_TRI_MATRIX,
@@ -369,12 +369,12 @@ static Gan_SquMatrix_f *
       long nel = A->size*(A->size+1)/2, onei = 1;
 
       /* copy and scale matrix */
-      scopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) scopy_ ( &nel, A->data, &onei, B->data, &onei );
       sscal_ ( &nel, &a, B->data, &onei );
    }
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   if ( !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B && !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_sscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -412,12 +412,12 @@ static Gan_SquMatrix_f *
       long nel = A->size*(A->size+1)/2, onei = 1;
 
       /* copy and scale matrix */
-      scopy_ ( &nel, A->data, &onei, B->data, &onei );
+      if(A!=B) scopy_ ( &nel, A->data, &onei, B->data, &onei );
       sscal_ ( &nel, &a, B->data, &onei );
    }
 #else /* !HAVE_LAPACK */
    /* copy and scale matrix */
-   if ( !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 ) ||
+   if ( (A!=B && !gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 )) ||
         !gan_sscal ( A->size*(A->size+1)/2, a, B->data, 1 ) )
    {
       gan_err_flush_trace();
@@ -780,7 +780,7 @@ static Gan_SquMatrix_f *
 /* function to compute matrix/vector product L*x = y, returning y */
 static Gan_Vector_f *
  lower_tri_matrix_multv (
-     Gan_SquMatrix_f *L, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *L, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector_f *x, Gan_Vector_f *y )
 {
    float total;
@@ -831,7 +831,7 @@ static Gan_Vector_f *
    {
       /* compute matrix/vector product y = L^-1*x */
       if ( x != y ) gan_scopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, L->data,
+      gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, L->data,
                   y->data, 1 );
    }
 #endif /* #ifdef HAVE_LAPACK */
@@ -856,7 +856,7 @@ static Gan_Vector_f *
       if ( x != y )
          gan_scopy ( x->rows, x->data, 1, y->data, 1 );
 
-      gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, L->data,
+      gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, L->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -868,7 +868,7 @@ static Gan_Vector_f *
 /* function to compute matrix/vector product L*x = y, returning y */
 static Gan_Vector_f *
  upper_tri_matrix_multv (
-     Gan_SquMatrix_f *U, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
+     const Gan_SquMatrix_f *U, Gan_TposeFlag A_tr, Gan_InvertFlag A_inv,
      Gan_Vector_f *x, Gan_Vector_f *y )
 {
    float total;
@@ -917,7 +917,7 @@ static Gan_Vector_f *
 #else /* !HAVE_LAPACK */
       /* compute matrix/vector product y = U^-1*x */
       if ( x != y ) gan_scopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, U->data,
+      gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, y->rows, U->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -940,7 +940,7 @@ static Gan_Vector_f *
 #else /* !HAVE_LAPACK */
       /* compute matrix/vector product y = U^-T*x */
       if ( x != y ) gan_scopy ( x->rows, x->data, 1, y->data, 1 );
-      gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, U->data,
+      gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, y->rows, U->data,
                   y->data, 1 );
 #endif /* #ifdef HAVE_LAPACK */
    }
@@ -954,7 +954,7 @@ static Gan_Vector_f *
  */
 static Gan_Matrix_f *
  lower_tri_matrix_rmult (
-     Gan_SquMatrix_f *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
+     const Gan_SquMatrix_f *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
      Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
@@ -1012,7 +1012,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
                         Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1038,7 +1038,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         L->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1069,7 +1069,7 @@ static Gan_Matrix_f *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows, L->data,
                         Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1098,7 +1098,7 @@ static Gan_Matrix_f *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         L->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1112,8 +1112,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  upper_tri_matrix_rmult (
-     Gan_SquMatrix_f *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Ccols = B_tr ? B->rows : B->cols;
    long     i, j, k;
@@ -1169,7 +1169,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1195,7 +1195,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1226,7 +1226,7 @@ static Gan_Matrix_f *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1255,7 +1255,7 @@ static Gan_Matrix_f *
 
          for ( j = 0, Cptr = C->data; (unsigned long) j < C->cols;
                j++, Cptr += C->rows )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->rows,
                         U->data, Cptr, 1 );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1269,8 +1269,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  lower_tri_matrix_lmult (
-     Gan_SquMatrix_f *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *L, Gan_TposeFlag L_tr, Gan_InvertFlag L_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    long     i, j, k;
@@ -1327,7 +1327,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         L->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1354,7 +1354,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1384,7 +1384,7 @@ static Gan_Matrix_f *
          if ( B != C ) gan_scopy ( Crows*L->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT,
                         C->cols, L->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1412,7 +1412,7 @@ static Gan_Matrix_f *
          if ( B != C ) gan_scopy ( Crows*L->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, L->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1426,8 +1426,8 @@ static Gan_Matrix_f *
  */
 static Gan_Matrix_f *
  upper_tri_matrix_lmult (
-     Gan_SquMatrix_f *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
-     Gan_Matrix_f    *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
+     const Gan_SquMatrix_f *U, Gan_TposeFlag U_tr, Gan_InvertFlag U_inv,
+     Gan_Matrix_f          *B, Gan_TposeFlag B_tr, Gan_Matrix_f *C )
 {
    unsigned long Crows = B_tr ? B->cols : B->rows;
    long     i, j, k;
@@ -1484,7 +1484,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1511,7 +1511,7 @@ static Gan_Matrix_f *
 #else /* !HAVE_LAPACK */
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         U->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1543,7 +1543,7 @@ static Gan_Matrix_f *
 
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_TRANSPOSE, GAN_NOUNIT, C->cols, U->data,
                         Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1571,7 +1571,7 @@ static Gan_Matrix_f *
          if ( B != C ) gan_scopy ( Crows*U->size, B->data, 1, C->data, 1 );
          for ( i = 0, Cptr = C->data; (unsigned long) i < C->rows;
                i++, Cptr++ )
-            gan_stpsv ( GAN_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
+            gan_stpsv ( GAN_MATRIXPART_UPPER, GAN_NOTRANSPOSE, GAN_NOUNIT, C->cols,
                         U->data, Cptr, C->rows );
 #endif /* #ifdef HAVE_LAPACK */
       }
@@ -1633,7 +1633,7 @@ static Gan_SquMatrix_f *
             gan_err_test_ptr ( info == 0, "lower_tri_matrix_rmult_squ",
                                GAN_ERROR_FAILURE, "" );
 #else /* !HAVE_LAPACK */
-            if ( !gan_spptri ( GAN_UPPER, C->size, C->data, &info ) )
+            if ( !gan_spptri ( GAN_MATRIXPART_UPPER, C->size, C->data, &info ) )
             {
                gan_err_flush_trace();
                gan_err_register ( "lower_tri_matrix_rmult_squ",
@@ -1774,7 +1774,7 @@ static Gan_SquMatrix_f *
             gan_err_test_ptr ( info == 0, "upper_tri_matrix_rmult_squ",
                                GAN_ERROR_FAILURE, "" );
 #else /* !HAVE_LAPACK */
-            if ( !gan_spptri ( GAN_UPPER, C->size, C->data, &info ) )
+            if ( !gan_spptri ( GAN_MATRIXPART_UPPER, C->size, C->data, &info ) )
             {
                gan_err_flush_trace();
                gan_err_register ( "upper_tri_matrix_rmult_squ",
@@ -1816,7 +1816,7 @@ static Gan_SquMatrix_f *
 
 static Gan_SquMatrix_f *
  triangular_matrix_lrmult (
-     Gan_SquMatrix_f *A, Gan_Matrix_f *B, Gan_TposeFlag B_tr,
+     const Gan_SquMatrix_f *A, const Gan_Matrix_f *B, Gan_TposeFlag B_tr,
      Gan_Matrix_f *C, Gan_SquMatrix_f *D )
 {
    gan_err_flush_trace();
@@ -1886,7 +1886,7 @@ static Gan_SquMatrix_f *
    gan_scopy ( A->size*(A->size+1)/2, A->data, 1, B->data, 1 );
 
    /* perform matrix inversion */
-   if ( !gan_stptri ( GAN_UPPER, GAN_NOUNIT, B->size, B->data, &info ) )
+   if ( !gan_stptri ( GAN_MATRIXPART_UPPER, GAN_NOUNIT, B->size, B->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -1917,7 +1917,7 @@ static Gan_SquMatrix_f *
 /* Compute squared Frobenius norm of triangular matrix A.
  */
 static float
- triangular_matrix_sumsqr ( Gan_SquMatrix_f *A )
+ triangular_matrix_sumsqr ( const Gan_SquMatrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -1931,7 +1931,7 @@ static float
 /* Compute Frobenius norm of triangular matrix A.
  */
 static float
- triangular_matrix_Fnorm ( Gan_SquMatrix_f *A )
+ triangular_matrix_Fnorm ( const Gan_SquMatrix_f *A )
 {
    long i;
    float total = 0.0F;
@@ -1944,7 +1944,7 @@ static float
 
 /* Convert lower triangular matrix to generic matrix */
 static Gan_Matrix_f *
- lower_tri_matrix_matrix ( Gan_SquMatrix_f *L, struct Gan_Matrix_f *B )
+ lower_tri_matrix_matrix ( const Gan_SquMatrix_f *L, struct Gan_Matrix_f *B )
 {
    long i, j;
 
@@ -1976,7 +1976,7 @@ static Gan_Matrix_f *
 
 /* Convert upper triangular matrix to generic matrix */
 static Gan_Matrix_f *
- upper_tri_matrix_matrix ( Gan_SquMatrix_f *U, struct Gan_Matrix_f *B )
+ upper_tri_matrix_matrix ( const Gan_SquMatrix_f *U, struct Gan_Matrix_f *B )
 {
    unsigned long i, j;
 
@@ -2287,7 +2287,7 @@ Gan_SquMatrix_f *
  * size of the matrix \a L.
  */
 Gan_Bool
- gan_ltmatf_read_va ( Gan_SquMatrix_f *L, unsigned long size, ... )
+ gan_ltmatf_read_va ( const Gan_SquMatrix_f *L, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
@@ -2317,7 +2317,7 @@ Gan_Bool
  * size of the matrix \a U.
  */
 Gan_Bool
- gan_utmatf_read_va ( Gan_SquMatrix_f *U, unsigned long size, ... )
+ gan_utmatf_read_va ( const Gan_SquMatrix_f *U, unsigned long size, ... )
 {
    unsigned int i, j;
    va_list ap;
