@@ -3,8 +3,8 @@
  * Module:        Fixed size matrix functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.21 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.24 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -50,7 +50,7 @@
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- GAN_MAT_PRINT ( GAN_MATTYPE *A,
+ GAN_MAT_PRINT ( const GAN_MATTYPE *A,
                  const char *prefix, int indent, const char *fmt )
 {
    return GAN_MAT_FPRINT ( stdout, A, prefix, indent, fmt );
@@ -67,7 +67,7 @@ Gan_Bool
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- GAN_MAT_FWRITE ( FILE *fp, GAN_MATTYPE *A, gan_ui32 magic_number )
+ GAN_MAT_FWRITE ( FILE *fp, const GAN_MATTYPE *A, gan_uint32 magic_number )
 {
    if ( !gan_fwrite_lendian_ui32 ( fp, &magic_number, 1 ) )
    {
@@ -75,23 +75,12 @@ Gan_Bool
       return GAN_FALSE;
    }
 
-#if (GAN_MAT_ELEMENT_TYPE == GAN_DOUBLE)
-   if ( !gan_fwrite_lendian_d ( fp, (double *)A,
-                                sizeof(GAN_MATTYPE)/sizeof(double) ) )
+   if ( !GAN_FWRITE_LENDIAN ( fp, (GAN_REALTYPE *)A,
+                              sizeof(GAN_MATTYPE)/sizeof(GAN_REALTYPE) ) )
    {
       gan_err_register ( "GAN_MAT_FWRITE", GAN_ERROR_CORRUPTED_FILE, "" );
       return GAN_FALSE;
    }
-#elif (GAN_MAT_ELEMENT_TYPE == GAN_FLOAT)
-   if ( !gan_fwrite_lendian_f ( fp, (float *)A,
-                                sizeof(GAN_MATTYPE)/sizeof(float) ) )
-   {
-      gan_err_register ( "GAN_MAT_FWRITE", GAN_ERROR_CORRUPTED_FILE, "" );
-      return GAN_FALSE;
-   }
-#else
-#error "unsupported size"      
-#endif
 
    return GAN_TRUE;
 }
@@ -107,7 +96,7 @@ Gan_Bool
  * \return #GAN_TRUE on success, #GAN_FALSE on failure.
  */
 Gan_Bool
- GAN_MAT_FREAD ( FILE *fp, GAN_MATTYPE *A, gan_ui32 *magic_number )
+ GAN_MAT_FREAD ( FILE *fp, GAN_MATTYPE *A, gan_uint32 *magic_number )
 {
    if ( !gan_fread_lendian_ui32 ( fp, magic_number, 1 ) )
    {
@@ -115,23 +104,12 @@ Gan_Bool
       return GAN_FALSE;
    }
 
-#if (GAN_MAT_ELEMENT_TYPE == GAN_DOUBLE)
-   if ( !gan_fread_lendian_d ( fp, (double *)A,
-                               sizeof(GAN_MATTYPE)/sizeof(double) ) )
+   if ( !GAN_FREAD_LENDIAN ( fp, (GAN_REALTYPE *)A,
+                             sizeof(GAN_MATTYPE)/sizeof(GAN_REALTYPE) ) )
    {
       gan_err_register ( "GAN_MAT_FREAD", GAN_ERROR_CORRUPTED_FILE, "" );
       return GAN_FALSE;
    }
-#elif (GAN_MAT_ELEMENT_TYPE == GAN_FLOAT)
-   if ( !gan_fread_lendian_f ( fp, (float *)A,
-                               sizeof(GAN_MATTYPE)/sizeof(float) ) )
-   {
-      gan_err_register ( "GAN_MAT_FREAD", GAN_ERROR_CORRUPTED_FILE, "" );
-      return GAN_FALSE;
-   }
-#else
-#error "unsupported size"      
-#endif
 
    /* set matrix type if specified */
 #ifdef GAN_FIXED_MATRIX_TYPE
@@ -183,7 +161,7 @@ GAN_MATTYPE
  * matrix.
  */
 GAN_MATTYPE
- GAN_MAT_COPY_S ( GAN_MATTYPE *A )
+ GAN_MAT_COPY_S ( const GAN_MATTYPE *A )
 {
    GAN_MATTYPE B;
 
@@ -208,7 +186,7 @@ GAN_MATTYPE
  * Return a copy of fixed size matrix \a A multiplied by scalar \a a.
  */
 GAN_MATTYPE
- GAN_MAT_SCALE_S ( GAN_MATTYPE *A, GAN_REALTYPE a )
+ GAN_MAT_SCALE_S ( const GAN_MATTYPE *A, GAN_REALTYPE a )
 {
    GAN_MATTYPE B;
 
@@ -224,7 +202,7 @@ GAN_MATTYPE
  * Return a copy of fixed size matrix \a A divided by scalar \a a.
  */
 GAN_MATTYPE
- GAN_MAT_DIVIDE_S ( GAN_MATTYPE *A, GAN_REALTYPE a )
+ GAN_MAT_DIVIDE_S ( const GAN_MATTYPE *A, GAN_REALTYPE a )
 {
    GAN_MATTYPE B;
 
@@ -240,7 +218,7 @@ GAN_MATTYPE
  * Return a negated copy of fixed size matrix \a A.
  */
 GAN_MATTYPE
- GAN_MAT_NEGATE_S ( GAN_MATTYPE *A )
+ GAN_MAT_NEGATE_S ( const GAN_MATTYPE *A )
 {
    GAN_MATTYPE B;
 
@@ -283,11 +261,11 @@ GAN_MATTYPE *
  * \return Scaled matrix \a A.
  */
 GAN_MATTYPE
- GAN_MAT_UNIT_S ( GAN_MATTYPE *A )
+ GAN_MAT_UNIT_S ( const GAN_MATTYPE *A )
 {
    GAN_MATTYPE B;
 
-   if ( GAN_MAT_UNIT_Q ( A, &B ) == NULL )
+   if ( GAN_MAT_UNIT_Q ( (GAN_MATTYPE *)A, &B ) == NULL )
       assert(0);
 
    return B;
@@ -311,7 +289,7 @@ GAN_MATTYPE
  * fixed size matrix \f$ A + B \f$.
  */
 GAN_MATTYPE
- GAN_MAT_ADD_S ( GAN_MATTYPE *A, GAN_MATTYPE *B )
+ GAN_MAT_ADD_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B )
 {
    GAN_MATTYPE C;
 
@@ -337,7 +315,7 @@ GAN_MATTYPE
  * fixed size matrix \f$ A - B \f$.
  */
 GAN_MATTYPE
- GAN_MAT_SUB_S ( GAN_MATTYPE *A, GAN_MATTYPE *B )
+ GAN_MAT_SUB_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B )
 {
    GAN_MATTYPE C;
 
@@ -362,7 +340,7 @@ GAN_MATTYPE
  * Returns the outer product \f$ p q^{\top} \f$ of two vectors \a p and \a q.
  */
 GAN_MATTYPE
- GAN_VEC_OUTER_S ( GAN_VECTYPE1 *p, GAN_VECTYPE2 *q )
+ GAN_VEC_OUTER_S ( const GAN_VECTYPE1 *p, const GAN_VECTYPE2 *q )
 {
    GAN_MATTYPE A;
 
@@ -378,7 +356,7 @@ GAN_MATTYPE
  * Returns the product \f$ A p \f$ of a matrix \a A and a vector \a p
  */
 GAN_VECTYPE1
- GAN_MAT_MULTV_S ( GAN_MATTYPE *A, GAN_VECTYPE2 *p )
+ GAN_MAT_MULTV_S ( const GAN_MATTYPE *A, const GAN_VECTYPE2 *p )
 {
    GAN_VECTYPE1 q;
 
@@ -395,7 +373,7 @@ GAN_VECTYPE1
  * vector \a p.
  */
 GAN_VECTYPE2
- GAN_MATT_MULTV_S ( GAN_MATTYPE *A, GAN_VECTYPE1 *p )
+ GAN_MATT_MULTV_S ( const GAN_MATTYPE *A, const GAN_VECTYPE1 *p )
 {
    GAN_VECTYPE2 q;
 
@@ -421,7 +399,7 @@ GAN_VECTYPE2
  * another matrix \a B. \a B is square but of generic type #GAN_MATTYPEL.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTM_S ( GAN_MATTYPE *A, GAN_MATTYPEL *B )
+ GAN_MAT_LMULTM_S ( const GAN_MATTYPE *A, const GAN_MATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -438,7 +416,7 @@ GAN_MATTYPE
  * with the transpose of a square matrix \a B of generic type #GAN_MATTYPEL.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTMT_S ( GAN_MATTYPE *A, GAN_MATTYPEL *B )
+ GAN_MAT_LMULTMT_S ( const GAN_MATTYPE *A, const GAN_MATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -455,7 +433,7 @@ GAN_MATTYPE
  * square matrix \a B of generic type #GAN_MATTYPER.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTM_S ( GAN_MATTYPE *A, GAN_MATTYPER *B )
+ GAN_MAT_RMULTM_S ( const GAN_MATTYPE *A, const GAN_MATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -472,7 +450,7 @@ GAN_MATTYPE
  * with the transpose of a square matrix \a B of generic type #GAN_MATTYPER.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTMT_S ( GAN_MATTYPE *A, GAN_MATTYPER *B )
+ GAN_MAT_RMULTMT_S ( const GAN_MATTYPE *A, const GAN_MATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -491,7 +469,7 @@ GAN_MATTYPE
  * so this function should be used with care.
  */
 GAN_SQUMATTYPER
- GAN_MAT_LMULTMT_SYM_S ( GAN_MATTYPE *A, GAN_MATTYPE *B )
+ GAN_MAT_LMULTMT_SYM_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B )
 {
    GAN_SQUMATTYPER C;
 
@@ -510,7 +488,7 @@ GAN_SQUMATTYPER
  * so this function should be used with care.
  */
 GAN_SQUMATTYPEL
- GAN_MAT_RMULTMT_SYM_S ( GAN_MATTYPE *A, GAN_MATTYPE *B )
+ GAN_MAT_RMULTMT_SYM_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B )
 {
    GAN_SQUMATTYPEL C;
 
@@ -528,7 +506,7 @@ GAN_SQUMATTYPEL
  *
  * \return The result \f$ A^{\top} A \f$ as a new matrix.
  */
-GAN_SQUMATTYPER GAN_MAT_SLMULTT_S ( GAN_MATTYPE *A )
+GAN_SQUMATTYPER GAN_MAT_SLMULTT_S ( const GAN_MATTYPE *A )
 {
    GAN_SQUMATTYPER B;
 
@@ -546,7 +524,7 @@ GAN_SQUMATTYPER GAN_MAT_SLMULTT_S ( GAN_MATTYPE *A )
  *
  * \return The result \f$ A A^{\top} \f$ as a new matrix.
  */
-GAN_SQUMATTYPEL GAN_MAT_SRMULTT_S ( GAN_MATTYPE *A )
+GAN_SQUMATTYPEL GAN_MAT_SRMULTT_S ( const GAN_MATTYPE *A )
 {
    GAN_SQUMATTYPEL B;
 
@@ -562,7 +540,7 @@ GAN_SQUMATTYPEL GAN_MAT_SRMULTT_S ( GAN_MATTYPE *A )
  * Returns the result \f$ B A \f$ of left-multiplying a matrix \a A with a
  * symmetric matrix \a B.
  */
-GAN_MATTYPE GAN_MAT_LMULTS_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
+GAN_MATTYPE GAN_MAT_LMULTS_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -579,7 +557,7 @@ GAN_MATTYPE GAN_MAT_LMULTS_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
  * symmetric matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTS_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B )
+ GAN_MAT_RMULTS_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -607,7 +585,7 @@ GAN_MATTYPE
  * you don't need this result.
  */
 GAN_SQUMATTYPER
- GAN_SYMMATL_LRMULT_S ( GAN_SQUMATTYPEL *A, GAN_MATTYPE *B, GAN_MATTYPE *C )
+ GAN_SYMMATL_LRMULT_S ( const GAN_SQUMATTYPEL *A, const GAN_MATTYPE *B, GAN_MATTYPE *C )
 {
    GAN_SQUMATTYPER D;
    GAN_MATTYPE Cp;
@@ -627,7 +605,7 @@ GAN_SQUMATTYPER
  * be passed as \c NULL if you don't need this result.
  */
 GAN_SQUMATTYPEL
- GAN_SYMMATR_LRMULT_S ( GAN_SQUMATTYPER *A, GAN_MATTYPE *B, GAN_MATTYPE *C )
+ GAN_SYMMATR_LRMULT_S ( const GAN_SQUMATTYPER *A, const GAN_MATTYPE *B, GAN_MATTYPE *C )
 {
    GAN_SQUMATTYPEL D;
    GAN_MATTYPE Cp;
@@ -654,7 +632,7 @@ GAN_SQUMATTYPEL
  * lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTL_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
+ GAN_MAT_LMULTL_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -671,7 +649,7 @@ GAN_MATTYPE
  * with the inverse of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTLI_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
+ GAN_MAT_LMULTLI_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -688,7 +666,7 @@ GAN_MATTYPE
  * with the transpose of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTLT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
+ GAN_MAT_LMULTLT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -705,7 +683,7 @@ GAN_MATTYPE
  * with the inverse transpose of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_LMULTLIT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B )
+ GAN_MAT_LMULTLIT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B )
 {
    GAN_MATTYPE C;
 
@@ -722,7 +700,7 @@ GAN_MATTYPE
  * lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTL_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B )
+ GAN_MAT_RMULTL_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -739,7 +717,7 @@ GAN_MATTYPE
  * with the inverse of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTLI_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B )
+ GAN_MAT_RMULTLI_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -756,7 +734,7 @@ GAN_MATTYPE
  * with the transpose of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTLT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B )
+ GAN_MAT_RMULTLT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -773,7 +751,7 @@ GAN_MATTYPE
  * with the inverse transpose of a lower-triangular matrix \a B.
  */
 GAN_MATTYPE
- GAN_MAT_RMULTLIT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B )
+ GAN_MAT_RMULTLIT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B )
 {
    GAN_MATTYPE C;
 
@@ -799,7 +777,7 @@ GAN_MATTYPE
  * function.
  */
 GAN_REALTYPE
- GAN_MAT_SUMSQR_S ( GAN_MATTYPE *A )
+ GAN_MAT_SUMSQR_S ( const GAN_MATTYPE *A )
 {
    return (GAN_REALTYPE) GAN_MAT_SUMSQR_Q ( A );
 }
@@ -813,7 +791,7 @@ GAN_REALTYPE
  * sum of squares of the elements of \a A. Implemented as a function.
  */
 GAN_REALTYPE
- GAN_MAT_FNORM_S ( GAN_MATTYPE *A )
+ GAN_MAT_FNORM_S ( const GAN_MATTYPE *A )
 {
    return (GAN_REALTYPE) GAN_MAT_FNORM_Q ( A );
 }
@@ -836,7 +814,7 @@ GAN_REALTYPE
  * returning the result as a new matrix.
  */
 GAN_MATTYPE
- GAN_MAT_FROM_MAT_S ( GAN_MATRIX_TYPE *A )
+ GAN_MAT_FROM_MAT_S ( const GAN_MATRIX_TYPE *A )
 {
    GAN_MATTYPE B;
 
@@ -855,7 +833,7 @@ GAN_MATTYPE
  * result as a new matrix.
  */
 GAN_MATTYPE
- GAN_MAT_FROM_SQUMAT_S ( GAN_SQUMATRIX_TYPE *A )
+ GAN_MAT_FROM_SQUMAT_S ( const GAN_SQUMATRIX_TYPE *A )
 {
    GAN_MATTYPE B;
 
@@ -874,7 +852,7 @@ GAN_MATTYPE
  * fixed size matrix.
  */
 GAN_SQUMATTYPEL
- GAN_SQUMAT_FROM_SQUMAT_S ( GAN_SQUMATRIX_TYPE *A )
+ GAN_SQUMAT_FROM_SQUMAT_S ( const GAN_SQUMATRIX_TYPE *A )
 {
    GAN_SQUMATTYPEL B;
 
