@@ -3,8 +3,8 @@
  * Module:        Fixed size matrix functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.19 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.22 $
+ * Last edited:   $Date: 2005/02/20 04:12:45 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  *
@@ -31,12 +31,12 @@
 #ifndef GAN_GENERATE_DOCUMENTATION
 
 #ifdef GAN_MAT_FPRINT
-Gan_Bool GAN_MAT_FPRINT ( FILE *fp, GAN_MATTYPE *A, const char *prefix,
+Gan_Bool GAN_MAT_FPRINT ( FILE *fp, const GAN_MATTYPE *A, const char *prefix,
                           int indent, const char *fmt );
 #endif
 
 #ifdef GAN_MAT_PRINT
-Gan_Bool GAN_MAT_PRINT ( GAN_MATTYPE *A, const char *prefix,
+Gan_Bool GAN_MAT_PRINT ( const GAN_MATTYPE *A, const char *prefix,
                          int indent, const char *fmt );
 #endif
 
@@ -46,11 +46,11 @@ Gan_Bool GAN_MAT_FSCANF ( FILE *fp, GAN_MATTYPE *A,
 #endif
 
 #ifdef GAN_MAT_FWRITE
-Gan_Bool GAN_MAT_FWRITE ( FILE *fp, GAN_MATTYPE *A, gan_ui32 magic_number );
+Gan_Bool GAN_MAT_FWRITE ( FILE *fp, const GAN_MATTYPE *A, gan_uint32 magic_number );
 #endif
 
 #ifdef GAN_MAT_FREAD
-Gan_Bool GAN_MAT_FREAD ( FILE *fp, GAN_MATTYPE *A, gan_ui32 *magic_number );
+Gan_Bool GAN_MAT_FREAD ( FILE *fp, GAN_MATTYPE *A, gan_uint32 *magic_number );
 #endif
 
 #ifdef GAN_MAT_ZERO_S
@@ -58,19 +58,19 @@ GAN_MATTYPE GAN_MAT_ZERO_S (void);
 #endif
 
 #ifdef GAN_MAT_COPY_S
-GAN_MATTYPE GAN_MAT_COPY_S ( GAN_MATTYPE *A );
+GAN_MATTYPE GAN_MAT_COPY_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_SCALE_S
-GAN_MATTYPE GAN_MAT_SCALE_S ( GAN_MATTYPE *A, GAN_REALTYPE a );
+GAN_MATTYPE GAN_MAT_SCALE_S ( const GAN_MATTYPE *A, GAN_REALTYPE a );
 #endif
 
 #ifdef GAN_MAT_DIVIDE_S
-GAN_MATTYPE GAN_MAT_DIVIDE_S ( GAN_MATTYPE *A, GAN_REALTYPE a );
+GAN_MATTYPE GAN_MAT_DIVIDE_S ( const GAN_MATTYPE *A, GAN_REALTYPE a );
 #endif
 
 #ifdef GAN_MAT_NEGATE_S
-GAN_MATTYPE GAN_MAT_NEGATE_S ( GAN_MATTYPE *A );
+GAN_MATTYPE GAN_MAT_NEGATE_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_UNIT_Q
@@ -78,129 +78,129 @@ GAN_MATTYPE *GAN_MAT_UNIT_Q ( GAN_MATTYPE *A, GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_MAT_UNIT_S
-GAN_MATTYPE GAN_MAT_UNIT_S ( GAN_MATTYPE *A );
+GAN_MATTYPE GAN_MAT_UNIT_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_ADD_S
-GAN_MATTYPE GAN_MAT_ADD_S ( GAN_MATTYPE *A, GAN_MATTYPE *B );
+GAN_MATTYPE GAN_MAT_ADD_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_MAT_SUB_S
-GAN_MATTYPE GAN_MAT_SUB_S ( GAN_MATTYPE *A, GAN_MATTYPE *B );
+GAN_MATTYPE GAN_MAT_SUB_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_VEC_OUTER_S
-GAN_MATTYPE GAN_VEC_OUTER_S ( GAN_VECTYPE1 *p, GAN_VECTYPE2 *q );
+GAN_MATTYPE GAN_VEC_OUTER_S ( const GAN_VECTYPE1 *p, const GAN_VECTYPE2 *q );
 #endif
 
 #ifdef GAN_MAT_MULTV_S
-GAN_VECTYPE1 GAN_MAT_MULTV_S ( GAN_MATTYPE *A, GAN_VECTYPE2 *p );
+GAN_VECTYPE1 GAN_MAT_MULTV_S ( const GAN_MATTYPE *A, const GAN_VECTYPE2 *p );
 #endif
 
 #ifdef GAN_MATT_MULTV_S
-GAN_VECTYPE2 GAN_MATT_MULTV_S ( GAN_MATTYPE *A, GAN_VECTYPE1 *p );
+GAN_VECTYPE2 GAN_MATT_MULTV_S ( const GAN_MATTYPE *A, const GAN_VECTYPE1 *p );
 #endif
 
 #ifdef GAN_MAT_LMULTM_S
-GAN_MATTYPE GAN_MAT_LMULTM_S ( GAN_MATTYPE *A, GAN_MATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTM_S ( const GAN_MATTYPE *A, const GAN_MATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTMT_S
-GAN_MATTYPE GAN_MAT_LMULTMT_S ( GAN_MATTYPE *A, GAN_MATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTMT_S ( const GAN_MATTYPE *A, const GAN_MATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_RMULTM_S
-GAN_MATTYPE GAN_MAT_RMULTM_S ( GAN_MATTYPE *A, GAN_MATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTM_S ( const GAN_MATTYPE *A, const GAN_MATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTMT_S
-GAN_MATTYPE GAN_MAT_RMULTMT_S ( GAN_MATTYPE *A, GAN_MATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTMT_S ( const GAN_MATTYPE *A, const GAN_MATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_LMULTMT_SYM_S
-GAN_SQUMATTYPER GAN_MAT_LMULTMT_SYM_S ( GAN_MATTYPE *A, GAN_MATTYPE *B );
+GAN_SQUMATTYPER GAN_MAT_LMULTMT_SYM_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_MAT_RMULTMT_SYM_S
-GAN_SQUMATTYPEL GAN_MAT_RMULTMT_SYM_S ( GAN_MATTYPE *A, GAN_MATTYPE *B );
+GAN_SQUMATTYPEL GAN_MAT_RMULTMT_SYM_S ( const GAN_MATTYPE *A, const GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_MAT_SRMULTT_S
-GAN_SQUMATTYPEL GAN_MAT_SRMULTT_S ( GAN_MATTYPE *A );
+GAN_SQUMATTYPEL GAN_MAT_SRMULTT_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_SLMULTT_S
-GAN_SQUMATTYPER GAN_MAT_SLMULTT_S ( GAN_MATTYPE *A );
+GAN_SQUMATTYPER GAN_MAT_SLMULTT_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_LMULTS_S
-GAN_MATTYPE GAN_MAT_LMULTS_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTS_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_RMULTS_S
-GAN_MATTYPE GAN_MAT_RMULTS_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTS_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_SYMMATL_LRMULT_S
-GAN_SQUMATTYPER GAN_SYMMATL_LRMULT_S ( GAN_SQUMATTYPEL *A, GAN_MATTYPE *B,
+GAN_SQUMATTYPER GAN_SYMMATL_LRMULT_S ( const GAN_SQUMATTYPEL *A, const GAN_MATTYPE *B,
                                        GAN_MATTYPE *C );
 #endif
 
 #ifdef GAN_SYMMATR_LRMULT_S
-GAN_SQUMATTYPEL GAN_SYMMATR_LRMULT_S ( GAN_SQUMATTYPER *A, GAN_MATTYPE *B,
+GAN_SQUMATTYPEL GAN_SYMMATR_LRMULT_S ( const GAN_SQUMATTYPER *A, const GAN_MATTYPE *B,
                                        GAN_MATTYPE *C );
 #endif
 
 #ifdef GAN_MAT_LMULTL_S
-GAN_MATTYPE GAN_MAT_LMULTL_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTL_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLI_S
-GAN_MATTYPE GAN_MAT_LMULTLI_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTLI_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLT_S
-GAN_MATTYPE GAN_MAT_LMULTLT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTLT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLIT_S
-GAN_MATTYPE GAN_MAT_LMULTLIT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE GAN_MAT_LMULTLIT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_RMULTL_S
-GAN_MATTYPE GAN_MAT_RMULTL_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTL_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLI_S
-GAN_MATTYPE GAN_MAT_RMULTLI_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTLI_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLT_S
-GAN_MATTYPE GAN_MAT_RMULTLT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTLT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLIT_S
-GAN_MATTYPE GAN_MAT_RMULTLIT_S ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE GAN_MAT_RMULTLIT_S ( const GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_SUMSQR_S
-GAN_REALTYPE GAN_MAT_SUMSQR_S ( GAN_MATTYPE *A );
+GAN_REALTYPE GAN_MAT_SUMSQR_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_FNORM_S
-GAN_REALTYPE GAN_MAT_FNORM_S ( GAN_MATTYPE *A );
+GAN_REALTYPE GAN_MAT_FNORM_S ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_FROM_MAT_S
-GAN_MATTYPE GAN_MAT_FROM_MAT_S ( GAN_MATRIX_TYPE *A );
+GAN_MATTYPE GAN_MAT_FROM_MAT_S ( const GAN_MATRIX_TYPE *A );
 #endif
 
 #ifdef GAN_MAT_FROM_SQUMAT_S
-GAN_MATTYPE GAN_MAT_FROM_SQUMAT_S ( GAN_SQUMATRIX_TYPE *A );
+GAN_MATTYPE GAN_MAT_FROM_SQUMAT_S ( const GAN_SQUMATRIX_TYPE *A );
 #endif
 
 #ifdef GAN_SQUMAT_FROM_SQUMAT_S
-GAN_SQUMATTYPEL GAN_SQUMAT_FROM_SQUMAT_S ( GAN_SQUMATRIX_TYPE *A );
+GAN_SQUMATTYPEL GAN_SQUMAT_FROM_SQUMAT_S ( const GAN_SQUMATRIX_TYPE *A );
 #endif
 
 #endif /* #ifndef GAN_GENERATE_DOCUMENTATION */
@@ -220,7 +220,7 @@ GAN_SQUMATTYPEL GAN_SQUMAT_FROM_SQUMAT_S ( GAN_SQUMATRIX_TYPE *A );
  *
  * Fill fixed size matrix \a A with zero, and return \a A.
  */
-GAN_MATTYPE *GAN_MAT_ZERO_Q ( GAN_MATTYPE *A );
+GAN_MATTYPE *GAN_MAT_ZERO_Q ( const GAN_MATTYPE *A );
 #endif
 
 /**
@@ -465,7 +465,7 @@ GAN_MATTYPE *GAN_MAT_DECREMENT ( GAN_MATTYPE *A, GAN_MATTYPE *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_VEC_OUTER_Q ( GAN_VECTYPE1 *p, GAN_VECTYPE2 *q,
+GAN_MATTYPE *GAN_VEC_OUTER_Q ( const GAN_VECTYPE1 *p, const GAN_VECTYPE2 *q,
                                GAN_MATTYPE *A );
 #endif
 
@@ -478,7 +478,7 @@ GAN_MATTYPE *GAN_VEC_OUTER_Q ( GAN_VECTYPE1 *p, GAN_VECTYPE2 *q,
  *
  * \return Pointer to result vector \a q.
  */
-GAN_VECTYPE1 *GAN_MAT_MULTV_Q ( GAN_MATTYPE *A, GAN_VECTYPE2 *p,
+GAN_VECTYPE1 *GAN_MAT_MULTV_Q ( const GAN_MATTYPE *A, GAN_VECTYPE2 *p,
                                 GAN_VECTYPE1 *q );
 #endif
 
@@ -491,7 +491,7 @@ GAN_VECTYPE1 *GAN_MAT_MULTV_Q ( GAN_MATTYPE *A, GAN_VECTYPE2 *p,
  *
  * \return Pointer to result vector \a p.
  */
-GAN_VECTYPE1 *GAN_MAT_MULTV_I ( GAN_MATTYPE *A, GAN_VECTYPE2 *p );
+GAN_VECTYPE1 *GAN_MAT_MULTV_I ( const GAN_MATTYPE *A, GAN_VECTYPE2 *p );
 #endif
 
 #ifdef GAN_MATT_MULTV_Q
@@ -503,7 +503,7 @@ GAN_VECTYPE1 *GAN_MAT_MULTV_I ( GAN_MATTYPE *A, GAN_VECTYPE2 *p );
  *
  * \return Pointer to result vector \a q.
  */
-GAN_VECTYPE2 *GAN_MATT_MULTV_Q ( GAN_MATTYPE *A, GAN_VECTYPE1 *p,
+GAN_VECTYPE2 *GAN_MATT_MULTV_Q ( const GAN_MATTYPE *A, GAN_VECTYPE1 *p,
                                  GAN_VECTYPE2 *q );
 #endif
 
@@ -516,7 +516,7 @@ GAN_VECTYPE2 *GAN_MATT_MULTV_Q ( GAN_MATTYPE *A, GAN_VECTYPE1 *p,
  *
  * \return Pointer to result vector \a p.
  */
-GAN_VECTYPE2 *GAN_MATT_MULTV_I ( GAN_MATTYPE *A, GAN_VECTYPE1 *p );
+GAN_VECTYPE2 *GAN_MATT_MULTV_I ( const GAN_MATTYPE *A, GAN_VECTYPE1 *p );
 #endif
 
 /**
@@ -538,7 +538,7 @@ GAN_VECTYPE2 *GAN_MATT_MULTV_I ( GAN_MATTYPE *A, GAN_VECTYPE1 *p );
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTM_Q ( GAN_MATTYPE *A, GAN_MATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTM_Q ( GAN_MATTYPE *A, const GAN_MATTYPEL *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -552,7 +552,7 @@ GAN_MATTYPE *GAN_MAT_LMULTM_Q ( GAN_MATTYPE *A, GAN_MATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTMT_Q ( GAN_MATTYPE *A, GAN_MATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTMT_Q ( GAN_MATTYPE *A, const GAN_MATTYPEL *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -566,7 +566,7 @@ GAN_MATTYPE *GAN_MAT_LMULTMT_Q ( GAN_MATTYPE *A, GAN_MATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTM_Q ( GAN_MATTYPE *A, GAN_MATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTM_Q ( GAN_MATTYPE *A, const GAN_MATTYPER *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -580,7 +580,7 @@ GAN_MATTYPE *GAN_MAT_RMULTM_Q ( GAN_MATTYPE *A, GAN_MATTYPER *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTMT_Q ( GAN_MATTYPE *A, GAN_MATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTMT_Q ( GAN_MATTYPE *A, const GAN_MATTYPER *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -596,7 +596,7 @@ GAN_MATTYPE *GAN_MAT_RMULTMT_Q ( GAN_MATTYPE *A, GAN_MATTYPER *B,
  *
  * \return Pointer to result symmetric matrix \a C.
  */
-GAN_SQUMATTYPER *GAN_MAT_LMULTMT_SYM_Q ( GAN_MATTYPE *A, GAN_MATTYPE *B,
+GAN_SQUMATTYPER *GAN_MAT_LMULTMT_SYM_Q ( const GAN_MATTYPE *A, const GAN_MATTYPE *B,
                                          GAN_SQUMATTYPER *C );
 #endif
 
@@ -612,7 +612,7 @@ GAN_SQUMATTYPER *GAN_MAT_LMULTMT_SYM_Q ( GAN_MATTYPE *A, GAN_MATTYPE *B,
  *
  * \return Pointer to result symmetric matrix \a C.
  */
-GAN_SQUMATTYPEL *GAN_MAT_RMULTMT_SYM_Q ( GAN_MATTYPE *A, GAN_MATTYPE *B,
+GAN_SQUMATTYPEL *GAN_MAT_RMULTMT_SYM_Q ( const GAN_MATTYPE *A, const GAN_MATTYPE *B,
                                          GAN_SQUMATTYPEL *C );
 #endif
 
@@ -625,7 +625,7 @@ GAN_SQUMATTYPEL *GAN_MAT_RMULTMT_SYM_Q ( GAN_MATTYPE *A, GAN_MATTYPE *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTS_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTS_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -638,7 +638,7 @@ GAN_MATTYPE *GAN_MAT_LMULTS_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTS_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTS_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -662,8 +662,8 @@ GAN_MATTYPE *GAN_MAT_RMULTS_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
  *
  * \return Pointer to result symmetric matrix \a D.
  */
-GAN_SQUMATTYPER *GAN_SYMMATL_LRMULT_Q ( GAN_SQUMATTYPEL *A, GAN_MATTYPE *B,
-                                        GAN_MATTYPE *C, GAN_SQUMATTYPER *D );
+GAN_SQUMATTYPER *GAN_SYMMATL_LRMULT_Q ( const GAN_SQUMATTYPEL *A, const GAN_MATTYPE *B,
+                                        const GAN_MATTYPE *C, GAN_SQUMATTYPER *D );
 #endif
 
 #ifdef  GAN_SYMMATR_LRMULT_Q
@@ -677,8 +677,8 @@ GAN_SQUMATTYPER *GAN_SYMMATL_LRMULT_Q ( GAN_SQUMATTYPEL *A, GAN_MATTYPE *B,
  *
  * \return Pointer to result symmetric matrix \a D.
  */
-GAN_SQUMATTYPEL *GAN_SYMMATR_LRMULT_Q ( GAN_SQUMATTYPER *A, GAN_MATTYPE *B,
-                                        GAN_MATTYPE *C, GAN_SQUMATTYPEL *D );
+GAN_SQUMATTYPEL *GAN_SYMMATR_LRMULT_Q ( const GAN_SQUMATTYPER *A, const GAN_MATTYPE *B,
+                                        const GAN_MATTYPE *C, GAN_SQUMATTYPEL *D );
 #endif
 
 /**
@@ -700,7 +700,7 @@ GAN_SQUMATTYPEL *GAN_SYMMATR_LRMULT_Q ( GAN_SQUMATTYPER *A, GAN_MATTYPE *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTL_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTL_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -714,7 +714,7 @@ GAN_MATTYPE *GAN_MAT_LMULTL_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLI_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTLI_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -728,7 +728,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLI_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTLT_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -742,7 +742,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLIT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
+GAN_MATTYPE *GAN_MAT_LMULTLIT_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B,
                                   GAN_MATTYPE *C );
 #endif
 
@@ -756,7 +756,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLIT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTL_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTL_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B,
                                 GAN_MATTYPE *C );
 #endif
 
@@ -770,7 +770,7 @@ GAN_MATTYPE *GAN_MAT_RMULTL_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLI_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTLI_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -784,7 +784,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLI_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTLT_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B,
                                  GAN_MATTYPE *C );
 #endif
 
@@ -798,7 +798,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
  *
  * \return Pointer to result matrix \a C.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLIT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
+GAN_MATTYPE *GAN_MAT_RMULTLIT_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B,
                                   GAN_MATTYPE *C );
 #endif
 
@@ -812,7 +812,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLIT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B,
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_LMULTL_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE *GAN_MAT_LMULTL_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLI_I
@@ -825,7 +825,7 @@ GAN_MATTYPE *GAN_MAT_LMULTL_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLI_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE *GAN_MAT_LMULTLI_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLT_I
@@ -838,7 +838,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLI_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE *GAN_MAT_LMULTLT_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_LMULTLIT_I
@@ -851,7 +851,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_LMULTLIT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_MATTYPE *GAN_MAT_LMULTLIT_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_RMULTL_I
@@ -864,7 +864,7 @@ GAN_MATTYPE *GAN_MAT_LMULTLIT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_RMULTL_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE *GAN_MAT_RMULTL_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLI_I
@@ -877,7 +877,7 @@ GAN_MATTYPE *GAN_MAT_RMULTL_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLI_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE *GAN_MAT_RMULTLI_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLT_I
@@ -890,7 +890,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLI_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE *GAN_MAT_RMULTLT_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_RMULTLIT_I
@@ -903,7 +903,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
  *
  * \return Pointer to result matrix \a A.
  */
-GAN_MATTYPE *GAN_MAT_RMULTLIT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_MATTYPE *GAN_MAT_RMULTLIT_I ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_SLMULTT_Q
@@ -915,7 +915,7 @@ GAN_MATTYPE *GAN_MAT_RMULTLIT_I ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
  *
  * \return Pointer to result matrix \a B.
  */
-GAN_SQUMATTYPER *GAN_MAT_SLMULTT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPER *B );
+GAN_SQUMATTYPER *GAN_MAT_SLMULTT_Q ( GAN_MATTYPE *A, const GAN_SQUMATTYPER *B );
 #endif
 
 #ifdef GAN_MAT_SLMULTT_I
@@ -940,7 +940,7 @@ GAN_MATTYPE *GAN_MAT_SLMULTT_I ( GAN_MATTYPE *A );
  *
  * \return Pointer to result matrix \a B.
  */
-GAN_SQUMATTYPEL *GAN_MAT_SRMULTT_Q ( GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
+GAN_SQUMATTYPEL *GAN_MAT_SRMULTT_Q ( const GAN_MATTYPE *A, GAN_SQUMATTYPEL *B );
 #endif
 
 #ifdef GAN_MAT_SRMULTT_I
@@ -971,7 +971,7 @@ GAN_MATTYPE *GAN_MAT_SRMULTT_I ( GAN_MATTYPE *A );
  *
  * Returns sum of squares of elements of matrix \a A.
  */
-GAN_REALTYPE GAN_MAT_SUMSQR_Q ( GAN_MATTYPE *A );
+GAN_REALTYPE GAN_MAT_SUMSQR_Q ( const GAN_MATTYPE *A );
 #endif
 
 #ifdef GAN_MAT_FNORM_Q
@@ -981,7 +981,7 @@ GAN_REALTYPE GAN_MAT_SUMSQR_Q ( GAN_MATTYPE *A );
  * Returns Frobenius norm of matrix \a A, defined as the square-root of the
  * sum of squares of the elements of \a A.
  */
-GAN_REALTYPE GAN_MAT_FNORM_Q ( GAN_MATTYPE *A );
+GAN_REALTYPE GAN_MAT_FNORM_Q ( const GAN_MATTYPE *A );
 #endif
 
 /**
@@ -994,39 +994,15 @@ GAN_REALTYPE GAN_MAT_FNORM_Q ( GAN_MATTYPE *A );
  */
 
 #ifdef GAN_MAT_FROM_MAT_Q
-/**
- * \brief Macro: Convert generic rectangular matrix to fixed size matrix structure.
- *
- * Convert generic rectangular matrix \a A to fixed size matrix structure
- * \a B, returning \a B.
- *
- * \return Pointer to result matrix \a B.
- */
-GAN_MATTYPE *GAN_MAT_FROM_MAT_Q ( GAN_MATRIX_TYPE *A, GAN_MATTYPE *B );
+GAN_MATTYPE *GAN_MAT_FROM_MAT_Q ( const GAN_MATRIX_TYPE *A, GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_MAT_FROM_SQUMAT_Q
-/**
- * \brief Macro: Convert square rectangular matrix to fixed size matrix structure.
- *
- * Convert square rectangular matrix \a A to fixed size matrix structure
- * \a B, returning \a B.
- *
- * \return Pointer to result matrix \a B.
- */
-GAN_MATTYPE *GAN_MAT_FROM_SQUMAT_Q ( GAN_SQUMATRIX_TYPE *A, GAN_MATTYPE *B );
+GAN_MATTYPE *GAN_MAT_FROM_SQUMAT_Q ( const GAN_SQUMATRIX_TYPE *A, GAN_MATTYPE *B );
 #endif
 
 #ifdef GAN_SQUMAT_FROM_SQUMAT_Q
-/**
- * \brief Macro: Convert square rectangular matrix to fixed size square matrix structure.
- *
- * Convert square rectangular matrix \a A to fixed size square matrix
- * structure \a B, returning \a B.
- *
- * \return Pointer to result matrix \a B.
- */
-GAN_SQUMATTYPEL *GAN_SQUMAT_FROM_SQUMAT_Q ( GAN_SQUMATRIX_TYPE *A,
+GAN_SQUMATTYPEL *GAN_SQUMAT_FROM_SQUMAT_Q ( const GAN_SQUMATRIX_TYPE *A,
                                             GAN_SQUMATTYPEL *B );
 #endif
 
