@@ -3,8 +3,8 @@
  * Module:        CLAPACK wrapper functions (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.8 $
- * Last edited:   $Date: 2003/01/31 18:57:11 $
+ * Revision:      $Revision: 1.10 $
+ * Last edited:   $Date: 2005/02/23 13:13:16 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -108,7 +108,7 @@ Gan_Bool
    }
 #else
    /* compute Cholesky factorisation */
-   if ( !gan_spptrf ( GAN_UPPER, A->size, A->data, &info ) )
+   if ( !gan_spptrf ( GAN_MATRIXPART_UPPER, A->size, A->data, &info ) )
    {
       if ( info < 0 )
       {
@@ -192,7 +192,7 @@ Gan_Bool
    }
 #else
    /* compute inverse */
-   if ( !gan_spptri ( GAN_UPPER, A->size, A->data, &info ) )
+   if ( !gan_spptri ( GAN_MATRIXPART_UPPER, A->size, A->data, &info ) )
    {
       gan_err_flush_trace();
       if ( info < 0 )
@@ -851,7 +851,7 @@ Gan_Bool
             (VL == NULL) ? NULL : VL->data,
             (VL == NULL || A->rows == 0) ? &onei : (long *)&A->rows,
             (VR == NULL) ? NULL : VR->data,
-            (VL == NULL || A->rows == 0) ? &onei : (long *)&A->rows,
+            (VR == NULL || A->rows == 0) ? &onei : (long *)&A->rows,
             work, (long *)&work_size, &info );
             
    if ( info != 0 )
