@@ -3,8 +3,8 @@
  * Module:        Generic column vectors (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.28 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.30 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -277,7 +277,7 @@ Gan_Vector *
  * Read elements of vector \a x from variable argument list.
  */
 Gan_Bool
- gan_vec_read_va ( Gan_Vector *x, unsigned long rows, ... )
+ gan_vec_read_va ( const Gan_Vector *x, unsigned long rows, ... )
 {
    unsigned int i;
    va_list ap;
@@ -312,7 +312,7 @@ Gan_Bool
  * Copy vector \a x to vector \a y, and return \a y.
  */
 Gan_Vector *
- gan_vec_copy_q ( Gan_Vector *x, Gan_Vector *y )
+ gan_vec_copy_q ( const Gan_Vector *x, Gan_Vector *y )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -565,7 +565,7 @@ Gan_Vector *
  * Returns the scalar product \f$ x.y \f$ of the two vectors \a x and \a y.
  */
 double
- gan_vec_dot ( Gan_Vector *x, Gan_Vector *y )
+ gan_vec_dot ( const Gan_Vector *x, const Gan_Vector *y )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -600,7 +600,7 @@ double
  * vectors \a x and \a y.
  */
 Gan_Matrix *
- gan_vec_outer_q ( Gan_Vector *x, Gan_Vector *y, Gan_Matrix *A )
+ gan_vec_outer_q ( const Gan_Vector *x, const Gan_Vector *y, Gan_Matrix *A )
 {
    int i, j;
 
@@ -632,7 +632,7 @@ Gan_Matrix *
  * \a x and itself.
  */
 Gan_SquMatrix *
- gan_vec_outer_sym_q ( Gan_Vector *x, Gan_SquMatrix *A )
+ gan_vec_outer_sym_q ( const Gan_Vector *x, Gan_SquMatrix *A )
 {
    int i, j;
 
@@ -674,7 +674,7 @@ Gan_SquMatrix *
  */
 Gan_Vector *
  gan_vec_insert ( Gan_Vector *x, unsigned long rx,
-                  Gan_Vector *y, unsigned long ry, unsigned long rows )
+                  const Gan_Vector *y, unsigned long ry, unsigned long rows )
 {
    int i;
 
@@ -698,8 +698,8 @@ Gan_Vector *
  */
 Gan_Vector *
  gan_vec_insert_mat ( Gan_Vector *x, unsigned long rx,
-                      Gan_Matrix *A, unsigned long ra,
-                                     unsigned long ca,
+                      const Gan_Matrix *A, unsigned long ra,
+                                           unsigned long ca,
                       unsigned long rows )
 {
    int i;
@@ -726,8 +726,8 @@ Gan_Vector *
  */
 Gan_Vector *
  gan_vec_insert_matT ( Gan_Vector *x, unsigned long rx,
-                       Gan_Matrix *A, unsigned long ra,
-                                      unsigned long ca,
+                       const Gan_Matrix *A, unsigned long ra,
+                                            unsigned long ca,
                        unsigned long rows )
 {
    int i;
@@ -859,7 +859,7 @@ Gan_Vector *
  * indentation \a indent and floating-point format \a fmt.
  */
 Gan_Bool
- gan_vec_fprint ( FILE *fp, Gan_Vector *x, const char *prefix,
+ gan_vec_fprint ( FILE *fp, const Gan_Vector *x, const char *prefix,
                   int indent, const char *fmt )
 {
    unsigned long i;
@@ -992,11 +992,11 @@ Gan_Vector *
  * magic number printed first.
  */
 Gan_Bool
- gan_vec_fwrite ( FILE *fp, Gan_Vector *x, gan_ui32 magic_number )
+ gan_vec_fwrite ( FILE *fp, const Gan_Vector *x, gan_uint32 magic_number )
 {
    if ( !gan_fwrite_lendian_ui32 ( fp, &magic_number, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &x->rows, 1 ) ||
-        !gan_fwrite_lendian_d ( fp, x->data, x->rows ) )
+        !gan_fwrite_lendian_f64 ( fp, x->data, x->rows ) )
    {
       gan_err_register ( "gan_vec_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -1016,7 +1016,7 @@ Gan_Bool
  * \sa gan_vecf_fread_s()
  */
 Gan_Vector *
- gan_vec_fread_q ( FILE *fp, Gan_Vector *x, gan_ui32 *magic_number )
+ gan_vec_fread_q ( FILE *fp, Gan_Vector *x, gan_uint32 *magic_number )
 {
    unsigned long rows;
 
@@ -1047,7 +1047,7 @@ Gan_Vector *
    }
 
    /* read vector data */
-   if ( !gan_fread_lendian_d ( fp, x->data, x->rows ) )
+   if ( !gan_fread_lendian_f64 ( fp, x->data, x->rows ) )
    {
       gan_err_register ( "gan_vec_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -1065,7 +1065,7 @@ Gan_Vector *
 /* for use in a debugger: use gan_vec_print() or gan_vec_fprint in
  * your program!
  */
-void gan_vec_db ( Gan_Vector *x )
+void gan_vec_db ( const Gan_Vector *x )
 {
    gan_vec_print ( x, "", 0, "%+g" );
 }
