@@ -3,8 +3,8 @@
  * Module:        Generic column vectors (double precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.24 $
- * Last edited:   $Date: 2002/04/18 14:56:25 $
+ * Revision:      $Revision: 1.26 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -90,35 +90,35 @@ Gan_Vector *gan_vec_fill_vap ( Gan_Vector *x, unsigned long rows,
                                va_list *aptr );
 Gan_Vector *gan_vec_fill_const_q ( Gan_Vector *x, unsigned long rows,
                                    double value );
-Gan_Bool gan_vec_read_va ( Gan_Vector *x, unsigned long rows, ... );
-Gan_Vector *gan_vec_copy_q ( Gan_Vector *x, Gan_Vector *y );
+Gan_Bool gan_vec_read_va ( const Gan_Vector *x, unsigned long rows, ... );
+Gan_Vector *gan_vec_copy_q ( const Gan_Vector *x, Gan_Vector *y );
 Gan_Vector *gan_vec_scale_q ( Gan_Vector *x, double a, Gan_Vector *y );
 Gan_Vector *gan_vec_add_q ( Gan_Vector *x, Gan_Vector *y, Gan_Vector *z );
 Gan_Vector *gan_vec_sub_q ( Gan_Vector *x, Gan_Vector *y, Gan_Vector *z );
-double gan_vec_dot ( Gan_Vector *x, Gan_Vector *y );
-struct Gan_Matrix *gan_vec_outer_q ( Gan_Vector *x, Gan_Vector *y,
+double gan_vec_dot ( const Gan_Vector *x, const Gan_Vector *y );
+struct Gan_Matrix *gan_vec_outer_q ( const Gan_Vector *x, const Gan_Vector *y,
                                      struct Gan_Matrix *A );
-struct Gan_SquMatrix *gan_vec_outer_sym_q ( Gan_Vector *x,
+struct Gan_SquMatrix *gan_vec_outer_sym_q ( const Gan_Vector *x,
                                             struct Gan_SquMatrix *A );
 Gan_Vector *gan_vec_insert ( Gan_Vector *x, unsigned long rx,
-                             Gan_Vector *y, unsigned long ry,
+                             const Gan_Vector *y, unsigned long ry,
                              unsigned long rows );
 void gan_vec_free_va ( Gan_Vector *x, ... );
 Gan_Vector *gan_vec_insert_mat ( Gan_Vector *x, unsigned long rx,
-                                 struct Gan_Matrix *A, unsigned long ra,
-                                                       unsigned long ca,
+                                 const struct Gan_Matrix *A, unsigned long ra,
+                                                             unsigned long ca,
                                  unsigned long rows );
 Gan_Vector *gan_vec_insert_matT ( Gan_Vector *x, unsigned long rx,
-                                  struct Gan_Matrix *A, unsigned long ra,
-                                                        unsigned long ca,
+                                  const struct Gan_Matrix *A, unsigned long ra,
+                                                              unsigned long ca,
                                   unsigned long rows );
-Gan_Bool gan_vec_fprint ( FILE *fp, Gan_Vector *x, const char *prefix,
+Gan_Bool gan_vec_fprint ( FILE *fp, const Gan_Vector *x, const char *prefix,
                           int indent, const char *fmt );
 Gan_Vector *gan_vec_fscanf_q ( FILE *fp, Gan_Vector *x,
                                char *prefix, int prefix_len );
-Gan_Bool gan_vec_fwrite ( FILE *fp, Gan_Vector *x, gan_ui32 magic_number );
+Gan_Bool gan_vec_fwrite ( FILE *fp, const Gan_Vector *x, gan_uint32 magic_number );
 Gan_Vector *gan_vec_fread_q ( FILE *fp, Gan_Vector *x,
-                              gan_ui32 *magic_number );
+                              gan_uint32 *magic_number );
 
 #ifdef GAN_GENERATE_DOCUMENTATION
 #define Gan_Matrix    struct Gan_Matrix
@@ -219,7 +219,7 @@ Gan_Bool gan_vec_set_el ( Gan_Vector *x, unsigned i, double value );
  * Macro to return element \a i of generic vector, or \c DBL_MAX on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_vec_get_el ( Gan_Vector *x, unsigned i );
+double gan_vec_get_el ( const Gan_Vector *x, unsigned i );
 #else
 #ifdef NDEBUG
 #define gan_vec_get_el(x,i) (x)->data[i]
@@ -292,7 +292,7 @@ Gan_Bool gan_vec_dec_el ( Gan_Vector *x, unsigned i, double value );
  * indentation \a indent and floating-point format \a fmt.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_vec_print ( Gan_Vector *x,
+Gan_Bool gan_vec_print ( const Gan_Vector *x,
                          const char *prefix, int indent, const char *fmt );
 #else
 #define gan_vec_print(x,p,i,f) (gan_vec_fprint(stdout,x,p,i,f))
@@ -369,7 +369,7 @@ Gan_Vector *gan_vec_fill_zero_s ( unsigned long rows );
  * Allocate and return a copy of vector \a x.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_vec_copy_s ( Gan_Vector *x );
+Gan_Vector *gan_vec_copy_s ( const Gan_Vector *x );
 #else
 #define gan_vec_copy_s(x) (gan_vec_copy_q((x),NULL))
 #endif
@@ -611,7 +611,7 @@ Gan_Vector *gan_vec_decrement ( Gan_Vector *x, Gan_Vector *y );
  * Compute outer product \f$ x y^{\top} \f$ of vectors \a x and \a y.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix *gan_vec_outer_s ( Gan_Vector *x, Gan_Vector *y );
+Gan_Matrix *gan_vec_outer_s ( const Gan_Vector *x, const Gan_Vector *y );
 #else
 #define gan_vec_outer_s(x,y) gan_vec_outer_q(x,y,NULL)
 #endif
@@ -624,7 +624,7 @@ Gan_Matrix *gan_vec_outer_s ( Gan_Vector *x, Gan_Vector *y );
  * itself.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix *gan_vec_outer_sym_s ( Gan_Vector *x );
+Gan_SquMatrix *gan_vec_outer_sym_s ( const Gan_Vector *x );
 #else
 #define gan_vec_outer_sym_s(x) gan_vec_outer_sym_q(x,NULL)
 #endif
@@ -645,7 +645,7 @@ Gan_SquMatrix *gan_vec_outer_sym_s ( Gan_Vector *x );
  * i.e. the squared length of \a x.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-double gan_vec_sqrlen ( Gan_Vector *x );
+double gan_vec_sqrlen ( const Gan_Vector *x );
 #else
 #define gan_vec_sqrlen(x) (gan_vec_dot(x,x))
 #endif
@@ -668,7 +668,7 @@ double gan_vec_sqrlen ( Gan_Vector *x );
  *         otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_vec_same_size ( Gan_Vector *x, Gan_Vector *y );
+Gan_Bool gan_vec_same_size ( const Gan_Vector *x, const Gan_Vector *y );
 #else
 #define gan_vec_same_size(x,y) ((x)->rows == (y)->rows)
 #endif
@@ -731,7 +731,7 @@ Gan_Vector *gan_vec_fscanf_s ( FILE *fp, const char *prefix, int prefix_len );
  * \sa gan_vec_fread_q()
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector *gan_vec_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_Vector *gan_vec_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_vec_fread_s(fp,ml) gan_vec_fread_q(fp,NULL,m)
 #endif
@@ -748,7 +748,7 @@ Gan_Vector *gan_vec_fread_s ( FILE *fp, gan_ui32 *magic_number );
 /* not user functions */
 Gan_Bool gan_vec_realloc ( Gan_Vector *x, unsigned long rows );
 #ifndef NDEBUG
-void gan_vec_db ( Gan_Vector *x ); /* for use in a debugger */
+void gan_vec_db ( const Gan_Vector *x ); /* for use in a debugger */
 #endif
 
 /**
