@@ -3,8 +3,8 @@
  * Module:        Generic column vectors (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.13 $
- * Last edited:   $Date: 2002/05/16 08:43:35 $
+ * Revision:      $Revision: 1.15 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -224,7 +224,7 @@ Gan_Vector_f *
  * Read elements of vector \a x from variable argument list.
  */
 Gan_Bool
- gan_vecf_read_va ( Gan_Vector_f *x, unsigned long rows, ... )
+ gan_vecf_read_va ( const Gan_Vector_f *x, unsigned long rows, ... )
 {
    unsigned int i;
    va_list ap;
@@ -259,7 +259,7 @@ Gan_Bool
  * Copy vector \a x to vector \a y, and return \a y.
  */
 Gan_Vector_f *
- gan_vecf_copy_q ( Gan_Vector_f *x, Gan_Vector_f *y )
+ gan_vecf_copy_q ( const Gan_Vector_f *x, Gan_Vector_f *y )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -512,7 +512,7 @@ Gan_Vector_f *
  * Returns the scalar product \f$ x.y \f$ of the two vectors \a x and \a y.
  */
 float
- gan_vecf_dot ( Gan_Vector_f *x, Gan_Vector_f *y )
+ gan_vecf_dot ( const Gan_Vector_f *x, const Gan_Vector_f *y )
 {
 #ifdef HAVE_LAPACK
    long onei = 1;
@@ -547,7 +547,7 @@ float
  * vectors \a x and \a y.
  */
 Gan_Matrix_f *
- gan_vecf_outer_q ( Gan_Vector_f *x, Gan_Vector_f *y, Gan_Matrix_f *A )
+ gan_vecf_outer_q ( const Gan_Vector_f *x, const Gan_Vector_f *y, Gan_Matrix_f *A )
 {
    int i, j;
 
@@ -579,7 +579,7 @@ Gan_Matrix_f *
  * \a x and itself.
  */
 Gan_SquMatrix_f *
- gan_vecf_outer_sym_q ( Gan_Vector_f *x, Gan_SquMatrix_f *A )
+ gan_vecf_outer_sym_q ( const Gan_Vector_f *x, Gan_SquMatrix_f *A )
 {
    int i, j;
 
@@ -621,7 +621,7 @@ Gan_SquMatrix_f *
  */
 Gan_Vector_f *
  gan_vecf_insert ( Gan_Vector_f *x, unsigned long rx,
-                   Gan_Vector_f *y, unsigned long ry, unsigned long rows )
+                   const Gan_Vector_f *y, unsigned long ry, unsigned long rows )
 {
    int i;
 
@@ -645,7 +645,7 @@ Gan_Vector_f *
  */
 Gan_Vector_f *
  gan_vecf_insert_mat ( Gan_Vector_f *x, unsigned long rx,
-                       Gan_Matrix_f *A, unsigned long ra,
+                       const Gan_Matrix_f *A, unsigned long ra,
                                         unsigned long ca,
                        unsigned long rows )
 {
@@ -673,7 +673,7 @@ Gan_Vector_f *
  */
 Gan_Vector_f *
  gan_vecf_insert_matT ( Gan_Vector_f *x, unsigned long rx,
-                        Gan_Matrix_f *A, unsigned long ra,
+                        const Gan_Matrix_f *A, unsigned long ra,
                                          unsigned long ca,
                         unsigned long rows )
 {
@@ -806,7 +806,7 @@ Gan_Vector_f *
  * indentation \a indent and floating-point format \a fmt.
  */
 Gan_Bool
- gan_vecf_fprint ( FILE *fp, Gan_Vector_f *x, const char *prefix,
+ gan_vecf_fprint ( FILE *fp, const Gan_Vector_f *x, const char *prefix,
                    int indent, const char *fmt )
 {
    unsigned long i;
@@ -939,11 +939,11 @@ Gan_Vector_f *
  * magic number printed first.
  */
 Gan_Bool
- gan_vecf_fwrite ( FILE *fp, Gan_Vector_f *x, gan_ui32 magic_number )
+ gan_vecf_fwrite ( FILE *fp, const Gan_Vector_f *x, gan_uint32 magic_number )
 {
    if ( !gan_fwrite_lendian_ui32 ( fp, &magic_number, 1 ) ||
         !gan_fwrite_lendian_ul ( fp, &x->rows, 1 ) ||
-        !gan_fwrite_lendian_f ( fp, x->data, x->rows ) )
+        !gan_fwrite_lendian_f32 ( fp, x->data, x->rows ) )
    {
       gan_err_register ( "gan_vecf_fwrite", GAN_ERROR_FAILURE, "" );
       return GAN_FALSE;
@@ -963,7 +963,7 @@ Gan_Bool
  * \sa gan_vecf_fread_s()
  */
 Gan_Vector_f *
- gan_vecf_fread_q ( FILE *fp, Gan_Vector_f *x, gan_ui32 *magic_number )
+ gan_vecf_fread_q ( FILE *fp, Gan_Vector_f *x, gan_uint32 *magic_number )
 {
    unsigned long rows;
 
@@ -994,7 +994,7 @@ Gan_Vector_f *
    }
 
    /* read vector data */
-   if ( !gan_fread_lendian_f ( fp, x->data, x->rows ) )
+   if ( !gan_fread_lendian_f32 ( fp, x->data, x->rows ) )
    {
       gan_err_register ( "gan_vecf_fread_q", GAN_ERROR_CORRUPTED_FILE, "" );
       return NULL;
@@ -1012,7 +1012,7 @@ Gan_Vector_f *
 /* for use in a debugger: use gan_vecf_print() or gan_vecf_fprint in
  * your program!
  */
-void gan_vecf_db ( Gan_Vector_f *x )
+void gan_vecf_db ( const Gan_Vector_f *x )
 {
    gan_vecf_print ( x, "", 0, "%+g" );
 }
