@@ -3,8 +3,8 @@
  * Module:        Generic column vectors (single precision)
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.9 $
- * Last edited:   $Date: 2002/04/18 14:56:26 $
+ * Revision:      $Revision: 1.11 $
+ * Last edited:   $Date: 2005/02/10 20:19:17 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -75,37 +75,37 @@ Gan_Vector_f *gan_vecf_fill_vap ( Gan_Vector_f *x, unsigned long rows,
                                va_list *aptr );
 Gan_Vector_f *gan_vecf_fill_const_q ( Gan_Vector_f *x, unsigned long rows,
                                       float value );
-Gan_Bool gan_vecf_read_va ( Gan_Vector_f *x, unsigned long rows, ... );
-Gan_Vector_f *gan_vecf_copy_q ( Gan_Vector_f *x, Gan_Vector_f *y );
+Gan_Bool gan_vecf_read_va ( const Gan_Vector_f *x, unsigned long rows, ... );
+Gan_Vector_f *gan_vecf_copy_q ( const Gan_Vector_f *x, Gan_Vector_f *y );
 Gan_Vector_f *gan_vecf_scale_q ( Gan_Vector_f *x, float a, Gan_Vector_f *y );
 Gan_Vector_f *gan_vecf_add_q ( Gan_Vector_f *x, Gan_Vector_f *y,
                                Gan_Vector_f *z );
 Gan_Vector_f *gan_vecf_sub_q ( Gan_Vector_f *x, Gan_Vector_f *y,
                                Gan_Vector_f *z );
-float gan_vecf_dot ( Gan_Vector_f *x, Gan_Vector_f *y );
-struct Gan_Matrix_f *gan_vecf_outer_q ( Gan_Vector_f *x, Gan_Vector_f *y,
+float gan_vecf_dot ( const Gan_Vector_f *x, const Gan_Vector_f *y );
+struct Gan_Matrix_f *gan_vecf_outer_q ( const Gan_Vector_f *x, const Gan_Vector_f *y,
                                         struct Gan_Matrix_f *A );
-struct Gan_SquMatrix_f *gan_vecf_outer_sym_q ( Gan_Vector_f *x,
+struct Gan_SquMatrix_f *gan_vecf_outer_sym_q ( const Gan_Vector_f *x,
                                                struct Gan_SquMatrix_f *A );
 Gan_Vector_f *gan_vecf_insert ( Gan_Vector_f *x, unsigned long rx,
-                                Gan_Vector_f *y, unsigned long ry,
+                                const Gan_Vector_f *y, unsigned long ry,
                                 unsigned long rows );
 void gan_vecf_free_va ( Gan_Vector_f *x, ... );
 Gan_Vector_f *gan_vecf_insert_mat ( Gan_Vector_f *x, unsigned long rx,
-                                    struct Gan_Matrix_f *A, unsigned long ra,
-                                                            unsigned long ca,
+                                    const struct Gan_Matrix_f *A, unsigned long ra,
+                                                                  unsigned long ca,
                                     unsigned long rows );
 Gan_Vector_f *gan_vecf_insert_matT ( Gan_Vector_f *x, unsigned long rx,
-                                     struct Gan_Matrix_f *A, unsigned long ra,
-                                                             unsigned long ca,
+                                     const struct Gan_Matrix_f *A, unsigned long ra,
+                                                                   unsigned long ca,
                                      unsigned long rows );
-Gan_Bool gan_vecf_fprint ( FILE *fp, Gan_Vector_f *x, const char *prefix,
+Gan_Bool gan_vecf_fprint ( FILE *fp, const Gan_Vector_f *x, const char *prefix,
                            int indent, const char *fmt );
 Gan_Vector_f *gan_vecf_fscanf_q ( FILE *fp, Gan_Vector_f *x,
                                   char *prefix, int prefix_len );
-Gan_Bool gan_vecf_fwrite ( FILE *fp, Gan_Vector_f *x, gan_ui32 magic_number );
+Gan_Bool gan_vecf_fwrite ( FILE *fp, const Gan_Vector_f *x, gan_uint32 magic_number );
 Gan_Vector_f *gan_vecf_fread_q ( FILE *fp, Gan_Vector_f *x,
-                                 gan_ui32 *magic_number );
+                                 gan_uint32 *magic_number );
 
 #ifdef GAN_GENERATE_DOCUMENTATION
 #define Gan_Matrix_f    struct Gan_Matrix_f
@@ -206,7 +206,7 @@ Gan_Bool gan_vecf_set_el ( Gan_Vector_f *x, unsigned i, float value );
  * Macro to return element \a i of generic vector, or \c FLT_MAX on failure.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-float gan_vecf_get_el ( Gan_Vector_f *x, unsigned i );
+float gan_vecf_get_el ( const Gan_Vector_f *x, unsigned i );
 #else
 #ifdef NDEBUG
 #define gan_vecf_get_el(x,i) (x)->data[i]
@@ -279,7 +279,7 @@ Gan_Bool gan_vecf_dec_el ( Gan_Vector_f *x, unsigned i, float value );
  * indentation \a indent and floating-point format \a fmt.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_vecf_print ( Gan_Vector_f *x,
+Gan_Bool gan_vecf_print ( const Gan_Vector_f *x,
                           const char *prefix, int indent, const char *fmt );
 #else
 #define gan_vecf_print(x,p,i,f) (gan_vecf_fprint(stdout,x,p,i,f))
@@ -356,7 +356,7 @@ Gan_Vector_f *gan_vecf_fill_zero_s ( unsigned long rows );
  * Allocate and return a copy of vector \a x.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_vecf_copy_s ( Gan_Vector_f *x );
+Gan_Vector_f *gan_vecf_copy_s ( const Gan_Vector_f *x );
 #else
 #define gan_vecf_copy_s(x) (gan_vecf_copy_q((x),NULL))
 #endif
@@ -598,7 +598,7 @@ Gan_Vector_f *gan_vecf_decrement ( Gan_Vector_f *x, Gan_Vector_f *y );
  * Compute outer product \f$ x y^{\top} \f$ of vectors \a x and \a y.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Matrix_f *gan_vecf_outer_s ( Gan_Vector_f *x, Gan_Vector_f *y );
+Gan_Matrix_f *gan_vecf_outer_s ( const Gan_Vector_f *x, Gan_Vector_f *y );
 #else
 #define gan_vecf_outer_s(x,y) gan_vecf_outer_q(x,y,NULL)
 #endif
@@ -611,7 +611,7 @@ Gan_Matrix_f *gan_vecf_outer_s ( Gan_Vector_f *x, Gan_Vector_f *y );
  * itself.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_SquMatrix_f *gan_vecf_outer_sym_s ( Gan_Vector_f *x );
+Gan_SquMatrix_f *gan_vecf_outer_sym_s ( const Gan_Vector_f *x );
 #else
 #define gan_vecf_outer_sym_s(x) gan_vecf_outer_sym_q(x,NULL)
 #endif
@@ -632,7 +632,7 @@ Gan_SquMatrix_f *gan_vecf_outer_sym_s ( Gan_Vector_f *x );
  * i.e. the squared length of \a x.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-float gan_vecf_sqrlen ( Gan_Vector_f *x );
+float gan_vecf_sqrlen ( const Gan_Vector_f *x );
 #else
 #define gan_vecf_sqrlen(x) (gan_vecf_dot(x,x))
 #endif
@@ -655,7 +655,7 @@ float gan_vecf_sqrlen ( Gan_Vector_f *x );
  *         otherwise.
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Bool gan_vecf_same_size ( Gan_Vector_f *x, Gan_Vector_f *y );
+Gan_Bool gan_vecf_same_size ( const Gan_Vector_f *x, const Gan_Vector_f *y );
 #else
 #define gan_vecf_same_size(x,y) ((x)->rows == (y)->rows)
 #endif
@@ -719,7 +719,7 @@ Gan_Vector_f *gan_vecf_fscanf_s ( FILE *fp, const char *prefix,
  * \sa gan_vecf_fread_q()
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
-Gan_Vector_f *gan_vecf_fread_s ( FILE *fp, gan_ui32 *magic_number );
+Gan_Vector_f *gan_vecf_fread_s ( FILE *fp, gan_uint32 *magic_number );
 #else
 #define gan_vecf_fread_s(fp,ml) gan_vecf_fread_q(fp,NULL,m)
 #endif
@@ -736,7 +736,7 @@ Gan_Vector_f *gan_vecf_fread_s ( FILE *fp, gan_ui32 *magic_number );
 /* not user functions */
 Gan_Bool gan_vecf_realloc ( Gan_Vector_f *x, unsigned long rows );
 #ifndef NDEBUG
-void gan_vecf_db ( Gan_Vector_f *x ); /* for use in a debugger */
+void gan_vecf_db ( const Gan_Vector_f *x ); /* for use in a debugger */
 #endif
 
 /**
