@@ -3,8 +3,8 @@
  * Module:        Preprocess source code for documentation generation
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.11 $
- * Last edited:   $Date: 2003/01/31 18:30:04 $
+ * Revision:      $Revision: 1.13 $
+ * Last edited:   $Date: 2005/05/21 22:49:00 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -62,6 +62,7 @@ typedef struct PPIfdefState
    struct PPIfdefState *next;
 } PPIfdefState;
 
+#if 0
 static void define_list_print(void)
 {
    PPDefinition *definition;
@@ -70,6 +71,7 @@ static void define_list_print(void)
          definition = definition->next )
       printf ( "Definition %s --> %s\n", definition->name, definition->def );
 }
+#endif
 
 static void define_list_add ( char *name, char *def )
 {
@@ -537,7 +539,13 @@ static void parse_file ( char *directory, char *input_file, FILE *fpout,
               (state->echo &&
                strcmp (line, "#endif /* #ifdef GAN_GENERATE_DOCUMENTATION */")
                != 0) )
-            fputs ( line, fpout );
+         {
+#if 0
+            if ( sscanf ( line, "#ifdef %s", s1 ) == 1 )
+               if(strncmp ( s1, "_GAN", 4 ) != 0)
+#endif
+                  fputs ( line, fpout );
+         }
 
          state = state->next;
          free(latest);
@@ -583,8 +591,8 @@ static void parse_file ( char *directory, char *input_file, FILE *fpout,
             }
             /* check for author definition line if at top level */
             else if ( top_level &&
-                      (sscanf ( line, "* Author: $Author: pm $", s1 ) == 1 ||
-                       sscanf ( line, " * Author: $Author: pm $", s1 ) == 1) )
+                      (sscanf ( line, "* Author: $Author: %s", s1 ) == 1 ||
+                       sscanf ( line, " * Author: $Author: %s", s1 ) == 1) )
             {
                /* output author line */
                fprintf ( fpout, " * \\author %s\n", s1 );
@@ -592,8 +600,8 @@ static void parse_file ( char *directory, char *input_file, FILE *fpout,
             }
             /* check for revision definition line if at top level */
             else if ( top_level &&
-                   (sscanf ( line, "* Revision: $Revision: 1.11 $", s1 ) == 1 ||
-                    sscanf ( line, " * Revision: $Revision: 1.11 $", s1 ) == 1) )
+                   (sscanf ( line, "* Revision: $Revision: %s", s1 ) == 1 ||
+                    sscanf ( line, " * Revision: $Revision: %s", s1 ) == 1) )
             {
                /* output version line */
                fprintf ( fpout, " * \\version %s\n", s1 );
@@ -601,8 +609,8 @@ static void parse_file ( char *directory, char *input_file, FILE *fpout,
             }
             /* check for date line if at top level */
             else if ( top_level &&
-             (sscanf ( line, "* Last edited: $Date: 2003/01/31 18:30:04 $", s1, s2 ) == 2 ||
-              sscanf ( line, " * Last edited: $Date: 2003/01/31 18:30:04 $", s1, s2 ) == 2) )
+                      (sscanf ( line, "* Last edited: $Date: %s %s", s1, s2 ) == 2 ||
+                       sscanf ( line, " * Last edited: $Date: %s %s", s1, s2 ) == 2) )
             {
                /* output date line */
                fprintf ( fpout, " * \\date %s %s\n", s1, s2 );
