@@ -3,8 +3,8 @@
  * Module:        Camera definitions and building functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.28 $
- * Last edited:   $Date: 2002/06/12 18:07:02 $
+ * Revision:      $Revision: 1.37 $
+ * Last edited:   $Date: 2005/04/22 13:16:23 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -47,6 +47,28 @@ extern "C" {
  * \{
  */
 
+/**
+ * /brief Radial basis function structure.
+ */
+typedef struct Gan_RadialBasisFunction
+{
+   /// Position of centre
+   double x, y;
+
+   /// Squared radius of radial basis function
+   double dist2_thres;
+
+   /// Weight attached to radial basis function
+   double weight;
+
+   /// Value used to normalise function
+   double normaliser;
+
+   /// Distortion vector
+   double dx, dy;
+
+} Gan_RadialBasisFunction;
+
 /**
  * \brief Structure containing camera parameters in double precision.
  */
@@ -58,7 +80,7 @@ typedef struct Gan_Camera
    /// parameters of linear camera
 
    /// focal distance in x/y pixels
-   double fx, fy;
+   double fx, fy, fy_over_fx;
 
    /// image centre x/y coordinates
    double x0, y0;
@@ -115,32 +137,47 @@ typedef struct Gan_Camera
       } radial3;
 
       struct { double cxx, cxy, cyx, cyy; } xydist4;
-   } nonlinear;
 
-   /// gamma value of images taken using this camera
-   double gamma;
+      struct
+      {
+         double xla, xlp, xha, xhp;
+         double yla, ylp, yha, yhp;
+      } fcwarp;
+
+      struct
+      {
+         /// skew and lower triangle components store in a scale-independent manner
+         double skew, kyx, kzx, kzy;
+
+         /// non-triangular camera matrix and its inverse
+         Gan_Matrix33 K, Kinv;
+
+         /// Cubic B-spline parameters
+         Gan_Vector2 weight[1 + (1 << GAN_CUBIC_BSPLINE_CAMERA_LEVEL)][1 + (1 << GAN_CUBIC_BSPLINE_CAMERA_LEVEL)];
+      } cbspline;
+   } nonlinear;
 
    /// point functions
    struct
    {
       /// point projection function
-      Gan_Bool (*project) ( struct Gan_Camera *camera,
+      Gan_Bool (*project) ( const struct Gan_Camera *camera,
                             Gan_Vector3 *X, Gan_Vector3 *p,
                             Gan_Matrix22 *HX, struct Gan_Camera *HC,
                             int *error_code );
 
       /// point back-projection function
-      Gan_Bool (*backproject) ( struct Gan_Camera *camera,
+      Gan_Bool (*backproject) ( const struct Gan_Camera *camera,
                                 Gan_Vector3 *p, Gan_Vector3 *X,
                                 int *error_code );
 
       /// function to add distortion to a point
-      Gan_Bool (*add_distortion) ( struct Gan_Camera *camera,
+      Gan_Bool (*add_distortion) ( const struct Gan_Camera *camera,
                                    Gan_Vector3 *pu, Gan_Vector3 *p,
                                    int *error_code );
 
       /// function to remove distortion from a point
-      Gan_Bool (*remove_distortion) ( struct Gan_Camera *camera,
+      Gan_Bool (*remove_distortion) ( const struct Gan_Camera *camera,
                                       Gan_Vector3 *p, Gan_Vector3 *pu,
                                       int *error_code);
    } point;
@@ -149,11 +186,11 @@ typedef struct Gan_Camera
    struct
    {
       /// line projection function
-      Gan_Bool (*project) ( struct Gan_Camera *camera,
+      Gan_Bool (*project) ( const struct Gan_Camera *camera,
                             Gan_Vector3 *L, Gan_Vector3 *l );
 
       /// line back-projection function
-      Gan_Bool (*backproject) ( struct Gan_Camera *camera,
+      Gan_Bool (*backproject) ( const struct Gan_Camera *camera,
                                 Gan_Vector3 *l, Gan_Vector3 *L );
    } line;
 } Gan_Camera;
@@ -162,9 +199,21 @@ Gan_Bool gan_camera_build_va ( Gan_Camera *camera,
                                Gan_CameraType type,
                                double zh, double fx, double fy,
                                double x0, double y0, ... );
+#if 0
+Gan_Bool gan_rbf_apply_adjustment ( Gan_Camera *camera,
+                                    Gan_Vector3 *p,
+                                    Gan_Matrix22 *HX, Gan_Camera HC[2],
+                                    int *error_code );
+#endif
 Gan_Bool gan_camera_internalize ( Gan_Camera *camera );
-Gan_Bool gan_camera_set_gamma ( Gan_Camera *camera, double gamma );
-Gan_SquMatrix33 gan_camera_fill_matrix_s ( Gan_Camera *camera );
+Gan_Bool gan_camera_identical ( Gan_Camera *camera1, Gan_Camera *pcamera2 );
+Gan_SquMatrix33 gan_camera_fill_matrix_s ( const Gan_Camera *camera );
+Gan_Matrix33    gan_camera_fill_fullmatrix_s ( const Gan_Camera *camera );
+
+Gan_Bool gan_camera_set_common_fields ( Gan_Camera *camera,
+                                        Gan_CameraType type,
+                                        double zh, double fx, double fy,
+                                        double x0, double y0 );
 
 /**
  * \brief Macro: Projects a point from the scene onto the image.
@@ -191,7 +240,7 @@ Gan_SquMatrix33 gan_camera_fill_matrix_s ( Gan_Camera *camera );
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_point_gen ( Gan_Camera *camera,
+ gan_camera_project_point_gen ( const Gan_Camera *camera,
                                 Gan_Vector3 *X, Gan_Vector3 *p,
                                 Gan_Matrix22 *HX, Gan_Camera HC[2],
                                 int *error_code );
@@ -218,7 +267,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_point ( Gan_Camera *camera,
+ gan_camera_project_point ( const Gan_Camera *camera,
                             Gan_Vector3 *X, Gan_Vector3 *p,
                             int *error_code );
 #else
@@ -246,7 +295,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_point_q ( Gan_Camera *camera,
+ gan_camera_project_point_q ( const Gan_Camera *camera,
                               Gan_Vector3 *X, Gan_Vector3 *p );
 #else
 #define gan_camera_project_point_q(camera,X,p)\
@@ -272,7 +321,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_point_i ( Gan_Camera *camera, Gan_Vector3 *X );
+ gan_camera_project_point_i ( const Gan_Camera *camera, Gan_Vector3 *X );
 #else
 #define gan_camera_project_point_i(camera,X)\
            gan_camera_project_point_q(camera,X,X)
@@ -299,7 +348,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_backproject_point ( Gan_Camera *camera,
+ gan_camera_backproject_point ( const Gan_Camera *camera,
                                 Gan_Vector3 *p, Gan_Vector3 *X,
                                 int *error_code );
 #else
@@ -329,7 +378,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_backproject_point_q ( Gan_Camera *camera,
+ gan_camera_backproject_point_q ( const Gan_Camera *camera,
                                   Gan_Vector3 *p, Gan_Vector3 *X );
 #else
 #define gan_camera_backproject_point_q(camera,p,X)\
@@ -357,7 +406,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_backproject_point_i ( Gan_Camera *camera,
+ gan_camera_backproject_point_i ( const Gan_Camera *camera,
                                   Gan_Vector3 *p );
 #else
 #define gan_camera_backproject_point_i(camera,p)\
@@ -382,7 +431,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_add_distortion ( Gan_Camera *camera,
+ gan_camera_add_distortion ( const Gan_Camera *camera,
                              Gan_Vector3 *pu, Gan_Vector3 *p,
                              int *error_code );
 #else
@@ -409,7 +458,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_add_distortion_q ( Gan_Camera *camera,
+ gan_camera_add_distortion_q ( const Gan_Camera *camera,
                                Gan_Vector3 *pu, Gan_Vector3 *p );
 #else
 #define gan_camera_add_distortion_q(camera,pu,p)\
@@ -433,7 +482,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_add_distortion_i ( Gan_Camera *camera, Gan_Vector3 *p );
+ gan_camera_add_distortion_i ( const Gan_Camera *camera, Gan_Vector3 *p );
 #else
 #define gan_camera_add_distortion_i(camera,p)\
            gan_camera_add_distortion_q(camera,p,p)
@@ -455,7 +504,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_remove_distortion ( Gan_Camera *camera,
+ gan_camera_remove_distortion ( const Gan_Camera *camera,
                                 Gan_Vector3 *p, Gan_Vector3 *pu,
                                 int *error_code );
 #else
@@ -481,7 +530,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_remove_distortion_q ( Gan_Camera *camera,
+ gan_camera_remove_distortion_q ( const Gan_Camera *camera,
                                   Gan_Vector3 *p, Gan_Vector3 *pu );
 #else
 #define gan_camera_remove_distortion_q(camera,p,pu)\
@@ -505,7 +554,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_remove_distortion_i ( Gan_Camera *camera, Gan_Vector3 *p );
+ gan_camera_remove_distortion_i ( const Gan_Camera *camera, Gan_Vector3 *p );
 #else
 #define gan_camera_remove_distortion_i(camera,p) \
            gan_camera_remove_distortion_q(camera,p,p)
@@ -529,7 +578,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_line_q ( Gan_Camera *camera,
+ gan_camera_project_line_q ( const Gan_Camera *camera,
                              Gan_Vector3 *L, Gan_Vector3 *l );
 #else
 #define gan_camera_project_line_q(camera,L,l)\
@@ -554,7 +603,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_project_line_i ( Gan_Camera *camera, Gan_Vector3 *L );
+ gan_camera_project_line_i ( const Gan_Camera *camera, Gan_Vector3 *L );
 #else
 #define gan_camera_project_line_i(camera,L)\
            gan_camera_project_line_q(camera,L,L)
@@ -581,7 +630,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_backproject_line_q ( Gan_Camera *camera,
+ gan_camera_backproject_line_q ( const Gan_Camera *camera,
                                  Gan_Vector3 *l, Gan_Vector3 *L );
 #else
 #define gan_camera_backproject_line_q(camera,l,L)\
@@ -611,7 +660,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_camera_backproject_line_i ( Gan_Camera *camera, Gan_Vector3 *l );
+ gan_camera_backproject_line_i ( const Gan_Camera *camera, Gan_Vector3 *l );
 #else
 #define gan_camera_backproject_line_i(camera,l)\
            gan_camera_backproject_line_q(camera,l,l)
