@@ -3,8 +3,8 @@
  * Module:        Functions for camera with 2 radial distortion parameters
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.8 $
- * Last edited:   $Date: 2003/08/28 09:12:40 $
+ * Revision:      $Revision: 1.11 $
+ * Last edited:   $Date: 2004/06/07 09:55:21 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -62,7 +62,7 @@
 
 /* point projection function */
 static Gan_Bool
- point_project ( Gan_Camera *camera,
+ point_project ( const Gan_Camera *camera,
                  Gan_Vector3 *X, Gan_Vector3 *p,
                  Gan_Matrix22 *HX, Gan_Camera HC[2],
                  int *error_code )
@@ -136,10 +136,22 @@ static Gan_Bool
 
       if ( HC != NULL )
       {
-         gan_camera_build_radial_distortion_2 ( &HC[0],
-                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 );
-         gan_camera_build_radial_distortion_2 ( &HC[1],
-                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 );
+         HC[0].zh = 0.0;
+         HC[0].fx = 0.0;
+         HC[0].fy = 0.0;
+         HC[0].x0 = 0.0;
+         HC[0].y0 = 0.0;
+         HC[0].type = GAN_RADIAL_DISTORTION_2;
+         HC[0].nonlinear.radial2.K1 = 0.0;
+         HC[0].nonlinear.radial2.K2 = 0.0;
+         HC[1].zh = 0.0;
+         HC[1].fx = 0.0;
+         HC[1].fy = 0.0;
+         HC[1].x0 = 0.0;
+         HC[1].y0 = 0.0;
+         HC[1].type = GAN_RADIAL_DISTORTION_2;
+         HC[1].nonlinear.radial2.K1 = 0.0;
+         HC[1].nonlinear.radial2.K2 = 0.0;
       }
 
       /* build homogeneous image coordinates of projected point */
@@ -167,10 +179,22 @@ static Gan_Bool
 
    if ( HC != NULL )
    {
-      gan_camera_build_radial_distortion_2 ( &HC[0],
-                          0.0,  d*XZ, 0.0, 1.0, 0.0, CFX*XZ*R2, CFX*XZ*R2*R2 );
-      gan_camera_build_radial_distortion_2 ( &HC[1],
-                          0.0, 0.0,  d*YZ, 0.0, 1.0, CFY*YZ*R2, CFY*YZ*R2*R2 );
+      HC[0].zh = 0.0;
+      HC[0].fx = d*XZ;
+      HC[0].fy = 0.0;
+      HC[0].x0 = 1.0;
+      HC[0].y0 = 0.0;
+      HC[0].type = GAN_RADIAL_DISTORTION_2;
+      HC[0].nonlinear.radial2.K1 = CFX*XZ*R2;
+      HC[0].nonlinear.radial2.K2 = CFX*XZ*R2*R2;
+      HC[1].zh = 0.0;
+      HC[1].fx = 0.0;
+      HC[1].fy = d*YZ;
+      HC[1].x0 = 0.0;
+      HC[1].y0 = 1.0;
+      HC[1].type = GAN_RADIAL_DISTORTION_2;
+      HC[1].nonlinear.radial2.K1 = CFY*YZ*R2;
+      HC[1].nonlinear.radial2.K2 = CFY*YZ*R2*R2;
    }
 
    /* build homogeneous image coordinates of projected point */
@@ -285,7 +309,7 @@ static Gan_Bool
 
 /* point back-projection function */
 static Gan_Bool
- point_backproject ( Gan_Camera *camera,
+ point_backproject ( const Gan_Camera *camera,
                      Gan_Vector3 *p, Gan_Vector3 *X,
                      int *error_code )
 {
@@ -348,7 +372,7 @@ static Gan_Bool
 
 /* function to add distortion to a point */
 static Gan_Bool
- point_add_distortion ( Gan_Camera *camera,
+ point_add_distortion ( const Gan_Camera *camera,
                         Gan_Vector3 *pu, Gan_Vector3 *p,
                         int *error_code )
 {
@@ -418,7 +442,7 @@ static Gan_Bool
    
 /* function to remove distortion from a point */
 static Gan_Bool
- point_remove_distortion ( Gan_Camera *camera,
+ point_remove_distortion ( const Gan_Camera *camera,
                            Gan_Vector3 *p, Gan_Vector3 *pu,
                            int *error_code)
 {
@@ -461,7 +485,7 @@ static Gan_Bool
 
 /* line projection function */
 static Gan_Bool
- line_project ( Gan_Camera *camera,
+ line_project ( const Gan_Camera *camera,
                 Gan_Vector3 *L, Gan_Vector3 *l )
 {
    Gan_Vector3 line;
@@ -495,7 +519,7 @@ static Gan_Bool
    
 /* line back-projection function */
 static Gan_Bool
- line_backproject ( Gan_Camera *camera, Gan_Vector3 *l, Gan_Vector3 *L )
+ line_backproject ( const Gan_Camera *camera, Gan_Vector3 *l, Gan_Vector3 *L )
 {
    Gan_Vector3 Line;
 
@@ -549,11 +573,13 @@ Gan_Bool
                                         double x0, double y0,
                                         double K1, double K2 )
 {
-   /* copy calibration type and linear camera parameters */
-   camera->type = GAN_RADIAL_DISTORTION_2;
-   camera->zh = zh; /* 3rd homogeneous image coordinate */
-   camera->fx = fx; camera->fy = fy; /* focal distances */
-   camera->x0 = x0; camera->y0 = y0; /* image centre coordinates */
+   if ( !gan_camera_set_common_fields ( camera, GAN_RADIAL_DISTORTION_2,
+                                        zh, fx, fy, x0, y0 ) )
+   {
+      gan_err_register ( "gan_camera_build_radial_distortion_2",
+                         GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
 
    /* copy non-linear camera parameters */
    CK1 = K1; /* radial distortion cubic term */
@@ -608,9 +634,6 @@ Gan_Bool
       COB = CTDR - COA*R;
    }
 
-   /* default gamma value */
-   camera->gamma = 1.0;
-
    /* add point projection/backprojection/distortion functions */
    camera->point.project           = point_project;
    camera->point.backproject       = point_backproject;
