@@ -3,8 +3,8 @@
  * Module:        Camera definitions and building functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.10 $
- * Last edited:   $Date: 2003/08/05 09:57:09 $
+ * Revision:      $Revision: 1.18 $
+ * Last edited:   $Date: 2005/04/22 13:16:23 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -47,6 +47,28 @@ extern "C" {
  * \{
  */
 
+/**
+ * /brief Radial basis function structure.
+ */
+typedef struct Gan_RadialBasisFunction_f
+{
+   /// Position of centre
+   float x, y;
+
+   /// Squared radius of radial basis function
+   float dist2_thres;
+
+   /// Weight attached to radial basis function
+   float weight;
+
+   /// Value used to normalise function
+   float normaliser;
+
+   /// Distortion vector
+   float dx, dy;
+
+} Gan_RadialBasisFunction_f;
+
 /**
  * \brief Structure containing camera parameters in single precision.
  */
@@ -58,7 +80,7 @@ typedef struct Gan_Camera_f
    /// parameters of linear camera
 
    /// focal distance in x/y pixels
-   float fx, fy;
+   float fx, fy, fy_over_fx;
 
    /// image centre x/y coordinates
    float x0, y0;
@@ -115,32 +137,47 @@ typedef struct Gan_Camera_f
       } radial3;
 
       struct { float cxx, cxy, cyx, cyy; } xydist4;
-   } nonlinear;
 
-   /// gamma value of images taken using this camera
-   float gamma;
+      struct
+      {
+         float xla, xlp, xha, xhp;
+         float yla, ylp, yha, yhp;
+      } fcwarp;
+
+      struct
+      {
+         /// skew and lower triangle components store in a scale-independent manner
+         float skew, kyx, kzx, kzy;
+
+         /// non-triangular camera matrix and its inverse
+         Gan_Matrix33_f K, Kinv;
+
+         /// Cubic B-spline parameters
+         Gan_Vector2_f weight[1 + (1 << GAN_CUBIC_BSPLINE_CAMERA_LEVEL)][1 + (1 << GAN_CUBIC_BSPLINE_CAMERA_LEVEL)];
+      } cbspline;
+   } nonlinear;
 
    /// point functions
    struct
    {
       /// point projection function
-      Gan_Bool (*project) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*project) ( const struct Gan_Camera_f *camera,
                             Gan_Vector3_f *X, Gan_Vector3_f *p,
                             Gan_Matrix22_f *HX, struct Gan_Camera_f *HC,
                             int *error_code );
 
       /// point back-projection function
-      Gan_Bool (*backproject) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*backproject) ( const struct Gan_Camera_f *camera,
                                 Gan_Vector3_f *p, Gan_Vector3_f *X,
                                 int *error_code );
 
       /// function to add distortion to a point
-      Gan_Bool (*add_distortion) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*add_distortion) ( const struct Gan_Camera_f *camera,
                                    Gan_Vector3_f *pu, Gan_Vector3_f *p,
                                    int *error_code );
 
       /// function to remove distortion from a point
-      Gan_Bool (*remove_distortion) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*remove_distortion) ( const struct Gan_Camera_f *camera,
                                       Gan_Vector3_f *p, Gan_Vector3_f *pu,
                                       int *error_code);
    } pointf;
@@ -149,11 +186,11 @@ typedef struct Gan_Camera_f
    struct
    {
       /// line projection function
-      Gan_Bool (*project) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*project) ( const struct Gan_Camera_f *camera,
                             Gan_Vector3_f *L, Gan_Vector3_f *l );
 
       /// line back-projection function
-      Gan_Bool (*backproject) ( struct Gan_Camera_f *camera,
+      Gan_Bool (*backproject) ( const struct Gan_Camera_f *camera,
                                 Gan_Vector3_f *l, Gan_Vector3_f *L );
    } linef;
 } Gan_Camera_f;
@@ -163,8 +200,14 @@ Gan_Bool gan_cameraf_build_va ( Gan_Camera_f *camera, Gan_CameraType type,
                                 float fx, float fy, float x0, float y0,
                                 ... );
 Gan_Bool gan_cameraf_internalize ( Gan_Camera_f *camera );
-Gan_Bool gan_cameraf_set_gamma ( Gan_Camera_f *camera, float gamma );
-Gan_SquMatrix33_f gan_cameraf_fill_matrix_s ( Gan_Camera_f *camera );
+Gan_Bool gan_cameraf_identical ( Gan_Camera_f *camera1, Gan_Camera_f *pcamera2 );
+Gan_SquMatrix33_f gan_cameraf_fill_matrix_s ( const Gan_Camera_f *camera );
+Gan_Matrix33_f    gan_cameraf_fill_fullmatrix_s ( const Gan_Camera_f *camera );
+
+Gan_Bool gan_cameraf_set_common_fields ( Gan_Camera_f *camera,
+                                         Gan_CameraType type,
+                                         float zh, float fx, float fy,
+                                         float x0, float y0 );
 
 /**
  * \brief Macro: Projects a point from the scene onto the image.
@@ -191,7 +234,7 @@ Gan_SquMatrix33_f gan_cameraf_fill_matrix_s ( Gan_Camera_f *camera );
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_point_gen ( Gan_Camera_f *camera,
+ gan_cameraf_project_point_gen ( const Gan_Camera_f *camera,
                                  Gan_Vector3_f *X, Gan_Vector3_f *p,
                                  Gan_Matrix22_f *HX, Gan_Camera_f HC[2],
                                  int *error_code );
@@ -218,7 +261,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_point ( Gan_Camera_f *camera,
+ gan_cameraf_project_point ( const Gan_Camera_f *camera,
                              Gan_Vector3_f *X, Gan_Vector3_f *p,
                              int *error_code );
 #else
@@ -246,7 +289,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_point_q ( Gan_Camera_f *camera,
+ gan_cameraf_project_point_q ( const Gan_Camera_f *camera,
                                Gan_Vector3_f *X, Gan_Vector3_f *p );
 #else
 #define gan_cameraf_project_point_q(camera,X,p)\
@@ -272,7 +315,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_point_i ( Gan_Camera_f *camera, Gan_Vector3_f *X );
+ gan_cameraf_project_point_i ( const Gan_Camera_f *camera, Gan_Vector3_f *X );
 #else
 #define gan_cameraf_project_point_i(camera,X)\
            gan_cameraf_project_point_q(camera,X,X)
@@ -299,7 +342,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_backproject_point ( Gan_Camera_f *camera,
+ gan_cameraf_backproject_point ( const Gan_Camera_f *camera,
                                  Gan_Vector3_f *p, Gan_Vector3_f *X,
                                  int *error_code );
 #else
@@ -329,7 +372,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_backproject_point_q ( Gan_Camera_f *camera,
+ gan_cameraf_backproject_point_q ( const Gan_Camera_f *camera,
                                    Gan_Vector3_f *p, Gan_Vector3_f *X );
 #else
 #define gan_cameraf_backproject_point_q(camera,p,X)\
@@ -357,7 +400,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_backproject_point_i ( Gan_Camera_f *camera,
+ gan_cameraf_backproject_point_i ( const Gan_Camera_f *camera,
                                    Gan_Vector3_f *p );
 #else
 #define gan_cameraf_backproject_point_i(camera,p)\
@@ -382,7 +425,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_add_distortion ( Gan_Camera_f *camera,
+ gan_cameraf_add_distortion ( const Gan_Camera_f *camera,
                               Gan_Vector3_f *pu, Gan_Vector3_f *p,
                               int *error_code );
 #else
@@ -409,7 +452,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_add_distortion_q ( Gan_Camera_f *camera,
+ gan_cameraf_add_distortion_q ( const Gan_Camera_f *camera,
                                 Gan_Vector3_f *pu, Gan_Vector3_f *p );
 #else
 #define gan_cameraf_add_distortion_q(camera,pu,p)\
@@ -433,7 +476,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_add_distortion_i ( Gan_Camera_f *camera, Gan_Vector3_f *p );
+ gan_cameraf_add_distortion_i ( const Gan_Camera_f *camera, Gan_Vector3_f *p );
 #else
 #define gan_cameraf_add_distortion_i(camera,p)\
            gan_cameraf_add_distortion_q(camera,p,p)
@@ -455,7 +498,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_remove_distortion ( Gan_Camera_f *camera,
+ gan_cameraf_remove_distortion ( const Gan_Camera_f *camera,
                                  Gan_Vector3_f *p, Gan_Vector3_f *pu,
                                  int *error_code );
 #else
@@ -481,7 +524,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_remove_distortion_q ( Gan_Camera_f *camera,
+ gan_cameraf_remove_distortion_q ( const Gan_Camera_f *camera,
                                    Gan_Vector3_f *p, Gan_Vector3_f *pu );
 #else
 #define gan_cameraf_remove_distortion_q(camera,p,pu)\
@@ -505,7 +548,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_remove_distortion_i ( Gan_Camera_f *camera, Gan_Vector3_f *p );
+ gan_cameraf_remove_distortion_i ( const Gan_Camera_f *camera, Gan_Vector3_f *p );
 #else
 #define gan_cameraf_remove_distortion_i(camera,p) \
            gan_cameraf_remove_distortion_q(camera,p,p)
@@ -529,7 +572,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_line_q ( Gan_Camera_f *camera,
+ gan_cameraf_project_line_q ( const Gan_Camera_f *camera,
                               Gan_Vector3_f *L, Gan_Vector3_f *l );
 #else
 #define gan_cameraf_project_line_q(camera,L,l)\
@@ -554,7 +597,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_project_line_i ( Gan_Camera_f *camera, Gan_Vector3_f *L );
+ gan_cameraf_project_line_i ( const Gan_Camera_f *camera, Gan_Vector3_f *L );
 #else
 #define gan_cameraf_project_line_i(camera,L)\
            gan_cameraf_project_line_q(camera,L,L)
@@ -581,7 +624,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_backproject_line_q ( Gan_Camera_f *camera,
+ gan_cameraf_backproject_line_q ( const Gan_Camera_f *camera,
                                   Gan_Vector3_f *l, Gan_Vector3_f *L );
 #else
 #define gan_cameraf_backproject_line_q(camera,l,L)\
@@ -611,7 +654,7 @@ Gan_Bool
  */
 #ifdef GAN_GENERATE_DOCUMENTATION
 Gan_Bool
- gan_cameraf_backproject_line_i ( Gan_Camera_f *camera, Gan_Vector3_f *l );
+ gan_cameraf_backproject_line_i ( const Gan_Camera_f *camera, Gan_Vector3_f *l );
 #else
 #define gan_cameraf_backproject_line_i(camera,l)\
            gan_cameraf_backproject_line_q(camera,l,l)
