@@ -3,8 +3,8 @@
  * Module:        Functions for camera with 3 radial distortion parameters
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.11 $
- * Last edited:   $Date: 2003/09/03 08:19:23 $
+ * Revision:      $Revision: 1.14 $
+ * Last edited:   $Date: 2004/06/07 09:55:23 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -63,7 +63,7 @@
 
 /* point projection function */
 static Gan_Bool
- point_project ( Gan_Camera_f *camera,
+ point_project ( const Gan_Camera_f *camera,
                  Gan_Vector3_f *X, Gan_Vector3_f *p,
                  Gan_Matrix22_f *HX, Gan_Camera_f HC[2],
                  int *error_code )
@@ -132,17 +132,29 @@ static Gan_Bool
       x.y = CY0 + d*CFY*YZ;
 
       /* compute Jacobians */
-      if ( HX != NULL || HC != NULL )
-      {
-         if ( error_code == NULL )
-         {
-            gan_err_flush_trace();
-            gan_err_register ( "point_project", GAN_ERROR_INCOMPATIBLE, "" );
-         }
-         else
-            *error_code = GAN_ERROR_INCOMPATIBLE;
+      if ( HX != NULL )
+         (void)gan_mat22f_zero_q(HX);
 
-         return GAN_FALSE;
+      if ( HC != NULL )
+      {
+         HC[0].zh = 0.0F;
+         HC[0].fx = 0.0F;
+         HC[0].fy = 0.0F;
+         HC[0].x0 = 0.0F;
+         HC[0].y0 = 0.0F;
+         HC[0].type = GAN_RADIAL_DISTORTION_3;
+         HC[0].nonlinear.radial3.K1 = 0.0F;
+         HC[0].nonlinear.radial3.K2 = 0.0F;
+         HC[0].nonlinear.radial3.K3 = 0.0F;
+         HC[1].zh = 0.0F;
+         HC[1].fx = 0.0F;
+         HC[1].fy = 0.0F;
+         HC[1].x0 = 0.0F;
+         HC[1].y0 = 0.0F;
+         HC[1].type = GAN_RADIAL_DISTORTION_3;
+         HC[1].nonlinear.radial3.K1 = 0.0F;
+         HC[1].nonlinear.radial3.K2 = 0.0F;
+         HC[1].nonlinear.radial3.K3 = 0.0F;
       }
 
       /* build homogeneous image coordinates of projected point */
@@ -166,14 +178,24 @@ static Gan_Bool
 
    if ( HC != NULL )
    {
-      gan_cameraf_build_radial_distortion_3 ( &HC[0],
-                                              0.0F, d*XZ, 0.0F, 1.0F, 0.0F,
-                                              CFX*XZ*R2, CFX*XZ*R2*R2,
-                                              CFX*XZ*R2*R2*R2 );
-      gan_cameraf_build_radial_distortion_3 ( &HC[1],
-                                              0.0F, 0.0F, d*YZ, 0.0F, 1.0F,
-                                              CFY*YZ*R2, CFY*YZ*R2*R2,
-                                              CFY*YZ*R2*R2*R2 );
+      HC[0].zh = 0.0F;
+      HC[0].fx = d*XZ;
+      HC[0].fy = 0.0F;
+      HC[0].x0 = 1.0F;
+      HC[0].y0 = 0.0F;
+      HC[0].type = GAN_RADIAL_DISTORTION_3;
+      HC[0].nonlinear.radial3.K1 = CFX*XZ*R2;
+      HC[0].nonlinear.radial3.K2 = CFX*XZ*R2*R2;
+      HC[0].nonlinear.radial3.K3 = CFX*XZ*R2*R2*R2;
+      HC[1].zh = 0.0F;
+      HC[1].fx = 0.0F;
+      HC[1].fy = d*YZ;
+      HC[1].x0 = 0.0F;
+      HC[1].y0 = 1.0F;
+      HC[1].type = GAN_RADIAL_DISTORTION_3;
+      HC[1].nonlinear.radial3.K1 = CFY*YZ*R2;
+      HC[1].nonlinear.radial3.K2 = CFY*YZ*R2*R2;
+      HC[1].nonlinear.radial3.K3 = CFY*YZ*R2*R2*R2;
    }
 
    /* build homogeneous image coordinates of projected point */
@@ -285,7 +307,7 @@ static Gan_Bool
 
 /* point back-projection function */
 static Gan_Bool
- point_backproject ( Gan_Camera_f *camera,
+ point_backproject ( const Gan_Camera_f *camera,
                      Gan_Vector3_f *p, Gan_Vector3_f *X,
                      int *error_code )
 {
@@ -348,7 +370,7 @@ static Gan_Bool
 
 /* function to add distortion to a point */
 static Gan_Bool
- point_add_distortion ( Gan_Camera_f *camera,
+ point_add_distortion ( const Gan_Camera_f *camera,
                         Gan_Vector3_f *pu, Gan_Vector3_f *p,
                         int *error_code )
 {
@@ -418,7 +440,7 @@ static Gan_Bool
    
 /* function to remove distortion from a point */
 static Gan_Bool
- point_remove_distortion ( Gan_Camera_f *camera,
+ point_remove_distortion ( const Gan_Camera_f *camera,
                            Gan_Vector3_f *p, Gan_Vector3_f *pu,
                            int *error_code)
 {
@@ -461,7 +483,7 @@ static Gan_Bool
 
 /* line projection function */
 static Gan_Bool
- line_project ( Gan_Camera_f *camera,
+ line_project ( const Gan_Camera_f *camera,
                 Gan_Vector3_f *L, Gan_Vector3_f *l )
 {
    Gan_Vector3_f line;
@@ -495,7 +517,7 @@ static Gan_Bool
    
 /* line back-projection function */
 static Gan_Bool
- line_backproject ( Gan_Camera_f *camera, Gan_Vector3_f *l, Gan_Vector3_f *L )
+ line_backproject ( const Gan_Camera_f *camera, Gan_Vector3_f *l, Gan_Vector3_f *L )
 {
    Gan_Vector3_f Line;
 
@@ -554,11 +576,13 @@ Gan_Bool
    Gan_Complex x[3];
    double minR2 = -1.0;
 
-   /* copy calibration type and linear camera parameters */
-   camera->type = GAN_RADIAL_DISTORTION_3;
-   camera->zh = zh; /* 3rd homogeneous image coordinate */
-   camera->fx = fx; camera->fy = fy; /* focal distances */
-   camera->x0 = x0; camera->y0 = y0; /* image centre coordinates */
+   if ( !gan_cameraf_set_common_fields ( camera, GAN_RADIAL_DISTORTION_3,
+                                         zh, fx, fy, x0, y0 ) )
+   {
+      gan_err_register ( "gan_cameraf_build_radial_distortion_3",
+                         GAN_ERROR_FAILURE, "" );
+      return GAN_FALSE;
+   }
 
    /* copy non-linear camera parameters */
    camera->nonlinear.radial3.K1 = K1; /* radial distortion cubic term */
@@ -601,9 +625,6 @@ Gan_Bool
    else
       CTR2 = CTDR = COA = COB = FLT_MAX;
 
-   /* default gamma value */
-   camera->gamma = 1.0F;
-
    /* add point projection/backprojection/distortion functions */
    camera->pointf.project           = point_project;
    camera->pointf.backproject       = point_backproject;
