@@ -3,8 +3,8 @@
  * Module:        Plessey corner detector algorithm
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.25 $
- * Last edited:   $Date: 2003/08/26 13:48:50 $
+ * Revision:      $Revision: 1.26 $
+ * Last edited:   $Date: 2004/05/19 09:43:40 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -184,6 +184,7 @@ static Gan_Image *
 
          /* smooth the image */
          smooth_y = gan_image_convolve1Dy_s ( image, GAN_ALL_CHANNELS,
+                                              GAN_EDGE_BEHAVIOUR_CLIP,
                                               image_filter_y );
          if ( smooth_y == NULL )
          {
@@ -203,6 +204,7 @@ static Gan_Image *
 
          /* smooth the image */
          smooth_x = gan_image_convolve1Dx_s ( image, GAN_ALL_CHANNELS,
+                                              GAN_EDGE_BEHAVIOUR_CLIP,
                                               image_filter_x );
          if ( smooth_x == NULL )
          {
@@ -221,6 +223,7 @@ static Gan_Image *
 
          /* smooth in x-direction */
          smooth_x = gan_image_convolve1Dx_s ( image, GAN_ALL_CHANNELS,
+                                              GAN_EDGE_BEHAVIOUR_CLIP,
                                               image_filter_x );
          if ( smooth_x == NULL )
          {
@@ -230,6 +233,7 @@ static Gan_Image *
 
          /* smooth in y-direction */
          smooth_xy = gan_image_convolve1Dy_s ( smooth_x, GAN_ALL_CHANNELS,
+                                               GAN_EDGE_BEHAVIOUR_CLIP,
                                                image_filter_y );
          if ( smooth_xy == NULL )
          {
@@ -502,15 +506,21 @@ Gan_CornerFeatureMap *
       field image separately to save some memory */
 
    /* smoothing the X plane allocates the smoothed gradient images */
-   imbf  = gan_image_convolve1Dx_s ( grad, GAN_X_CHANNEL, grad_filter_x );
-   smgxy = gan_image_convolve1Dy_s ( imbf, GAN_X_CHANNEL, grad_filter_y );
+   imbf  = gan_image_convolve1Dx_s ( grad, GAN_X_CHANNEL,
+                                     GAN_EDGE_BEHAVIOUR_CLIP, grad_filter_x );
+   smgxy = gan_image_convolve1Dy_s ( imbf, GAN_X_CHANNEL,
+                                     GAN_EDGE_BEHAVIOUR_CLIP, grad_filter_y );
 
    /* smooth the Y and Z planes */
-   gan_image_convolve1Dx_q ( grad, GAN_Y_CHANNEL, grad_filter_x, imbf );
-   gan_image_convolve1Dy_q ( imbf, GAN_Y_CHANNEL, grad_filter_y, smgxy );
+   gan_image_convolve1Dx_q ( grad, GAN_Y_CHANNEL, GAN_EDGE_BEHAVIOUR_CLIP,
+                             grad_filter_x, imbf );
+   gan_image_convolve1Dy_q ( imbf, GAN_Y_CHANNEL, GAN_EDGE_BEHAVIOUR_CLIP,
+                             grad_filter_y, smgxy );
 
-   gan_image_convolve1Dx_q ( grad, GAN_Z_CHANNEL, grad_filter_x, imbf );
-   gan_image_convolve1Dy_q ( imbf, GAN_Z_CHANNEL, grad_filter_y, smgxy );
+   gan_image_convolve1Dx_q ( grad, GAN_Z_CHANNEL, GAN_EDGE_BEHAVIOUR_CLIP,
+                             grad_filter_x, imbf );
+   gan_image_convolve1Dy_q ( imbf, GAN_Z_CHANNEL, GAN_EDGE_BEHAVIOUR_CLIP,
+                             grad_filter_y, smgxy );
 
    /* free gradient image */
    gan_image_free ( grad );
