@@ -3,8 +3,8 @@
  * Module:        Levenberg-Marquardt non-linear minimisation algorithm
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.16 $
- * Last edited:   $Date: 2002/05/16 08:43:38 $
+ * Revision:      $Revision: 1.17 $
+ * Last edited:   $Date: 2005/05/21 22:49:00 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -437,11 +437,10 @@ static double
  *
  * After passing some observations to the structure using gan_lev_marq_obs_h()
  * or gan_lev_marq_obs_F(), this function invokes the callback function
- * #init_func to set the state to initial values, before applying
+ * \a init_func to set the state to initial values, before applying
  * Levenberg-Marquardt iterations using gan_lev_marq_iteration().
  *
- * \return #GAN_TRUE on successful initialisation of structure, #GAN_FALSE
- *         on failure.
+ * \return #GAN_TRUE on successful initialisation of structure, #GAN_FALSE on failure.
  * \sa gan_lev_marq_iteration().
  */
 Gan_Bool
@@ -474,12 +473,10 @@ Gan_Bool
  * actually applied to the state vector \f$x\f$ if the residual has been
  * reduced from the existing value.
  *
- * \return #GAN_TRUE on successful completion of iteration, #GAN_FALSE on
- *         failure.
+ * \return #GAN_TRUE on successful completion of iteration, #GAN_FALSE on failure.
  */
 Gan_Bool
- gan_lev_marq_iteration ( Gan_LevMarqStruct *lm,
-                          double lambda, double *residualp )
+ gan_lev_marq_iteration ( Gan_LevMarqStruct *lm, double lambda, double *residualp )
 {
    int i;
    Gan_LevMarqObs *obs;
@@ -617,16 +614,9 @@ Gan_Bool
       }
    }
 
-#if 1
    /* add lambda*I to A */
    for ( i = (int)lm->x.rows-1; i >= 0; i-- )
       gan_squmat_inc_el ( &lm->A, i, i, lambda );
-#else
-   /* scale diagonal of A by 1+lambda */
-   for ( i = (int)lm->x.rows-1; i >= 0; i-- )
-      gan_squmat_set_el ( &lm->A, i, i,
-                          (1.0+lambda)*gan_squmat_get_el(&lm->A,i,i) );
-#endif
 
    /* compute Cholesky factorisation of the information matrix */
    if ( gan_squmat_cholesky_i ( &lm->A ) == NULL )
