@@ -3,9 +3,9 @@
  * Module:        Local blocked feature map definition & functions
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.21 $
- * Last edited:   $Date: 2003/08/26 13:48:51 $
- * Author:        $Author: pm $
+ * Revision:      $Revision: 1.22 $
+ * Last edited:   $Date: 2004/11/11 14:02:11 $
+ * Author:        $Author: jps $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
 
@@ -87,7 +87,7 @@ Gan_LocalFeatureMap *
    {
       gan_err_test_ptr ( pms->xstep != 0, "gan_local_feature_map_form",
                          GAN_ERROR_INCOMPATIBLE, "step parameters" );
-      gan_err_test_bool ( pms->bsize % 2 == 1,
+      gan_err_test_ptr ( pms->bsize % 2 == 1,
                           "gan_local_feature_map_form",GAN_ERROR_INCOMPATIBLE,
                           "local feature pmap catchment area should be odd");
       xblocks = (width + pms->xstep - 1)/pms->xstep + pms->bsize - 1;
