@@ -3,8 +3,8 @@
  * Module:        Vision package test program
  * Part of:       Gandalf Library
  *
- * Revision:      $Revision: 1.30 $
- * Last edited:   $Date: 2003/09/09 16:07:45 $
+ * Revision:      $Revision: 1.35 $
+ * Last edited:   $Date: 2004/09/30 11:19:53 $
  * Author:        $Author: pm $
  * Copyright:     (c) 2000 Imagineer Software Limited
  */
@@ -31,8 +31,12 @@
 #include <gandalf/vision/mask1D.h>
 #include <gandalf/vision/harris_corner.h>
 #include <gandalf/vision/canny_edge.h>
+#include <gandalf/vision/segment_strings.h>
 #include <gandalf/vision/orthog_line.h>
 #include <gandalf/vision/lev_marq.h>
+#include <gandalf/vision/essential.h>
+#include <gandalf/vision/fundamental.h>
+#include <gandalf/vision/camera_linear.h>
 #include <gandalf/linalg/mat_symmetric.h>
 #include <gandalf/linalg/3x3matrix.h>
 #include <gandalf/linalg/3vector.h>
@@ -409,6 +413,75 @@ static Gan_Bool lev_marq_test(void)
    return GAN_TRUE;
 }
 
+#define ESSENTIAL_TEST_ANGLE  0.1
+#define ESSENTIAL_TEST_AXIS_X 0.0
+#define ESSENTIAL_TEST_AXIS_Y 1.0
+#define ESSENTIAL_TEST_AXIS_Z 0.0
+#define ESSENTIAL_TEST_TX     3.0
+#define ESSENTIAL_TEST_TY     0.0
+#define ESSENTIAL_TEST_TZ     0.0
+#define ESSENTIAL_TEST_NO_POINTS 100
+
+/* test essential matrix algorithm */
+static Gan_Bool essential_matrix_test()
+{
+   Gan_Euclid3D pose, result;
+   Gan_Camera camera1, camera2;
+   int i;
+   Gan_Vector3 X[ESSENTIAL_TEST_NO_POINTS];
+   Gan_Vector3 p1[ESSENTIAL_TEST_NO_POINTS], p2[ESSENTIAL_TEST_NO_POINTS];
+
+   /* set up ground truth pose */
+   cu_assert(gan_rot3D_build_angle_axis(&pose.rot,
+                                        ESSENTIAL_TEST_ANGLE,
+                                        ESSENTIAL_TEST_AXIS_X,
+                                        ESSENTIAL_TEST_AXIS_Y,
+                                        ESSENTIAL_TEST_AXIS_Z));
+   (void)gan_vec3_fill_q(&pose.trans,
+                         ESSENTIAL_TEST_TX,
+                         ESSENTIAL_TEST_TY,
+                         ESSENTIAL_TEST_TZ);
+
+   /* now build cameras */
+   cu_assert(gan_camera_build_linear(&camera1, 100.0, 50.0, 50.0, 10.0, 10.0));
+   cu_assert(gan_camera_build_linear(&camera2, 150.0, 10.0, 100.0, 20.0, -20.0));
+
+   /* now build 3D points */
+   for(i=ESSENTIAL_TEST_NO_POINTS-1; i >= 0; i--)
+      (void) gan_vec3_fill_q(&X[i],
+                             20.0*gan_random_m11(),
+                             20.0*gan_random_m11(),
+                             50.0 + 20.0*gan_random_01());
+
+   /* project the points onto the images */
+   for(i=ESSENTIAL_TEST_NO_POINTS-1; i >= 0; i--)
+   {
+      /* first camera */
+      cu_assert(gan_camera_project_point_q(&camera1, &X[i], &p1[i]));
+
+      /* second camera */
+      p2[i] = gan_vec3_sub_s(&X[i], &pose.trans);
+      p2[i] = gan_rot3D_apply_v3_s(&pose.rot, &p2[i]);
+      if(p2[i].z <= 0.0)
+         printf("Here\n");
+
+      cu_assert(gan_camera_project_point_i(&camera2, &p2[i]));
+   }
+
+   /* now compute essential matrix */
+   cu_assert(gan_essential_matrix_fit(p1, p2, ESSENTIAL_TEST_NO_POINTS, &camera1, &camera2, NULL, &result));
+
+   result.rot = gan_rot3D_convert_s(&result.rot, GAN_ROT3D_ANGLE_AXIS);
+
+   return GAN_TRUE;
+}
+
+/* test fundamental matrix algorithm */
+static Gan_Bool fundamental_matrix_test()
+{
+   return GAN_TRUE;
+}
+
 static Gan_Bool setup_test(void)
 {
    /* set default Gandalf error handler without trace handling */
@@ -434,6 +507,12 @@ static Gan_Bool run_test(void)
    /* test Levenberg-Marquardt algorithm */
    cu_assert ( lev_marq_test() );
 
+   /* test essential matrix algorithm */
+   cu_assert ( essential_matrix_test() );
+
+   /* test fundamental matrix algorithm */
+   cu_assert ( fundamental_matrix_test() );
+
    /* read image from file */
    img_rgb = gan_image_read ( image_file, file_format, NULL );
    cu_assert(img_rgb != NULL);
@@ -448,8 +527,8 @@ static Gan_Bool run_test(void)
    cu_assert ( mask != NULL );
 
    /* compute edges of the image */
-   emap = gan_canny_edge_s ( img_grey, NULL, mask, mask, 0.008, 0.024, 10,
-                             NULL, NULL, &lpms );
+   emap = gan_canny_edge_s ( img_grey, NULL, mask, mask, NULL, NULL,
+                             0.008, 0.024, 10, NULL, NULL, &lpms, NULL, NULL );
    cu_assert ( emap != NULL );
 
    /* compute corners of the image */
@@ -458,19 +537,24 @@ static Gan_Bool run_test(void)
    cu_assert ( cmap != NULL );
 
    /* compute lines of the image */
-   lmap = gan_orthog_line_s ( emap, 10, 2, 1.0, &lpms, GAN_TRUE );
+   lmap = gan_orthog_line_s ( emap, 10, 2, 1.0, &lpms, GAN_TRUE, NULL, NULL );
    cu_assert ( lmap != NULL );
 
+#if 0
+   /* segment the edge strings */
+   cu_assert ( gan_segment_strings_q ( emap, 9, 0.5F, 10 ) );
+#endif
+
    /* free convolution mask */
    gan_mask1D_free ( mask );
 
-   /* number of edges is 22465 on all architectures tested so far */
+   /* number of edges is 22389 on all architectures tested so far */
    cu_assert ( emap->nedges >= 22384 && emap->nedges <= 22394 );
 
    /* number of corners is 1546 on all architectures tested so far */
    cu_assert ( cmap->ncorners >= 1534 && cmap->ncorners <= 1540 );
 
-   /* number of lines found in Linux & Windows is 678, but 679 on IRIX! */
+   /* number of lines found is 679 */
    cu_assert ( lmap->nlines >= 676 && lmap->nlines <= 682 );
 
 #ifdef VISION_TEST_MAIN
